#ifndef lint
static char const 
yyrcsid[] = "$FreeBSD: src/usr.bin/yacc/skeleton.c,v 1.28 2000/01/17 02:04:06 bde Exp $";
#endif
#include <stdlib.h>
#define YYBYACC 1
#define YYMAJOR 1
#define YYMINOR 9
#define YYLEX yylex()
#define YYEMPTY -1
#define yyclearin (yychar=(YYEMPTY))
#define yyerrok (yyerrflag=0)
#define YYRECOVERING() (yyerrflag!=0)
static int yygrowstack();
#define YYPREFIX "yy"
#line 14 "parse.y"

#define YYDEBUG 1

#include "ruby.h"
#include "env.h"
#include "intern.h"
#include "node.h"
#include "st.h"
#include <stdio.h>
#include <errno.h>
#include <ctype.h>

#define yyparse ruby_yyparse
#define yylex ruby_yylex
#define yyerror ruby_yyerror
#define yylval ruby_yylval
#define yychar ruby_yychar
#define yydebug ruby_yydebug

#define ID_SCOPE_SHIFT 3
#define ID_SCOPE_MASK 0x07
#define ID_LOCAL    0x01
#define ID_INSTANCE 0x02
#define ID_GLOBAL   0x03
#define ID_ATTRSET  0x04
#define ID_CONST    0x05
#define ID_CLASS    0x06
#define ID_JUNK     0x07
#define ID_INTERNAL ID_JUNK

#define is_notop_id(id) ((id)>tLAST_TOKEN)
#define is_local_id(id) (is_notop_id(id)&&((id)&ID_SCOPE_MASK)==ID_LOCAL)
#define is_global_id(id) (is_notop_id(id)&&((id)&ID_SCOPE_MASK)==ID_GLOBAL)
#define is_instance_id(id) (is_notop_id(id)&&((id)&ID_SCOPE_MASK)==ID_INSTANCE)
#define is_attrset_id(id) (is_notop_id(id)&&((id)&ID_SCOPE_MASK)==ID_ATTRSET)
#define is_const_id(id) (is_notop_id(id)&&((id)&ID_SCOPE_MASK)==ID_CONST)
#define is_class_id(id) (is_notop_id(id)&&((id)&ID_SCOPE_MASK)==ID_CLASS)
#define is_junk_id(id) (is_notop_id(id)&&((id)&ID_SCOPE_MASK)==ID_JUNK)

#define is_asgn_or_id(id) ((is_notop_id(id)) && \
	(((id)&ID_SCOPE_MASK) == ID_GLOBAL || \
	 ((id)&ID_SCOPE_MASK) == ID_INSTANCE || \
	 ((id)&ID_SCOPE_MASK) == ID_CLASS))

NODE *ruby_eval_tree_begin = 0;
NODE *ruby_eval_tree = 0;

char *ruby_sourcefile;		/* current source file */
int   ruby_sourceline;		/* current line no. */

static int yylex();
static int yyerror();

static enum lex_state {
    EXPR_BEG,			/* ignore newline, +/- is a sign. */
    EXPR_END,			/* newline significant, +/- is a operator. */
    EXPR_ARG,			/* newline significant, +/- is a operator. */
    EXPR_CMDARG,		/* newline significant, +/- is a operator. */
    EXPR_ENDARG,		/* newline significant, +/- is a operator. */
    EXPR_MID,			/* newline significant, +/- is a operator. */
    EXPR_FNAME,			/* ignore newline, no reserved words. */
    EXPR_DOT,			/* right after `.' or `::', no reserved words. */
    EXPR_CLASS,			/* immediate after `class', no here document. */
} lex_state;
static NODE *lex_strterm;

#ifdef HAVE_LONG_LONG
typedef unsigned LONG_LONG stack_type;
#else
typedef unsigned long stack_type;
#endif

#define BITSTACK_PUSH(stack, n)	(stack = (stack<<1)|((n)&1))
#define BITSTACK_POP(stack)	(stack >>= 1)
#define BITSTACK_LEXPOP(stack)	(stack = (stack >> 1) | (stack & 1))
#define BITSTACK_SET_P(stack)	(stack&1)

static stack_type cond_stack = 0;
#define COND_PUSH(n)	BITSTACK_PUSH(cond_stack, n)
#define COND_POP()	BITSTACK_POP(cond_stack)
#define COND_LEXPOP()	BITSTACK_LEXPOP(cond_stack)
#define COND_P()	BITSTACK_SET_P(cond_stack)

static stack_type cmdarg_stack = 0;
#define CMDARG_PUSH(n)	BITSTACK_PUSH(cmdarg_stack, n)
#define CMDARG_POP()	BITSTACK_POP(cmdarg_stack)
#define CMDARG_LEXPOP()	BITSTACK_LEXPOP(cmdarg_stack)
#define CMDARG_P()	BITSTACK_SET_P(cmdarg_stack)

static int class_nest = 0;
static int in_single = 0;
static int in_def = 0;
static int compile_for_eval = 0;
static ID cur_mid = 0;

static NODE *cond();
static NODE *logop();
static int cond_negative();

static NODE *newline_node();
static void fixpos();

static int value_expr0();
static void void_expr0();
static void void_stmts();
static NODE *remove_begin();
#define value_expr(node) value_expr0((node) = remove_begin(node))
#define void_expr(node) void_expr0((node) = remove_begin(node))

static NODE *block_append();
static NODE *list_append();
static NODE *list_concat();
static NODE *arg_concat();
static NODE *arg_prepend();
static NODE *literal_concat();
static NODE *new_evstr();
static NODE *evstr2dstr();
static NODE *call_op();
static int in_defined = 0;

static NODE *negate_lit();
static NODE *ret_args();
static NODE *arg_blk_pass();
static NODE *new_call();
static NODE *new_fcall();
static NODE *new_super();
static NODE *new_yield();

static NODE *gettable();
static NODE *assignable();
static NODE *aryset();
static NODE *attrset();
static void rb_backref_error();
static NODE *node_assign();

static NODE *match_gen();
static void local_push();
static void local_pop();
static int  local_append();
static int  local_cnt();
static int  local_id();
static ID  *local_tbl();
static ID   internal_id();

static struct RVarmap *dyna_push();
static void dyna_pop();
static int dyna_in_block();
static NODE *dyna_init();

static void top_local_init();
static void top_local_setup();

#define RE_OPTION_ONCE 0x80

#define NODE_STRTERM NODE_ZARRAY	/* nothing to gc */
#define NODE_HEREDOC NODE_ARRAY 	/* 1, 3 to gc */
#define SIGN_EXTEND(x,n) (((1<<(n))-1-((x)&~(~0<<(n))))^~(~0<<(n)))
#define nd_func u1.id
#if SIZEOF_SHORT == 2
#define nd_term(node) ((signed short)(node)->u2.id)
#else
#define nd_term(node) SIGN_EXTEND((node)->u2.id, CHAR_BIT*2)
#endif
#define nd_paren(node) (char)((node)->u2.id >> CHAR_BIT*2)
#define nd_nest u3.id

/* Older versions of Yacc set YYMAXDEPTH to a very low value by default (150,
   for instance).  This is too low for Ruby to parse some files, such as
   date/format.rb, therefore bump the value up to at least Bison's default. */
#ifdef OLD_YACC
#ifndef YYMAXDEPTH
#define YYMAXDEPTH 10000
#endif
#endif

#line 191 "parse.y"
typedef union {
    NODE *node;
    ID id;
    int num;
    struct RVarmap *vars;
} YYSTYPE;
#line 200 "y.tab.c"
#define YYERRCODE 256
#define kCLASS 257
#define kMODULE 258
#define kDEF 259
#define kUNDEF 260
#define kBEGIN 261
#define kRESCUE 262
#define kENSURE 263
#define kEND 264
#define kIF 265
#define kUNLESS 266
#define kTHEN 267
#define kELSIF 268
#define kELSE 269
#define kCASE 270
#define kWHEN 271
#define kWHILE 272
#define kUNTIL 273
#define kFOR 274
#define kBREAK 275
#define kNEXT 276
#define kREDO 277
#define kRETRY 278
#define kIN 279
#define kDO 280
#define kDO_COND 281
#define kDO_BLOCK 282
#define kRETURN 283
#define kYIELD 284
#define kSUPER 285
#define kSELF 286
#define kNIL 287
#define kTRUE 288
#define kFALSE 289
#define kAND 290
#define kOR 291
#define kNOT 292
#define kIF_MOD 293
#define kUNLESS_MOD 294
#define kWHILE_MOD 295
#define kUNTIL_MOD 296
#define kRESCUE_MOD 297
#define kALIAS 298
#define kDEFINED 299
#define klBEGIN 300
#define klEND 301
#define k__LINE__ 302
#define k__FILE__ 303
#define tIDENTIFIER 304
#define tFID 305
#define tGVAR 306
#define tIVAR 307
#define tCONSTANT 308
#define tCVAR 309
#define tINTEGER 310
#define tFLOAT 311
#define tSTRING_CONTENT 312
#define tNTH_REF 313
#define tBACK_REF 314
#define tREGEXP_END 315
#define tUPLUS 316
#define tUMINUS 317
#define tPOW 318
#define tCMP 319
#define tEQ 320
#define tEQQ 321
#define tNEQ 322
#define tGEQ 323
#define tLEQ 324
#define tANDOP 325
#define tOROP 326
#define tMATCH 327
#define tNMATCH 328
#define tDOT2 329
#define tDOT3 330
#define tAREF 331
#define tASET 332
#define tLSHFT 333
#define tRSHFT 334
#define tCOLON2 335
#define tCOLON3 336
#define tOP_ASGN 337
#define tASSOC 338
#define tLPAREN 339
#define tLPAREN_ARG 340
#define tRPAREN 341
#define tLBRACK 342
#define tLBRACE 343
#define tLBRACE_ARG 344
#define tSTAR 345
#define tAMPER 346
#define tSYMBEG 347
#define tSTRING_BEG 348
#define tXSTRING_BEG 349
#define tREGEXP_BEG 350
#define tWORDS_BEG 351
#define tQWORDS_BEG 352
#define tSTRING_DBEG 353
#define tSTRING_DVAR 354
#define tSTRING_END 355
#define tLOWEST 356
#define tUMINUS_NUM 357
#define tLAST_TOKEN 358
const short yylhs[] = {                                        -1,
   95,    0,   19,   20,   21,   21,   21,   21,   98,   22,
   22,   22,   22,   22,   22,   22,   22,   22,   22,   99,
   22,   22,   22,   22,   22,   22,   22,   22,   22,   22,
   22,   22,   22,   22,   23,   23,   23,   23,   23,   23,
   29,   27,   27,   27,   27,   27,   56,   56,   56,  100,
  101,   72,   26,   26,   26,   26,   26,   26,   26,   26,
   76,   76,   79,   79,   78,   78,   78,   78,   78,   78,
   80,   80,   77,   77,   81,   81,   81,   81,   81,   81,
   81,   81,   74,   74,   74,   74,   74,   74,   74,   74,
   89,   89,   18,   18,   18,   90,   90,   90,   90,   90,
   82,   82,   66,  103,   66,   91,   91,   91,   91,   91,
   91,   91,   91,   91,   91,   91,   91,   91,   91,   91,
   91,   91,   91,   91,   91,   91,   91,   91,   91,   91,
   91,  102,  102,  102,  102,  102,  102,  102,  102,  102,
  102,  102,  102,  102,  102,  102,  102,  102,  102,  102,
  102,  102,  102,  102,  102,  102,  102,  102,  102,  102,
  102,  102,  102,  102,  102,  102,  102,  102,  102,  102,
  102,  102,   24,   24,   24,   24,   24,   24,   24,   24,
   24,   24,   24,   24,   24,   24,   24,   24,   24,   24,
   24,   24,   24,   24,   24,   24,   24,   24,   24,   24,
   24,   24,   24,   24,   24,   24,   24,   24,   24,   24,
   24,   24,   24,  105,   24,   24,   24,   30,   48,   48,
   48,   48,   48,   48,   45,   45,   45,   45,   46,   46,
   42,   42,   42,   42,   42,   42,   42,   42,   42,   43,
   43,   43,   43,   43,   43,   43,   43,   43,   43,   43,
   43,  107,   47,   44,  108,   44,  109,   44,   50,   49,
   49,   40,   40,   53,   53,   53,   25,   25,   25,   25,
   25,   25,   25,   25,   25,  110,   25,  111,   25,   25,
   25,   25,   25,   25,   25,   25,   25,   25,   25,  112,
   25,   25,   25,   25,   25,   25,  114,  116,   25,  117,
  118,   25,   25,   25,   25,   25,  119,  120,   25,  121,
   25,  123,  124,   25,  125,   25,  126,   25,  128,  129,
   25,   25,   25,   25,   25,   31,  113,  113,  113,  113,
  115,  115,  115,   32,   32,   33,   33,   69,   69,   70,
   70,   70,   70,  130,  131,   73,   55,   55,   55,   28,
   28,   28,   28,   28,   28,  132,  133,   71,  134,  135,
   71,   34,   41,   41,   41,   35,   35,   36,   36,   37,
   37,   37,   38,   38,   39,   39,   15,   15,   15,    2,
    3,    3,    4,    5,    6,   10,   10,   12,   12,   14,
   14,   11,   11,   13,   13,    7,    7,    8,    8,    9,
  136,    9,  137,    9,   68,   68,   68,   68,   85,   84,
   84,   84,   84,   17,   16,   16,   16,   16,   83,   83,
   83,   83,   83,   83,   83,   83,   83,   83,   83,   51,
   52,   67,   67,   54,  138,   54,   54,   57,   57,   58,
   58,   58,   58,   58,   58,   58,   58,   58,   93,   93,
   93,   93,   93,   94,   94,   60,   59,   59,  139,  139,
   92,   92,  140,  140,   61,   62,   62,    1,  141,    1,
   63,   63,   63,   64,   64,   65,   86,   86,   86,   87,
   87,   87,   87,   88,   88,   88,  127,  127,   96,   96,
  104,  104,  106,  106,  106,  122,  122,   97,   97,   75,
};
const short yylen[] = {                                         2,
    0,    2,    4,    2,    1,    1,    3,    2,    0,    4,
    3,    3,    3,    2,    3,    3,    3,    3,    3,    0,
    5,    4,    3,    3,    3,    6,    5,    5,    5,    3,
    3,    3,    3,    1,    1,    3,    3,    2,    2,    1,
    1,    1,    1,    2,    2,    2,    1,    4,    4,    0,
    0,    6,    2,    3,    4,    5,    4,    5,    2,    2,
    1,    3,    1,    3,    1,    2,    3,    2,    2,    1,
    1,    3,    2,    3,    1,    4,    3,    3,    3,    3,
    2,    1,    1,    4,    3,    3,    3,    3,    2,    1,
    1,    1,    2,    1,    3,    1,    1,    1,    1,    1,
    1,    1,    1,    0,    4,    1,    1,    1,    1,    1,
    1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
    1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
    1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
    1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
    1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
    1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
    1,    1,    3,    5,    3,    6,    5,    5,    5,    5,
    4,    3,    3,    3,    3,    3,    3,    3,    3,    3,
    4,    4,    2,    2,    3,    3,    3,    3,    3,    3,
    3,    3,    3,    3,    3,    3,    3,    2,    2,    3,
    3,    3,    3,    0,    4,    5,    1,    1,    1,    2,
    2,    5,    2,    3,    3,    4,    4,    6,    1,    1,
    1,    2,    5,    2,    5,    4,    7,    3,    1,    4,
    3,    5,    7,    2,    5,    4,    6,    7,    9,    3,
    1,    0,    2,    1,    0,    3,    0,    4,    2,    2,
    1,    1,    3,    3,    4,    2,    1,    1,    1,    1,
    1,    1,    1,    1,    1,    0,    4,    0,    5,    3,
    3,    2,    4,    3,    3,    1,    4,    3,    1,    0,
    6,    2,    1,    2,    6,    6,    0,    0,    7,    0,
    0,    7,    5,    6,    4,    5,    0,    0,    9,    0,
    6,    0,    0,    8,    0,    5,    0,    6,    0,    0,
    9,    1,    1,    1,    1,    1,    1,    1,    1,    2,
    1,    1,    1,    1,    5,    1,    2,    1,    1,    1,
    2,    1,    3,    0,    0,    6,    2,    4,    4,    2,
    4,    4,    3,    2,    1,    0,    0,    6,    0,    0,
    6,    5,    1,    4,    2,    1,    1,    6,    1,    1,
    1,    1,    2,    1,    2,    1,    1,    1,    1,    1,
    1,    2,    3,    3,    3,    3,    3,    0,    3,    1,
    2,    3,    3,    0,    3,    0,    2,    0,    2,    1,
    0,    3,    0,    4,    1,    1,    1,    1,    2,    1,
    1,    1,    1,    3,    1,    1,    2,    2,    1,    1,
    1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
    1,    1,    1,    1,    0,    4,    2,    4,    2,    6,
    4,    4,    2,    4,    2,    2,    1,    0,    1,    1,
    1,    1,    1,    1,    3,    3,    1,    3,    1,    1,
    2,    1,    1,    1,    2,    2,    1,    1,    0,    5,
    1,    2,    2,    1,    3,    3,    1,    1,    1,    1,
    1,    1,    1,    1,    1,    1,    1,    1,    0,    1,
    0,    1,    0,    1,    1,    1,    1,    1,    2,    0,
};
const short yydefred[] = {                                      1,
    0,    0,    0,    0,    0,    0,    0,  276,    0,    0,
    0,  297,  300,    0,    0,    0,  324,  325,    0,    0,
    0,  425,  424,  426,  427,    0,    0,    0,   20,    0,
  429,  428,    0,    0,  421,  420,    0,  423,  415,  416,
  432,  433,    0,    0,    0,    0,    0,    0,    0,    0,
    0,  396,  398,  398,    0,    0,    0,    0,    0,  268,
    0,  381,  269,  270,  271,  272,  267,  377,  379,    2,
    0,    0,    0,    0,    0,    0,   35,    0,    0,  273,
    0,    0,   43,    0,    0,    5,    0,    0,   61,    0,
   71,    0,  378,    0,    0,  322,  323,  286,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,  326,    0,
  274,  430,    0,   94,  315,  141,  152,  142,  165,  138,
  158,  148,  147,  163,  146,  145,  140,  166,  150,  139,
  153,  157,  159,  151,  144,  160,  167,  162,    0,    0,
    0,    0,  137,  156,  155,  168,  169,  170,  171,  172,
  136,  143,  134,  135,    0,    0,    0,   98,    0,  127,
  128,  125,  109,  110,  111,  114,  116,  112,  129,  130,
  117,  118,  122,  113,  115,  106,  107,  108,  119,  120,
  121,  123,  124,  126,  131,  469,    0,  468,  317,   99,
  100,  161,  154,  164,  149,  132,  133,   96,   97,    0,
    0,  103,  102,  101,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,  497,  496,    0,    0,    0,  498,
    0,    0,    0,    0,    0,    0,  338,  339,    0,    0,
    0,    0,    0,  231,    0,    0,   45,  239,    0,  474,
   46,   44,    0,   60,    0,    0,  354,   59,   38,    0,
    9,  492,    0,    0,    0,  193,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,  219,
    0,    0,    0,  471,    0,    0,    0,   69,    0,  412,
  411,  413,    0,  409,  410,    0,    0,    0,    0,    0,
    0,    0,    0,    0,  208,   39,  209,  382,    4,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,  344,  347,  359,  356,  294,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
   73,  350,    0,  292,    0,    0,   91,    0,   93,  417,
  418,    0,  435,  310,  434,    0,    0,    0,    0,    0,
  488,  487,  319,    0,  104,    0,    0,    0,  329,  328,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,  499,    0,    0,    0,    0,    0,    0,
  307,    0,  259,    0,    0,  232,  261,    0,  234,  288,
    0,    0,  254,  253,    0,    0,    0,    0,    0,   11,
   13,   12,    0,  290,    0,    0,    0,    0,    0,    0,
    0,  280,    0,    0,    0,  220,    0,  494,  221,  284,
    0,  223,    0,  473,  285,  472,    0,    0,    0,    0,
  400,  403,  401,  414,  399,  383,  397,  384,  385,  386,
  387,  390,    0,  392,    0,  393,    0,    0,    0,   15,
   16,   17,   18,   19,   36,   37,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,  353,
    0,    0,    0,  482,    0,    0,  483,    0,   25,  480,
  481,    0,    0,    0,   30,    0,    0,   23,  262,    0,
   31,   24,    0,   33,    0,   67,   74,   50,   54,    0,
  437,    0,    0,    0,    0,    0,   95,    0,    0,    0,
    0,    0,    0,  451,  450,  449,  452,  460,  464,  463,
  459,    0,    0,    0,    0,  457,  447,    0,  454,    0,
    0,    0,    0,  277,    0,    0,  369,    0,  330,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,  305,  333,  332,  298,  331,  301,    0,    0,
    0,    0,    0,    0,    0,  238,  476,    0,    0,  260,
    0,    0,    0,  475,  287,    0,    0,    0,  257,  251,
    0,    0,    0,    0,    0,    0,    0,  225,   10,    0,
    0,    0,   22,    0,    0,    0,  224,    0,  263,    0,
    0,    0,    0,    0,    0,    0,  389,  391,  395,    0,
    0,    0,  342,    0,  345,  340,  360,  357,    0,    0,
  352,    0,    0,    0,    0,  230,  351,    0,  229,  349,
   49,  348,   48,  266,    0,    0,   72,    0,  313,    0,
    0,  283,  316,    0,  320,    0,    0,    0,  439,    0,
  445,  467,    0,  446,    0,  443,  461,  465,  105,    0,
    0,  371,  372,    0,    0,  336,    0,    0,  334,    0,
    0,    0,    0,    0,    0,  303,  306,  365,    0,    0,
    0,    0,    0,    0,    0,    0,  236,    0,    0,    0,
    0,    0,  244,  256,    0,    0,  226,    0,    0,  227,
    0,   21,  279,    0,    0,    0,  405,  406,  407,  408,
  402,    0,  341,    0,    0,    0,    0,    0,   29,    0,
   58,    0,    0,   27,    0,   28,   56,    0,    0,    0,
   51,    0,  436,  311,  470,    0,  456,    0,  318,    0,
  458,  466,    0,    0,    0,  455,    0,    0,  374,  337,
    0,    3,  376,    0,  295,    0,  296,  304,    0,    0,
    0,    0,  308,  233,    0,  235,  250,    0,    0,  241,
    0,  258,    0,    0,  291,  222,  404,  343,    0,    0,
    0,    0,   26,  265,    0,    0,    0,  438,  444,    0,
  441,  442,    0,    0,    0,  373,    0,    0,  375,    0,
  364,  366,  367,  362,  299,  302,    0,    0,    0,    0,
  240,    0,  246,    0,  228,  346,  361,  358,    0,  314,
    0,    0,    0,    0,    0,    0,    0,    0,    0,  237,
  242,    0,    0,    0,  245,   52,  321,  440,    0,    0,
    0,    0,    0,  368,  335,  309,    0,    0,  247,    0,
    0,  243,    0,  248,    0,  249,
};
const short yydgoto[] = {                                       1,
  187,   60,   61,   62,   63,   64,  286,  283,  455,   65,
   66,  290,  292,  463,   67,   68,   69,  108,  376,  377,
   71,   72,   73,   74,   75,   76,   77,   78,  208,  235,
  257,  708,  709,  393,  844,  576,  701,  788,  792,  267,
  592,  237,  619,  414,  666,  667,  244,  268,  406,  610,
   80,  210,  531,  364,   82,   83,  563,  564,  565,  566,
  782,  691,  272,  269,  240,  201,  211,  751,  226,  655,
  354,  539,  335,  212,   86,   87,   88,   89,  263,   90,
   91,  202,  213,  284,   93,  113,  546,  510,  114,  204,
  517,  568,  569,  570,    2,  218,  219,  423,  254,  678,
  825,  191,  573,  253,  425,  439,  245,  622,  731,  205,
  434,  630,  381,  221,  596,  721,  222,  722,  604,  847,
  543,  382,  540,  772,  369,  374,  373,  552,  776,  503,
  755,  505,  757,  504,  756,  646,  645,  542,  571,  572,
  370,
};
const short yysindex[] = {                                      0,
    0,13838,14260,18736,19027,17365,17066,    0,15624,15624,
 7014,    0,    0,18833,14460,14460,    0,    0,14460,  108,
  111,    0,    0,    0,    0,15624,16975,   60,    0,   61,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,16497,16497, -201,14058,15624,15721,16497,19124,
17445,    0,    0,    0,  167,  192, -154,16594,16497,    0,
 -109,    0,    0,    0,    0,    0,    0,    0,    0,    0,
   96,  744,  -22,17116,    0,  -18,    0,  -35,   23,    0,
  -60,   63,    0,  -53,  227,    0,  229,18930,    0,  276,
    0,    0,    0,   68,  744,    0,    0,    0,  108,  111,
   60,    0,    0,15624, -102,13838,  248,   36,    0,   71,
    0,    0,   68,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,   97,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,17445,
  293,    0,    0,    0,13838,   31,  -22,  513,   74,    8,
   41,  375,    0,  513,    0,    0,   96,   92,  383,    0,
15624,15624,  143,   78,    0,  175,    0,    0,    0,16497,
16497,16497,17116,    0,  119,  429,    0,    0,  433,    0,
    0,    0,14363,    0,14557,14460,    0,    0,    0, -168,
    0,    0,  441,  385,13838,    0,   95,  194,  202,14058,
  514,    0,  539,  -22,16497,   60,  232,  483,  439,    0,
  465,  422,  439,    0,  278,  120,    0,    0,    0,    0,
    0,    0,  398,    0,    0,  434,  473,  605,  237,  581,
  241, -185,  294,  316,    0,    0,    0,    0,    0,14159,
15624,15624,15624,15624,14260,15624,15624,16497,16497,16497,
16497,16497,16497,16497,16497,16497,16497,16497,16497,16497,
16497,16497,16497,16497,16497,16497,16497,16497,16497,16497,
16497,16497,16497,    0,    0,    0,    0,    0,17491,15721,
17790,16594,17829,17829,16594,15818,15818,14058,19124,  608,
    0,    0,  314,    0,  441,  -22,    0,    0,    0,    0,
    0,   96,    0,    0,    0,17860,15721,17829,13838,15624,
    0,    0,    0,  245,    0,  395,  403,  202,    0,    0,
13838,  400,18159,15721,18190,16497,16497,16497,13838,  275,
13838,15915,  406,    0,   57,   57,    0,18228,15721,18259,
    0,  630,    0,16497,14654,    0,    0,14751,    0,    0,
  638, 5885,    0,    0,  -18,  636,   60,   56,  647,    0,
    0,    0,17066,    0,16497,13838,  564,18159,18190,16497,
  649,    0,    0,   60, 5337,    0,16012,    0,    0,    0,
16497,    0,16497,    0,    0,    0,    0,18297,15721,18328,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,   58,    0,  659,    0,16497,16497,  744,    0,
    0,    0,    0,    0,    0,    0,  194, 1435, 1435, 1435,
 1435, 1218, 1218, 8413, 7928, 1435, 1435, 1789, 1789,  206,
  206, 5735, 1218, 1218,  491,  491,  158,  171,  171,  194,
  194,  194,  -56,  -56,  -56,  359,    0,  363,  111,    0,
    0,  609,  366,    0,  368,  111,    0,17116,    0,    0,
    0,  111,  111,17116,    0,16497, 9112,    0,    0,  672,
    0,    0,    0,    0,  677,    0,    0,    0,    0,   96,
    0,15624,13838,    0,    0,  111,    0,  632,  111,  463,
  197,17744,  673,    0,    0,    0,    0,    0,    0,    0,
    0,  152,13838,   96,  698,    0,    0,  704,    0,  706,
  460,  469,17066,    0,16109,  500,    0,  299,    0,  437,
  363,  682,  440,  442, 9112,  500,13838,  518,  522,16497,
  734,  513,    0,    0,    0,    0,    0,    0,    0,    0,
  703,    0,    0,15624,  445,    0,    0,16497,  119,    0,
  758,16497,  119,    0,    0,16497,17116,    9,    0,    0,
  761,  765,14848,  767,17829,17829,  769,    0,    0,15624,
17116,  689,    0,17116,    0,  774,    0,16497,    0,    0,
    0,  740,    0,    0,13838,  561,    0,    0,    0,  194,
  194,16497,    0,18502,    0,    0,    0,    0,16594,16497,
    0,  314,  504,16594,16594,    0,    0,  314,    0,    0,
    0,    0,    0,    0,16497,16206,    0,  -56,    0,   96,
  585,    0,    0,  811,    0,16497,   60,  589,    0,  121,
    0,    0,  -25,    0,  152,    0,    0,    0,    0,    0,
  517,    0,    0,13838,  606,    0,15624,  613,    0,16497,
  543,16497,16497,  620,  627,    0,    0,    0,16303,13838,
13838,13838,    0,   57,  630,14945,    0,  630,  630,15042,
  848,15139,    0,    0,  -18,   56,    0,  111,  111,    0,
  188,    0,    0, 5337,    0,  770,    0,    0,    0,    0,
    0,17116,    0,  772,13838,13838,13838,17116,    0,17116,
    0,16594,17116,    0,17116,    0,    0,17116,16497,    0,
    0,13838,    0,    0,    0,  245,    0,  867,    0,  673,
    0,    0,  704,  870,  704,    0,19221,  513,    0,    0,
13838,    0,    0,  513,    0,16497,    0,    0,16497,  331,
  658,  661,    0,    0,16497,    0,    0,16497,  883,    0,
  886,    0,16497,  891,    0,    0,    0,    0,  669,  686,
  816,17116,    0,    0,13838,  690,13838,    0,    0,  121,
    0,    0,  648,  130,    0,    0,    0,13838,    0,13838,
    0,    0,    0,    0,    0,    0,13838,13842,  630,15236,
    0,15333,    0,  630,    0,    0,    0,    0,  830,    0,
  693,  704,    0,18627,15721,18658,  403,  299,  700,    0,
    0,16497,  919,16497,    0,    0,    0,    0,    0,    0,
  882,    0,    0,    0,    0,    0,  630,15430,    0,  630,
    0,    0,16497,    0,  630,    0,
};
const short yyrindex[] = {                                      0,
    0,  579,    0,    0,    0,    0,    0,    0,    0,    0,
  347,    0,    0,    0,13048,13133,    0,    0,13229, 4390,
 3865,    0,    0,    0,    0,    0,    0,16400,    0,    0,
    0,    0, 2153, 3106,    0,    0, 2250,    0,    0,    0,
    0,    0,    0,    0,    0,  573,    0,  884,  853,  225,
  641,    0,    0,    0,  662, -184,    0,    0,    0,    0,
 8300,    0,    0,    0,    0,    0,    0,    0,    0,    0,
 1310, 1994, 2084, 6263, 8385, 9601,    0, 8481,    0,    0,
    0,13240,    0,13604,    0,    0,    0,  377,    0,    0,
    0,13528,    0,15527, 5633,    0,    0,    0, 8785, 7332,
  941, 5679, 5992,    0,    0,  573,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,  139, 1014,
 1060, 1479,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0, 1546, 1596, 1654,    0, 1663,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
 6200,    0,    0,    0,  567,    0, 7223,    0,    0,    0,
 7815,    0, 7421,    0,    0,    0,  347,    0,  382,    0,
    0,    0,    0,    0,  748,    0,    0,    0,  855,    0,
    0,    0, 6853,    0,12727, 8115,    0,    0, 8115,    0,
    0,    0,    0,    0,    0,  939,    0,    0,    0,    0,
    0,    0,16691,    0,  506,    0,    0, 9270,13678,  573,
    0,   39,    0,  132,    0,  890,  892,    0,  892,    0,
  862,    0,  862,    0,    0,    0,  602,    0, 1002,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0, 8870, 8966,    0,    0,    0,    0,    0, 1616,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,  884,
    0,    0,    0,    0,    0,    0,    0,  573,  449,  454,
    0,    0, 6365,    0,    0,  102,    0, 6509,    0,    0,
    0,    0,    0,    0,    0,    0,  884,    0,  567,    0,
    0,    0,    0,  113,    0,    0,  364, 7900,    0,    0,
  535, 6615,    0,  884,    0,    0,    0,    0,  123,    0,
  124,    0,    0,    0,    0,    0,  858,    0,  884,    0,
    0, 8115,    0,    0,    0,    0,    0,    0,    0,    0,
    0,  947,    0,    0,  148,  250,  952,  952,    0,    0,
    0,    0,    0,    0,    0,  506,    0,    0,    0,    0,
    0,    0,  611,  952,  890,    0,  909,    0,    0,    0,
   42,    0,  880,    0,    0,    0, 1189,    0,  884,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0, 6465,    0,
    0,    0,    0,    0,    0,    0, 9355,11179,11268,11358,
11452,10812,10906,11541,11814,11631,11725,11904,11940,10328,
10417,    0,10995,11085,10633,10722,10507,10113,10202, 9450,
 9692, 9781, 6729, 6729, 6900, 4721, 3437, 5246,15527,    0,
 3534,    0, 4818,    0, 5149, 3962,    0,12000,    0,    0,
    0, 5577, 5577,12087,    0,    0, 9657,    0,    0,    0,
    0,    0,  903,    0,    0,    0,    0,    0,    0,    0,
    0,    0,  567, 6094, 6407,    0,    0,    0, 7511,    0,
  952,    0,  699,    0,    0,    0,    0,    0,    0,    0,
    0,  430,  567,    0,  573,    0,    0,  573,    0,  573,
  835,    0,    0,    0,   15,  342,    0,  728,    0, 2581,
 4293,    0, 2678, 3009,12788,  728,  124,    0,    0,    0,
  540,    0,    0,    0,    0,    0,    0,    0,  563,  575,
    0, 1672, 1938,    0,    0,    0,    0,    0,12941,    0,
 8115,    0,    0,    0,    0,    0,   38,    0,    0,    0,
  939,    0,    0,    0,    0,    0,    0,    0,    0,    0,
12147,    0,    0,12215,  722,    0,    0,    0,    0,  899,
 2035,    0, 1500, 1904,  506,    0,    0,    0,    0, 9871,
10023,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0, 8600,13977,    0,    0,    0,    0, 9085,    0,    0,
    0,    0,    0,    0,    0,    0,    0, 6900,    0,    0,
    0,    0,    0,    0,    0,    0,  952,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,   67,
  625,    0,    0,  230,  728,    0,    0,    0,    0,    0,
 7996,    0,    0,    0,    0,    0,    0,    0,    0,  199,
  124,  124,  940,    0, 8115,    0,    0, 8115,  939,    0,
    0,    0,    0,    0,    0,  952,    0,   30,   30,    0,
    0,    0,    0,  890, 1223,    0,    0,    0,    0,    0,
    0,12270,    0,    0,  124,  124,  506,12331,    0,12422,
    0,    0,12513,    0,12574,    0,    0,12629,    0, 1750,
    0,  567,    0,    0,    0,  113,    0,    0,    0,    0,
    0,    0,  573,  573,  573,    0,    0,    0,    0,    0,
  124,    0,    0,    0,    0,    0,    0,    0,    0,  728,
    0,    0,    0,    0,    0,    0,    0,    0,  939,    0,
  939,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,12666,    0,    0,  506,    0,  567,    0,    0,    0,
    0,    0,    0,    0,  129,    0,  134,  567,    0,  535,
    0,    0,    0,    0,    0,    0,  124, 8115,  939,    0,
    0,    0,    0,  939,    0,    0,    0,    0,    0,    0,
    0,  573,  164,    0,  884,    0,  364,  728,    0,    0,
    0,    0,  939,    0,    0,    0,    0,    0,  571,   73,
    0,  709,  951,    0,    0,    0,  939,    0,    0,  939,
  168,    0,    0,    0,  939,    0,
};
const short yygindex[] = {                                      0,
    0,    0,    0,  946,    0,    0,    0,  586,  -49,    0,
    0,    0,    0,    0,    0,    0,    0, 1013, -331,   91,
    0,   75,  697,  -15,   10,    1,  -23,    0,   53, 1516,
  516,  151, -426, -371,    0,  153,    0,    0,    0,   35,
    0,  486,    0,    0,   -3,   18,   -9, -295,  223,   11,
 1015, 1330, -242,    0, -224,    0,  249,  464,  334, -568,
 -343, -512,    0,  -10, -366,    0, 1085,    0,  376, -352,
  967, -532,    0, 1302,  875,   -7,    0,   19, -208,  961,
  -13,  -16, 1139,    0,   14,  666, -282,    0,  -24,    4,
   -4, -596,  356,    0,    0,   50,  982,    0,    0,    0,
    0,    0,    0,  -14,    0,   -8,    0,    0,    0,    0,
    0,    0, -206,    0, -360,    0,    0,    0,    0,    0,
    0,   21,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,
};
#define YYTABLESIZE 19578
const short yytable[] = {                                     233,
  233,  190,  190,  233,  239,  239,  228,  389,  239,  189,
  251,  248,  560,  109,  109,  234,  234,  247,  588,  234,
  203,  418,  190,  109,  500,  238,  238,  256,  258,  238,
  567,  220,  233,  233,  296,  598,  278,  550,  273,  500,
  203,  614,  295,  297,  512,  215,  190,   40,  266,  236,
  236,  431,  730,  236,  285,  694,  509,  696,  516,  109,
  522,  523,  214,  217,  262,  252,  215,  654,  341,  252,
  500,  548,  500,  500,  614,  500,  370,   95,   40,   63,
  359,  218,   88,  271,  353,  549,  355,  337,  582,  647,
  352,  220,   70,  783,  216,  363,  247,  109,  785,   61,
  509,  626,  516,  601,  534,  215,  259,  246,  344,  352,
  262,  312,  481,  340,  595,  216,  368,  549,  281,  385,
  299,  781,  448,  400,  370,  370,  465,  394,  365,  761,
   88,   88,  500,  500,  495,  767,  261,  420,   90,  535,
  429,  278,  372,   83,  421,  422,  549,  243,  161,  705,
  246,  657,  658,  642,  216,  293,  294,  231,  560,  714,
  312,  367,  561,  281,  384,  450,  495,  549,  399,  466,
  394,  448,  278,   89,  274,  866,  161,   84,  161,  430,
  161,  500,  500,  255,  425,  384,   90,   90,  231,  560,
  337,   83,   83,  561,  333,  190,  261,  161,  289,  331,
  329,  357,  330,  285,  332,  358,  252,  333,  500,  282,
  449,  681,  331,  283,  233,  233,  295,  332,  567,  274,
  865,   89,   89,  291,  430,   84,   84,  233,  815,  233,
  233,  688,  239,  862,  239,  239,  457,  220,   52,  500,
  462,  438,  333,  234,  336,  234,  415,  331,  329,  435,
  330,  436,  332,  238,  282,  238,  238,  500,  283,  500,
  442,  781,  444,  334,  446,   70,  390,  306,  307,  653,
  829,  831,  832,  395,  396,  437,  342,  236,  262,  236,
  416,  500,  560,  345,  562,   70,  561,  346,  500,  347,
  500,  362,  477,  478,  479,  480,  481,  482,  483,  484,
  485,  486,  487,  488,  489,  490,  491,  492,  493,  494,
  495,  496,  497,  498,  499,  500,  501,  502,  519,  351,
  559,  525,  528,  532,  233,  771,  518,   40,   40,  524,
  527,  233,  702,  370,  511,  536,  375,  594,  378,   88,
  266,  547,  738,  739,  386,  427,  404,  336,   70,  878,
  261,  233,  500,  470,  471,  472,  473,  339,  109,  614,
  391,  511,  392,  803,  500,  614,  262,  266,  233,  451,
  518,  524,  585,  842,  469,  218,  233,  387,  511,  474,
  530,  530,  541,  233,  266,  720,  500,  500,  233,  233,
  625,  500,  233,  511,  611,   90,  617,  343,  736,  266,
   83,  621,  624,  627,  370,  366,  629,  281,  383,  631,
  452,  453,  398,  648,  634,  597,  597,   65,  190,  636,
  637,  233,  620,  511,  780,  233,  591,  233,  843,  428,
   89,  371,  567,  233,   84,  388,  203,   65,  261,  448,
  826,  394,  161,  511,  161,  161,  161,  161,  438,  266,
  397,  650,  651,  401,  448,  553,  404,  554,  555,  556,
  557,  409,  500,  274,  864,  558,  559,  500,  430,  500,
  448,  578,  405,  425,  438,  308,  408,  306,  307,  586,
  424,  589,  441,  161,  161,  614,  306,  307,  308,   68,
  321,  322,  500,  500,   66,  861,  558,  559,  282,  662,
   65,  241,  283,   70,  242,  661,  668,  426,  443,   68,
  233,  308,  671,  673,   66,  500,  632,   79,   79,  110,
  110,  614,  215,  308,  209,  209,  209,  333,  328,  224,
  209,  209,  331,  329,  209,  330,  684,  332,  430,  670,
  672,  209,  661,  587,  500,  392,  445,  190,  553,  363,
  554,  555,  556,  557,  432,  685,  699,  360,  361,  233,
  679,   79,  209,  209,  500,  276,  707,  704,  190,  881,
  380,  216,   68,  209,  233,  440,  500,   66,  500,  433,
   86,  838,  500,  549,  689,  447,  203,  840,  500,  558,
  559,  460,  233,  500,  680,  464,  233,  363,  363,  704,
  233,  392,  480,  276,  500,  500,   78,  233,  484,  530,
  480,  467,  611,  500,  481,  489,  484,  489,   80,  209,
  281,   79,  744,  735,  606,  500,  500,  500,   86,   86,
  500,  500,  500,  468,  500,  759,  752,  500,  287,  288,
  764,  766,   82,  758,  760,   82,  228,  274,  763,  765,
  490,  537,  490,  484,   72,   65,  724,  538,  574,  768,
  233,  484,   82,  109,  575,  281,  579,   94,   94,  593,
  233,   62,  778,  605,   94,   94,   94,  715,  615,  623,
   94,   94,  500,  500,   94,  484,   86,  628,  633,  635,
  649,   94,  274,  484,  758,  659,  763,  765,   88,  660,
  773,  663,  664,  233,  665,  207,  207,  207,  453,  451,
  233,   94,   94,   94,  233,  676,  233,  677,   85,  811,
   79,  814,  249,   94,  682,   82,  683,   68,  411,  816,
  413,  417,   66,  686,   62,  746,  209,  209,  823,  453,
  810,  690,  453,  264,  597,  451,  822,  693,  480,  695,
  452,  453,  454,  233,  480,  670,  672,  453,  209,  794,
  209,  209,   64,  697,  809,   72,   85,   85,  704,   94,
   79,   94,  698,  710,  711,   79,  712,  719,  713,  379,
  822,  716,   62,  233,  451,  717,  452,  453,  456,  848,
  231,   82,  233,  274,  790,  723,  109,  233,  500,  480,
  356,  726,  500,  500,  732,  734,  363,  737,  308,  740,
  800,  801,  802,  742,  743,   79,  209,  209,  209,  209,
   79,  209,  209,  321,  322,  452,  453,  458,  500,  500,
  500,  480,  745,  727,  233,  500,  233,   86,  274,  873,
  762,   86,  484,  733,  462,  819,  820,  821,  774,  233,
  484,  775,  779,   88,  787,  209,  233,  209,  233,  511,
  209,  209,  209,   79,  276,  266,  747,  748,  791,  749,
   94,   90,  233,   41,   42,  462,  795,  233,  462,  796,
   82,  839,  209,  797,   79,  209,   94,   94,  812,   62,
  798,  500,  451,  462,  817,  818,   79,  484,   75,  209,
  430,   81,   32,  282,   79,  484,   79,  828,   94,  281,
   94,   94,   32,  830,  209,  859,  451,  207,  207,  459,
   94,  845,  270,  274,  846,   94,  850,  209,  867,  852,
  868,  855,  856,  452,  453,  461,  274,  869,  480,   78,
  858,   79,   78,   32,  484,  430,  262,  804,  282,  857,
  806,  807,  398,  860,  876,  863,  877,  452,  453,   78,
   87,   32,  888,  886,  209,   94,   94,   94,   94,   94,
   94,   94,   94,  388,  891,   85,  500,  500,   83,  500,
  491,   89,  491,   76,  493,  283,  493,  255,  480,  484,
  481,  500,  491,  398,  398,  398,  481,  207,  207,  207,
  207,  495,  475,  476,  495,   94,  298,   94,   87,   87,
   94,   94,   94,   94,  388,  388,  388,  115,  885,  884,
  188,  484,   78,  154,  827,  687,   90,   32,  784,  754,
  283,  851,   94,  853,   94,   94,  301,  302,  303,  304,
  305,  481,   75,  480,  338,   75,   94,  430,  350,   94,
  786,  154,  300,  154,   94,  154,   94,  209,   79,  424,
    0,    0,   75,   84,   94,    0,  551,    0,    0,  164,
  870,  871,  154,  481,    0,    0,  875,   94,   79,    0,
    0,    0,  274,    0,    0,    0,   84,   84,  111,  111,
    0,   94,  430,    0,    0,  889,    0,  164,  225,  164,
    0,  164,   79,    0,    0,  426,    0,    0,  264,  892,
  407,    0,  894,  407,   94,    0,    0,  896,  164,  209,
  419,    0,    0,    0,    0,   75,    0,    0,    0,    0,
   84,    0,    0,   83,  277,    0,   89,    0,  209,    0,
   92,   92,  112,  112,  112,  209,    0,    0,    0,    0,
    0,    0,  229,    0,    0,    0,    0,    0,    0,    0,
   79,    0,    0,    0,   32,   32,   32,    0,    0,  224,
   32,   32,  277,   32,  209,    0,    0,   78,  484,  209,
  209,    0,    0,    0,   92,    0,    0,    0,  279,  430,
   84,    0,  282,    0,    0,   32,   32,   32,   32,   32,
    0,    0,    0,    0,    0,    0,    0,   94,   94,    0,
    0,    0,    0,    0,  270,    0,    0,   87,   84,   79,
    0,    0,  209,    0,    0,    0,  279,    0,   94,   81,
  481,    0,   81,  484,  282,   79,   79,   79,  207,    0,
    0,  270,    0,    0,   92,    0,    0,    0,    0,   81,
    0,  577,   94,    0,  333,  328,    0,    0,  270,  331,
  329,    0,  330,   76,  332,    0,   76,    0,  283,   94,
   79,   79,   79,  270,  283,    0,  407,  209,    0,  282,
   75,    0,    0,   76,    0,  481,    0,   79,   94,   84,
  407,    0,    0,    0,    0,   94,    0,    0,    0,    0,
  207,    0,  834,   85,   85,    0,   79,    0,    0,  489,
   94,  327,   81,  283,    0,  227,    0,  154,    0,  154,
  154,  154,  154,  270,   94,    0,  741,    0,    0,   94,
   94,   81,   81,    0,    0,    0,  430,    0,    0,   84,
   79,  326,   79,   92,   84,    0,   76,   85,  424,    0,
  489,    0,    0,   79,    0,   79,    0,    0,  154,  154,
    0,    0,   79,  164,    0,  164,  164,  164,  164,   94,
    0,    0,   94,    0,    0,   81,    0,  656,  656,  656,
  209,    0,    0,    0,   84,   94,   94,   94,    0,   84,
  669,    0,    0,   92,  426,    0,  669,  669,   92,    0,
    0,    0,    0,  207,  164,  164,    0,   85,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
   94,   94,   94,  669,    0,    0,    0,   94,    0,    0,
    0,    0,   84,  277,  489,   81,    0,   94,   92,  692,
    0,    0,  692,   92,  692,    0,    0,    0,    0,  703,
  706,    0,  706,   84,    0,    0,   94,    0,    0,    0,
  706,    0,    0,    0,    0,   84,    0,   81,    0,    0,
    0,  333,  328,   84,    0,   84,  331,  329,    0,  330,
    0,  332,    0,    0,    0,  407,   92,  279,  149,    0,
   94,    0,   94,    0,  325,  407,  324,    0,    0,    0,
    0,   76,    0,   94,    0,   94,   85,   92,    0,    0,
   84,    0,   94,    0,    0,    0,  149,    0,  149,   92,
  149,    0,    0,  282,  427,    0,    0,   92,  327,   92,
   94,    0,    0,    0,   81,  308,    0,  149,    0,  480,
   77,    0,    0,   77,    0,  480,    0,    0,    0,    0,
  321,  322,  656,    0,    0,  132,   85,  283,  326,    0,
   77,   85,    0,    0,   92,    0,    0,    0,    0,    0,
    0,  489,  489,  489,    0,  789,    0,  489,  489,  793,
  489,    0,    0,  132,   81,  132,    0,  132,    0,   81,
  480,  429,    0,    0,    0,    0,    0,    0,    0,  407,
    0,   85,  407,  407,  132,  133,   85,    0,    0,    0,
    0,    0,  669,  669,    0,  490,    0,    0,    0,    0,
    0,    0,  480,   77,    0,    0,    0,   84,    0,   81,
    0,    0,    0,  133,   81,  133,    0,  133,    0,    0,
    0,  428,    0,    0,    0,    0,    0,   84,    0,   85,
    0,    0,    0,    0,  133,    0,  490,  692,  692,  692,
    0,    0,    0,   96,    0,    0,    0,    0,    0,    0,
   85,   84,   97,    0,  706,    0,    0,   81,    0,    0,
    0,   92,   85,  407,    0,  407,    0,    0,    0,    0,
   85,   96,   85,   96,    0,   96,    0,    0,   81,  419,
   97,   92,   97,    0,   97,    0,    0,    0,  422,    0,
   81,  480,   96,    0,    0,   77,    0,  480,   81,    0,
   81,   97,  407,  407,    0,   92,    0,   85,  407,   84,
  750,    0,    0,    0,    0,    0,  692,    0,  225,  270,
  490,  577,  706,    0,    0,  402,  403,  407,    0,  264,
    0,    0,  308,    0,    0,   81,    0,  313,  314,  264,
    0,  407,  480,    0,  407,    0,    0,  321,  322,  407,
    0,    0,    0,    0,    0,    0,    0,    0,   77,  480,
    0,    0,  149,   92,  149,  149,  149,  149,   84,    0,
  264,    0,  229,  263,  480,   85,    0,    0,    0,    0,
    0,    0,    0,    0,   84,   84,   84,  264,  264,    0,
    0,    0,    0,  427,    0,    0,    0,    0,    0,    0,
    0,    0,    0,  149,  149,  333,  328,    0,    0,    0,
  331,  329,    0,  330,  480,  332,    0,    0,    0,   84,
   84,   84,   92,    0,   85,    0,    0,    0,  325,  132,
  324,  132,  132,  132,  132,    0,   84,    0,   92,   92,
   92,  529,  533,    0,   85,    0,    0,    0,    0,    0,
    0,  835,   81,    0,  264,   84,    0,  490,  490,  490,
  429,    0,  327,  490,  490,    0,  490,    0,   85,    0,
  132,  132,   81,   92,   92,   92,    0,    0,    0,  133,
    0,  133,  133,  133,  133,    0,    0,  529,    0,   84,
   92,   84,  326,    0,    0,    0,   81,    0,    0,  607,
  609,    0,   84,  613,   84,  837,    0,  618,    0,   92,
  428,   84,    0,    0,    0,    0,    0,    0,    0,    0,
  133,  133,    0,  481,   79,    0,   85,   79,    0,  481,
   85,  480,  639,    0,    0,  227,  613,   96,  639,   96,
   96,   96,   96,   92,   79,   92,   97,    0,   97,   97,
   97,   97,    0,    0,   81,    0,   92,  481,   92,    0,
    0,   79,    0,  481,    0,   92,    0,    0,  419,    0,
    0,    0,    0,    6,  481,    0,    0,  422,   96,   96,
    0,    0,    0,    6,    0,   85,  480,   97,   97,    0,
    0,  264,  264,  264,    0,    0,  264,  264,  264,    0,
  264,   85,   85,   85,    0,    0,  481,   79,  481,    0,
    0,    0,    0,   81,    6,    0,    0,    0,    0,    0,
    0,  674,  264,  264,  264,  264,  264,    0,    0,   81,
   81,   81,    6,    0,    0,    0,   85,   85,   85,    0,
  481,   87,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,   85,  481,   80,    0,    0,   80,    0,
  281,    0,    0,   34,   81,   81,   81,  264,  836,    0,
  700,    0,   85,   34,    0,   80,    0,    0,    0,    0,
    0,   81,    0,    0,    0,  718,  308,  309,  310,  311,
  312,  313,  314,  315,  316,  317,  318,    0,    6,    0,
   81,  321,  322,  725,   34,  281,   85,  728,   85,    0,
    0,  729,    0,    0,    0,    0,    0,    0,  609,   85,
    0,   85,   34,    0,    0,    0,    0,    0,   85,    0,
    0,    0,  419,    0,   81,    0,   81,    0,   80,    0,
    0,    0,  419,    0,    0,    0,    0,   81,    0,   81,
    0,    0,    0,    0,    0,    0,   81,    0,    0,    0,
    0,    0,   79,  481,    0,  477,    0,    0,    0,  419,
  419,  770,  477,  419,  419,  419,  419,  419,  419,  419,
    0,  777,    0,    0,    0,    0,    0,    0,   34,    0,
  419,  419,  419,  419,  419,  419,   87,  481,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,  639,    0,    0,    0,  481,    0,
    0,  613,    0,  419,    0,  419,  419,  613,    0,  422,
    0,    0,    0,    0,    0,    6,    6,    6,    0,  422,
    0,    6,    6,    0,    6,    0,    0,    0,    0,    0,
    0,    0,  481,    0,    0,  477,  419,  419,  477,    0,
    0,    0,  478,    0,  824,    0,  422,  422,    0,  478,
  422,  422,  422,  422,  422,  422,  422,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,  422,  422,  422,
  422,  422,  422,   80,  841,    0,    0,    0,    0,    0,
    0,    0,    0,  849,    0,    0,    0,    0,  854,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
  422,    0,  422,  422,    0,   34,   34,   34,    0,    0,
    0,   34,   34,    0,   34,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,  609,    0,  613,    0,  281,
    0,    0,  478,  422,  422,  478,   34,   34,   34,   34,
   34,    0,    0,    0,    0,    0,    0,  887,    0,  890,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,  613,    0,    0,    0,    0,  895,  477,
  477,  477,    0,  477,  419,  419,  419,  477,  477,  419,
  419,  419,  477,  419,  477,  477,  477,  477,  477,  477,
  477,  419,  477,  419,  419,  477,  477,  477,  477,  477,
  477,  477,  419,  419,    0,  419,  419,  419,  419,  419,
    0,  477,    0,    0,  477,  477,  477,  477,  477,  477,
  477,  477,  477,  477,    0,  477,  477,    0,  477,  477,
  419,  419,  419,  419,  419,  419,  419,  419,  419,  419,
  419,  419,  419,    0,    0,  419,  419,  419,  477,  419,
  419,  477,  477,    0,  477,  477,  419,  477,  477,  477,
  477,  477,  477,  477,  477,    0,  478,  478,  478,  477,
  478,  422,  422,  422,  478,  478,  422,  422,  422,  478,
  422,  478,  478,  478,  478,  478,  478,  478,  422,  478,
  422,  422,  478,  478,  478,  478,  478,  478,  478,  422,
  422,    0,  422,  422,  422,  422,  422,    0,  478,    0,
    0,  478,  478,  478,  478,  478,  478,  478,  478,  478,
  478,    0,  478,  478,    0,  478,  478,  422,  422,  422,
  422,  422,  422,  422,  422,  422,  422,  422,  422,  422,
  484,    0,  422,  422,  422,  478,  422,  422,  478,  478,
  484,  478,  478,  422,  478,  478,  478,  478,  478,  478,
  478,  478,    0,    0,    0,    0,  478,    0,    0,    0,
    0,    0,    0,  480,    0,    0,    0,  484,  484,    0,
  480,  484,  484,  484,  484,  484,  484,  484,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,  484,  484,
  484,   86,  484,  484,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,  484,    0,  484,  484,    0,    0,  480,    0,    0,
    0,    0,    0,    0,    0,    0,    0,  480,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,  484,  484,  484,  480,    0,    0,    0,
  480,    0,    0,    0,  480,  480,    0,  480,  480,  480,
  480,  480,  480,  480,  480,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,  480,  480,  480,   85,  480,
  480,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,  480,    0,
  480,  480,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
  480,  480,  480,  480,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,  480,  480,  480,
    0,  480,  484,  484,  484,  480,  480,  484,  484,  484,
  480,  484,  480,  480,  480,  480,  480,  480,  480,    0,
  484,  484,  484,  480,  480,  480,  480,  480,  480,  480,
  484,  484,    0,  484,  484,  484,  484,  484,    0,  480,
    0,    0,  480,  480,  480,  480,  480,  480,  480,  480,
  480,  480,    0,  480,  480,    0,  480,  480,  484,  484,
  484,  484,  484,  484,  484,  484,  484,  484,  484,  484,
  484,    0,    0,  484,  484,  484,  480,    0,  484,  480,
  480,    0,  480,  480,  484,  480,  480,  480,  480,  480,
  480,  480,  480,    0,  480,  480,  480,  480,  480,  480,
  480,  480,  480,  480,  480,  480,  480,  480,  480,  480,
  480,  480,  480,  480,  480,  480,    0,  480,  480,  480,
  480,  480,  480,  480,  480,  480,  480,  480,  480,    0,
  480,  480,  480,  480,  480,    0,  480,    0,    0,  480,
  480,  480,  480,  480,  480,  480,  480,  480,  480,    0,
  480,  480,    0,  480,  480,  480,  480,  480,  480,  480,
  480,  480,  480,  480,  480,  480,  480,  480,  481,    0,
  480,  480,  480,  480,    0,  480,  480,  480,  481,  480,
  480,  480,  480,  480,  480,  480,  480,  480,  480,  480,
    0,    0,    0,    0,  480,    0,    0,    0,    0,    0,
    0,  481,    0,    0,    0,  481,  481,    0,  481,  481,
  481,  481,  481,  481,  481,  481,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,  481,  481,  481,   87,
  481,  481,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,  481,
    0,  481,  481,    0,    0,  275,    0,    0,    0,    0,
    0,    0,    0,    0,    0,  275,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,  481,  481,  481,  481,    0,    0,    0,  479,    0,
    0,    0,  275,  275,    0,  479,  275,  275,  275,  275,
  275,  275,  275,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,  275,  275,  275,    0,  275,  275,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,  275,    0,  275,  275,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,  479,  275,
  275,  479,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,  481,  481,  481,    0,  481,
  481,  481,  481,  481,  481,  481,  481,  481,  481,  481,
  481,  481,  481,  481,  481,  481,  481,    0,  481,  481,
  481,  481,  481,  481,  481,  481,  481,  481,  481,  481,
    0,  481,  481,  481,  481,  481,    0,  481,    0,    0,
  481,  481,  481,  481,  481,  481,  481,  481,  481,  481,
    0,  481,  481,    0,  481,  481,  481,  481,  481,  481,
  481,  481,  481,  481,  481,  481,  481,  481,  481,    0,
    0,  481,  481,  481,  481,    0,  481,  481,  481,    0,
  481,  481,  481,  481,  481,  481,  481,  481,  481,  481,
  481,    0,  479,  479,  479,  481,  479,  275,  275,  275,
  479,  479,  275,  275,  275,  479,  275,  479,  479,  479,
  479,  479,  479,  479,    0,  479,  275,  275,  479,  479,
  479,  479,  479,  479,  479,  275,  275,    0,  275,  275,
  275,  275,  275,    0,  479,    0,    0,  479,  479,  479,
  479,  479,  479,  479,  479,  479,  479,    0,  479,  479,
    0,  479,  479,  275,  275,  275,  275,  275,  275,  275,
  275,  275,  275,  275,  275,  275,  485,    0,  275,  275,
  275,  479,    0,  275,  479,  479,  485,  479,  479,  275,
  479,  479,  479,  479,  479,  479,  479,  479,    0,    0,
    0,    0,  479,    0,    0,    0,    0,    0,    0,  482,
    0,    0,    0,  485,  485,    0,  482,  485,  485,  485,
  485,  485,  485,  485,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,  485,  485,  485,    0,  485,  485,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,  485,    0,  485,
  485,    0,    0,  486,    0,    0,    0,    0,    0,    0,
    0,    0,    0,  486,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,  485,
  485,  485,  482,    0,    0,    0,  483,    0,    0,    0,
  486,  486,    0,  483,  486,  486,  486,  486,  486,  486,
  486,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,  486,  486,  486,    0,  486,  486,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,  486,    0,  486,  486,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,  486,  486,  486,  483,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,  482,  482,  482,    0,  482,  485,  485,
  485,  482,  482,  485,  485,  485,  482,  485,  482,  482,
  482,  482,  482,  482,  482,    0,  485,  485,  485,  482,
  482,  482,  482,  482,  482,  482,  485,  485,    0,  485,
  485,  485,  485,  485,    0,  482,    0,    0,  482,  482,
  482,  482,  482,  482,  482,  482,  482,  482,    0,  482,
  482,    0,  482,  482,  485,  485,  485,  485,  485,  485,
  485,  485,  485,  485,  485,  485,  485,    0,    0,  485,
  485,  485,  482,    0,  485,  482,  482,    0,  482,  482,
  485,  482,  482,  482,  482,  482,  482,  482,  482,    0,
  483,  483,  483,  482,  483,  486,  486,  486,  483,  483,
  486,  486,  486,  483,  486,  483,  483,  483,  483,  483,
  483,  483,    0,  486,  486,  486,  483,  483,  483,  483,
  483,  483,  483,  486,  486,    0,  486,  486,  486,  486,
  486,    0,  483,    0,    0,  483,  483,  483,  483,  483,
  483,  483,  483,  483,  483,    0,  483,  483,    0,  483,
  483,  486,  486,  486,  486,  486,  486,  486,  486,  486,
  486,  486,  486,  486,  355,    0,  486,  486,  486,  483,
    0,  486,  483,  483,  355,  483,  483,  486,  483,  483,
  483,  483,  483,  483,  483,  483,    0,    0,    0,    0,
  483,    0,    0,    0,    0,    0,    0,  252,    0,    0,
    0,  355,  355,    0,    0,  355,  355,  355,  355,  355,
  355,  355,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,  355,  355,  355,    0,  355,  355,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,  355,    0,  355,  355,    0,
    0,  500,    0,    0,    0,    0,    0,    0,    0,    0,
    0,  500,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,  355,  355,  355,
  252,    0,    0,    0,  252,    0,    0,    0,  500,  500,
    0,    0,  500,  500,  500,  500,  500,  500,  500,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,  500,
  500,  500,    0,  500,  500,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,  500,    0,  500,  500,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,  500,  500,  500,  252,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,  252,  252,  252,    0,  252,  355,  355,  355,  252,
  252,  355,  355,  355,  252,  355,  252,  252,  252,  252,
  252,  252,  252,    0,  355,  355,  355,  252,  252,  252,
  252,  252,  252,  252,  355,  355,    0,  355,  355,  355,
  355,  355,    0,  252,    0,    0,  252,  252,  252,  252,
  252,  252,  252,  252,  252,  252,    0,  252,  252,    0,
  252,  252,  355,  355,  355,  355,  355,  355,  355,  355,
  355,  355,  355,  355,  355,    0,    0,  355,  355,  355,
  252,    0,  355,  252,  252,    0,  252,  252,  355,  252,
  252,  252,  252,  252,  252,  252,  252,    0,  252,  252,
  252,  252,  252,  500,  500,  500,  252,  252,  500,  500,
  500,  252,  500,  252,  252,  252,  252,  252,  252,  252,
    0,  500,  500,  500,  252,  252,  252,  252,  252,  252,
  252,  500,  500,    0,  500,  500,  500,  500,  500,    0,
  252,    0,    0,  252,  252,  252,  252,  252,  252,  252,
  252,  252,  252,    0,  252,  252,    0,  252,  252,  500,
  500,  500,  500,  500,  500,  500,  500,  500,  500,  500,
  500,  500,  281,    0,  500,  500,  500,  252,    0,  500,
  252,  252,  281,  252,  252,  500,  252,  252,  252,  252,
  252,  252,  252,  252,    0,    0,    0,    0,  252,    0,
    0,    0,    0,    0,    0,  481,    0,    0,    0,  281,
  281,    0,  481,  281,  281,  281,  281,  281,  281,  281,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
  281,  281,  281,   88,  281,  281,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,  281,    0,  281,  281,    0,    0,  289,
    0,    0,    0,    0,    0,    0,    0,    0,    0,  289,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,  281,  281,  481,    0,
    0,    0,  252,    0,    0,    0,  289,  289,    0,    0,
  289,  289,  289,  289,  289,  289,  289,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,  289,  289,  289,
    0,  289,  289,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
  289,    0,  289,  289,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,  289,  289,  252,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,  481,
  481,  481,    0,  481,  281,  281,  281,  481,  481,  281,
  281,  281,  481,  281,  481,  481,  481,  481,  481,  481,
  481,    0,    0,  281,  281,  481,  481,  481,  481,  481,
  481,  481,  281,  281,    0,  281,  281,  281,  281,  281,
    0,  481,    0,    0,  481,  481,  481,  481,  481,  481,
  481,  481,  481,  481,    0,  481,  481,    0,  481,  481,
  281,  281,  281,  281,  281,  281,  281,  281,  281,  281,
  281,  281,  281,    0,    0,  281,  281,  281,  481,    0,
  281,  481,  481,    0,  481,  481,  281,  481,  481,  481,
  481,  481,  481,  481,  481,    0,  252,  252,  252,  481,
  252,  289,  289,  289,  252,  252,  289,  289,  289,  252,
  289,  252,  252,  252,  252,  252,  252,  252,    0,    0,
  289,  289,  252,  252,  252,  252,  252,  252,  252,  289,
  289,    0,  289,  289,  289,  289,  289,    0,  252,    0,
    0,  252,  252,  252,  252,  252,  252,  252,  252,  252,
  252,    0,  252,  252,    0,  252,  252,  289,  289,  289,
  289,  289,  289,  289,  289,  289,  289,  289,  289,  289,
  484,    0,  289,  289,  289,  252,    0,  289,  252,  252,
  484,  252,  252,  289,  252,  252,  252,  252,  252,  252,
  252,  252,    0,    0,    0,    0,  252,    0,    0,    0,
    0,    0,    0,  480,    0,    0,    0,  484,  484,    0,
  480,  484,  484,  484,   78,  484,  484,  484,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,  484,
  484,   86,  484,  484,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,  484,    0,    0,  484,    0,    0,  480,    0,    0,
    0,    0,    0,    0,    0,    0,    0,  480,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,  484,  484,  484,  480,    0,    0,    0,
  480,    0,    0,    0,  480,  480,    0,  480,  480,  480,
  480,   77,  480,  480,  480,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,  480,  480,   85,  480,
  480,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,  480,    0,
    0,  480,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
  480,  480,  480,  480,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,  480,  480,  480,
    0,  480,  484,  484,  484,  480,  480,    0,  484,  484,
  480,  484,  480,  480,  480,  480,  480,  480,  480,    0,
  484,    0,    0,  480,  480,  480,  480,  480,  480,  480,
  484,  484,    0,  484,  484,  484,  484,  484,    0,  480,
    0,    0,  480,  480,  480,  480,  480,  480,  480,  480,
  480,  480,    0,  480,  480,    0,  480,  480,  484,  484,
  484,  484,  484,  484,  484,  484,  484,  484,  484,  484,
  484,    0,    0,  484,  484,  484,  480,    0,    0,  480,
  480,    0,  480,  480,    0,  480,  480,  480,  480,  480,
  480,  480,  480,    0,  480,  480,  480,  480,  480,  480,
  480,  480,  480,  480,    0,  480,  480,  480,  480,  480,
  480,  480,  480,  480,  480,  480,    0,  480,    0,    0,
  480,  480,  480,  480,  480,  480,  480,  480,  480,    0,
  480,  480,  480,  480,  480,    0,  480,    0,    0,  480,
  480,  480,  480,  480,  480,  480,  480,  480,  480,    0,
  480,  480,    0,  480,  480,  480,  480,  480,  480,  480,
  480,  480,  480,  480,  480,  480,  480,  480,  481,    0,
  480,  480,  480,  480,    0,    0,  480,  480,  481,  480,
  480,    0,  480,  480,  480,  480,  480,  480,  480,  480,
    0,    0,    0,    0,  480,    0,    0,    0,    0,    0,
    0,  481,    0,    0,    0,  481,  481,    0,  481,  481,
  481,  481,   79,  481,  481,  481,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,  481,  481,   87,
  481,  481,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,  481,
    0,    0,  481,    0,    0,  281,    0,    0,    0,    0,
    0,    0,    0,    0,    0,  281,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,  481,  481,  481,  481,    0,    0,    0,  481,    0,
    0,    0,  281,  281,    0,  481,  281,  281,  281,   80,
  281,  281,  281,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,  281,  281,   88,  281,  281,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,  281,    0,    0,  281,
    0,    0,    0,    0,    0,    0,  252,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,  281,
  281,  481,    0,  333,  328,    0,    0,    0,  331,  329,
    0,  330,    0,  332,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,  325,    0,  324,  323,
    0,    0,    0,    0,    0,  481,  481,  481,    0,  481,
  481,  481,  481,  481,  481,    0,  481,  481,  481,  481,
  481,  481,  481,  481,  481,  481,  481,    0,  481,    0,
  327,  481,  481,  481,  481,  481,  481,  481,  481,  481,
    0,  481,  481,  481,  481,  481,    0,  481,    0,    0,
  481,  481,  481,  481,  481,  481,  481,  481,  481,  481,
  326,  481,  481,    0,  481,  481,  481,  481,  481,  481,
  481,  481,  481,  481,  481,  481,  481,  481,  481,    0,
    0,  481,  481,  481,  481,    0,    0,  481,  481,    0,
  481,  481,    0,  481,  481,  481,  481,  481,  481,  481,
  481,    0,  481,  481,  481,  481,  481,  281,  281,  281,
  481,  481,    0,  281,  281,  481,  281,  481,  481,  481,
  481,  481,  481,  481,    0,    0,    0,    0,  481,  481,
  481,  481,  481,  481,  481,  281,  281,    0,  281,  281,
  281,  281,  281,    0,  481,    0,    0,  481,  481,  481,
  481,  481,  481,  481,  481,  481,  481,    0,  481,  481,
    0,  481,  481,  281,  281,  281,  281,  281,  281,  281,
  281,  281,  281,  281,  281,  281,  500,    0,  281,  281,
  281,  481,    0,    0,  481,  481,  500,  481,  481,    0,
  481,  481,  481,  481,  481,  481,  481,  481,    0,    0,
    0,    0,  481,    0,    0,    0,    0,    0,    0,  252,
    0,    0,    0,    0,    0,    0,    0,  500,    0,    0,
    0,    0,  500,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    8,    0,  500,  500,    0,    0,    0,    0,
    0,    0,    8,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,  308,  309,  310,  311,  312,  313,
  314,  315,  316,  317,  318,  319,  320,    0,    0,  321,
  322,    0,    0,    8,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,   91,    0,
    0,    8,    0,    0,    0,    0,    0,    0,    0,    0,
    0,  500,  252,    0,    0,    0,    0,    0,    0,    0,
    0,   91,    0,    0,    0,    0,    0,    0,  477,    0,
    0,    0,    0,    0,  419,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,   91,   91,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    8,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,  419,
    0,  333,  328,    0,    0,    0,  331,  329,    0,  330,
    0,  332,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,  652,    0,  325,    0,  324,  323,    0,    0,
    0,  477,    0,    0,   91,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,  327,    0,
    0,    0,    0,  252,  252,  252,    0,  252,  500,  500,
  500,  252,  252,  500,  500,  500,  252,  500,  252,  252,
  252,  252,  252,  252,  252,    0,    0,  500,  326,  252,
  252,  252,  252,  252,  252,  252,  500,  500,    0,  500,
  500,  500,  500,  500,    0,  252,    0,    0,  252,  252,
  252,  252,  252,  252,  252,  252,  252,  252,    0,  252,
  252,    0,  252,  252,    8,    8,    8,    0,    0,    0,
    8,    8,    0,    8,    0,    0,    0,    0,    0,    0,
    0,  500,  252,    0,    0,  252,  252,   58,  252,  252,
    0,  252,  252,  252,  252,  252,  252,  252,  252,    0,
    0,    0,    0,  252,   91,   91,   91,   91,   91,   91,
   91,   91,   91,   91,   91,    0,    0,   91,   91,    0,
   91,   91,   91,   91,   91,   91,   91,    0,  477,    0,
    0,   91,   91,   91,   91,   91,   91,   91,    0,    0,
   91,    0,    0,    0,    0,    0,   91,   91,   91,   91,
   91,   91,   91,   91,   91,   91,   91,   91,   91,   91,
    0,   91,   91,    0,   91,   91,    0,    0,    0,    0,
    0,   92,    0,    0,    0,    0,    0,    0,    0,    0,
   59,    0,    0,  419,   91,    0,    0,   91,   91,    0,
   91,   91,    0,   91,   92,   91,   91,   91,   91,   91,
   91,  478,    0,    0,    0,   91,    0,  422,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
   92,   92,  308,  309,  310,  311,  312,  313,  314,  315,
  316,  317,  318,  319,  320,    0,    0,  321,  322,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,  422,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,   91,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,  478,    0,    0,   92,    0,    0,
    0,    0,    0,    0,    0,    0,   91,    0,    0,    0,
    0,    0,    0,  480,    0,    0,    0,    0,    0,  484,
    0,    4,    5,    6,    0,    8,    0,    0,    0,    9,
   10,    0,   91,   91,   11,    0,   12,   13,   14,   15,
   16,   17,   18,    0,    0,    0,    0,   19,   20,   21,
   22,   23,   24,   25,    0,    0,   26,    0,    0,    0,
    0,    0,    0,   28,  484,    0,   31,   32,   33,   34,
   35,   36,   37,   38,   39,   40,    0,   41,   42,   14,
   43,   44,    0,    0,    0,    0,    0,    0,    0,   14,
    0,    0,    0,    0,    0,    0,  484,    0,    0,   91,
  206,    0,    0,  106,   47,    0,   48,   49,    0,  616,
  231,   51,   52,   53,   54,   55,   56,    0,    0,    0,
   14,   57,    0,    0,    0,    0,    0,   92,   92,   92,
   92,   92,   92,   92,   92,   92,   92,   92,   14,    0,
   92,   92,   40,   92,   92,   92,   92,   92,   92,   92,
    0,  478,   40,    0,   92,   92,   92,   92,   92,   92,
   92,    0,    0,   92,    0,    0,    0,    0,    0,   92,
   92,   92,   92,   92,   92,   92,   92,   92,   92,   92,
   92,   92,   92,   40,   92,   92,    0,   92,   92,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
   40,   40,    0,    0,   14,    0,  422,   92,    0,    0,
   92,   92,    0,   92,   92,    0,   92,    0,   92,   92,
   92,   92,   92,   92,    0,    0,    0,    0,   92,   91,
   91,   91,   91,   91,   91,   91,   91,   91,   91,   91,
    0,    0,   91,   91,   53,   91,   91,   91,   91,   91,
   91,   91,    0,  484,   53,    0,   91,   91,   91,   91,
   91,   91,   91,    0,    0,   91,    0,   40,    0,    0,
    0,   91,   91,   91,   91,   91,   91,   91,   91,   91,
   91,   91,   91,   91,   91,   53,   91,   91,    0,   91,
   91,    0,    0,    0,    0,    0,   92,    0,    0,    0,
    0,    0,   53,   53,    0,    0,    0,    0,  484,   91,
    0,    0,   91,   91,    0,   91,   91,    0,   91,   92,
   91,   91,   91,   91,   91,   91,  481,    0,    0,    0,
   91,    0,  281,    0,    0,    0,    0,   53,    0,    0,
    0,   14,   14,   14,    7,   92,   92,   14,   14,    0,
   14,    0,    0,    0,    7,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,   53,
    0,    0,   14,   14,   14,   14,   14,  281,    0,    0,
    0,    0,    0,    0,    0,    7,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,   92,    0,
    0,    0,    0,    7,   40,   40,   40,    0,    0,   40,
   40,   40,   92,   40,    0,    0,    0,    0,    0,    0,
    0,   92,    0,   40,    0,    0,    0,    0,    0,    0,
    0,    0,   40,   40,  282,   40,   40,   40,   40,   40,
    0,    0,    0,    0,    0,    0,    0,   92,   92,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    7,
    0,    0,    0,    0,    0,    0,    0,    0,    0,  282,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,  327,    0,   53,   53,   53,    0,
    0,   53,   53,   53,   92,   53,    0,    0,    0,    0,
    0,    0,    0,    0,    0,   53,   53,  327,    0,    0,
    0,    0,    0,    0,   53,   53,    0,   53,   53,   53,
   53,   53,   92,   92,   92,   92,   92,   92,   92,   92,
   92,   92,   92,  327,    0,   92,   92,    0,   92,   92,
   92,   92,   92,   92,   92,    0,    0,    0,    0,   92,
   92,   92,   92,   92,   92,   92,    0,    0,   92,    0,
    0,    0,    0,    0,   92,   92,   92,   92,   92,   92,
   92,   92,   92,   92,   92,   92,   92,   92,    0,   92,
   92,    0,   92,   92,    0,    0,    7,    7,    7,    0,
    0,    0,    7,    7,    0,    7,    0,    0,  500,    0,
  327,  281,   92,    0,    0,   92,   92,    0,   92,   92,
    0,   92,    0,   92,   92,   92,   92,   92,   92,    0,
    0,  500,    0,   92,   92,   92,   92,   92,   92,   92,
   92,   92,   92,   92,   92,    0,    0,   92,   92,    0,
   92,   92,   92,   92,   92,   92,   92,  500,    0,    0,
    0,   92,   92,   92,   92,   92,   92,   92,    0,    0,
   92,    0,    0,    0,    0,    0,   92,   92,   92,   92,
   92,   92,   92,   92,   92,   92,   92,   92,   92,   92,
    0,   92,   92,    0,   92,   92,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,  282,   92,    0,    0,   92,   92,    0,
   92,   92,  218,   92,  500,   92,   92,   92,   92,   92,
   92,    0,  218,    0,    0,   92,    0,    0,    0,    0,
  327,  327,  327,  327,  327,  327,  327,  327,  327,  327,
  327,    0,  327,  327,  327,  327,  327,  327,  327,  327,
  327,  327,  327,  218,    0,    0,  218,  327,  327,  327,
  327,  327,  327,  327,    0,    0,  327,    0,    0,  500,
  218,  218,  327,  327,  327,  327,  327,  327,  327,  327,
  327,  327,  327,  327,  327,  327,    0,  327,  327,    0,
  327,  327,  500,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,  218,    0,    0,    0,    0,
  327,    0,    0,  327,  327,    0,  327,  327,  500,  327,
    0,  327,  327,  327,  327,  327,  327,    0,    0,    0,
    0,  327,    0,    0,    0,    0,    0,  218,    0,    0,
    0,    0,    0,    0,  500,  500,  500,  500,  500,  500,
    0,    0,  500,  500,  500,    0,    0,    0,  500,    0,
  500,  500,  500,  500,  500,  500,  500,    0,    0,    0,
    0,  500,  500,  500,  500,  500,  500,  500,    0,    0,
  500,    0,    0,  215,  500,  500,  500,  500,  500,  500,
  500,  500,  500,  500,  500,  500,  500,  500,  500,  500,
    0,  500,  500,    0,  500,  500,   58,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,  500,    0,    0,  500,  500,    0,
  500,  500,  216,  500,    0,  500,  500,  500,  500,  500,
  500,    0,    0,    0,    0,  500,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,  218,  218,  218,    0,    0,  218,
  218,  218,    0,  218,    0,    0,    0,    0,    0,    0,
    0,    0,    0,  218,  218,    0,    0,    0,    0,   59,
    0,    0,  218,  218,    0,  218,  218,  218,  218,  218,
    0,    0,    0,    0,    0,  500,  500,  500,  500,  500,
  500,    0,    0,    0,  500,  500,    0,    0,    0,  500,
    0,  500,  500,  500,  500,  500,  500,  500,    0,    0,
    0,    0,  500,  500,  500,  500,  500,  500,  500,    0,
  218,  500,    0,    0,    0,    0,  218,  500,  500,  500,
  500,  500,  500,  500,  500,  500,  500,  500,  500,  500,
  500,    0,  500,  500,    0,  500,  500,    0,    0,    0,
    0,    0,   41,    0,    0,    0,    0,    0,    0,    0,
    0,    0,   41,    0,    0,  500,    0,    0,  500,  500,
    0,  500,  500,    0,  500,    0,  500,  500,  500,  500,
  500,  500,    0,    0,    0,    0,  500,    0,    0,    0,
    0,    0,    0,   41,    0,    0,    0,    0,    0,    0,
    4,    5,    6,    0,    8,    0,    0,    0,    9,   10,
   41,   41,    0,   11,    0,   12,   13,   14,   15,   16,
   17,   18,    0,    0,    0,    0,   19,   20,   21,   22,
   23,   24,   25,    0,    0,   26,    0,    0,    0,    0,
    0,    0,   28,    0,    0,   31,   32,   33,   34,   35,
   36,   37,   38,   39,   40,    0,   41,   42,    0,   43,
   44,  355,    0,    0,    0,    0,    0,    0,    0,    0,
    0,  355,    0,    0,    0,    0,    0,   41,    0,  206,
    0,    0,  106,   47,    0,   48,   49,    0,    0,    0,
   51,   52,   53,   54,   55,   56,    0,    0,  355,  355,
   57,    0,  355,  355,  355,  355,  355,  355,  355,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,  355,
  355,  355,    0,  355,  355,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
  430,    0,  355,    0,  355,  355,    0,    0,    0,    0,
  430,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,  355,  355,  355,  430,  430,    0,
    0,  430,  430,  430,  430,  430,  430,  430,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,  430,  430,
  430,   83,  430,  430,   41,   41,   41,    0,    0,   41,
   41,   41,    0,   41,    0,    0,    0,    0,    0,    0,
    0,    0,    0,   41,    0,    0,    0,    0,    0,    0,
  500,  430,    0,  430,  430,   41,   41,   41,   41,   41,
  500,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,  430,  430,    0,  500,  500,    0,
    0,  500,  500,  500,  500,  500,  500,  500,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,  500,  500,
  500,    0,  500,  500,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,  355,  355,  355,    0,    0,  355,  355,
  355,  500,  355,  500,  500,    0,    0,    0,    0,    0,
    0,  355,  355,  355,    0,    0,    0,    0,    0,    0,
    0,  355,  355,    0,  355,  355,  355,  355,  355,    0,
    0,    0,    0,  500,  500,  500,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,  355,
  355,  355,  355,  355,  355,  355,  355,  355,  355,  355,
  355,  355,    0,    0,  355,  355,  355,    0,    0,  355,
    0,    0,    0,    0,    0,  355,    0,    0,    0,    0,
    0,    0,  430,  430,  430,    0,    0,  430,  430,  430,
    0,  430,    0,    0,    0,    0,    0,    0,    0,    0,
    0,  430,  430,    0,    0,    0,    0,    0,    0,    0,
  430,  430,    0,  430,  430,  430,  430,  430,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,  430,  430,
  430,  430,  430,  430,  430,  430,  430,  430,  430,  430,
  430,    0,    0,  430,  430,  430,    0,  431,  430,    0,
    0,    0,    0,    0,  430,    0,    0,    0,    0,    0,
    0,    0,  500,  500,  500,    0,    0,  500,  500,  500,
    0,  500,    0,    0,    0,    0,    0,    0,    0,    0,
  500,  500,  500,    0,    0,    0,    0,    0,    0,    0,
  500,  500,    0,  500,  500,  500,  500,  500,    0,    0,
    0,    0,    0,    0,  274,    0,    0,    0,    0,    0,
    0,    0,    0,    0,  274,    0,    0,    0,  500,  500,
  500,  500,  500,  500,  500,  500,  500,  500,  500,  500,
  500,    0,    0,  500,  500,  500,    0,    0,  500,    0,
    0,  274,  274,    0,  500,  274,  274,  274,  274,  274,
  274,  274,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,  274,  274,  274,   90,  274,  274,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,  282,
    0,    0,    0,    0,    0,  274,    0,  274,  274,  282,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,  282,  282,  274,  274,
  282,  282,  282,  282,  282,  282,  282,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,  282,  282,  282,
   89,  282,  282,    0,  333,  328,    0,    0,    0,  331,
  329,    0,  330,    0,  332,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,  325,    0,  324,
  282,    0,  282,  282,    0,  283,    0,    0,    0,    0,
    0,    0,    0,    0,    0,  283,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,  327,    0,  282,  282,    0,    0,    0,    0,    0,
    0,    0,  283,  283,    0,    0,  283,  283,  283,  283,
  283,  283,  283,    0,    0,    0,    0,    0,    0,    0,
    0,  326,    0,  283,  283,  283,   84,  283,  283,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,  274,  274,  274,    0,
    0,  274,  274,  274,    0,  274,  283,    0,  283,  283,
    0,    0,    0,    0,    0,  274,  274,    0,    0,    0,
    0,    0,    0,    0,  274,  274,    0,  274,  274,  274,
  274,  274,    0,    0,  500,    0,    0,    0,    0,  283,
  283,    0,    0,    0,  500,    0,    0,    0,    0,    0,
    0,    0,  274,  274,  274,  274,  274,  274,  274,  274,
  274,  274,  274,  274,  274,    0,    0,  274,  274,  274,
    0,    0,  274,    0,    0,  500,    0,    0,  274,    0,
    0,  282,  282,  282,    0,    0,  282,  282,  282,    0,
  282,    0,  500,  500,    0,    0,    0,    0,    0,    0,
  282,  282,    0,    0,    0,    0,    0,    0,    0,  282,
  282,    0,  282,  282,  282,  282,  282,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,  500,    0,    0,
    0,    0,    0,    0,    0,    0,    0,  282,  282,  282,
  282,  282,  282,  282,  282,  282,  282,  282,  282,  282,
    0,    0,  282,  282,  282,    0,    0,  282,    0,  500,
    0,    0,    0,  282,    0,  308,  309,  310,  311,  312,
  313,  314,  315,    0,  317,  318,    0,  283,  283,  283,
  321,  322,  283,  283,  283,    0,  283,    0,    0,    0,
    0,    0,    0,    0,    0,    0,  283,  283,    0,    0,
    0,    0,    0,    0,    0,  283,  283,    0,  283,  283,
  283,  283,  283,    0,    0,    0,    0,    0,    0,  380,
    0,    0,    0,    0,    0,    0,    0,    0,    0,  380,
    0,    0,    0,  283,  283,  283,  283,  283,  283,  283,
  283,  283,  283,  283,  283,  283,    0,    0,  283,  283,
  283,    0,    0,  283,    0,    0,  380,  380,    0,  283,
  380,  380,  380,  380,  380,  380,  380,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,  380,  380,  380,
    0,  380,  380,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,  500,  500,  500,    0,
    0,  500,  500,  500,  217,  500,    0,    0,    0,    0,
  380,    0,  380,  380,  217,  500,  500,    0,    0,    0,
    0,    0,    0,    0,  500,  500,    0,  500,  500,  500,
  500,  500,    0,    0,    0,    0,    0,    0,    0,    0,
    0,  217,  217,  380,  380,  217,  217,  217,  217,  217,
  326,  217,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,  217,  217,  217,    0,  217,  217,    0,  333,
  328,    0,    0,    0,  331,  329,    0,  330,  500,  332,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,  325,    0,  324,  326,    0,  217,  217,    0,
  293,    0,    0,    0,    0,    0,    0,    0,    0,    0,
  293,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,  327,    0,  217,  217,
    0,    0,    0,    0,    0,    0,    0,  293,  293,    0,
    0,  293,  293,  293,  293,  293,  293,  293,    0,    0,
    0,    0,    0,    0,    0,    0,  326,    0,  293,  293,
  293,    0,  293,  293,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,  380,  380,  380,    0,    0,  380,  380,  380,    0,
  380,  293,    0,  293,  293,    0,    0,    0,    0,    0,
  380,  380,    0,    0,    0,    0,    0,    0,    0,  380,
  380,    0,  380,  380,  380,  380,  380,    0,    0,   57,
    0,    0,    0,    0,  293,  293,    0,    0,    0,   57,
    0,    0,    0,    0,    0,    0,    0,  380,  380,  380,
  380,  380,  380,  380,  380,  380,  380,  380,  380,  380,
    0,    0,  380,  380,  380,    0,    0,  380,    0,    0,
   57,    0,    0,  380,    0,    0,  217,  217,  217,    0,
    0,  217,  217,  217,    0,  217,    0,   57,   57,    0,
    0,    0,    0,    0,    0,  217,  217,    0,    0,    0,
    0,    0,    0,    0,  217,  217,    0,  217,  217,  217,
  217,  217,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,   57,    0,    0,    0,    0,    0,    0,    0,
    0,    0,  217,  217,  217,  217,  217,  217,  217,  217,
  217,  217,  217,  217,  217,    0,    0,  217,  217,  326,
    0,    0,  217,    0,   57,    0,    0,    0,  217,    0,
  308,  309,  310,  311,  312,  313,  314,    0,    0,  317,
  318,    0,  293,  293,  293,  321,  322,  293,  293,  293,
    0,  293,    0,    0,    0,    0,    0,    0,    0,    0,
    0,  293,  293,    0,    0,    0,    0,    0,    0,    0,
  293,  293,    0,  293,  293,  293,  293,  293,    0,    0,
    0,    0,    0,    0,  289,    0,    0,    0,    0,    0,
    0,    0,    0,    0,  289,    0,    0,    0,  293,  293,
  293,  293,  293,  293,  293,  293,  293,  293,  293,  293,
  293,    0,    0,  293,  293,  293,    0,    0,  293,    0,
    0,  289,  289,    0,  293,  289,  289,  289,  289,  289,
  289,  289,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,  289,  289,  289,    0,  289,  289,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,   57,   57,   57,    0,    0,   57,   57,   57,  417,
   57,    0,    0,    0,    0,  289,    0,  289,  289,  417,
   57,   57,    0,    0,    0,    0,    0,    0,    0,   57,
   57,    0,   57,   57,   57,   57,   57,    0,    0,    0,
    0,    0,    0,    0,    0,    0,  417,  417,  289,  289,
  417,  417,  417,  417,  417,  417,  417,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,  417,  417,  417,
    0,  417,  417,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
  417,    0,  417,  417,    0,  418,    0,    0,    0,    0,
    0,    0,    0,    0,    0,  418,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,  417,  417,    0,    0,    0,    0,    0,
    0,    0,  418,  418,    0,    0,  418,  418,  418,  418,
  418,  418,  418,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,  418,  418,  418,    0,  418,  418,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,  289,  289,  289,    0,
    0,  289,  289,  289,    0,  289,  418,    0,  418,  418,
    0,    0,    0,    0,    0,  289,  289,    0,    0,    0,
    0,    0,    0,    0,  289,  289,    0,  289,  289,  289,
  289,  289,    0,    0,   55,    0,    0,    0,    0,  418,
  418,    0,    0,    0,   55,    0,    0,    0,    0,    0,
    0,    0,  289,  289,  289,  289,  289,  289,  289,  289,
  289,  289,  289,  289,  289,    0,    0,  289,  289,  289,
    0,    0,  289,    0,    0,   55,    0,    0,  289,    0,
    0,  417,  417,  417,    0,    0,  417,  417,  417,    0,
  417,    0,   55,   55,    0,    0,    0,    0,  333,  328,
  417,  417,    0,  331,  329,    0,  330,    0,  332,  417,
  417,    0,  417,  417,  417,  417,  417,    0,    0,    0,
    0,  325,    0,  324,  323,    0,    0,   55,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,  417,  417,
  417,  417,  417,  417,  417,  417,  417,  417,  417,  417,
    0,    0,  417,  417,  417,  327,    0,  417,    0,   55,
    0,    0,    0,  417,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,  418,  418,  418,
    0,    0,  418,  418,  418,  326,  418,    0,    0,    0,
    0,    0,    0,    0,    0,    0,  418,  418,    0,    0,
    0,    0,    0,    0,    0,  418,  418,    0,  418,  418,
  418,  418,  418,    0,    0,    0,    0,    0,    0,  194,
    0,    0,    0,    0,    0,    0,    0,    0,    0,  194,
    0,    0,    0,    0,  418,  418,  418,  418,  418,  418,
  418,  418,  418,  418,  418,  418,    0,    0,  418,  418,
  418,    0,    0,  418,    0,    0,  194,  194,    0,  418,
  194,  194,  194,  194,  194,    0,  194,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,  194,  194,  194,
    0,  194,  194,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,   55,   55,   55,    0,
    0,   55,   55,   55,  190,   55,    0,    0,    0,    0,
    0,    0,  194,  194,  190,   55,   55,    0,    0,    0,
    0,    0,    0,    0,   55,   55,    0,   55,   55,   55,
   55,   55,    0,    0,    0,    0,    0,    0,    0,    0,
    0,  190,  190,  194,  194,  190,  190,  190,  190,  190,
    0,  190,    0,    0,    0,    0,    0,    0,  675,    0,
    0,    0,  190,  190,  190,    0,  190,  190,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,  308,
  309,  310,  311,  312,  313,  314,  315,  316,  317,  318,
  319,  320,    0,    0,  321,  322,    0,  190,  190,  187,
    0,    0,    0,    0,    0,    0,    0,    0,    0,  187,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,  190,  190,
    0,    0,    0,    0,    0,    0,  187,  187,    0,    0,
  187,  187,  187,  187,  187,    0,  187,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,  187,  187,  187,
    0,  187,  187,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,  194,  194,  194,    0,    0,  194,  194,  194,    0,
  194,    0,  187,  187,    0,    0,    0,    0,    0,    0,
  194,  194,    0,    0,    0,    0,    0,    0,    0,  194,
  194,    0,  194,  194,  194,  194,  194,    0,    0,    0,
    0,    0,    0,  187,  187,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,  194,  194,
  194,  194,  194,  194,  194,  194,  194,  194,  194,  194,
   42,    0,  194,  194,    0,    0,    0,  194,    0,    0,
   42,    0,    0,  194,    0,    0,  190,  190,  190,    0,
    0,  190,  190,  190,    0,  190,    0,    0,    0,    0,
    0,    0,    0,    0,    0,  190,  190,    0,    0,    0,
    0,   42,    0,    0,  190,  190,    0,  190,  190,  190,
  190,  190,    0,    0,    0,    0,  173,    0,   42,   42,
    0,    0,    0,    0,    0,    0,  173,    0,    0,    0,
    0,    0,    0,  190,  190,  190,  190,  190,  190,  190,
  190,  190,  190,  190,  190,    0,    0,  190,  190,    0,
    0,  188,  190,    0,    0,    0,    0,  173,  190,    0,
  218,  188,    0,    0,    0,    0,    0,    0,    0,    0,
    0,  187,  187,  187,    0,  173,  187,  187,  187,    0,
  187,    0,    0,    0,    0,   42,    0,    0,  188,  188,
  187,  187,  188,  188,  188,  188,  188,    0,  188,  187,
  187,    0,  187,  187,  187,  187,  187,    0,    0,  188,
  188,  188,    0,  188,  188,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,  187,  187,
  187,  187,  187,  187,  187,  187,  187,  187,  187,  187,
  189,  173,  187,  187,  188,  188,    0,  187,    0,    0,
  189,    0,    0,  187,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,  188,  188,  189,  189,    0,
    0,  189,  189,  189,  189,  189,    0,  189,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,  189,  189,
  189,    0,  189,  189,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,   42,   42,   42,    0,    0,   42,   42,   42,
  191,   42,    0,  189,  189,    0,    0,    0,    0,    0,
  191,   42,    0,    0,    0,    0,    0,    0,    0,    0,
   42,   42,    0,   42,   42,   42,   42,   42,    0,    0,
    0,    0,    0,    0,  189,  189,    0,  191,  191,    0,
    0,  191,  191,  191,  191,  191,    0,  191,  173,  173,
  173,    0,    0,    0,  173,  173,    0,  173,  191,  191,
  191,    0,  191,  191,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,  173,  173,    0,  173,
  173,  173,  173,  188,  188,  188,    0,    0,  188,  188,
  188,    0,  188,  191,  191,    0,    0,    0,    0,    0,
    0,    0,  188,  188,    0,    0,    0,    0,    0,    0,
    0,  188,  188,    0,  188,  188,  188,  188,  188,    0,
    0,    0,    0,    0,  191,  191,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
  188,  188,  188,  188,  188,  188,  188,  188,  188,  188,
  188,  188,  192,    0,  188,  188,    0,    0,    0,  188,
    0,    0,  192,    0,    0,  188,    0,    0,    0,    0,
    0,    0,  189,  189,  189,    0,    0,  189,  189,  189,
    0,  189,    0,    0,    0,    0,    0,    0,    0,  192,
  192,  189,  189,  192,  192,  192,  192,  192,    0,  192,
  189,  189,    0,  189,  189,  189,  189,  189,    0,    0,
  192,  192,  192,    0,  192,  192,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,  189,
  189,  189,  189,  189,  189,  189,  189,  189,  189,  189,
  189,    0,  185,  189,  189,  192,  192,    0,  189,    0,
    0,    0,  185,    0,  189,    0,    0,    0,    0,    0,
    0,    0,  191,  191,  191,    0,    0,  191,  191,  191,
    0,  191,    0,    0,    0,    0,  192,  192,    0,    0,
  185,  191,  191,  185,    0,  185,  185,  185,    0,    0,
  191,  191,    0,  191,  191,  191,  191,  191,    0,    0,
  185,  185,  185,    0,  185,  185,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,  191,
  191,  191,  191,  191,  191,  191,  191,  191,  191,  191,
  191,  186,    0,  191,  191,  185,  185,    0,  191,    0,
    0,  186,    0,    0,  191,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,  185,  185,    0,  186,
    0,    0,  186,    0,  186,  186,  186,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,  186,
  186,  186,    0,  186,  186,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,  192,  192,  192,    0,    0,  192,
  192,  192,    0,  192,  186,  186,    0,    0,    0,    0,
    0,    0,    0,  192,  192,    0,    0,    0,    0,    0,
    0,    0,  192,  192,    0,  192,  192,  192,  192,  192,
    0,    0,    0,    0,    0,  186,  186,  210,    0,    0,
    0,    0,    0,    0,    0,    0,    0,  210,    0,    0,
    0,  192,  192,  192,  192,  192,  192,  192,  192,  192,
  192,  192,  192,    0,    0,  192,  192,    0,    0,    0,
  192,    0,    0,    0,    0,  210,  192,    0,  210,    0,
    0,  210,    0,    0,  185,  185,  185,    0,    0,  185,
  185,  185,    0,  185,    0,  210,  210,  210,    0,  210,
  210,    0,    0,  185,  185,    0,    0,    0,    0,    0,
    0,    0,  185,  185,    0,  185,  185,  185,  185,  185,
    0,    0,    0,    0,    0,    0,  211,    0,    0,    0,
  210,  210,    0,    0,    0,    0,  211,    0,    0,    0,
    0,  185,  185,  185,  185,  185,  185,  185,  185,  185,
  185,  185,  185,    0,    0,  185,  185,    0,    0,    0,
  185,  210,  210,    0,  211,    0,  185,  211,    0,    0,
  211,    0,    0,  186,  186,  186,    0,    0,  186,  186,
  186,    0,  186,    0,  211,  211,  211,    0,  211,  211,
    0,    0,  186,  186,    0,    0,    0,    0,    0,    0,
    0,  186,  186,    0,  186,  186,  186,  186,  186,    0,
    0,    0,    0,    0,    0,    0,  197,    0,    0,  211,
  211,    0,    0,    0,    0,    0,  197,    0,    0,    0,
  186,  186,  186,  186,  186,  186,  186,  186,  186,  186,
  186,  186,    0,    0,  186,  186,    0,    0,    0,  186,
  211,  211,    0,    0,  197,  186,    0,  197,    0,    0,
  197,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,  197,  197,  197,    0,  197,  197,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,  210,
  210,  210,    0,    0,  210,  210,  210,    0,  210,  197,
  197,    0,    0,    0,    0,    0,    0,    0,  210,  210,
    0,    0,    0,    0,    0,    0,    0,  210,  210,    0,
  210,  210,  210,  210,  210,    0,    0,    0,    0,    0,
  197,  197,  195,    0,    0,    0,    0,    0,    0,    0,
    0,    0,  195,    0,    0,    0,  210,  210,  210,  210,
  210,  210,  210,  210,  210,  210,  210,  210,    0,    0,
  210,  210,    0,    0,    0,  210,    0,    0,    0,    0,
    0,  210,    0,  195,    0,    0,  195,    0,  211,  211,
  211,    0,    0,  211,  211,  211,    0,  211,    0,    0,
  195,  195,  195,    0,  195,  195,    0,  211,  211,    0,
    0,    0,    0,    0,    0,    0,  211,  211,    0,  211,
  211,  211,  211,  211,    0,    0,    0,    0,    0,    0,
    0,  196,    0,    0,    0,  195,  195,    0,    0,    0,
    0,  196,    0,    0,    0,  211,  211,  211,  211,  211,
  211,  211,  211,  211,  211,  211,  211,    0,    0,  211,
  211,    0,    0,    0,  211,    0,  195,  195,    0,    0,
  211,    0,  196,    0,    0,  196,    0,    0,  197,  197,
  197,    0,    0,  197,  197,  197,    0,  197,    0,  196,
  196,  196,    0,  196,  196,    0,    0,  197,  197,    0,
    0,    0,    0,    0,    0,    0,  197,  197,    0,  197,
  197,  197,  197,  197,    0,    0,    0,    0,    0,    0,
    0,  200,    0,    0,  196,  196,    0,    0,    0,    0,
    0,  200,    0,    0,    0,  197,  197,  197,  197,  197,
  197,  197,  197,  197,  197,  197,  197,    0,    0,    0,
    0,    0,    0,    0,  197,  196,  196,    0,    0,    0,
  197,    0,  200,    0,    0,  200,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,  200,
  200,  200,    0,  200,  200,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,  195,  195,  195,    0,    0,  195,
  195,  195,    0,  195,  200,  202,    0,    0,    0,    0,
    0,    0,    0,  195,  195,  202,    0,    0,    0,    0,
    0,    0,  195,  195,    0,  195,  195,  195,  195,  195,
    0,    0,    0,    0,    0,    0,  200,    0,    0,    0,
    0,    0,    0,    0,    0,    0,  202,    0,    0,  202,
    0,  195,  195,  195,  195,  195,  195,  195,  195,  195,
  195,  195,  195,  202,  202,  202,    0,  202,  202,    0,
  195,    0,    0,    0,    0,    0,  195,    0,    0,    0,
    0,    0,    0,  196,  196,  196,    0,    0,  196,  196,
  196,    0,  196,    0,  199,    0,    0,    0,  202,    0,
    0,    0,  196,  196,  199,    0,    0,    0,    0,    0,
    0,  196,  196,    0,  196,  196,  196,  196,  196,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
  202,    0,    0,    0,    0,  199,    0,    0,  199,    0,
  196,  196,  196,  196,  196,  196,  196,  196,  196,  196,
  196,  196,  199,  199,  199,    0,  199,  199,    0,  196,
    0,    0,    0,    0,    0,  196,    0,    0,    0,    0,
    0,    0,    0,  200,  200,  200,    0,    0,  200,  200,
  200,    0,  200,    0,  201,    0,    0,  199,    0,    0,
    0,    0,  200,  200,  201,    0,    0,    0,    0,    0,
    0,  200,  200,    0,  200,  200,  200,  200,  200,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,  199,
    0,    0,    0,    0,    0,  201,    0,    0,  201,    0,
  200,  200,  200,  200,  200,  200,  200,  200,  200,  200,
  200,  200,  201,  201,  201,    0,  201,  201,    0,  200,
    0,    0,    0,    0,    0,  200,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,  202,  202,  202,
    0,    0,  202,  202,  202,    0,  202,  201,  198,    0,
    0,    0,    0,    0,    0,    0,  202,  202,  198,    0,
    0,    0,    0,    0,    0,  202,  202,    0,  202,  202,
  202,  202,  202,    0,    0,    0,    0,    0,    0,  201,
    0,    0,    0,    0,    0,    0,    0,    0,    0,  198,
    0,    0,  198,    0,  202,  202,  202,  202,  202,  202,
  202,  202,  202,  202,  202,  202,  198,  198,    0,    0,
    0,  198,    0,  202,    0,    0,    0,    0,    0,  202,
    0,    0,    0,    0,    0,    0,  199,  199,  199,    0,
    0,  199,  199,  199,    0,  199,    0,  203,    0,    0,
    0,  198,    0,    0,    0,  199,  199,  203,    0,    0,
    0,    0,    0,    0,  199,  199,    0,  199,  199,  199,
  199,  199,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,  198,    0,    0,    0,    0,  203,    0,
    0,  203,    0,  199,  199,  199,  199,  199,  199,  199,
  199,  199,  199,  199,  199,  203,  203,    0,    0,    0,
  203,    0,  199,    0,    0,    0,    0,    0,  199,    0,
    0,    0,    0,    0,    0,    0,  201,  201,  201,    0,
    0,  201,  201,  201,    0,  201,    0,  204,    0,    0,
  203,    0,    0,    0,    0,  201,  201,  204,    0,    0,
    0,    0,    0,    0,  201,  201,    0,  201,  201,  201,
  201,  201,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,  203,    0,    0,    0,    0,    0,  204,    0,
    0,  204,    0,  201,  201,  201,  201,  201,  201,  201,
  201,  201,  201,  201,  201,  204,  204,    0,    0,    0,
  204,    0,  201,    0,    0,    0,    0,    0,  201,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
  198,  198,  198,    0,    0,  198,  198,  198,    0,  198,
  204,  205,    0,    0,    0,    0,    0,    0,    0,  198,
  198,  205,    0,    0,    0,    0,    0,    0,  198,  198,
    0,  198,  198,  198,  198,  198,    0,    0,    0,    0,
    0,    0,  204,    0,    0,    0,    0,    0,    0,    0,
    0,    0,  205,    0,    0,  205,    0,    0,    0,    0,
    0,    0,    0,  198,  198,    0,    0,  198,  198,  205,
  205,    0,    0,    0,  205,    0,  198,    0,    0,    0,
    0,    0,  198,    0,    0,    0,    0,    0,    0,  203,
  203,  203,    0,    0,  203,  203,  203,    0,  203,    0,
  212,    0,    0,    0,  205,    0,    0,    0,  203,  203,
  212,    0,    0,    0,    0,    0,    0,  203,  203,    0,
  203,  203,  203,  203,  203,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,  205,    0,    0,    0,
    0,  212,    0,    0,  212,    0,    0,    0,    0,    0,
    0,    0,  203,  203,    0,    0,  203,  203,  212,  212,
    0,    0,    0,  212,    0,  203,    0,    0,    0,    0,
    0,  203,    0,    0,    0,    0,    0,    0,    0,  204,
  204,  204,    0,    0,  204,  204,  204,    0,  204,    0,
  206,    0,    0,  212,    0,    0,    0,    0,  204,  204,
  206,    0,    0,    0,    0,    0,    0,  204,  204,    0,
  204,  204,  204,  204,  204,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,  212,    0,    0,    0,    0,
    0,  206,    0,    0,  206,    0,    0,    0,    0,    0,
    0,    0,  204,  204,    0,    0,  204,  204,  206,  206,
    0,    0,    0,  206,    0,  204,    0,    0,    0,    0,
    0,  204,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,  205,  205,  205,    0,    0,  205,  205,
  205,    0,  205,  206,  207,    0,    0,    0,    0,    0,
    0,    0,  205,  205,  207,    0,    0,    0,    0,    0,
    0,  205,  205,    0,  205,  205,  205,  205,  205,    0,
    0,    0,    0,    0,    0,  206,    0,    0,    0,    0,
    0,    0,    0,    0,    0,  207,    0,    0,  207,    0,
    0,    0,    0,    0,    0,    0,  205,  205,    0,    0,
  205,  205,  207,  207,    0,    0,    0,  207,    0,  205,
    0,    0,    0,    0,    0,  205,    0,    0,    0,    0,
    0,    0,  212,  212,  212,    0,    0,  212,  212,  212,
    0,  212,    0,  213,    0,    0,    0,  207,    0,    0,
    0,  212,  212,  213,    0,    0,    0,    0,    0,    0,
  212,  212,    0,  212,  212,  212,  212,  212,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,  207,
    0,    0,    0,    0,  213,    0,    0,  213,    0,    0,
    0,    0,    0,    0,    0,  212,  212,    0,    0,  212,
  212,  213,  213,    0,    0,    0,  213,    0,  212,    0,
    0,    0,    0,    0,  212,    0,    0,    0,    0,    0,
    0,    0,  206,  206,  206,    0,    0,  206,  206,  206,
    0,  206,    0,  183,    0,    0,  213,    0,    0,    0,
    0,  206,  206,  183,    0,    0,    0,    0,    0,    0,
  206,  206,    0,  206,  206,  206,  206,  206,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,  213,  184,
    0,    0,    0,    0,  183,    0,    0,  183,    0,  184,
    0,    0,    0,    0,    0,  206,  206,    0,    0,  206,
  206,  183,  183,    0,    0,    0,  183,    0,  206,    0,
    0,    0,    0,    0,  206,    0,    0,    0,    0,    0,
  184,    0,    0,  184,    0,    0,  207,  207,  207,    0,
    0,  207,  207,  207,    0,  207,  183,  184,  184,  175,
    0,    0,  184,    0,    0,  207,  207,    0,    0,  175,
    0,    0,    0,    0,  207,  207,    0,  207,  207,  207,
  207,  207,    0,    0,    0,    0,    0,    0,  183,    0,
    0,    0,  184,    0,    0,    0,    0,    0,    0,    0,
  175,    0,    0,  175,    0,    0,    0,    0,    0,  207,
  207,    0,    0,  207,  207,    0,    0,  175,  175,    0,
    0,    0,  207,    0,  184,    0,    0,    0,  207,    0,
    0,    0,    0,    0,    0,  213,  213,  213,    0,    0,
  213,  213,  213,    0,  213,    0,  182,    0,    0,    0,
    0,    0,  175,    0,  213,  213,  182,    0,    0,    0,
    0,    0,    0,  213,  213,    0,  213,  213,  213,  213,
  213,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,  175,    0,    0,  182,    0,    0,
  182,    0,    0,    0,    0,    0,    0,    0,    0,  213,
    0,    0,  213,  213,  182,  182,  215,    0,    0,    0,
    0,  213,    0,    0,    0,    0,  215,  213,    0,    0,
    0,    0,    0,    0,    0,  183,  183,  183,    0,    0,
  183,  183,  183,    0,  183,    0,    0,    0,    0,  182,
    0,    0,    0,    0,  183,  183,    0,  215,    0,    0,
  215,    0,    0,  183,  183,    0,  183,  183,  183,  183,
  183,  184,  184,  184,  215,  215,  184,  184,  184,    0,
  184,  182,    0,    0,  181,    0,    0,    0,    0,    0,
  184,  184,    0,    0,  181,    0,    0,    0,    0,  184,
  184,    0,  184,  184,  184,  184,  184,    0,    0,  215,
    0,  183,    0,    0,    0,    0,    0,  183,    0,    0,
    0,    0,    0,    0,    0,  181,    0,    0,  181,    0,
    0,  175,  175,  175,    0,    0,  175,  175,  175,  216,
  175,  215,  181,  181,    0,    0,    0,  184,    0,  216,
  175,  175,    0,  184,    0,    0,    0,    0,    0,  175,
  175,    0,  175,  175,  175,  175,  175,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,  181,    0,    0,
  216,    0,    0,  216,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,  216,  216,    0,
  179,    0,    0,    0,    0,    0,    0,  175,    0,  181,
  179,    0,    0,  175,    0,    0,    0,    0,  182,  182,
  182,    0,    0,  182,  182,  182,    0,  182,    0,    0,
    0,    0,  216,    0,    0,    0,    0,  182,  182,    0,
    0,  179,    0,    0,  179,    0,  182,  182,    0,  182,
  182,  182,  182,  182,    0,    0,    0,    0,  179,  179,
    0,    0,    0,    0,  216,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,  215,  215,
  215,    0,    0,  215,  215,  215,    0,  215,    0,    0,
    0,  180,    0,  179,  182,    0,    0,  215,  215,    0,
  182,  180,    0,    0,    0,    0,  215,  215,    0,  215,
  215,  215,  215,  215,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,  179,    0,    0,    0,    0,
    0,    0,  180,    0,    0,  180,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,  181,  181,  181,  180,
  180,  181,  181,  181,  215,  181,    0,    0,    0,    0,
  215,    0,    0,    0,    0,  181,  181,    0,    0,    0,
    0,    0,    0,    0,  181,  181,    0,  181,  181,  181,
  181,  181,  177,    0,  180,    0,    0,    0,    0,    0,
    0,    0,  177,    0,    0,    0,    0,    0,    0,    0,
    0,  216,  216,  216,    0,    0,  216,  216,  216,    0,
  216,    0,    0,    0,    0,    0,  180,    0,    0,    0,
  216,  216,  181,  177,    0,    0,  177,    0,  181,  216,
  216,    0,  216,  216,  216,  216,  216,    0,    0,    0,
  177,  177,    0,  178,    0,    0,    0,    0,    0,    0,
    0,    0,    0,  178,    0,    0,    0,    0,    0,    0,
    0,    0,  179,  179,  179,    0,    0,  179,  179,  179,
    0,  179,    0,    0,    0,  177,    0,  216,    0,    0,
    0,  179,  179,  216,  178,    0,    0,  178,    0,    0,
  179,  179,    0,  179,  179,  179,  179,  179,  174,    0,
    0,  178,  178,    0,    0,    0,    0,  177,  174,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,  176,  178,    0,  179,  174,
    0,    0,  174,    0,  179,  176,    0,    0,    0,    0,
    0,    0,    0,  180,  180,  180,  174,  174,  180,  180,
  180,    0,  180,    0,    0,    0,    0,    0,  178,    0,
    0,    0,  180,  180,    0,    0,  176,    0,    0,  176,
    0,  180,  180,    0,  180,  180,  180,  180,  180,    0,
    0,  174,    0,  176,  176,    0,  262,    0,    0,    0,
    0,    0,    0,    0,    0,    0,  262,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,  174,    0,    0,    0,    0,  176,  180,
    0,    0,    0,    0,    0,  180,    0,  262,    0,    0,
  262,    0,    0,    0,  177,  177,  177,    0,    0,  177,
  177,  177,    0,  177,  262,  262,    0,  173,    0,    0,
  176,    0,    0,  177,  177,    0,    0,  173,    0,    0,
    0,    0,  177,  177,    0,  177,  177,  177,  177,  177,
    0,    0,    0,    0,    0,    0,    0,    0,    0,  262,
    0,    0,    0,    0,    0,    0,    0,    0,  173,    0,
    0,  173,    0,    0,    0,  178,  178,  178,    0,    0,
  178,  178,  178,    0,  178,  173,  173,    0,    0,    0,
  177,  262,    0,    0,  178,  178,  177,    0,    0,    0,
    0,    0,    0,  178,  178,    0,  178,  178,  178,  178,
  178,    0,    0,    0,    0,    0,    0,    0,    0,    0,
  173,    0,    0,    0,    0,    0,    0,    0,    0,    0,
  174,  174,  174,    0,    0,  174,  174,  174,    0,  174,
    0,    0,    0,    0,    0,    0,    0,    0,    0,  174,
  174,  178,  173,    0,    0,    0,    0,  178,  174,  174,
    0,  174,  174,  174,  174,  174,    0,  176,  176,  176,
    0,    0,  176,  176,  176,    0,  176,    0,    0,    0,
  263,    0,    0,    0,    0,    0,  176,  176,    0,    0,
  263,    0,    0,    0,    0,  176,  176,    0,  176,  176,
  176,  176,  176,    0,    0,    0,  174,    0,    0,    0,
    0,    0,  174,    0,    0,    0,    0,    0,    0,    0,
    0,  263,    0,    0,  263,    0,    0,    0,  262,  262,
  262,    0,    0,  262,  262,  262,    0,  262,  263,  263,
    0,    0,    0,  176,    0,    0,    0,  262,  262,  176,
    0,    0,    0,    0,    0,    0,  262,  262,    0,  262,
  262,  262,  262,  262,    0,    0,    0,    0,    0,    0,
    0,    0,    0,  263,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,  322,    0,  173,
  173,  173,    0,    0,  173,  173,  173,  322,  173,    0,
    0,    0,    0,    0,    0,  263,    0,    0,  173,  173,
  262,    0,    0,    0,    0,    0,    0,  173,  173,    0,
  173,  173,  173,  173,  322,  322,    0,    0,  322,  322,
  322,  322,  322,  322,  322,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,  322,  322,  322,    0,  322,
  322,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,  173,    0,    0,    0,    0,
    0,  173,  323,    0,    0,    0,    0,    0,  322,    0,
    0,  322,  323,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,  323,
  323,  322,  322,  323,  323,  323,  323,  323,  323,  323,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
  323,  323,  323,    0,  323,  323,    0,    0,    0,    0,
    0,    0,  263,  263,  263,    0,    0,  263,  263,  263,
    0,  263,    0,    0,    0,    0,    0,    0,    0,    0,
    0,  263,  263,  323,    0,    0,  323,    0,  286,    0,
  263,  263,    0,  263,  263,  263,  263,  263,  286,   47,
    0,    0,    0,    0,    0,    0,    0,    0,    0,   47,
    0,    0,    0,    0,    0,    0,  323,  323,    0,    0,
    0,    0,    0,    0,    0,  286,  286,    0,    0,  286,
  286,  286,  286,  286,  286,  286,    0,    0,    0,    0,
   47,    0,    0,    0,  263,    0,  286,  286,  286,    0,
  286,  286,    0,    0,    0,    0,    0,   47,   47,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,  322,
  322,  322,    0,    0,  322,  322,  322,    0,  322,  286,
    0,    0,  286,    0,    0,    0,    0,    0,  322,    0,
    0,    0,    0,    0,    0,    0,    0,  322,  322,    0,
  322,  322,  322,  322,  322,    0,    0,    0,    0,    0,
    0,    0,  286,  286,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,   47,  322,  322,  322,  322,  322,
  322,  322,  322,  322,  322,  322,  322,  322,    0,    0,
  322,  322,  322,    0,    0,  322,    0,    0,    0,    0,
    0,    0,    0,    0,  323,  323,  323,    0,    0,  323,
  323,  323,    0,  323,    0,    0,    0,    0,    0,    0,
    0,    0,    0,  323,    0,    0,    0,    0,    0,    0,
    0,    0,  323,  323,    0,  323,  323,  323,  323,  323,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
  323,  323,  323,  323,  323,  323,  323,  323,  323,  323,
  323,  323,  323,    0,    0,  323,  323,  323,    0,    0,
  323,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
  286,  286,  286,    0,    0,  286,  286,  286,    0,  286,
    0,   47,   47,   47,    0,    0,   47,   47,   47,  286,
   47,    0,    0,    0,    0,    0,    0,    0,  286,  286,
   47,  286,  286,  286,  286,  286,    0,  430,    0,   47,
   47,    0,   47,   47,   47,   47,   47,  430,    0,    0,
    0,    0,    0,    0,    0,    0,  286,  286,  286,  286,
  286,  286,  286,  286,  286,  286,  286,  286,  286,    0,
    0,  286,  286,  286,  430,  430,  286,    0,  430,  430,
  430,   75,  430,  430,  430,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,  430,  430,   83,  430,
  430,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,  274,    0,    0,    0,    0,    0,    0,
    0,    0,    0,  274,    0,    0,    0,    0,  430,    0,
    0,  430,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
  274,  274,    0,    0,  274,  274,  274,   82,  274,  274,
  274,  430,  430,    0,    0,    0,    0,    0,    0,    0,
    0,    0,  274,  274,   90,  274,  274,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,  282,    0,    0,
    0,    0,    0,    0,    0,    0,    0,  282,    0,    0,
    0,    0,    0,    0,  274,    0,    0,  274,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,  282,  282,    0,    0,  282,  282,
  282,   81,  282,  282,  282,    0,    0,  274,  274,    0,
    0,    0,    0,    0,    0,    0,  282,  282,   89,  282,
  282,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,  282,    0,
    0,  282,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,  430,
  430,  430,    0,    0,    0,  430,  430,    0,  430,    0,
    0,  282,  282,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,  430,  430,    0,
  430,  430,  430,  430,  430,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,  430,  430,  430,  430,  430,
  430,  430,  430,  430,  430,  430,  430,  430,    0,    0,
  430,  430,  430,    0,  431,  274,  274,  274,    0,    0,
   58,  274,  274,    0,  274,    0,    0,    0,  333,  328,
    0,    0,    0,  331,  329,  605,  330,    0,  332,    0,
    0,    0,    0,  274,  274,    0,  274,  274,  274,  274,
  274,  325,    0,  324,  323,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,  274,  274,  274,  274,  274,  274,  274,  274,  274,
  274,  274,  274,  274,    0,  327,  274,  274,  274,  282,
  282,  282,    0,    0,    0,  282,  282,    0,  282,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,   59,    0,  326,    0,  282,  282,    0,
  282,  282,  282,  282,  282,    0,  283,    0,    0,    0,
    0,    0,    0,    0,    0,    0,  283,    0,    0,    0,
    0,    0,    0,    0,    0,  282,  282,  282,  282,  282,
  282,  282,  282,  282,  282,  282,  282,  282,    0,    0,
  282,  282,  282,  283,  283,    0,    0,  283,  283,  283,
   76,  283,  283,  283,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,  283,  283,   84,  283,  283,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,  283,    0,    0,
  283,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
   58,    0,    0,    3,    4,    5,    6,    7,    8,    0,
  283,  283,    9,   10,    0,    0,    0,   11,    0,   12,
   13,   14,   15,   16,   17,   18,    0,    0,    0,    0,
   19,   20,   21,   22,   23,   24,   25,    0,    0,   26,
    0,    0,    0,    0,    0,   27,   28,   29,   30,   31,
   32,   33,   34,   35,   36,   37,   38,   39,   40,    0,
   41,   42,    0,   43,   44,    0,    0,    0,    0,  308,
  309,  310,  311,  312,  313,  314,  315,  316,  317,  318,
  319,  320,    0,   45,  321,  322,   46,   47,    0,   48,
   49,    0,   50,   59,   51,   52,   53,   54,   55,   56,
    0,   58,    0,    0,   57,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,  394,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,  283,  283,
  283,    0,    0,    0,  283,  283,    0,  283,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,  283,  283,    0,  283,
  283,  283,  283,  283,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,   59,    0,    0,    0,    0,    0,
    0,    0,   58,    0,  283,  283,  283,  283,  283,  283,
  283,  283,  283,  283,  283,  283,  283,    0,    0,  283,
  283,  283,    0,    3,    4,    5,    6,    7,    8,    0,
    0,    0,    9,   10,    0,    0,    0,   11,    0,   12,
   13,   14,   15,   16,   17,   18,    0,    0,    0,    0,
   19,   20,   21,   22,   23,   24,   25,    0,    0,   26,
    0,    0,    0,    0,    0,   27,   28,   29,   30,   31,
   32,   33,   34,   35,   36,   37,   38,   39,   40,    0,
   41,   42,    0,   43,   44,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,   59,    0,    0,    0,    0,
    0,    0,    0,   45,    0,  232,  260,   47,    0,   48,
   49,    0,   50,  410,   51,   52,   53,   54,   55,   56,
    0,    0,    0,    0,   57,    4,    5,    6,    7,    8,
    0,    0,    0,    9,   10,    0,    0,    0,   11,    0,
   12,   13,   14,   15,   16,   17,   18,    0,    0,    0,
    0,   19,   20,   21,   22,   23,   24,   25,    0,    0,
   26,    0,    0,    0,    0,    0,   27,   28,   29,   30,
   31,   32,   33,   34,   35,   36,   37,   38,   39,   40,
    0,   41,   42,    0,   43,   44,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,   59,    0,
    0,    0,  232,    0,   45,    0,    0,   46,   47,    0,
   48,   49,    0,   50,    0,   51,   52,   53,   54,   55,
   56,    0,    0,    0,    0,   57,    4,    5,    6,    7,
    8,    0,    0,    0,    9,   10,    0,    0,    0,   11,
    0,   12,   13,   14,   15,   16,   17,   18,    0,    0,
    0,    0,   19,   20,   21,   22,   23,   24,   25,    0,
    0,   26,    0,    0,    0,    0,    0,   27,   28,   29,
   30,   31,   32,   33,   34,   35,   36,   37,   38,   39,
   40,    0,   41,   42,    0,   43,   44,    0,    0,    0,
    0,    0,    0,    0,    0,   59,    0,    0,    0,  232,
    0,    0,    0,    0,    0,   45,    0,    0,   46,   47,
    0,   48,   49,    0,   50,    0,   51,   52,   53,   54,
   55,   56,    0,    0,    0,    0,   57,    0,    0,    4,
    5,    6,    0,    8,    0,    0,    0,    9,   10,    0,
    0,    0,   11,    0,   12,   13,   14,   96,   97,   17,
   18,    0,    0,    0,    0,   98,   20,   21,   22,   23,
   24,   25,    0,    0,    0,    0,    0,    0,    0,    0,
    0,   28,    0,    0,   31,   32,   33,   34,   35,   36,
   37,   38,   39,   40,    0,   41,   42,    0,   43,   44,
    0,    0,   59,    0,    0,    0,  232,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,  206,    0,
    0,  106,   47,    0,   48,   49,    0,  230,  231,   51,
   52,   53,   54,   55,   56,    0,    4,    5,    6,   57,
    8,    0,    0,    0,    9,   10,    0,    0,    0,   11,
    0,   12,   13,   14,   96,   97,   17,   18,    0,    0,
    0,    0,   98,   20,   21,   22,   23,   24,   25,    0,
    0,    0,    0,    0,    0,    0,    0,    0,   28,    0,
    0,   31,   32,   33,   34,   35,   36,   37,   38,   39,
   40,    0,   41,   42,    0,   43,   44,    0,    0,   59,
    0,    0,    0,  232,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,  206,    0,    0,  106,   47,
    0,   48,   49,    0,  230,  231,   51,   52,   53,   54,
   55,   56,    0,    4,    5,    6,   57,    8,    0,    0,
    0,    9,   10,    0,    0,    0,   11,    0,   12,   13,
   14,   96,   97,   17,   18,    0,    0,    0,    0,   98,
   20,   21,   22,   23,   24,   25,    0,    0,    0,    0,
    0,    0,    0,    0,    0,   28,    0,    0,   31,   32,
   33,   34,   35,   36,   37,   38,   39,   40,    0,   41,
   42,    0,   43,   44,    0,    0,   59,    0,    0,    0,
  232,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,  206,    0,    0,  106,  412,    0,   48,   49,
    0,  230,  231,   51,   52,   53,   54,   55,   56,    0,
    4,    5,    6,   57,    8,    0,    0,    0,    9,   10,
    0,    0,    0,   11,    0,   12,   13,   14,   96,   97,
   17,   18,    0,    0,    0,    0,   98,   99,  100,   22,
   23,   24,   25,    0,    0,    0,    0,    0,    0,    0,
    0,    0,   28,    0,    0,   31,   32,   33,   34,   35,
   36,   37,   38,   39,   40,    0,   41,   42,    0,   43,
   44,    0,    0,   59,    0,    0,    0,  232,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,  206,
    0,    0,  106,   47,    0,   48,   49,    0,  608,  231,
   51,   52,   53,   54,   55,   56,    0,    4,    5,    6,
   57,    8,    0,    0,    0,    9,   10,    0,    0,    0,
   11,    0,   12,   13,   14,   96,   97,   17,   18,    0,
    0,    0,    0,   98,   99,  100,   22,   23,   24,   25,
    0,    0,    0,    0,    0,    0,    0,    0,    0,   28,
    0,    0,   31,   32,   33,   34,   35,   36,   37,   38,
   39,   40,    0,   41,   42,    0,   43,   44,    0,    0,
   59,    0,    0,    0,  232,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,  206,    0,    0,  106,
   47,    0,   48,   49,    0,  612,  231,   51,   52,   53,
   54,   55,   56,    0,    4,    5,    6,   57,    8,    0,
    0,    0,    9,   10,    0,    0,    0,   11,    0,   12,
   13,   14,   96,   97,   17,   18,    0,    0,    0,    0,
   98,   20,   21,   22,   23,   24,   25,    0,    0,    0,
    0,    0,    0,    0,    0,    0,   28,    0,    0,   31,
   32,   33,   34,   35,   36,   37,   38,   39,   40,    0,
   41,   42,    0,   43,   44,    0,    0,   59,    0,    0,
    0,  232,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,  206,    0,    0,  106,   47,    0,   48,
   49,    0,  608,  231,   51,   52,   53,   54,   55,   56,
    0,    4,    5,    6,   57,    8,    0,    0,    0,    9,
   10,    0,    0,    0,   11,    0,   12,   13,   14,   96,
   97,   17,   18,    0,    0,    0,    0,   98,   99,  100,
   22,   23,   24,   25,    0,    0,    0,    0,    0,    0,
    0,    0,    0,   28,    0,    0,   31,   32,   33,   34,
   35,   36,   37,   38,   39,   40,    0,   41,   42,    0,
   43,   44,    0,    0,   59,    0,    0,    0,  232,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
  206,    0,    0,  106,   47,    0,   48,   49,    0,  805,
  231,   51,   52,   53,   54,   55,   56,    0,    4,    5,
    6,   57,    8,    0,    0,    0,    9,   10,    0,    0,
    0,   11,    0,   12,   13,   14,   96,   97,   17,   18,
    0,    0,    0,    0,   98,   99,  100,   22,   23,   24,
   25,    0,    0,    0,    0,    0,    0,    0,    0,    0,
   28,    0,    0,   31,   32,   33,   34,   35,   36,   37,
   38,   39,   40,    0,   41,   42,    0,   43,   44,    0,
    0,   59,    0,    0,    0,  232,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,  206,    0,    0,
  106,   47,    0,   48,   49,    0,  808,  231,   51,   52,
   53,   54,   55,   56,    0,    4,    5,    6,   57,    8,
    0,    0,    0,    9,   10,    0,    0,    0,   11,    0,
   12,   13,   14,   96,   97,   17,   18,    0,    0,    0,
    0,   98,   99,  100,   22,   23,   24,   25,    0,    0,
    0,    0,    0,    0,    0,    0,    0,   28,    0,    0,
   31,   32,   33,   34,   35,   36,   37,   38,   39,   40,
    0,   41,   42,    0,   43,   44,    0,    0,   59,    0,
    0,    0,  232,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,  206,    0,    0,  106,   47,    0,
   48,   49,    0,  813,  231,   51,   52,   53,   54,   55,
   56,    0,    4,    5,    6,   57,    8,    0,    0,    0,
    9,   10,    0,    0,    0,   11,    0,   12,   13,   14,
   96,   97,   17,   18,    0,    0,    0,    0,   98,   99,
  100,   22,   23,   24,   25,    0,    0,    0,    0,    0,
    0,    0,    0,    0,   28,    0,    0,   31,   32,   33,
   34,   35,   36,   37,   38,   39,   40,    0,   41,   42,
    0,   43,   44,    0,    0,   59,    0,    0,    0,  252,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,  206,    0,    0,  106,   47,    0,   48,   49,    0,
  872,  231,   51,   52,   53,   54,   55,   56,    0,    4,
    5,    6,   57,    8,    0,    0,    0,    9,   10,    0,
    0,    0,   11,    0,   12,   13,   14,   96,   97,   17,
   18,    0,    0,    0,    0,   98,   99,  100,   22,   23,
   24,   25,    0,    0,    0,    0,    0,    0,    0,    0,
    0,   28,    0,    0,   31,   32,   33,   34,   35,   36,
   37,   38,   39,   40,    0,   41,   42,    0,   43,   44,
    0,    0,  252,    0,    0,    0,   58,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,  206,    0,
    0,  106,   47,    0,   48,   49,    0,  874,  231,   51,
   52,   53,   54,   55,   56,    0,    4,    5,    6,   57,
    8,    0,    0,    0,    9,   10,    0,    0,    0,   11,
    0,   12,   13,   14,   96,   97,   17,   18,    0,    0,
    0,    0,   98,   99,  100,   22,   23,   24,   25,    0,
    0,    0,    0,    0,    0,    0,    0,    0,   28,    0,
    0,   31,   32,   33,   34,   35,   36,   37,   38,   39,
   40,    0,   41,   42,    0,   43,   44,    0,    0,   59,
    0,    0,    0,  232,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,  206,    0,    0,  106,   47,
    0,   48,   49,    0,  893,  231,   51,   52,   53,   54,
   55,   56,    0,  252,  252,  252,   57,  252,    0,    0,
    0,  252,  252,    0,    0,    0,  252,    0,  252,  252,
  252,  252,  252,  252,  252,    0,    0,    0,    0,  252,
  252,  252,  252,  252,  252,  252,    0,    0,    0,    0,
    0,    0,    0,    0,    0,  252,    0,    0,  252,  252,
  252,  252,  252,  252,  252,  252,  252,  252,    0,  252,
  252,    0,  252,  252,    0,    0,   59,    0,    0,    0,
  232,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,  252,    0,    0,  252,  252,    0,  252,  252,
    0,  252,  252,  252,  252,  252,  252,  252,  252,    0,
    4,    5,    6,  252,    8,    0,    0,    0,    9,   10,
    0,    0,    0,   11,    0,   12,   13,   14,   15,   16,
   17,   18,    0,    0,    0,    0,   19,   20,   21,   22,
   23,   24,   25,    0,    0,   26,    0,    0,    0,    0,
    0,    0,   28,    0,    0,   31,   32,   33,   34,   35,
   36,   37,   38,   39,   40,    0,   41,   42,    0,   43,
   44,    0,    0,   59,    0,    0,    0,  232,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,  206,
    0,    0,  106,   47,    0,   48,   49,    0,    0,    0,
   51,   52,   53,   54,   55,   56,    0,    4,    5,    6,
   57,    8,    0,    0,    0,    9,   10,    0,    0,    0,
   11,    0,   12,   13,   14,   96,   97,   17,   18,    0,
    0,    0,    0,   98,   20,   21,   22,   23,   24,   25,
    0,    0,    0,    0,    0,    0,    0,    0,    0,   28,
    0,    0,   31,   32,   33,   34,   35,   36,   37,   38,
   39,   40,    0,   41,   42,    0,   43,   44,    0,    0,
   59,    0,    0,    0,  232,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,  206,    0,    0,  106,
   47,    0,   48,   49,    0,  265,    0,   51,   52,   53,
   54,   55,   56,    0,    4,    5,    6,   57,    8,    0,
    0,    0,    9,   10,    0,    0,    0,   11,    0,   12,
   13,   14,   15,   16,   17,   18,    0,    0,    0,    0,
   19,   20,   21,   22,   23,   24,   25,    0,    0,    0,
    0,    0,    0,    0,    0,    0,   28,    0,    0,   31,
   32,   33,   34,   35,   36,   37,   38,   39,   40,    0,
   41,   42,    0,   43,   44,    0,    0,   59,    0,    0,
    0,  232,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,  206,    0,    0,  106,   47,    0,   48,
   49,    0,  526,    0,   51,   52,   53,   54,   55,   56,
    0,    4,    5,    6,   57,    8,    0,    0,    0,    9,
   10,    0,    0,    0,   11,    0,   12,   13,   14,   96,
   97,   17,   18,    0,    0,    0,    0,   98,   99,  100,
   22,   23,   24,   25,    0,    0,    0,    0,    0,    0,
    0,    0,    0,   28,    0,    0,   31,   32,   33,   34,
   35,   36,   37,   38,   39,   40,    0,   41,   42,    0,
   43,   44,    0,    0,   59,    0,    0,    0,  232,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
  206,    0,    0,  106,   47,    0,   48,   49,    0,  590,
    0,   51,   52,   53,   54,   55,   56,    0,    4,    5,
    6,   57,    8,    0,    0,    0,    9,   10,    0,    0,
    0,   11,    0,   12,   13,   14,   96,   97,   17,   18,
    0,    0,    0,    0,   98,   99,  100,   22,   23,   24,
   25,    0,    0,    0,    0,    0,    0,    0,    0,    0,
   28,    0,    0,   31,   32,   33,   34,   35,   36,   37,
   38,   39,   40,    0,   41,   42,    0,   43,   44,    0,
    0,   59,    0,    0,    0,  232,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,  206,    0,    0,
  106,   47,    0,   48,   49,    0,  638,    0,   51,   52,
   53,   54,   55,   56,    0,    4,    5,    6,   57,    8,
    0,    0,    0,    9,   10,    0,    0,    0,   11,    0,
   12,   13,   14,   96,   97,   17,   18,    0,    0,    0,
    0,   98,   99,  100,   22,   23,   24,   25,    0,    0,
    0,    0,    0,    0,    0,    0,    0,   28,    0,    0,
   31,   32,   33,   34,   35,   36,   37,   38,   39,   40,
    0,   41,   42,    0,   43,   44,    0,    0,   59,    0,
    0,    0,  491,    0,    0,    0,    0,    0,    0,  491,
    0,    0,    0,    0,  206,    0,    0,  106,   47,    0,
   48,   49,    0,  526,    0,   51,   52,   53,   54,   55,
   56,    0,    4,    5,    6,   57,    8,    0,    0,    0,
    9,   10,    0,    0,    0,   11,    0,   12,   13,   14,
   96,   97,   17,   18,    0,    0,    0,    0,   98,   99,
  100,   22,   23,   24,   25,    0,    0,    0,    0,    0,
    0,    0,    0,    0,   28,    0,    0,   31,   32,   33,
   34,   35,   36,   37,   38,   39,   40,    0,   41,   42,
    0,   43,   44,    0,    0,  491,    0,    0,    0,  232,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,  206,    0,    0,  106,   47,    0,   48,   49,    0,
  769,    0,   51,   52,   53,   54,   55,   56,    0,    4,
    5,    6,   57,    8,    0,    0,    0,    9,   10,    0,
    0,    0,   11,    0,   12,   13,   14,   96,   97,   17,
   18,    0,    0,    0,    0,   98,   99,  100,   22,   23,
   24,   25,    0,    0,    0,    0,    0,    0,    0,    0,
    0,   28,    0,    0,   31,   32,   33,   34,   35,   36,
   37,   38,   39,   40,    0,   41,   42,    0,   43,   44,
    0,    0,   59,    0,    0,    0,  232,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,  206,    0,
    0,  106,   47,    0,   48,   49,    0,  799,    0,   51,
   52,   53,   54,   55,   56,    0,  491,  491,  491,   57,
  491,    0,    0,    0,  491,  491,    0,    0,    0,  491,
    0,  491,  491,  491,  491,  491,  491,  491,    0,    0,
    0,    0,  491,  491,  491,  491,  491,  491,  491,    0,
    0,    0,    0,    0,    0,    0,    0,    0,  491,    0,
    0,  491,  491,  491,  491,  491,  491,  491,  491,  491,
  491,    0,  491,  491,    0,  491,  491,    0,    0,   59,
    0,    0,    0,  214,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,  491,    0,    0,  491,  491,
    0,  491,  491,    0,    0,    0,  491,  491,  491,  491,
  491,  491,    0,    4,    5,    6,  491,    8,    0,    0,
    0,    9,   10,    0,    0,    0,   11,    0,   12,   13,
   14,   96,   97,   17,   18,    0,    0,    0,    0,   98,
   99,  100,   22,   23,   24,   25,    0,    0,    0,    0,
    0,    0,    0,    0,    0,   28,    0,    0,   31,   32,
   33,   34,   35,   36,   37,   38,   39,   40,    0,   41,
   42,    0,   43,   44,    0,    0,  214,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,  206,    0,    0,  106,   47,    0,   48,   49,
    0,    0,    0,   51,   52,   53,   54,   55,   56,    0,
    4,    5,    6,   57,    8,    0,    0,    0,    9,   10,
    0,    0,    0,   11,    0,   12,   13,   14,   15,   16,
   17,   18,    0,    0,    0,    0,   19,   20,   21,   22,
   23,   24,   25,    0,    0,    0,    0,    0,    0,    0,
    0,    0,   28,    0,    0,   31,   32,   33,   34,   35,
   36,   37,   38,   39,   40,    0,   41,   42,    0,   43,
   44,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,  206,
    0,    0,  106,   47,    0,   48,   49,    0,    0,    0,
   51,   52,   53,   54,   55,   56,    0,  214,  214,  214,
   57,  214,    0,    0,    0,  214,  214,    0,    0,    0,
  214,    0,  214,  214,  214,  214,  214,  214,  214,    0,
    0,    0,    0,  214,  214,  214,  214,  214,  214,  214,
    0,    0,    0,    0,    0,    0,    0,    0,    0,  214,
    0,    0,  214,  214,  214,  214,  214,  214,  214,  214,
  214,  214,    0,  214,  214,    0,  214,  214,    0,    0,
    0,  183,  178,    0,    0,    0,  181,  179,    0,  180,
    0,  182,    0,    0,    0,    0,  214,    0,    0,  214,
  214,    0,  214,  214,  175,    0,  174,  214,  214,  214,
  214,  214,  214,    0,    0,    0,    0,  214,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,  177,    0,
  185,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,  176,    0,
  184,    0,  183,  178,    0,    0,    0,  181,  179,    0,
  180,    0,  182,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,  175,    0,  174,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,  333,  328,    0,    0,    0,  331,  329,  177,
  330,  185,  332,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,  325,    0,  324,  323,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,  176,
    0,  184,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,  327,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,  116,  117,  118,  119,  120,  121,  122,  123,  326,
    0,  124,  125,  126,  127,  128,    0,    0,  129,  130,
  131,  132,  133,  134,  135,    0,    0,  136,  137,  138,
  192,  193,  194,  195,  143,  144,  145,  146,  147,  148,
  149,  150,  151,  152,  153,  154,  196,  197,  198,  158,
  250,    0,  199,    0,    0,    0,    0,    0,    0,    0,
  160,  161,  162,  163,  164,  165,    0,  166,  167,    0,
    0,  168,    0,    0,    0,  169,  170,  171,  172,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,  173,
    0,  200,  116,  117,  118,  119,  120,  121,  122,  123,
    0,    0,  124,  125,  126,  127,  128,    0,    0,  129,
  130,  131,  132,  133,  134,  135,    0,    0,  136,  137,
  138,  192,  193,  194,  195,  143,  144,  145,  146,  147,
  148,  149,  150,  151,  152,  153,  154,  196,  197,  198,
  158,    0,    0,  199,    0,    0,    0,    0,    0,    0,
    0,  160,  161,  162,  163,  164,  165,    0,  166,  167,
    0,    0,  168,    0,    0,    0,  169,  170,  171,  172,
    0,  183,  178,    0,  186,    0,  181,  179,    0,  180,
  173,  182,  200,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,  175,    0,  174,    0,    0,    0,
    0,    0,    0,  308,  309,  310,  311,  312,  313,  314,
  315,  316,  317,  318,  319,  320,    0,    0,  321,  322,
    0,    0,    0,    0,    0,    0,    0,    0,  177,    0,
  185,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,  183,  178,    0,    0,    0,  181,  179,  176,  180,
  184,  182,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,  175,    0,  174,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,  183,  178,    0,
    0,    0,  181,  179,    0,  180,    0,  182,  177,    0,
  185,    0,    0,    0,    0,    0,    0,    0,    0,    0,
  175,    0,  174,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,  176,    0,
  184,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,  177,    0,  185,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,  176,    0,  184,    0,    0,    0,
    0,  116,  117,  118,  119,  120,  121,  122,  123,    0,
    0,  124,  125,  126,  127,  128,    0,    0,  129,  130,
  131,  132,  133,  134,  135,    0,    0,  136,  137,  138,
  139,  140,  141,  142,  143,  144,  145,  146,  147,  148,
  149,  150,  151,  152,  153,  154,  155,  156,  157,  158,
   35,   36,  159,   38,    0,    0,    0,    0,    0,    0,
  160,  161,  162,  163,  164,  165,    0,  166,  167,    0,
    0,  168,    0,    0,    0,  169,  170,  171,  172,    0,
    0,  116,  117,  118,  119,  120,  121,  122,  123,  173,
    0,  124,  125,  126,  127,  128,    0,    0,  129,  130,
  131,  132,  133,  134,  135,    0,    0,  136,  137,  138,
  192,  193,  194,  195,  143,  144,  145,  146,  147,  148,
  149,  150,  151,  152,  153,  154,  196,  197,  198,  158,
  280,  281,  199,  282,    0,    0,    0,    0,    0,    0,
  160,  161,  162,  163,  164,  165,    0,  166,  167,    0,
    0,  168,    0,    0,    0,  169,  170,  171,  172,    0,
  183,  178,    0,    0,    0,  181,  179,    0,  180,  173,
  182,    0,    0,    0,  506,  507,    0,    0,  508,    0,
    0,    0,    0,  175,    0,  174,  160,  161,  162,  163,
  164,  165,    0,  166,  167,    0,    0,  168,    0,    0,
    0,  169,  170,  171,  172,    0,  183,  178,    0,    0,
    0,  181,  179,    0,  180,  173,  182,  177,    0,  185,
    0,    0,    0,    0,    0,    0,    0,    0,    0,  175,
    0,  174,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,  183,  178,  176,    0,  184,
  181,  179,    0,  180,    0,  182,    0,    0,    0,    0,
    0,    0,    0,  177,    0,  185,    0,    0,  175,    0,
  174,    0,    0,    0,    0,    0,  183,  178,    0,    0,
    0,  181,  179,    0,  180,    0,  182,    0,    0,    0,
    0,    0,    0,  176,    0,  184,    0,    0,    0,  175,
    0,  174,  177,    0,  185,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,  176,  177,  184,  185,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,  176,    0,  184,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
  116,  117,  118,  119,  120,  121,  122,  123,    0,    0,
  124,  125,  126,  127,  128,    0,    0,  129,  130,  131,
  132,  133,  134,  135,    0,    0,  136,  137,  138,  192,
  193,  194,  195,  143,  144,  145,  146,  147,  148,  149,
  150,  151,  152,  153,  154,  196,  197,  198,  158,    0,
    0,  199,    0,    0,    0,    0,    0,    0,    0,  160,
  161,  162,  163,  164,  165,    0,  166,  167,    0,    0,
  168,    0,    0,    0,  169,  170,  171,  172,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,  173,    0,
    0,    0,    0,  513,  514,    0,    0,  515,    0,    0,
    0,    0,    0,    0,    0,  160,  161,  162,  163,  164,
  165,    0,  166,  167,    0,    0,  168,    0,    0,    0,
  169,  170,  171,  172,    0,    0,    0,    0,    0,    0,
    0,    0,  520,  514,  173,    0,  521,    0,    0,    0,
    0,    0,    0,    0,  160,  161,  162,  163,  164,  165,
    0,  166,  167,    0,    0,  168,    0,    0,    0,  169,
  170,  171,  172,  544,  507,    0,    0,  545,    0,    0,
    0,    0,    0,  173,    0,  160,  161,  162,  163,  164,
  165,    0,  166,  167,    0,    0,  168,    0,    0,    0,
  169,  170,  171,  172,    0,  183,  178,    0,    0,    0,
  181,  179,    0,  180,  173,  182,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,  175,    0,
  174,    0,    0,    0,    0,    0,  183,  178,    0,    0,
    0,  181,  179,    0,  180,    0,  182,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,  175,
    0,  174,  177,    0,  185,    0,    0,    0,    0,    0,
    0,    0,    0,    0,  183,  178,    0,    0,    0,  181,
  179,    0,  180,    0,  182,    0,    0,    0,    0,    0,
    0,    0,  176,  177,  184,  185,    0,  175,    0,  174,
    0,    0,    0,    0,    0,  183,  178,    0,    0,    0,
  181,  179,    0,  180,    0,  182,    0,    0,    0,    0,
    0,    0,    0,  176,    0,  184,    0,    0,  175,    0,
  174,  177,    0,  185,    0,    0,    0,    0,    0,    0,
    0,    0,    0,  183,  178,    0,    0,    0,  181,  179,
    0,  180,    0,  182,    0,    0,    0,    0,    0,    0,
    0,  176,  177,  184,  185,    0,  175,    0,  174,    0,
    0,    0,    0,    0,  183,  178,    0,    0,    0,  181,
  179,    0,  180,    0,  182,    0,    0,    0,    0,    0,
    0,    0,  176,    0,  184,    0,    0,  175,    0,  174,
  177,    0,  185,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
  176,  177,  184,  185,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,  176,    0,  184,    0,    0,    0,    0,    0,    0,
    0,    0,  580,  507,    0,    0,  581,    0,    0,    0,
    0,    0,    0,    0,  160,  161,  162,  163,  164,  165,
    0,  166,  167,    0,    0,  168,    0,    0,    0,  169,
  170,  171,  172,  583,  514,    0,    0,  584,    0,    0,
    0,    0,    0,  173,    0,  160,  161,  162,  163,  164,
  165,    0,  166,  167,    0,    0,  168,    0,    0,    0,
  169,  170,  171,  172,    0,    0,    0,    0,    0,    0,
    0,  599,  507,    0,  173,  600,    0,    0,    0,    0,
    0,    0,    0,  160,  161,  162,  163,  164,  165,    0,
  166,  167,    0,    0,  168,    0,    0,    0,  169,  170,
  171,  172,  602,  514,    0,    0,  603,    0,    0,    0,
    0,    0,  173,    0,  160,  161,  162,  163,  164,  165,
    0,  166,  167,    0,    0,  168,    0,    0,    0,  169,
  170,  171,  172,    0,    0,    0,    0,    0,    0,    0,
  640,  507,    0,  173,  641,    0,    0,    0,    0,    0,
    0,    0,  160,  161,  162,  163,  164,  165,    0,  166,
  167,    0,    0,  168,    0,  753,    0,  169,  170,  171,
  172,  643,  514,    0,    0,  644,    0,    0,    0,    0,
    0,  173,    0,  160,  161,  162,  163,  164,  165,    0,
  166,  167,    0,    0,  168,    0,    0,    0,  169,  170,
  171,  172,    0,  183,  178,    0,    0,    0,  181,  179,
    0,  180,  173,  182,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,  175,    0,  174,    0,
    0,    0,    0,    0,  183,  178,    0,    0,    0,  181,
  179,    0,  180,    0,  182,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,  175,    0,  174,
  177,    0,  185,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
  176,  177,  184,  185,    0,    0,    0,    0,    4,    5,
    6,    0,    8,    0,    0,    0,    9,   10,    0,    0,
    0,   11,    0,   12,   13,   14,   96,   97,   17,   18,
    0,  176,    0,  184,   98,   99,  100,   22,   23,   24,
   25,    0,    0,    0,    0,    0,    0,    0,    0,    0,
  101,    0,    0,   31,   32,   33,   34,   35,   36,   37,
   38,   39,   40,    0,   41,   42,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,  223,    0,    0,
   46,   47,    0,   48,   49,    0,   50,    0,   51,   52,
   53,   54,   55,   56,    0,    0,    0,    0,  107,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
  879,  507,    0,    0,  880,    0,    0,    0,    0,    0,
    0,    0,  160,  161,  162,  163,  164,  165,    0,  166,
  167,    0,    0,  168,    0,    0,    0,  169,  170,  171,
  172,  882,  514,    0,    0,  883,    0,    0,    0,    0,
    0,  173,    0,  160,  161,  162,  163,  164,  165,    0,
  166,  167,    0,    0,  168,    0,    0,    0,  169,  170,
  171,  172,    4,    5,    6,    0,    8,    0,    0,    0,
    9,   10,  173,    0,    0,   11,    0,   12,   13,   14,
   96,   97,   17,   18,    0,    0,    0,    0,   98,   99,
  100,   22,   23,   24,   25,    0,    0,    0,    0,    0,
    0,    0,    0,    0,  101,    0,    0,   31,   32,  102,
   34,   35,   36,  103,   38,   39,   40,    0,   41,   42,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,  104,    0,
    0,  105,    0,    0,  106,   47,    0,   48,   49,    0,
    0,    0,   51,   52,   53,   54,   55,   56,    0,    4,
    5,    6,  107,    8,    0,    0,    0,    9,   10,    0,
    0,    0,   11,    0,   12,   13,   14,   96,   97,   17,
   18,    0,    0,    0,    0,   98,   99,  100,   22,   23,
   24,   25,    0,    0,    0,    0,    0,    0,    0,    0,
    0,  101,    0,    0,   31,   32,   33,   34,   35,   36,
   37,   38,   39,   40,    0,   41,   42,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,  223,    0,
    0,   46,   47,    0,   48,   49,    0,   50,    0,   51,
   52,   53,   54,   55,   56,    0,    4,    5,    6,  107,
    8,    0,    0,    0,    9,   10,    0,    0,    0,   11,
    0,   12,   13,   14,   96,   97,   17,   18,    0,    0,
    0,    0,   98,   99,  100,   22,   23,   24,   25,    0,
    0,    0,    0,    0,    0,    0,    0,    0,  101,    0,
    0,   31,   32,   33,   34,   35,   36,   37,   38,   39,
   40,    0,   41,   42,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,  275,    0,    0,  348,   47,
    0,   48,   49,    0,  349,    0,   51,   52,   53,   54,
   55,   56,    0,    4,    5,    6,  107,    8,    0,    0,
    0,    9,   10,    0,    0,    0,   11,    0,   12,   13,
   14,   96,   97,   17,   18,    0,    0,    0,    0,   98,
   99,  100,   22,   23,   24,   25,    0,    0,    0,    0,
    0,    0,    0,    0,    0,  101,    0,    0,   31,   32,
  102,   34,   35,   36,  103,   38,   39,   40,    0,   41,
   42,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,  105,    0,    0,  106,   47,    0,   48,   49,
    0,    0,    0,   51,   52,   53,   54,   55,   56,    0,
    4,    5,    6,  107,    8,    0,    0,    0,    9,   10,
    0,    0,    0,   11,    0,   12,   13,   14,   96,   97,
   17,   18,    0,    0,    0,    0,   98,   99,  100,   22,
   23,   24,   25,    0,    0,    0,    0,    0,    0,    0,
    0,    0,  101,    0,    0,   31,   32,   33,   34,   35,
   36,   37,   38,   39,   40,    0,   41,   42,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,  275,
    0,    0,  106,   47,    0,   48,   49,    0,    0,    0,
   51,   52,   53,   54,   55,   56,    0,    4,    5,    6,
  107,    8,    0,    0,    0,    9,   10,    0,    0,    0,
   11,    0,   12,   13,   14,   96,   97,   17,   18,    0,
    0,    0,    0,   98,   99,  100,   22,   23,   24,   25,
    0,    0,    0,    0,    0,    0,    0,    0,    0,  101,
    0,    0,   31,   32,   33,   34,   35,   36,   37,   38,
   39,   40,    0,   41,   42,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,  833,    0,    0,  106,
   47,    0,   48,   49,    0,    0,    0,   51,   52,   53,
   54,   55,   56,    0,    0,    0,    0,  107,
};
const short yycheck[] = {                                      15,
   16,    6,    7,   19,   15,   16,   14,  214,   19,    6,
   27,   21,   38,    4,    5,   15,   16,   21,  390,   19,
    7,  246,   27,   14,   10,   15,   16,   43,   44,   19,
  374,   11,   48,   49,   58,  396,   50,  369,   49,   10,
   27,  408,   58,   59,  340,   10,   51,   10,   48,   15,
   16,  260,   44,   19,   51,  568,  339,  570,  341,   50,
  343,  344,   10,   11,   46,   10,   10,  124,   46,   10,
   41,  367,   58,   59,  441,   46,   10,    3,   41,   41,
  105,   44,   10,   49,   94,  368,  101,  123,  384,   32,
   94,   71,    2,  690,   59,   60,  100,   88,  695,   61,
  383,   46,  385,  399,  347,   10,  308,   40,   46,  113,
   44,   10,   40,   91,   58,   59,   46,  400,   46,   46,
   71,  690,   10,   46,   58,   59,  312,  312,  108,  662,
   58,   59,   10,   10,   93,  668,   46,  306,   10,  348,
   46,   10,   46,   10,  313,  314,  429,   40,   10,  576,
   40,  504,  505,  449,   59,  310,  311,   10,   38,  586,
   59,   91,   42,   91,   91,   46,  125,  450,   91,  355,
  355,   59,   41,   10,   46,   46,   38,   10,   40,   46,
   42,   59,   59,  123,   46,   91,   58,   59,   41,   38,
  123,   58,   59,   42,   37,  200,  106,   59,   32,   42,
   43,  304,   45,  200,   47,  308,   10,   37,   10,   46,
   91,  543,   42,   46,  230,  231,  232,   47,  562,   91,
   91,   58,   59,   32,   91,   58,   59,  243,   41,  245,
  246,  563,  243,  830,  245,  246,  286,  217,  348,   10,
  290,   10,   37,  243,  280,  245,  246,   42,   43,  265,
   45,  266,   47,  243,   91,  245,  246,   59,   91,   10,
  269,  830,  271,  282,  273,   41,  217,  290,  291,  326,
  783,  784,  785,  221,  222,   44,  337,  243,  260,  245,
  246,  267,   38,  337,   40,   61,   42,   61,   59,   61,
   41,  256,  308,  309,  310,  311,  312,  313,  314,  315,
  316,  317,  318,  319,  320,  321,  322,  323,  324,  325,
  326,  327,  328,  329,  330,  331,  332,  333,  342,   44,
  346,  345,  346,  347,  340,  678,  342,  290,  291,  345,
  346,  347,  575,  267,  339,  349,   44,  281,  308,  267,
  340,  366,  625,  626,  337,  255,  338,  280,  124,  862,
  260,  367,  338,  301,  302,  303,  304,  335,  349,  726,
  269,  366,  271,  724,  335,  732,  348,  367,  384,  312,
  386,  387,  388,  800,  300,  338,  392,  337,  383,  305,
  346,  347,  362,  399,  384,  592,  264,  264,  404,  405,
  335,  269,  408,  398,  405,  267,  412,  335,  623,  399,
  267,  412,  417,  418,  338,  335,  423,  335,  335,  425,
  353,  354,  335,  463,  430,  395,  396,   41,  423,  434,
  435,  437,  412,  428,  304,  441,  392,  443,  800,  335,
  267,  335,  776,  449,  267,   61,  423,   61,  348,   10,
  772,   59,  304,  448,  306,  307,  308,  309,   10,  449,
  308,  467,  468,  279,  335,  304,  338,  306,  307,  308,
  309,  239,  264,  335,  335,  345,  346,  269,  335,  271,
   41,  381,   44,  335,   10,  318,   44,  290,  291,  389,
   40,  391,   44,  345,  346,  852,  290,  291,  318,   41,
  333,  334,  263,  264,   41,  827,  345,  346,  335,  509,
  124,   16,  335,  279,   19,  509,  516,  123,   44,   61,
  526,  318,  522,  523,   61,   10,  426,    2,    3,    4,
    5,  888,   10,  318,    9,   10,   11,   37,   38,   14,
   15,   16,   42,   43,   19,   45,  551,   47,  337,  522,
  523,   26,  546,  269,   10,  271,  125,  552,  304,   10,
  306,  307,  308,  309,   41,  552,  573,  310,  311,  575,
  540,   46,   47,   48,   59,   50,  268,  269,  573,  865,
   58,   59,  124,   58,  590,   93,   10,  124,    0,   41,
   10,  788,   10,  866,  564,  308,  573,  794,   10,  345,
  346,  355,  608,   59,  542,  355,  612,   58,   59,  269,
  616,  271,   40,   88,  263,  264,   44,  623,   46,  575,
   40,  318,  623,   41,   40,  269,   46,  271,   44,  104,
   46,  106,  638,  623,  402,   59,  263,  264,   58,   59,
  125,   59,  269,  318,   10,  659,  652,   59,   53,   54,
  664,  665,   41,  659,  660,   44,  654,   46,  664,  665,
  269,   44,  271,   91,   44,  279,  604,  344,  264,  675,
  676,   91,   61,  654,  262,   91,  267,    2,    3,  264,
  686,   61,  687,   44,    9,   10,   11,  587,   41,   44,
   15,   16,   58,   59,   19,  123,  124,   41,  125,   41,
   32,   26,   91,  123,  710,  337,  712,  713,  124,  337,
  680,   93,  337,  719,  337,    9,   10,   11,   10,  312,
  726,   46,   47,   48,  730,   44,  732,   41,   10,  730,
  205,  736,   26,   58,   93,  124,  264,  279,  243,  744,
  245,  246,  279,   61,  124,  645,  221,  222,  762,   41,
  730,   44,   44,   47,  724,  312,  762,   44,   40,   44,
  353,  354,  355,  769,   46,  738,  739,   59,  243,  707,
  245,  246,   41,  304,  730,   44,   58,   59,  269,  104,
  255,  106,  304,  337,   93,  260,  337,   44,  337,  267,
  796,  264,   61,  799,  312,  264,  353,  354,  355,  805,
  346,   44,  808,   46,  704,   93,  787,  813,  264,   91,
  104,   44,  268,  269,   44,   41,  267,   41,  318,   41,
  720,  721,  722,  125,   41,  300,  301,  302,  303,  304,
  305,  306,  307,  333,  334,  353,  354,  355,  262,  263,
  264,  123,   93,  611,  850,  269,  852,  267,   91,  850,
  337,  279,  280,  621,   10,  755,  756,  757,  264,  865,
  280,   41,  264,  279,  338,  340,  872,  342,  874,  864,
  345,  346,  347,  348,  349,  865,  306,  307,  263,  309,
  205,  124,  888,  313,  314,   41,  264,  893,   44,  337,
  279,  791,  367,  264,  369,  370,  221,  222,   41,  279,
  264,  267,  312,   59,  125,  124,  381,  335,   44,  384,
   46,   44,    0,   46,  389,  335,  391,   41,  243,  335,
  245,  246,   10,   44,  399,  825,  312,  221,  222,  315,
  255,  264,   48,   49,  264,  260,   44,  412,  838,   44,
  840,   41,  264,  353,  354,  355,  335,  847,   40,   41,
  125,  426,   44,   41,   46,   91,   44,  725,   91,  264,
  728,  729,  312,  264,  125,  308,  264,  353,  354,   61,
   10,   59,   44,  264,  449,  300,  301,  302,  303,  304,
  305,  306,  307,  312,   93,  267,   93,  125,  124,   41,
   40,  124,   93,   44,   93,   46,  125,   41,  280,   91,
   40,  264,   41,  353,  354,  355,   46,  301,  302,  303,
  304,   93,  306,  307,  125,  340,   61,  342,   58,   59,
  345,  346,  347,  348,  353,  354,  355,    5,  868,  867,
    6,  123,  124,   10,  776,  562,  279,  125,  695,  654,
   91,  809,  367,  811,  369,  370,  293,  294,  295,  296,
  297,   91,   41,  335,   78,   44,  381,   46,   88,  384,
  695,   38,   71,   40,  389,   42,  391,  542,  543,   46,
   -1,   -1,   61,  124,  399,   -1,  370,   -1,   -1,   10,
  848,  849,   59,  123,   -1,   -1,  854,  412,  563,   -1,
   -1,   -1,  335,   -1,   -1,   -1,    2,    3,    4,    5,
   -1,  426,   91,   -1,   -1,  873,   -1,   38,   14,   40,
   -1,   42,  587,   -1,   -1,   46,   -1,   -1,  412,  887,
  236,   -1,  890,  239,  449,   -1,   -1,  895,   59,  604,
  246,   -1,   -1,   -1,   -1,  124,   -1,   -1,   -1,   -1,
   46,   -1,   -1,  279,   50,   -1,  279,   -1,  623,   -1,
    2,    3,    4,    5,    6,  630,   -1,   -1,   -1,   -1,
   -1,   -1,   14,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  645,   -1,   -1,   -1,  262,  263,  264,   -1,   -1,  654,
  268,  269,   88,  271,  659,   -1,   -1,  279,  280,  664,
  665,   -1,   -1,   -1,   46,   -1,   -1,   -1,   50,  335,
  106,   -1,  335,   -1,   -1,  293,  294,  295,  296,  297,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,  542,  543,   -1,
   -1,   -1,   -1,   -1,  340,   -1,   -1,  267,  279,  704,
   -1,   -1,  707,   -1,   -1,   -1,   88,   -1,  563,   41,
  280,   -1,   44,  335,   46,  720,  721,  722,  542,   -1,
   -1,  367,   -1,   -1,  106,   -1,   -1,   -1,   -1,   61,
   -1,  377,  587,   -1,   37,   38,   -1,   -1,  384,   42,
   43,   -1,   45,   41,   47,   -1,   44,   -1,   46,  604,
  755,  756,  757,  399,  335,   -1,  402,  762,   -1,   91,
  279,   -1,   -1,   61,   -1,  335,   -1,  772,  623,  205,
  416,   -1,   -1,   -1,   -1,  630,   -1,   -1,   -1,   -1,
  604,   -1,  787,    2,    3,   -1,  791,   -1,   -1,    0,
  645,   94,  124,   91,   -1,   14,   -1,  304,   -1,  306,
  307,  308,  309,  449,  659,   -1,  630,   -1,   -1,  664,
  665,    2,    3,   -1,   -1,   -1,  335,   -1,   -1,  255,
  825,  124,  827,  205,  260,   -1,  124,   46,  335,   -1,
   41,   -1,   -1,  838,   -1,  840,   -1,   -1,  345,  346,
   -1,   -1,  847,  304,   -1,  306,  307,  308,  309,  704,
   -1,   -1,  707,   -1,   -1,   46,   -1,  503,  504,  505,
  865,   -1,   -1,   -1,  300,  720,  721,  722,   -1,  305,
  516,   -1,   -1,  255,  335,   -1,  522,  523,  260,   -1,
   -1,   -1,   -1,  707,  345,  346,   -1,  106,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  755,  756,  757,  549,   -1,   -1,   -1,  762,   -1,   -1,
   -1,   -1,  348,  349,  125,  106,   -1,  772,  300,  565,
   -1,   -1,  568,  305,  570,   -1,   -1,   -1,   -1,  575,
  576,   -1,  578,  369,   -1,   -1,  791,   -1,   -1,   -1,
  586,   -1,   -1,   -1,   -1,  381,   -1,  279,   -1,   -1,
   -1,   37,   38,  389,   -1,  391,   42,   43,   -1,   45,
   -1,   47,   -1,   -1,   -1,  611,  348,  349,   10,   -1,
  825,   -1,  827,   -1,   60,  621,   62,   -1,   -1,   -1,
   -1,  279,   -1,  838,   -1,  840,  205,  369,   -1,   -1,
  426,   -1,  847,   -1,   -1,   -1,   38,   -1,   40,  381,
   42,   -1,   -1,  335,   46,   -1,   -1,  389,   94,  391,
  865,   -1,   -1,   -1,  205,  318,   -1,   59,   -1,   40,
   41,   -1,   -1,   44,   -1,   46,   -1,   -1,   -1,   -1,
  333,  334,  678,   -1,   -1,   10,  255,  335,  124,   -1,
   61,  260,   -1,   -1,  426,   -1,   -1,   -1,   -1,   -1,
   -1,  262,  263,  264,   -1,  701,   -1,  268,  269,  705,
  271,   -1,   -1,   38,  255,   40,   -1,   42,   -1,  260,
   91,   46,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  725,
   -1,  300,  728,  729,   59,   10,  305,   -1,   -1,   -1,
   -1,   -1,  738,  739,   -1,    0,   -1,   -1,   -1,   -1,
   -1,   -1,  123,  124,   -1,   -1,   -1,  543,   -1,  300,
   -1,   -1,   -1,   38,  305,   40,   -1,   42,   -1,   -1,
   -1,   46,   -1,   -1,   -1,   -1,   -1,  563,   -1,  348,
   -1,   -1,   -1,   -1,   59,   -1,   41,  783,  784,  785,
   -1,   -1,   -1,   10,   -1,   -1,   -1,   -1,   -1,   -1,
  369,  587,   10,   -1,  800,   -1,   -1,  348,   -1,   -1,
   -1,  543,  381,  809,   -1,  811,   -1,   -1,   -1,   -1,
  389,   38,  391,   40,   -1,   42,   -1,   -1,  369,   46,
   38,  563,   40,   -1,   42,   -1,   -1,   -1,   46,   -1,
  381,   40,   59,   -1,   -1,   44,   -1,   46,  389,   -1,
  391,   59,  848,  849,   -1,  587,   -1,  426,  854,  645,
  646,   -1,   -1,   -1,   -1,   -1,  862,   -1,  654,  865,
  125,  867,  868,   -1,   -1,  230,  231,  873,   -1,    0,
   -1,   -1,  318,   -1,   -1,  426,   -1,  323,  324,   10,
   -1,  887,   91,   -1,  890,   -1,   -1,  333,  334,  895,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  279,  280,
   -1,   -1,  304,  645,  306,  307,  308,  309,  704,   -1,
   41,   -1,  654,   44,  123,  124,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  720,  721,  722,   58,   59,   -1,
   -1,   -1,   -1,  335,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,  345,  346,   37,   38,   -1,   -1,   -1,
   42,   43,   -1,   45,  335,   47,   -1,   -1,   -1,  755,
  756,  757,  704,   -1,  543,   -1,   -1,   -1,   60,  304,
   62,  306,  307,  308,  309,   -1,  772,   -1,  720,  721,
  722,  346,  347,   -1,  563,   -1,   -1,   -1,   -1,   -1,
   -1,  787,  543,   -1,  125,  791,   -1,  262,  263,  264,
  335,   -1,   94,  268,  269,   -1,  271,   -1,  587,   -1,
  345,  346,  563,  755,  756,  757,   -1,   -1,   -1,  304,
   -1,  306,  307,  308,  309,   -1,   -1,  392,   -1,  825,
  772,  827,  124,   -1,   -1,   -1,  587,   -1,   -1,  404,
  405,   -1,  838,  408,  840,  787,   -1,  412,   -1,  791,
  335,  847,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  345,  346,   -1,   40,   41,   -1,  645,   44,   -1,   46,
  279,  280,  437,   -1,   -1,  654,  441,  304,  443,  306,
  307,  308,  309,  825,   61,  827,  304,   -1,  306,  307,
  308,  309,   -1,   -1,  645,   -1,  838,   40,  840,   -1,
   -1,   44,   -1,   46,   -1,  847,   -1,   -1,  335,   -1,
   -1,   -1,   -1,    0,   91,   -1,   -1,  335,  345,  346,
   -1,   -1,   -1,   10,   -1,  704,  335,  345,  346,   -1,
   -1,  262,  263,  264,   -1,   -1,  267,  268,  269,   -1,
  271,  720,  721,  722,   -1,   -1,  123,  124,   91,   -1,
   -1,   -1,   -1,  704,   41,   -1,   -1,   -1,   -1,   -1,
   -1,  526,  293,  294,  295,  296,  297,   -1,   -1,  720,
  721,  722,   59,   -1,   -1,   -1,  755,  756,  757,   -1,
  123,  124,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,  772,   40,   41,   -1,   -1,   44,   -1,
   46,   -1,   -1,    0,  755,  756,  757,  338,  787,   -1,
  575,   -1,  791,   10,   -1,   61,   -1,   -1,   -1,   -1,
   -1,  772,   -1,   -1,   -1,  590,  318,  319,  320,  321,
  322,  323,  324,  325,  326,  327,  328,   -1,  125,   -1,
  791,  333,  334,  608,   41,   91,  825,  612,  827,   -1,
   -1,  616,   -1,   -1,   -1,   -1,   -1,   -1,  623,  838,
   -1,  840,   59,   -1,   -1,   -1,   -1,   -1,  847,   -1,
   -1,   -1,    0,   -1,  825,   -1,  827,   -1,  124,   -1,
   -1,   -1,   10,   -1,   -1,   -1,   -1,  838,   -1,  840,
   -1,   -1,   -1,   -1,   -1,   -1,  847,   -1,   -1,   -1,
   -1,   -1,  279,  280,   -1,   33,   -1,   -1,   -1,   37,
   38,  676,   40,   41,   42,   43,   44,   45,   46,   47,
   -1,  686,   -1,   -1,   -1,   -1,   -1,   -1,  125,   -1,
   58,   59,   60,   61,   62,   63,  279,  280,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  719,   -1,   -1,   -1,  335,   -1,
   -1,  726,   -1,   91,   -1,   93,   94,  732,   -1,    0,
   -1,   -1,   -1,   -1,   -1,  262,  263,  264,   -1,   10,
   -1,  268,  269,   -1,  271,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  335,   -1,   -1,  123,  124,  125,  126,   -1,
   -1,   -1,   33,   -1,  769,   -1,   37,   38,   -1,   40,
   41,   42,   43,   44,   45,   46,   47,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   58,   59,   60,
   61,   62,   63,  279,  799,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,  808,   -1,   -1,   -1,   -1,  813,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   91,   -1,   93,   94,   -1,  262,  263,  264,   -1,   -1,
   -1,  268,  269,   -1,  271,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  850,   -1,  852,   -1,  335,
   -1,   -1,  123,  124,  125,  126,  293,  294,  295,  296,
  297,   -1,   -1,   -1,   -1,   -1,   -1,  872,   -1,  874,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,  888,   -1,   -1,   -1,   -1,  893,  257,
  258,  259,   -1,  261,  262,  263,  264,  265,  266,  267,
  268,  269,  270,  271,  272,  273,  274,  275,  276,  277,
  278,  279,  280,  281,  282,  283,  284,  285,  286,  287,
  288,  289,  290,  291,   -1,  293,  294,  295,  296,  297,
   -1,  299,   -1,   -1,  302,  303,  304,  305,  306,  307,
  308,  309,  310,  311,   -1,  313,  314,   -1,  316,  317,
  318,  319,  320,  321,  322,  323,  324,  325,  326,  327,
  328,  329,  330,   -1,   -1,  333,  334,  335,  336,  337,
  338,  339,  340,   -1,  342,  343,  344,  345,  346,  347,
  348,  349,  350,  351,  352,   -1,  257,  258,  259,  357,
  261,  262,  263,  264,  265,  266,  267,  268,  269,  270,
  271,  272,  273,  274,  275,  276,  277,  278,  279,  280,
  281,  282,  283,  284,  285,  286,  287,  288,  289,  290,
  291,   -1,  293,  294,  295,  296,  297,   -1,  299,   -1,
   -1,  302,  303,  304,  305,  306,  307,  308,  309,  310,
  311,   -1,  313,  314,   -1,  316,  317,  318,  319,  320,
  321,  322,  323,  324,  325,  326,  327,  328,  329,  330,
    0,   -1,  333,  334,  335,  336,  337,  338,  339,  340,
   10,  342,  343,  344,  345,  346,  347,  348,  349,  350,
  351,  352,   -1,   -1,   -1,   -1,  357,   -1,   -1,   -1,
   -1,   -1,   -1,   33,   -1,   -1,   -1,   37,   38,   -1,
   40,   41,   42,   43,   44,   45,   46,   47,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   58,   59,
   60,   61,   62,   63,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   91,   -1,   93,   94,   -1,   -1,    0,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   10,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,  123,  124,  125,  126,   -1,   -1,   -1,
   33,   -1,   -1,   -1,   37,   38,   -1,   40,   41,   42,
   43,   44,   45,   46,   47,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   58,   59,   60,   61,   62,
   63,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   91,   -1,
   93,   94,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  123,  124,  125,  126,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,  257,  258,  259,
   -1,  261,  262,  263,  264,  265,  266,  267,  268,  269,
  270,  271,  272,  273,  274,  275,  276,  277,  278,   -1,
  280,  281,  282,  283,  284,  285,  286,  287,  288,  289,
  290,  291,   -1,  293,  294,  295,  296,  297,   -1,  299,
   -1,   -1,  302,  303,  304,  305,  306,  307,  308,  309,
  310,  311,   -1,  313,  314,   -1,  316,  317,  318,  319,
  320,  321,  322,  323,  324,  325,  326,  327,  328,  329,
  330,   -1,   -1,  333,  334,  335,  336,   -1,  338,  339,
  340,   -1,  342,  343,  344,  345,  346,  347,  348,  349,
  350,  351,  352,   -1,  257,  258,  259,  357,  261,  262,
  263,  264,  265,  266,  267,  268,  269,  270,  271,  272,
  273,  274,  275,  276,  277,  278,   -1,  280,  281,  282,
  283,  284,  285,  286,  287,  288,  289,  290,  291,   -1,
  293,  294,  295,  296,  297,   -1,  299,   -1,   -1,  302,
  303,  304,  305,  306,  307,  308,  309,  310,  311,   -1,
  313,  314,   -1,  316,  317,  318,  319,  320,  321,  322,
  323,  324,  325,  326,  327,  328,  329,  330,    0,   -1,
  333,  334,  335,  336,   -1,  338,  339,  340,   10,  342,
  343,  344,  345,  346,  347,  348,  349,  350,  351,  352,
   -1,   -1,   -1,   -1,  357,   -1,   -1,   -1,   -1,   -1,
   -1,   33,   -1,   -1,   -1,   37,   38,   -1,   40,   41,
   42,   43,   44,   45,   46,   47,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   58,   59,   60,   61,
   62,   63,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   91,
   -1,   93,   94,   -1,   -1,    0,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   10,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  123,  124,  125,  126,   -1,   -1,   -1,   33,   -1,
   -1,   -1,   37,   38,   -1,   40,   41,   42,   43,   44,
   45,   46,   47,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   58,   59,   60,   -1,   62,   63,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   91,   -1,   93,   94,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  123,  124,
  125,  126,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  257,  258,  259,   -1,  261,
  262,  263,  264,  265,  266,  267,  268,  269,  270,  271,
  272,  273,  274,  275,  276,  277,  278,   -1,  280,  281,
  282,  283,  284,  285,  286,  287,  288,  289,  290,  291,
   -1,  293,  294,  295,  296,  297,   -1,  299,   -1,   -1,
  302,  303,  304,  305,  306,  307,  308,  309,  310,  311,
   -1,  313,  314,   -1,  316,  317,  318,  319,  320,  321,
  322,  323,  324,  325,  326,  327,  328,  329,  330,   -1,
   -1,  333,  334,  335,  336,   -1,  338,  339,  340,   -1,
  342,  343,  344,  345,  346,  347,  348,  349,  350,  351,
  352,   -1,  257,  258,  259,  357,  261,  262,  263,  264,
  265,  266,  267,  268,  269,  270,  271,  272,  273,  274,
  275,  276,  277,  278,   -1,  280,  281,  282,  283,  284,
  285,  286,  287,  288,  289,  290,  291,   -1,  293,  294,
  295,  296,  297,   -1,  299,   -1,   -1,  302,  303,  304,
  305,  306,  307,  308,  309,  310,  311,   -1,  313,  314,
   -1,  316,  317,  318,  319,  320,  321,  322,  323,  324,
  325,  326,  327,  328,  329,  330,    0,   -1,  333,  334,
  335,  336,   -1,  338,  339,  340,   10,  342,  343,  344,
  345,  346,  347,  348,  349,  350,  351,  352,   -1,   -1,
   -1,   -1,  357,   -1,   -1,   -1,   -1,   -1,   -1,   33,
   -1,   -1,   -1,   37,   38,   -1,   40,   41,   42,   43,
   44,   45,   46,   47,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   58,   59,   60,   -1,   62,   63,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   91,   -1,   93,
   94,   -1,   -1,    0,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   10,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  123,
  124,  125,  126,   -1,   -1,   -1,   33,   -1,   -1,   -1,
   37,   38,   -1,   40,   41,   42,   43,   44,   45,   46,
   47,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   58,   59,   60,   -1,   62,   63,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   91,   -1,   93,   94,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  123,  124,  125,  126,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,  257,  258,  259,   -1,  261,  262,  263,
  264,  265,  266,  267,  268,  269,  270,  271,  272,  273,
  274,  275,  276,  277,  278,   -1,  280,  281,  282,  283,
  284,  285,  286,  287,  288,  289,  290,  291,   -1,  293,
  294,  295,  296,  297,   -1,  299,   -1,   -1,  302,  303,
  304,  305,  306,  307,  308,  309,  310,  311,   -1,  313,
  314,   -1,  316,  317,  318,  319,  320,  321,  322,  323,
  324,  325,  326,  327,  328,  329,  330,   -1,   -1,  333,
  334,  335,  336,   -1,  338,  339,  340,   -1,  342,  343,
  344,  345,  346,  347,  348,  349,  350,  351,  352,   -1,
  257,  258,  259,  357,  261,  262,  263,  264,  265,  266,
  267,  268,  269,  270,  271,  272,  273,  274,  275,  276,
  277,  278,   -1,  280,  281,  282,  283,  284,  285,  286,
  287,  288,  289,  290,  291,   -1,  293,  294,  295,  296,
  297,   -1,  299,   -1,   -1,  302,  303,  304,  305,  306,
  307,  308,  309,  310,  311,   -1,  313,  314,   -1,  316,
  317,  318,  319,  320,  321,  322,  323,  324,  325,  326,
  327,  328,  329,  330,    0,   -1,  333,  334,  335,  336,
   -1,  338,  339,  340,   10,  342,  343,  344,  345,  346,
  347,  348,  349,  350,  351,  352,   -1,   -1,   -1,   -1,
  357,   -1,   -1,   -1,   -1,   -1,   -1,   33,   -1,   -1,
   -1,   37,   38,   -1,   -1,   41,   42,   43,   44,   45,
   46,   47,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   58,   59,   60,   -1,   62,   63,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   91,   -1,   93,   94,   -1,
   -1,    0,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   10,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,  123,  124,  125,
  126,   -1,   -1,   -1,   33,   -1,   -1,   -1,   37,   38,
   -1,   -1,   41,   42,   43,   44,   45,   46,   47,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   58,
   59,   60,   -1,   62,   63,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   91,   -1,   93,   94,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  123,  124,  125,  126,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  257,  258,  259,   -1,  261,  262,  263,  264,  265,
  266,  267,  268,  269,  270,  271,  272,  273,  274,  275,
  276,  277,  278,   -1,  280,  281,  282,  283,  284,  285,
  286,  287,  288,  289,  290,  291,   -1,  293,  294,  295,
  296,  297,   -1,  299,   -1,   -1,  302,  303,  304,  305,
  306,  307,  308,  309,  310,  311,   -1,  313,  314,   -1,
  316,  317,  318,  319,  320,  321,  322,  323,  324,  325,
  326,  327,  328,  329,  330,   -1,   -1,  333,  334,  335,
  336,   -1,  338,  339,  340,   -1,  342,  343,  344,  345,
  346,  347,  348,  349,  350,  351,  352,   -1,  257,  258,
  259,  357,  261,  262,  263,  264,  265,  266,  267,  268,
  269,  270,  271,  272,  273,  274,  275,  276,  277,  278,
   -1,  280,  281,  282,  283,  284,  285,  286,  287,  288,
  289,  290,  291,   -1,  293,  294,  295,  296,  297,   -1,
  299,   -1,   -1,  302,  303,  304,  305,  306,  307,  308,
  309,  310,  311,   -1,  313,  314,   -1,  316,  317,  318,
  319,  320,  321,  322,  323,  324,  325,  326,  327,  328,
  329,  330,    0,   -1,  333,  334,  335,  336,   -1,  338,
  339,  340,   10,  342,  343,  344,  345,  346,  347,  348,
  349,  350,  351,  352,   -1,   -1,   -1,   -1,  357,   -1,
   -1,   -1,   -1,   -1,   -1,   33,   -1,   -1,   -1,   37,
   38,   -1,   40,   41,   42,   43,   44,   45,   46,   47,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   58,   59,   60,   61,   62,   63,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   91,   -1,   93,   94,   -1,   -1,    0,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   10,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  124,  125,  126,   -1,
   -1,   -1,   33,   -1,   -1,   -1,   37,   38,   -1,   -1,
   41,   42,   43,   44,   45,   46,   47,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   58,   59,   60,
   -1,   62,   63,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   91,   -1,   93,   94,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,  124,  125,  126,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  257,
  258,  259,   -1,  261,  262,  263,  264,  265,  266,  267,
  268,  269,  270,  271,  272,  273,  274,  275,  276,  277,
  278,   -1,   -1,  281,  282,  283,  284,  285,  286,  287,
  288,  289,  290,  291,   -1,  293,  294,  295,  296,  297,
   -1,  299,   -1,   -1,  302,  303,  304,  305,  306,  307,
  308,  309,  310,  311,   -1,  313,  314,   -1,  316,  317,
  318,  319,  320,  321,  322,  323,  324,  325,  326,  327,
  328,  329,  330,   -1,   -1,  333,  334,  335,  336,   -1,
  338,  339,  340,   -1,  342,  343,  344,  345,  346,  347,
  348,  349,  350,  351,  352,   -1,  257,  258,  259,  357,
  261,  262,  263,  264,  265,  266,  267,  268,  269,  270,
  271,  272,  273,  274,  275,  276,  277,  278,   -1,   -1,
  281,  282,  283,  284,  285,  286,  287,  288,  289,  290,
  291,   -1,  293,  294,  295,  296,  297,   -1,  299,   -1,
   -1,  302,  303,  304,  305,  306,  307,  308,  309,  310,
  311,   -1,  313,  314,   -1,  316,  317,  318,  319,  320,
  321,  322,  323,  324,  325,  326,  327,  328,  329,  330,
    0,   -1,  333,  334,  335,  336,   -1,  338,  339,  340,
   10,  342,  343,  344,  345,  346,  347,  348,  349,  350,
  351,  352,   -1,   -1,   -1,   -1,  357,   -1,   -1,   -1,
   -1,   -1,   -1,   33,   -1,   -1,   -1,   37,   38,   -1,
   40,   41,   42,   43,   44,   45,   46,   47,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   59,
   60,   61,   62,   63,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   91,   -1,   -1,   94,   -1,   -1,    0,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   10,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,  123,  124,  125,  126,   -1,   -1,   -1,
   33,   -1,   -1,   -1,   37,   38,   -1,   40,   41,   42,
   43,   44,   45,   46,   47,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   59,   60,   61,   62,
   63,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   91,   -1,
   -1,   94,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  123,  124,  125,  126,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,  257,  258,  259,
   -1,  261,  262,  263,  264,  265,  266,   -1,  268,  269,
  270,  271,  272,  273,  274,  275,  276,  277,  278,   -1,
  280,   -1,   -1,  283,  284,  285,  286,  287,  288,  289,
  290,  291,   -1,  293,  294,  295,  296,  297,   -1,  299,
   -1,   -1,  302,  303,  304,  305,  306,  307,  308,  309,
  310,  311,   -1,  313,  314,   -1,  316,  317,  318,  319,
  320,  321,  322,  323,  324,  325,  326,  327,  328,  329,
  330,   -1,   -1,  333,  334,  335,  336,   -1,   -1,  339,
  340,   -1,  342,  343,   -1,  345,  346,  347,  348,  349,
  350,  351,  352,   -1,  257,  258,  259,  357,  261,  262,
  263,  264,  265,  266,   -1,  268,  269,  270,  271,  272,
  273,  274,  275,  276,  277,  278,   -1,  280,   -1,   -1,
  283,  284,  285,  286,  287,  288,  289,  290,  291,   -1,
  293,  294,  295,  296,  297,   -1,  299,   -1,   -1,  302,
  303,  304,  305,  306,  307,  308,  309,  310,  311,   -1,
  313,  314,   -1,  316,  317,  318,  319,  320,  321,  322,
  323,  324,  325,  326,  327,  328,  329,  330,    0,   -1,
  333,  334,  335,  336,   -1,   -1,  339,  340,   10,  342,
  343,   -1,  345,  346,  347,  348,  349,  350,  351,  352,
   -1,   -1,   -1,   -1,  357,   -1,   -1,   -1,   -1,   -1,
   -1,   33,   -1,   -1,   -1,   37,   38,   -1,   40,   41,
   42,   43,   44,   45,   46,   47,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   59,   60,   61,
   62,   63,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   91,
   -1,   -1,   94,   -1,   -1,    0,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   10,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  123,  124,  125,  126,   -1,   -1,   -1,   33,   -1,
   -1,   -1,   37,   38,   -1,   40,   41,   42,   43,   44,
   45,   46,   47,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   59,   60,   61,   62,   63,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   91,   -1,   -1,   94,
   -1,   -1,   -1,   -1,   -1,   -1,   10,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  124,
  125,  126,   -1,   37,   38,   -1,   -1,   -1,   42,   43,
   -1,   45,   -1,   47,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   60,   -1,   62,   63,
   -1,   -1,   -1,   -1,   -1,  257,  258,  259,   -1,  261,
  262,  263,  264,  265,  266,   -1,  268,  269,  270,  271,
  272,  273,  274,  275,  276,  277,  278,   -1,  280,   -1,
   94,  283,  284,  285,  286,  287,  288,  289,  290,  291,
   -1,  293,  294,  295,  296,  297,   -1,  299,   -1,   -1,
  302,  303,  304,  305,  306,  307,  308,  309,  310,  311,
  124,  313,  314,   -1,  316,  317,  318,  319,  320,  321,
  322,  323,  324,  325,  326,  327,  328,  329,  330,   -1,
   -1,  333,  334,  335,  336,   -1,   -1,  339,  340,   -1,
  342,  343,   -1,  345,  346,  347,  348,  349,  350,  351,
  352,   -1,  257,  258,  259,  357,  261,  262,  263,  264,
  265,  266,   -1,  268,  269,  270,  271,  272,  273,  274,
  275,  276,  277,  278,   -1,   -1,   -1,   -1,  283,  284,
  285,  286,  287,  288,  289,  290,  291,   -1,  293,  294,
  295,  296,  297,   -1,  299,   -1,   -1,  302,  303,  304,
  305,  306,  307,  308,  309,  310,  311,   -1,  313,  314,
   -1,  316,  317,  318,  319,  320,  321,  322,  323,  324,
  325,  326,  327,  328,  329,  330,    0,   -1,  333,  334,
  335,  336,   -1,   -1,  339,  340,   10,  342,  343,   -1,
  345,  346,  347,  348,  349,  350,  351,  352,   -1,   -1,
   -1,   -1,  357,   -1,   -1,   -1,   -1,   -1,   -1,   33,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   41,   -1,   -1,
   -1,   -1,   46,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,    0,   -1,   58,   59,   -1,   -1,   -1,   -1,
   -1,   -1,   10,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  318,  319,  320,  321,  322,  323,
  324,  325,  326,  327,  328,  329,  330,   -1,   -1,  333,
  334,   -1,   -1,   41,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   10,   -1,
   -1,   59,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  125,  126,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   33,   -1,   -1,   -1,   -1,   -1,   -1,   40,   -1,
   -1,   -1,   -1,   -1,   46,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   59,   60,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,  125,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   91,
   -1,   37,   38,   -1,   -1,   -1,   42,   43,   -1,   45,
   -1,   47,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   58,   -1,   60,   -1,   62,   63,   -1,   -1,
   -1,  123,   -1,   -1,  126,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   94,   -1,
   -1,   -1,   -1,  257,  258,  259,   -1,  261,  262,  263,
  264,  265,  266,  267,  268,  269,  270,  271,  272,  273,
  274,  275,  276,  277,  278,   -1,   -1,  281,  124,  283,
  284,  285,  286,  287,  288,  289,  290,  291,   -1,  293,
  294,  295,  296,  297,   -1,  299,   -1,   -1,  302,  303,
  304,  305,  306,  307,  308,  309,  310,  311,   -1,  313,
  314,   -1,  316,  317,  262,  263,  264,   -1,   -1,   -1,
  268,  269,   -1,  271,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  335,  336,   -1,   -1,  339,  340,   33,  342,  343,
   -1,  345,  346,  347,  348,  349,  350,  351,  352,   -1,
   -1,   -1,   -1,  357,  256,  257,  258,  259,  260,  261,
  262,  263,  264,  265,  266,   -1,   -1,  269,  270,   -1,
  272,  273,  274,  275,  276,  277,  278,   -1,  280,   -1,
   -1,  283,  284,  285,  286,  287,  288,  289,   -1,   -1,
  292,   -1,   -1,   -1,   -1,   -1,  298,  299,  300,  301,
  302,  303,  304,  305,  306,  307,  308,  309,  310,  311,
   -1,  313,  314,   -1,  316,  317,   -1,   -1,   -1,   -1,
   -1,   10,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  126,   -1,   -1,  335,  336,   -1,   -1,  339,  340,   -1,
  342,  343,   -1,  345,   33,  347,  348,  349,  350,  351,
  352,   40,   -1,   -1,   -1,  357,   -1,   46,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   59,   60,  318,  319,  320,  321,  322,  323,  324,  325,
  326,  327,  328,  329,  330,   -1,   -1,  333,  334,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   91,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   10,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  123,   -1,   -1,  126,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   33,   -1,   -1,   -1,
   -1,   -1,   -1,   40,   -1,   -1,   -1,   -1,   -1,   46,
   -1,  257,  258,  259,   -1,  261,   -1,   -1,   -1,  265,
  266,   -1,   59,   60,  270,   -1,  272,  273,  274,  275,
  276,  277,  278,   -1,   -1,   -1,   -1,  283,  284,  285,
  286,  287,  288,  289,   -1,   -1,  292,   -1,   -1,   -1,
   -1,   -1,   -1,  299,   91,   -1,  302,  303,  304,  305,
  306,  307,  308,  309,  310,  311,   -1,  313,  314,    0,
  316,  317,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   10,
   -1,   -1,   -1,   -1,   -1,   -1,  123,   -1,   -1,  126,
  336,   -1,   -1,  339,  340,   -1,  342,  343,   -1,  345,
  346,  347,  348,  349,  350,  351,  352,   -1,   -1,   -1,
   41,  357,   -1,   -1,   -1,   -1,   -1,  256,  257,  258,
  259,  260,  261,  262,  263,  264,  265,  266,   59,   -1,
  269,  270,    0,  272,  273,  274,  275,  276,  277,  278,
   -1,  280,   10,   -1,  283,  284,  285,  286,  287,  288,
  289,   -1,   -1,  292,   -1,   -1,   -1,   -1,   -1,  298,
  299,  300,  301,  302,  303,  304,  305,  306,  307,  308,
  309,  310,  311,   41,  313,  314,   -1,  316,  317,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   58,   59,   -1,   -1,  125,   -1,  335,  336,   -1,   -1,
  339,  340,   -1,  342,  343,   -1,  345,   -1,  347,  348,
  349,  350,  351,  352,   -1,   -1,   -1,   -1,  357,  256,
  257,  258,  259,  260,  261,  262,  263,  264,  265,  266,
   -1,   -1,  269,  270,    0,  272,  273,  274,  275,  276,
  277,  278,   -1,  280,   10,   -1,  283,  284,  285,  286,
  287,  288,  289,   -1,   -1,  292,   -1,  125,   -1,   -1,
   -1,  298,  299,  300,  301,  302,  303,  304,  305,  306,
  307,  308,  309,  310,  311,   41,  313,  314,   -1,  316,
  317,   -1,   -1,   -1,   -1,   -1,   10,   -1,   -1,   -1,
   -1,   -1,   58,   59,   -1,   -1,   -1,   -1,  335,  336,
   -1,   -1,  339,  340,   -1,  342,  343,   -1,  345,   33,
  347,  348,  349,  350,  351,  352,   40,   -1,   -1,   -1,
  357,   -1,   46,   -1,   -1,   -1,   -1,   93,   -1,   -1,
   -1,  262,  263,  264,    0,   59,   60,  268,  269,   -1,
  271,   -1,   -1,   -1,   10,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  125,
   -1,   -1,  293,  294,  295,  296,  297,   91,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   41,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   10,   -1,
   -1,   -1,   -1,   59,  262,  263,  264,   -1,   -1,  267,
  268,  269,  126,  271,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   33,   -1,  281,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  290,  291,   46,  293,  294,  295,  296,  297,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   59,   60,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  125,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   91,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   10,   -1,  262,  263,  264,   -1,
   -1,  267,  268,  269,  126,  271,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  281,  282,   33,   -1,   -1,
   -1,   -1,   -1,   -1,  290,  291,   -1,  293,  294,  295,
  296,  297,  256,  257,  258,  259,  260,  261,  262,  263,
  264,  265,  266,   59,   -1,  269,  270,   -1,  272,  273,
  274,  275,  276,  277,  278,   -1,   -1,   -1,   -1,  283,
  284,  285,  286,  287,  288,  289,   -1,   -1,  292,   -1,
   -1,   -1,   -1,   -1,  298,  299,  300,  301,  302,  303,
  304,  305,  306,  307,  308,  309,  310,  311,   -1,  313,
  314,   -1,  316,  317,   -1,   -1,  262,  263,  264,   -1,
   -1,   -1,  268,  269,   -1,  271,   -1,   -1,   10,   -1,
  126,  335,  336,   -1,   -1,  339,  340,   -1,  342,  343,
   -1,  345,   -1,  347,  348,  349,  350,  351,  352,   -1,
   -1,   33,   -1,  357,  256,  257,  258,  259,  260,  261,
  262,  263,  264,  265,  266,   -1,   -1,  269,  270,   -1,
  272,  273,  274,  275,  276,  277,  278,   59,   -1,   -1,
   -1,  283,  284,  285,  286,  287,  288,  289,   -1,   -1,
  292,   -1,   -1,   -1,   -1,   -1,  298,  299,  300,  301,
  302,  303,  304,  305,  306,  307,  308,  309,  310,  311,
   -1,  313,  314,   -1,  316,  317,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,  335,  336,   -1,   -1,  339,  340,   -1,
  342,  343,    0,  345,  126,  347,  348,  349,  350,  351,
  352,   -1,   10,   -1,   -1,  357,   -1,   -1,   -1,   -1,
  256,  257,  258,  259,  260,  261,  262,  263,  264,  265,
  266,   -1,  268,  269,  270,  271,  272,  273,  274,  275,
  276,  277,  278,   41,   -1,   -1,   44,  283,  284,  285,
  286,  287,  288,  289,   -1,   -1,  292,   -1,   -1,   10,
   58,   59,  298,  299,  300,  301,  302,  303,  304,  305,
  306,  307,  308,  309,  310,  311,   -1,  313,  314,   -1,
  316,  317,   33,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   93,   -1,   -1,   -1,   -1,
  336,   -1,   -1,  339,  340,   -1,  342,  343,   59,  345,
   -1,  347,  348,  349,  350,  351,  352,   -1,   -1,   -1,
   -1,  357,   -1,   -1,   -1,   -1,   -1,  125,   -1,   -1,
   -1,   -1,   -1,   -1,  256,  257,  258,  259,  260,  261,
   -1,   -1,  264,  265,  266,   -1,   -1,   -1,  270,   -1,
  272,  273,  274,  275,  276,  277,  278,   -1,   -1,   -1,
   -1,  283,  284,  285,  286,  287,  288,  289,   -1,   -1,
  292,   -1,   -1,   10,  125,  126,  298,  299,  300,  301,
  302,  303,  304,  305,  306,  307,  308,  309,  310,  311,
   -1,  313,  314,   -1,  316,  317,   33,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  336,   -1,   -1,  339,  340,   -1,
  342,  343,   59,  345,   -1,  347,  348,  349,  350,  351,
  352,   -1,   -1,   -1,   -1,  357,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  262,  263,  264,   -1,   -1,  267,
  268,  269,   -1,  271,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,  281,  282,   -1,   -1,   -1,   -1,  126,
   -1,   -1,  290,  291,   -1,  293,  294,  295,  296,  297,
   -1,   -1,   -1,   -1,   -1,  256,  257,  258,  259,  260,
  261,   -1,   -1,   -1,  265,  266,   -1,   -1,   -1,  270,
   -1,  272,  273,  274,  275,  276,  277,  278,   -1,   -1,
   -1,   -1,  283,  284,  285,  286,  287,  288,  289,   -1,
  338,  292,   -1,   -1,   -1,   -1,  344,  298,  299,  300,
  301,  302,  303,  304,  305,  306,  307,  308,  309,  310,
  311,   -1,  313,  314,   -1,  316,  317,   -1,   -1,   -1,
   -1,   -1,    0,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   10,   -1,   -1,  336,   -1,   -1,  339,  340,
   -1,  342,  343,   -1,  345,   -1,  347,  348,  349,  350,
  351,  352,   -1,   -1,   -1,   -1,  357,   -1,   -1,   -1,
   -1,   -1,   -1,   41,   -1,   -1,   -1,   -1,   -1,   -1,
  257,  258,  259,   -1,  261,   -1,   -1,   -1,  265,  266,
   58,   59,   -1,  270,   -1,  272,  273,  274,  275,  276,
  277,  278,   -1,   -1,   -1,   -1,  283,  284,  285,  286,
  287,  288,  289,   -1,   -1,  292,   -1,   -1,   -1,   -1,
   -1,   -1,  299,   -1,   -1,  302,  303,  304,  305,  306,
  307,  308,  309,  310,  311,   -1,  313,  314,   -1,  316,
  317,    0,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   10,   -1,   -1,   -1,   -1,   -1,  125,   -1,  336,
   -1,   -1,  339,  340,   -1,  342,  343,   -1,   -1,   -1,
  347,  348,  349,  350,  351,  352,   -1,   -1,   37,   38,
  357,   -1,   41,   42,   43,   44,   45,   46,   47,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   58,
   59,   60,   -1,   62,   63,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
    0,   -1,   91,   -1,   93,   94,   -1,   -1,   -1,   -1,
   10,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  123,  124,  125,   37,   38,   -1,
   -1,   41,   42,   43,   44,   45,   46,   47,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   58,   59,
   60,   61,   62,   63,  262,  263,  264,   -1,   -1,  267,
  268,  269,   -1,  271,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,  281,   -1,   -1,   -1,   -1,   -1,   -1,
    0,   91,   -1,   93,   94,  293,  294,  295,  296,  297,
   10,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  124,  125,   -1,   37,   38,   -1,
   -1,   41,   42,   43,   44,   45,   46,   47,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   58,   59,
   60,   -1,   62,   63,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,  262,  263,  264,   -1,   -1,  267,  268,
  269,   91,  271,   93,   94,   -1,   -1,   -1,   -1,   -1,
   -1,  280,  281,  282,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  290,  291,   -1,  293,  294,  295,  296,  297,   -1,
   -1,   -1,   -1,  123,  124,  125,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  318,
  319,  320,  321,  322,  323,  324,  325,  326,  327,  328,
  329,  330,   -1,   -1,  333,  334,  335,   -1,   -1,  338,
   -1,   -1,   -1,   -1,   -1,  344,   -1,   -1,   -1,   -1,
   -1,   -1,  262,  263,  264,   -1,   -1,  267,  268,  269,
   -1,  271,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  281,  282,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  290,  291,   -1,  293,  294,  295,  296,  297,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  318,  319,
  320,  321,  322,  323,  324,  325,  326,  327,  328,  329,
  330,   -1,   -1,  333,  334,  335,   -1,  337,  338,   -1,
   -1,   -1,   -1,   -1,  344,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  262,  263,  264,   -1,   -1,  267,  268,  269,
   -1,  271,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  280,  281,  282,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  290,  291,   -1,  293,  294,  295,  296,  297,   -1,   -1,
   -1,   -1,   -1,   -1,    0,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   10,   -1,   -1,   -1,  318,  319,
  320,  321,  322,  323,  324,  325,  326,  327,  328,  329,
  330,   -1,   -1,  333,  334,  335,   -1,   -1,  338,   -1,
   -1,   37,   38,   -1,  344,   41,   42,   43,   44,   45,
   46,   47,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   58,   59,   60,   61,   62,   63,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,    0,
   -1,   -1,   -1,   -1,   -1,   91,   -1,   93,   94,   10,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   37,   38,  124,  125,
   41,   42,   43,   44,   45,   46,   47,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   58,   59,   60,
   61,   62,   63,   -1,   37,   38,   -1,   -1,   -1,   42,
   43,   -1,   45,   -1,   47,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   60,   -1,   62,
   91,   -1,   93,   94,   -1,    0,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   10,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   94,   -1,  124,  125,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   37,   38,   -1,   -1,   41,   42,   43,   44,
   45,   46,   47,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  124,   -1,   58,   59,   60,   61,   62,   63,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  262,  263,  264,   -1,
   -1,  267,  268,  269,   -1,  271,   91,   -1,   93,   94,
   -1,   -1,   -1,   -1,   -1,  281,  282,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  290,  291,   -1,  293,  294,  295,
  296,  297,   -1,   -1,    0,   -1,   -1,   -1,   -1,  124,
  125,   -1,   -1,   -1,   10,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  318,  319,  320,  321,  322,  323,  324,  325,
  326,  327,  328,  329,  330,   -1,   -1,  333,  334,  335,
   -1,   -1,  338,   -1,   -1,   41,   -1,   -1,  344,   -1,
   -1,  262,  263,  264,   -1,   -1,  267,  268,  269,   -1,
  271,   -1,   58,   59,   -1,   -1,   -1,   -1,   -1,   -1,
  281,  282,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  290,
  291,   -1,  293,  294,  295,  296,  297,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   93,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,  318,  319,  320,
  321,  322,  323,  324,  325,  326,  327,  328,  329,  330,
   -1,   -1,  333,  334,  335,   -1,   -1,  338,   -1,  125,
   -1,   -1,   -1,  344,   -1,  318,  319,  320,  321,  322,
  323,  324,  325,   -1,  327,  328,   -1,  262,  263,  264,
  333,  334,  267,  268,  269,   -1,  271,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  281,  282,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  290,  291,   -1,  293,  294,
  295,  296,  297,   -1,   -1,   -1,   -1,   -1,   -1,    0,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   10,
   -1,   -1,   -1,  318,  319,  320,  321,  322,  323,  324,
  325,  326,  327,  328,  329,  330,   -1,   -1,  333,  334,
  335,   -1,   -1,  338,   -1,   -1,   37,   38,   -1,  344,
   41,   42,   43,   44,   45,   46,   47,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   58,   59,   60,
   -1,   62,   63,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  262,  263,  264,   -1,
   -1,  267,  268,  269,    0,  271,   -1,   -1,   -1,   -1,
   91,   -1,   93,   94,   10,  281,  282,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  290,  291,   -1,  293,  294,  295,
  296,  297,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   37,   38,  124,  125,   41,   42,   43,   44,   45,
   46,   47,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   58,   59,   60,   -1,   62,   63,   -1,   37,
   38,   -1,   -1,   -1,   42,   43,   -1,   45,  344,   47,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   60,   -1,   62,   91,   -1,   93,   94,   -1,
    0,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   10,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   94,   -1,  124,  125,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   37,   38,   -1,
   -1,   41,   42,   43,   44,   45,   46,   47,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  124,   -1,   58,   59,
   60,   -1,   62,   63,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  262,  263,  264,   -1,   -1,  267,  268,  269,   -1,
  271,   91,   -1,   93,   94,   -1,   -1,   -1,   -1,   -1,
  281,  282,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  290,
  291,   -1,  293,  294,  295,  296,  297,   -1,   -1,    0,
   -1,   -1,   -1,   -1,  124,  125,   -1,   -1,   -1,   10,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,  318,  319,  320,
  321,  322,  323,  324,  325,  326,  327,  328,  329,  330,
   -1,   -1,  333,  334,  335,   -1,   -1,  338,   -1,   -1,
   41,   -1,   -1,  344,   -1,   -1,  262,  263,  264,   -1,
   -1,  267,  268,  269,   -1,  271,   -1,   58,   59,   -1,
   -1,   -1,   -1,   -1,   -1,  281,  282,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  290,  291,   -1,  293,  294,  295,
  296,  297,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   93,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  318,  319,  320,  321,  322,  323,  324,  325,
  326,  327,  328,  329,  330,   -1,   -1,  333,  334,  335,
   -1,   -1,  338,   -1,  125,   -1,   -1,   -1,  344,   -1,
  318,  319,  320,  321,  322,  323,  324,   -1,   -1,  327,
  328,   -1,  262,  263,  264,  333,  334,  267,  268,  269,
   -1,  271,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  281,  282,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  290,  291,   -1,  293,  294,  295,  296,  297,   -1,   -1,
   -1,   -1,   -1,   -1,    0,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   10,   -1,   -1,   -1,  318,  319,
  320,  321,  322,  323,  324,  325,  326,  327,  328,  329,
  330,   -1,   -1,  333,  334,  335,   -1,   -1,  338,   -1,
   -1,   37,   38,   -1,  344,   41,   42,   43,   44,   45,
   46,   47,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   58,   59,   60,   -1,   62,   63,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  262,  263,  264,   -1,   -1,  267,  268,  269,    0,
  271,   -1,   -1,   -1,   -1,   91,   -1,   93,   94,   10,
  281,  282,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  290,
  291,   -1,  293,  294,  295,  296,  297,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   37,   38,  124,  125,
   41,   42,   43,   44,   45,   46,   47,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   58,   59,   60,
   -1,   62,   63,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   91,   -1,   93,   94,   -1,    0,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   10,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,  124,  125,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   37,   38,   -1,   -1,   41,   42,   43,   44,
   45,   46,   47,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   58,   59,   60,   -1,   62,   63,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  262,  263,  264,   -1,
   -1,  267,  268,  269,   -1,  271,   91,   -1,   93,   94,
   -1,   -1,   -1,   -1,   -1,  281,  282,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  290,  291,   -1,  293,  294,  295,
  296,  297,   -1,   -1,    0,   -1,   -1,   -1,   -1,  124,
  125,   -1,   -1,   -1,   10,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  318,  319,  320,  321,  322,  323,  324,  325,
  326,  327,  328,  329,  330,   -1,   -1,  333,  334,  335,
   -1,   -1,  338,   -1,   -1,   41,   -1,   -1,  344,   -1,
   -1,  262,  263,  264,   -1,   -1,  267,  268,  269,   -1,
  271,   -1,   58,   59,   -1,   -1,   -1,   -1,   37,   38,
  281,  282,   -1,   42,   43,   -1,   45,   -1,   47,  290,
  291,   -1,  293,  294,  295,  296,  297,   -1,   -1,   -1,
   -1,   60,   -1,   62,   63,   -1,   -1,   93,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  319,  320,
  321,  322,  323,  324,  325,  326,  327,  328,  329,  330,
   -1,   -1,  333,  334,  335,   94,   -1,  338,   -1,  125,
   -1,   -1,   -1,  344,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,  262,  263,  264,
   -1,   -1,  267,  268,  269,  124,  271,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  281,  282,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  290,  291,   -1,  293,  294,
  295,  296,  297,   -1,   -1,   -1,   -1,   -1,   -1,    0,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   10,
   -1,   -1,   -1,   -1,  319,  320,  321,  322,  323,  324,
  325,  326,  327,  328,  329,  330,   -1,   -1,  333,  334,
  335,   -1,   -1,  338,   -1,   -1,   37,   38,   -1,  344,
   41,   42,   43,   44,   45,   -1,   47,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   58,   59,   60,
   -1,   62,   63,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  262,  263,  264,   -1,
   -1,  267,  268,  269,    0,  271,   -1,   -1,   -1,   -1,
   -1,   -1,   93,   94,   10,  281,  282,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  290,  291,   -1,  293,  294,  295,
  296,  297,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   37,   38,  124,  125,   41,   42,   43,   44,   45,
   -1,   47,   -1,   -1,   -1,   -1,   -1,   -1,  297,   -1,
   -1,   -1,   58,   59,   60,   -1,   62,   63,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  318,
  319,  320,  321,  322,  323,  324,  325,  326,  327,  328,
  329,  330,   -1,   -1,  333,  334,   -1,   93,   94,    0,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   10,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  124,  125,
   -1,   -1,   -1,   -1,   -1,   -1,   37,   38,   -1,   -1,
   41,   42,   43,   44,   45,   -1,   47,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   58,   59,   60,
   -1,   62,   63,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  262,  263,  264,   -1,   -1,  267,  268,  269,   -1,
  271,   -1,   93,   94,   -1,   -1,   -1,   -1,   -1,   -1,
  281,  282,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  290,
  291,   -1,  293,  294,  295,  296,  297,   -1,   -1,   -1,
   -1,   -1,   -1,  124,  125,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  319,  320,
  321,  322,  323,  324,  325,  326,  327,  328,  329,  330,
    0,   -1,  333,  334,   -1,   -1,   -1,  338,   -1,   -1,
   10,   -1,   -1,  344,   -1,   -1,  262,  263,  264,   -1,
   -1,  267,  268,  269,   -1,  271,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  281,  282,   -1,   -1,   -1,
   -1,   41,   -1,   -1,  290,  291,   -1,  293,  294,  295,
  296,  297,   -1,   -1,   -1,   -1,    0,   -1,   58,   59,
   -1,   -1,   -1,   -1,   -1,   -1,   10,   -1,   -1,   -1,
   -1,   -1,   -1,  319,  320,  321,  322,  323,  324,  325,
  326,  327,  328,  329,  330,   -1,   -1,  333,  334,   -1,
   -1,    0,  338,   -1,   -1,   -1,   -1,   41,  344,   -1,
   44,   10,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  262,  263,  264,   -1,   59,  267,  268,  269,   -1,
  271,   -1,   -1,   -1,   -1,  125,   -1,   -1,   37,   38,
  281,  282,   41,   42,   43,   44,   45,   -1,   47,  290,
  291,   -1,  293,  294,  295,  296,  297,   -1,   -1,   58,
   59,   60,   -1,   62,   63,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  319,  320,
  321,  322,  323,  324,  325,  326,  327,  328,  329,  330,
    0,  125,  333,  334,   93,   94,   -1,  338,   -1,   -1,
   10,   -1,   -1,  344,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  124,  125,   37,   38,   -1,
   -1,   41,   42,   43,   44,   45,   -1,   47,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   58,   59,
   60,   -1,   62,   63,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  262,  263,  264,   -1,   -1,  267,  268,  269,
    0,  271,   -1,   93,   94,   -1,   -1,   -1,   -1,   -1,
   10,  281,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  290,  291,   -1,  293,  294,  295,  296,  297,   -1,   -1,
   -1,   -1,   -1,   -1,  124,  125,   -1,   37,   38,   -1,
   -1,   41,   42,   43,   44,   45,   -1,   47,  262,  263,
  264,   -1,   -1,   -1,  268,  269,   -1,  271,   58,   59,
   60,   -1,   62,   63,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  290,  291,   -1,  293,
  294,  295,  296,  262,  263,  264,   -1,   -1,  267,  268,
  269,   -1,  271,   93,   94,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  281,  282,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  290,  291,   -1,  293,  294,  295,  296,  297,   -1,
   -1,   -1,   -1,   -1,  124,  125,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  319,  320,  321,  322,  323,  324,  325,  326,  327,  328,
  329,  330,    0,   -1,  333,  334,   -1,   -1,   -1,  338,
   -1,   -1,   10,   -1,   -1,  344,   -1,   -1,   -1,   -1,
   -1,   -1,  262,  263,  264,   -1,   -1,  267,  268,  269,
   -1,  271,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   37,
   38,  281,  282,   41,   42,   43,   44,   45,   -1,   47,
  290,  291,   -1,  293,  294,  295,  296,  297,   -1,   -1,
   58,   59,   60,   -1,   62,   63,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  319,
  320,  321,  322,  323,  324,  325,  326,  327,  328,  329,
  330,   -1,    0,  333,  334,   93,   94,   -1,  338,   -1,
   -1,   -1,   10,   -1,  344,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  262,  263,  264,   -1,   -1,  267,  268,  269,
   -1,  271,   -1,   -1,   -1,   -1,  124,  125,   -1,   -1,
   38,  281,  282,   41,   -1,   43,   44,   45,   -1,   -1,
  290,  291,   -1,  293,  294,  295,  296,  297,   -1,   -1,
   58,   59,   60,   -1,   62,   63,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  319,
  320,  321,  322,  323,  324,  325,  326,  327,  328,  329,
  330,    0,   -1,  333,  334,   93,   94,   -1,  338,   -1,
   -1,   10,   -1,   -1,  344,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  124,  125,   -1,   38,
   -1,   -1,   41,   -1,   43,   44,   45,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   58,
   59,   60,   -1,   62,   63,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  262,  263,  264,   -1,   -1,  267,
  268,  269,   -1,  271,   93,   94,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,  281,  282,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  290,  291,   -1,  293,  294,  295,  296,  297,
   -1,   -1,   -1,   -1,   -1,  124,  125,    0,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   10,   -1,   -1,
   -1,  319,  320,  321,  322,  323,  324,  325,  326,  327,
  328,  329,  330,   -1,   -1,  333,  334,   -1,   -1,   -1,
  338,   -1,   -1,   -1,   -1,   38,  344,   -1,   41,   -1,
   -1,   44,   -1,   -1,  262,  263,  264,   -1,   -1,  267,
  268,  269,   -1,  271,   -1,   58,   59,   60,   -1,   62,
   63,   -1,   -1,  281,  282,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  290,  291,   -1,  293,  294,  295,  296,  297,
   -1,   -1,   -1,   -1,   -1,   -1,    0,   -1,   -1,   -1,
   93,   94,   -1,   -1,   -1,   -1,   10,   -1,   -1,   -1,
   -1,  319,  320,  321,  322,  323,  324,  325,  326,  327,
  328,  329,  330,   -1,   -1,  333,  334,   -1,   -1,   -1,
  338,  124,  125,   -1,   38,   -1,  344,   41,   -1,   -1,
   44,   -1,   -1,  262,  263,  264,   -1,   -1,  267,  268,
  269,   -1,  271,   -1,   58,   59,   60,   -1,   62,   63,
   -1,   -1,  281,  282,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  290,  291,   -1,  293,  294,  295,  296,  297,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,    0,   -1,   -1,   93,
   94,   -1,   -1,   -1,   -1,   -1,   10,   -1,   -1,   -1,
  319,  320,  321,  322,  323,  324,  325,  326,  327,  328,
  329,  330,   -1,   -1,  333,  334,   -1,   -1,   -1,  338,
  124,  125,   -1,   -1,   38,  344,   -1,   41,   -1,   -1,
   44,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   58,   59,   60,   -1,   62,   63,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  262,
  263,  264,   -1,   -1,  267,  268,  269,   -1,  271,   93,
   94,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  281,  282,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,  290,  291,   -1,
  293,  294,  295,  296,  297,   -1,   -1,   -1,   -1,   -1,
  124,  125,    0,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   10,   -1,   -1,   -1,  319,  320,  321,  322,
  323,  324,  325,  326,  327,  328,  329,  330,   -1,   -1,
  333,  334,   -1,   -1,   -1,  338,   -1,   -1,   -1,   -1,
   -1,  344,   -1,   41,   -1,   -1,   44,   -1,  262,  263,
  264,   -1,   -1,  267,  268,  269,   -1,  271,   -1,   -1,
   58,   59,   60,   -1,   62,   63,   -1,  281,  282,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  290,  291,   -1,  293,
  294,  295,  296,  297,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,    0,   -1,   -1,   -1,   93,   94,   -1,   -1,   -1,
   -1,   10,   -1,   -1,   -1,  319,  320,  321,  322,  323,
  324,  325,  326,  327,  328,  329,  330,   -1,   -1,  333,
  334,   -1,   -1,   -1,  338,   -1,  124,  125,   -1,   -1,
  344,   -1,   41,   -1,   -1,   44,   -1,   -1,  262,  263,
  264,   -1,   -1,  267,  268,  269,   -1,  271,   -1,   58,
   59,   60,   -1,   62,   63,   -1,   -1,  281,  282,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  290,  291,   -1,  293,
  294,  295,  296,  297,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,    0,   -1,   -1,   93,   94,   -1,   -1,   -1,   -1,
   -1,   10,   -1,   -1,   -1,  319,  320,  321,  322,  323,
  324,  325,  326,  327,  328,  329,  330,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  338,  124,  125,   -1,   -1,   -1,
  344,   -1,   41,   -1,   -1,   44,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   58,
   59,   60,   -1,   62,   63,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  262,  263,  264,   -1,   -1,  267,
  268,  269,   -1,  271,   93,    0,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,  281,  282,   10,   -1,   -1,   -1,   -1,
   -1,   -1,  290,  291,   -1,  293,  294,  295,  296,  297,
   -1,   -1,   -1,   -1,   -1,   -1,  125,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   41,   -1,   -1,   44,
   -1,  319,  320,  321,  322,  323,  324,  325,  326,  327,
  328,  329,  330,   58,   59,   60,   -1,   62,   63,   -1,
  338,   -1,   -1,   -1,   -1,   -1,  344,   -1,   -1,   -1,
   -1,   -1,   -1,  262,  263,  264,   -1,   -1,  267,  268,
  269,   -1,  271,   -1,    0,   -1,   -1,   -1,   93,   -1,
   -1,   -1,  281,  282,   10,   -1,   -1,   -1,   -1,   -1,
   -1,  290,  291,   -1,  293,  294,  295,  296,  297,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  125,   -1,   -1,   -1,   -1,   41,   -1,   -1,   44,   -1,
  319,  320,  321,  322,  323,  324,  325,  326,  327,  328,
  329,  330,   58,   59,   60,   -1,   62,   63,   -1,  338,
   -1,   -1,   -1,   -1,   -1,  344,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,  262,  263,  264,   -1,   -1,  267,  268,
  269,   -1,  271,   -1,    0,   -1,   -1,   93,   -1,   -1,
   -1,   -1,  281,  282,   10,   -1,   -1,   -1,   -1,   -1,
   -1,  290,  291,   -1,  293,  294,  295,  296,  297,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  125,
   -1,   -1,   -1,   -1,   -1,   41,   -1,   -1,   44,   -1,
  319,  320,  321,  322,  323,  324,  325,  326,  327,  328,
  329,  330,   58,   59,   60,   -1,   62,   63,   -1,  338,
   -1,   -1,   -1,   -1,   -1,  344,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,  262,  263,  264,
   -1,   -1,  267,  268,  269,   -1,  271,   93,    0,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  281,  282,   10,   -1,
   -1,   -1,   -1,   -1,   -1,  290,  291,   -1,  293,  294,
  295,  296,  297,   -1,   -1,   -1,   -1,   -1,   -1,  125,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   41,
   -1,   -1,   44,   -1,  319,  320,  321,  322,  323,  324,
  325,  326,  327,  328,  329,  330,   58,   59,   -1,   -1,
   -1,   63,   -1,  338,   -1,   -1,   -1,   -1,   -1,  344,
   -1,   -1,   -1,   -1,   -1,   -1,  262,  263,  264,   -1,
   -1,  267,  268,  269,   -1,  271,   -1,    0,   -1,   -1,
   -1,   93,   -1,   -1,   -1,  281,  282,   10,   -1,   -1,
   -1,   -1,   -1,   -1,  290,  291,   -1,  293,  294,  295,
  296,  297,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,  125,   -1,   -1,   -1,   -1,   41,   -1,
   -1,   44,   -1,  319,  320,  321,  322,  323,  324,  325,
  326,  327,  328,  329,  330,   58,   59,   -1,   -1,   -1,
   63,   -1,  338,   -1,   -1,   -1,   -1,   -1,  344,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  262,  263,  264,   -1,
   -1,  267,  268,  269,   -1,  271,   -1,    0,   -1,   -1,
   93,   -1,   -1,   -1,   -1,  281,  282,   10,   -1,   -1,
   -1,   -1,   -1,   -1,  290,  291,   -1,  293,  294,  295,
  296,  297,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  125,   -1,   -1,   -1,   -1,   -1,   41,   -1,
   -1,   44,   -1,  319,  320,  321,  322,  323,  324,  325,
  326,  327,  328,  329,  330,   58,   59,   -1,   -1,   -1,
   63,   -1,  338,   -1,   -1,   -1,   -1,   -1,  344,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  262,  263,  264,   -1,   -1,  267,  268,  269,   -1,  271,
   93,    0,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  281,
  282,   10,   -1,   -1,   -1,   -1,   -1,   -1,  290,  291,
   -1,  293,  294,  295,  296,  297,   -1,   -1,   -1,   -1,
   -1,   -1,  125,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   41,   -1,   -1,   44,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,  325,  326,   -1,   -1,  329,  330,   58,
   59,   -1,   -1,   -1,   63,   -1,  338,   -1,   -1,   -1,
   -1,   -1,  344,   -1,   -1,   -1,   -1,   -1,   -1,  262,
  263,  264,   -1,   -1,  267,  268,  269,   -1,  271,   -1,
    0,   -1,   -1,   -1,   93,   -1,   -1,   -1,  281,  282,
   10,   -1,   -1,   -1,   -1,   -1,   -1,  290,  291,   -1,
  293,  294,  295,  296,  297,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  125,   -1,   -1,   -1,
   -1,   41,   -1,   -1,   44,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  325,  326,   -1,   -1,  329,  330,   58,   59,
   -1,   -1,   -1,   63,   -1,  338,   -1,   -1,   -1,   -1,
   -1,  344,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  262,
  263,  264,   -1,   -1,  267,  268,  269,   -1,  271,   -1,
    0,   -1,   -1,   93,   -1,   -1,   -1,   -1,  281,  282,
   10,   -1,   -1,   -1,   -1,   -1,   -1,  290,  291,   -1,
  293,  294,  295,  296,  297,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  125,   -1,   -1,   -1,   -1,
   -1,   41,   -1,   -1,   44,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  325,  326,   -1,   -1,  329,  330,   58,   59,
   -1,   -1,   -1,   63,   -1,  338,   -1,   -1,   -1,   -1,
   -1,  344,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,  262,  263,  264,   -1,   -1,  267,  268,
  269,   -1,  271,   93,    0,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  281,  282,   10,   -1,   -1,   -1,   -1,   -1,
   -1,  290,  291,   -1,  293,  294,  295,  296,  297,   -1,
   -1,   -1,   -1,   -1,   -1,  125,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   41,   -1,   -1,   44,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  325,  326,   -1,   -1,
  329,  330,   58,   59,   -1,   -1,   -1,   63,   -1,  338,
   -1,   -1,   -1,   -1,   -1,  344,   -1,   -1,   -1,   -1,
   -1,   -1,  262,  263,  264,   -1,   -1,  267,  268,  269,
   -1,  271,   -1,    0,   -1,   -1,   -1,   93,   -1,   -1,
   -1,  281,  282,   10,   -1,   -1,   -1,   -1,   -1,   -1,
  290,  291,   -1,  293,  294,  295,  296,  297,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  125,
   -1,   -1,   -1,   -1,   41,   -1,   -1,   44,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  325,  326,   -1,   -1,  329,
  330,   58,   59,   -1,   -1,   -1,   63,   -1,  338,   -1,
   -1,   -1,   -1,   -1,  344,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  262,  263,  264,   -1,   -1,  267,  268,  269,
   -1,  271,   -1,    0,   -1,   -1,   93,   -1,   -1,   -1,
   -1,  281,  282,   10,   -1,   -1,   -1,   -1,   -1,   -1,
  290,  291,   -1,  293,  294,  295,  296,  297,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  125,    0,
   -1,   -1,   -1,   -1,   41,   -1,   -1,   44,   -1,   10,
   -1,   -1,   -1,   -1,   -1,  325,  326,   -1,   -1,  329,
  330,   58,   59,   -1,   -1,   -1,   63,   -1,  338,   -1,
   -1,   -1,   -1,   -1,  344,   -1,   -1,   -1,   -1,   -1,
   41,   -1,   -1,   44,   -1,   -1,  262,  263,  264,   -1,
   -1,  267,  268,  269,   -1,  271,   93,   58,   59,    0,
   -1,   -1,   63,   -1,   -1,  281,  282,   -1,   -1,   10,
   -1,   -1,   -1,   -1,  290,  291,   -1,  293,  294,  295,
  296,  297,   -1,   -1,   -1,   -1,   -1,   -1,  125,   -1,
   -1,   -1,   93,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   41,   -1,   -1,   44,   -1,   -1,   -1,   -1,   -1,  325,
  326,   -1,   -1,  329,  330,   -1,   -1,   58,   59,   -1,
   -1,   -1,  338,   -1,  125,   -1,   -1,   -1,  344,   -1,
   -1,   -1,   -1,   -1,   -1,  262,  263,  264,   -1,   -1,
  267,  268,  269,   -1,  271,   -1,    0,   -1,   -1,   -1,
   -1,   -1,   93,   -1,  281,  282,   10,   -1,   -1,   -1,
   -1,   -1,   -1,  290,  291,   -1,  293,  294,  295,  296,
  297,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  125,   -1,   -1,   41,   -1,   -1,
   44,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  326,
   -1,   -1,  329,  330,   58,   59,    0,   -1,   -1,   -1,
   -1,  338,   -1,   -1,   -1,   -1,   10,  344,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  262,  263,  264,   -1,   -1,
  267,  268,  269,   -1,  271,   -1,   -1,   -1,   -1,   93,
   -1,   -1,   -1,   -1,  281,  282,   -1,   41,   -1,   -1,
   44,   -1,   -1,  290,  291,   -1,  293,  294,  295,  296,
  297,  262,  263,  264,   58,   59,  267,  268,  269,   -1,
  271,  125,   -1,   -1,    0,   -1,   -1,   -1,   -1,   -1,
  281,  282,   -1,   -1,   10,   -1,   -1,   -1,   -1,  290,
  291,   -1,  293,  294,  295,  296,  297,   -1,   -1,   93,
   -1,  338,   -1,   -1,   -1,   -1,   -1,  344,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   41,   -1,   -1,   44,   -1,
   -1,  262,  263,  264,   -1,   -1,  267,  268,  269,    0,
  271,  125,   58,   59,   -1,   -1,   -1,  338,   -1,   10,
  281,  282,   -1,  344,   -1,   -1,   -1,   -1,   -1,  290,
  291,   -1,  293,  294,  295,  296,  297,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   93,   -1,   -1,
   41,   -1,   -1,   44,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   58,   59,   -1,
    0,   -1,   -1,   -1,   -1,   -1,   -1,  338,   -1,  125,
   10,   -1,   -1,  344,   -1,   -1,   -1,   -1,  262,  263,
  264,   -1,   -1,  267,  268,  269,   -1,  271,   -1,   -1,
   -1,   -1,   93,   -1,   -1,   -1,   -1,  281,  282,   -1,
   -1,   41,   -1,   -1,   44,   -1,  290,  291,   -1,  293,
  294,  295,  296,  297,   -1,   -1,   -1,   -1,   58,   59,
   -1,   -1,   -1,   -1,  125,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  262,  263,
  264,   -1,   -1,  267,  268,  269,   -1,  271,   -1,   -1,
   -1,    0,   -1,   93,  338,   -1,   -1,  281,  282,   -1,
  344,   10,   -1,   -1,   -1,   -1,  290,  291,   -1,  293,
  294,  295,  296,  297,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  125,   -1,   -1,   -1,   -1,
   -1,   -1,   41,   -1,   -1,   44,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  262,  263,  264,   58,
   59,  267,  268,  269,  338,  271,   -1,   -1,   -1,   -1,
  344,   -1,   -1,   -1,   -1,  281,  282,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  290,  291,   -1,  293,  294,  295,
  296,  297,    0,   -1,   93,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   10,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  262,  263,  264,   -1,   -1,  267,  268,  269,   -1,
  271,   -1,   -1,   -1,   -1,   -1,  125,   -1,   -1,   -1,
  281,  282,  338,   41,   -1,   -1,   44,   -1,  344,  290,
  291,   -1,  293,  294,  295,  296,  297,   -1,   -1,   -1,
   58,   59,   -1,    0,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   10,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  262,  263,  264,   -1,   -1,  267,  268,  269,
   -1,  271,   -1,   -1,   -1,   93,   -1,  338,   -1,   -1,
   -1,  281,  282,  344,   41,   -1,   -1,   44,   -1,   -1,
  290,  291,   -1,  293,  294,  295,  296,  297,    0,   -1,
   -1,   58,   59,   -1,   -1,   -1,   -1,  125,   10,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,    0,   93,   -1,  338,   41,
   -1,   -1,   44,   -1,  344,   10,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,  262,  263,  264,   58,   59,  267,  268,
  269,   -1,  271,   -1,   -1,   -1,   -1,   -1,  125,   -1,
   -1,   -1,  281,  282,   -1,   -1,   41,   -1,   -1,   44,
   -1,  290,  291,   -1,  293,  294,  295,  296,  297,   -1,
   -1,   93,   -1,   58,   59,   -1,    0,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   10,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,  125,   -1,   -1,   -1,   -1,   93,  338,
   -1,   -1,   -1,   -1,   -1,  344,   -1,   41,   -1,   -1,
   44,   -1,   -1,   -1,  262,  263,  264,   -1,   -1,  267,
  268,  269,   -1,  271,   58,   59,   -1,    0,   -1,   -1,
  125,   -1,   -1,  281,  282,   -1,   -1,   10,   -1,   -1,
   -1,   -1,  290,  291,   -1,  293,  294,  295,  296,  297,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   93,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   41,   -1,
   -1,   44,   -1,   -1,   -1,  262,  263,  264,   -1,   -1,
  267,  268,  269,   -1,  271,   58,   59,   -1,   -1,   -1,
  338,  125,   -1,   -1,  281,  282,  344,   -1,   -1,   -1,
   -1,   -1,   -1,  290,  291,   -1,  293,  294,  295,  296,
  297,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   93,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  262,  263,  264,   -1,   -1,  267,  268,  269,   -1,  271,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  281,
  282,  338,  125,   -1,   -1,   -1,   -1,  344,  290,  291,
   -1,  293,  294,  295,  296,  297,   -1,  262,  263,  264,
   -1,   -1,  267,  268,  269,   -1,  271,   -1,   -1,   -1,
    0,   -1,   -1,   -1,   -1,   -1,  281,  282,   -1,   -1,
   10,   -1,   -1,   -1,   -1,  290,  291,   -1,  293,  294,
  295,  296,  297,   -1,   -1,   -1,  338,   -1,   -1,   -1,
   -1,   -1,  344,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   41,   -1,   -1,   44,   -1,   -1,   -1,  262,  263,
  264,   -1,   -1,  267,  268,  269,   -1,  271,   58,   59,
   -1,   -1,   -1,  338,   -1,   -1,   -1,  281,  282,  344,
   -1,   -1,   -1,   -1,   -1,   -1,  290,  291,   -1,  293,
  294,  295,  296,  297,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   93,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,    0,   -1,  262,
  263,  264,   -1,   -1,  267,  268,  269,   10,  271,   -1,
   -1,   -1,   -1,   -1,   -1,  125,   -1,   -1,  281,  282,
  344,   -1,   -1,   -1,   -1,   -1,   -1,  290,  291,   -1,
  293,  294,  295,  296,   37,   38,   -1,   -1,   41,   42,
   43,   44,   45,   46,   47,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   58,   59,   60,   -1,   62,
   63,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  338,   -1,   -1,   -1,   -1,
   -1,  344,    0,   -1,   -1,   -1,   -1,   -1,   91,   -1,
   -1,   94,   10,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   37,
   38,  124,  125,   41,   42,   43,   44,   45,   46,   47,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   58,   59,   60,   -1,   62,   63,   -1,   -1,   -1,   -1,
   -1,   -1,  262,  263,  264,   -1,   -1,  267,  268,  269,
   -1,  271,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  281,  282,   91,   -1,   -1,   94,   -1,    0,   -1,
  290,  291,   -1,  293,  294,  295,  296,  297,   10,    0,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   10,
   -1,   -1,   -1,   -1,   -1,   -1,  124,  125,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   37,   38,   -1,   -1,   41,
   42,   43,   44,   45,   46,   47,   -1,   -1,   -1,   -1,
   41,   -1,   -1,   -1,  344,   -1,   58,   59,   60,   -1,
   62,   63,   -1,   -1,   -1,   -1,   -1,   58,   59,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  262,
  263,  264,   -1,   -1,  267,  268,  269,   -1,  271,   91,
   -1,   -1,   94,   -1,   -1,   -1,   -1,   -1,  281,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,  290,  291,   -1,
  293,  294,  295,  296,  297,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  124,  125,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  125,  318,  319,  320,  321,  322,
  323,  324,  325,  326,  327,  328,  329,  330,   -1,   -1,
  333,  334,  335,   -1,   -1,  338,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  262,  263,  264,   -1,   -1,  267,
  268,  269,   -1,  271,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,  281,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  290,  291,   -1,  293,  294,  295,  296,  297,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  318,  319,  320,  321,  322,  323,  324,  325,  326,  327,
  328,  329,  330,   -1,   -1,  333,  334,  335,   -1,   -1,
  338,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  262,  263,  264,   -1,   -1,  267,  268,  269,   -1,  271,
   -1,  262,  263,  264,   -1,   -1,  267,  268,  269,  281,
  271,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  290,  291,
  281,  293,  294,  295,  296,  297,   -1,    0,   -1,  290,
  291,   -1,  293,  294,  295,  296,  297,   10,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  318,  319,  320,  321,
  322,  323,  324,  325,  326,  327,  328,  329,  330,   -1,
   -1,  333,  334,  335,   37,   38,  338,   -1,   41,   42,
   43,   44,   45,   46,   47,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   59,   60,   61,   62,
   63,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,    0,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   10,   -1,   -1,   -1,   -1,   91,   -1,
   -1,   94,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   37,   38,   -1,   -1,   41,   42,   43,   44,   45,   46,
   47,  124,  125,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   59,   60,   61,   62,   63,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,    0,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   10,   -1,   -1,
   -1,   -1,   -1,   -1,   91,   -1,   -1,   94,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   37,   38,   -1,   -1,   41,   42,
   43,   44,   45,   46,   47,   -1,   -1,  124,  125,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   59,   60,   61,   62,
   63,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   91,   -1,
   -1,   94,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  262,
  263,  264,   -1,   -1,   -1,  268,  269,   -1,  271,   -1,
   -1,  124,  125,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,  290,  291,   -1,
  293,  294,  295,  296,  297,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  318,  319,  320,  321,  322,
  323,  324,  325,  326,  327,  328,  329,  330,   -1,   -1,
  333,  334,  335,   -1,  337,  262,  263,  264,   -1,   -1,
   33,  268,  269,   -1,  271,   -1,   -1,   -1,   37,   38,
   -1,   -1,   -1,   42,   43,   44,   45,   -1,   47,   -1,
   -1,   -1,   -1,  290,  291,   -1,  293,  294,  295,  296,
  297,   60,   -1,   62,   63,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  318,  319,  320,  321,  322,  323,  324,  325,  326,
  327,  328,  329,  330,   -1,   94,  333,  334,  335,  262,
  263,  264,   -1,   -1,   -1,  268,  269,   -1,  271,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,  126,   -1,  124,   -1,  290,  291,   -1,
  293,  294,  295,  296,  297,   -1,    0,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   10,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  318,  319,  320,  321,  322,
  323,  324,  325,  326,  327,  328,  329,  330,   -1,   -1,
  333,  334,  335,   37,   38,   -1,   -1,   41,   42,   43,
   44,   45,   46,   47,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   59,   60,   61,   62,   63,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   91,   -1,   -1,
   94,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   33,   -1,   -1,  256,  257,  258,  259,  260,  261,   -1,
  124,  125,  265,  266,   -1,   -1,   -1,  270,   -1,  272,
  273,  274,  275,  276,  277,  278,   -1,   -1,   -1,   -1,
  283,  284,  285,  286,  287,  288,  289,   -1,   -1,  292,
   -1,   -1,   -1,   -1,   -1,  298,  299,  300,  301,  302,
  303,  304,  305,  306,  307,  308,  309,  310,  311,   -1,
  313,  314,   -1,  316,  317,   -1,   -1,   -1,   -1,  318,
  319,  320,  321,  322,  323,  324,  325,  326,  327,  328,
  329,  330,   -1,  336,  333,  334,  339,  340,   -1,  342,
  343,   -1,  345,  126,  347,  348,  349,  350,  351,  352,
   -1,   33,   -1,   -1,  357,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   59,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  262,  263,
  264,   -1,   -1,   -1,  268,  269,   -1,  271,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  290,  291,   -1,  293,
  294,  295,  296,  297,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  126,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   33,   -1,  318,  319,  320,  321,  322,  323,
  324,  325,  326,  327,  328,  329,  330,   -1,   -1,  333,
  334,  335,   -1,  256,  257,  258,  259,  260,  261,   -1,
   -1,   -1,  265,  266,   -1,   -1,   -1,  270,   -1,  272,
  273,  274,  275,  276,  277,  278,   -1,   -1,   -1,   -1,
  283,  284,  285,  286,  287,  288,  289,   -1,   -1,  292,
   -1,   -1,   -1,   -1,   -1,  298,  299,  300,  301,  302,
  303,  304,  305,  306,  307,  308,  309,  310,  311,   -1,
  313,  314,   -1,  316,  317,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  126,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,  336,   -1,   33,  339,  340,   -1,  342,
  343,   -1,  345,   41,  347,  348,  349,  350,  351,  352,
   -1,   -1,   -1,   -1,  357,  257,  258,  259,  260,  261,
   -1,   -1,   -1,  265,  266,   -1,   -1,   -1,  270,   -1,
  272,  273,  274,  275,  276,  277,  278,   -1,   -1,   -1,
   -1,  283,  284,  285,  286,  287,  288,  289,   -1,   -1,
  292,   -1,   -1,   -1,   -1,   -1,  298,  299,  300,  301,
  302,  303,  304,  305,  306,  307,  308,  309,  310,  311,
   -1,  313,  314,   -1,  316,  317,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  126,   -1,
   -1,   -1,   33,   -1,  336,   -1,   -1,  339,  340,   -1,
  342,  343,   -1,  345,   -1,  347,  348,  349,  350,  351,
  352,   -1,   -1,   -1,   -1,  357,  257,  258,  259,  260,
  261,   -1,   -1,   -1,  265,  266,   -1,   -1,   -1,  270,
   -1,  272,  273,  274,  275,  276,  277,  278,   -1,   -1,
   -1,   -1,  283,  284,  285,  286,  287,  288,  289,   -1,
   -1,  292,   -1,   -1,   -1,   -1,   -1,  298,  299,  300,
  301,  302,  303,  304,  305,  306,  307,  308,  309,  310,
  311,   -1,  313,  314,   -1,  316,  317,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  126,   -1,   -1,   -1,   33,
   -1,   -1,   -1,   -1,   -1,  336,   -1,   -1,  339,  340,
   -1,  342,  343,   -1,  345,   -1,  347,  348,  349,  350,
  351,  352,   -1,   -1,   -1,   -1,  357,   -1,   -1,  257,
  258,  259,   -1,  261,   -1,   -1,   -1,  265,  266,   -1,
   -1,   -1,  270,   -1,  272,  273,  274,  275,  276,  277,
  278,   -1,   -1,   -1,   -1,  283,  284,  285,  286,  287,
  288,  289,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  299,   -1,   -1,  302,  303,  304,  305,  306,  307,
  308,  309,  310,  311,   -1,  313,  314,   -1,  316,  317,
   -1,   -1,  126,   -1,   -1,   -1,   33,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  336,   -1,
   -1,  339,  340,   -1,  342,  343,   -1,  345,  346,  347,
  348,  349,  350,  351,  352,   -1,  257,  258,  259,  357,
  261,   -1,   -1,   -1,  265,  266,   -1,   -1,   -1,  270,
   -1,  272,  273,  274,  275,  276,  277,  278,   -1,   -1,
   -1,   -1,  283,  284,  285,  286,  287,  288,  289,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  299,   -1,
   -1,  302,  303,  304,  305,  306,  307,  308,  309,  310,
  311,   -1,  313,  314,   -1,  316,  317,   -1,   -1,  126,
   -1,   -1,   -1,   33,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  336,   -1,   -1,  339,  340,
   -1,  342,  343,   -1,  345,  346,  347,  348,  349,  350,
  351,  352,   -1,  257,  258,  259,  357,  261,   -1,   -1,
   -1,  265,  266,   -1,   -1,   -1,  270,   -1,  272,  273,
  274,  275,  276,  277,  278,   -1,   -1,   -1,   -1,  283,
  284,  285,  286,  287,  288,  289,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  299,   -1,   -1,  302,  303,
  304,  305,  306,  307,  308,  309,  310,  311,   -1,  313,
  314,   -1,  316,  317,   -1,   -1,  126,   -1,   -1,   -1,
   33,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  336,   -1,   -1,  339,  340,   -1,  342,  343,
   -1,  345,  346,  347,  348,  349,  350,  351,  352,   -1,
  257,  258,  259,  357,  261,   -1,   -1,   -1,  265,  266,
   -1,   -1,   -1,  270,   -1,  272,  273,  274,  275,  276,
  277,  278,   -1,   -1,   -1,   -1,  283,  284,  285,  286,
  287,  288,  289,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  299,   -1,   -1,  302,  303,  304,  305,  306,
  307,  308,  309,  310,  311,   -1,  313,  314,   -1,  316,
  317,   -1,   -1,  126,   -1,   -1,   -1,   33,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  336,
   -1,   -1,  339,  340,   -1,  342,  343,   -1,  345,  346,
  347,  348,  349,  350,  351,  352,   -1,  257,  258,  259,
  357,  261,   -1,   -1,   -1,  265,  266,   -1,   -1,   -1,
  270,   -1,  272,  273,  274,  275,  276,  277,  278,   -1,
   -1,   -1,   -1,  283,  284,  285,  286,  287,  288,  289,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  299,
   -1,   -1,  302,  303,  304,  305,  306,  307,  308,  309,
  310,  311,   -1,  313,  314,   -1,  316,  317,   -1,   -1,
  126,   -1,   -1,   -1,   33,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  336,   -1,   -1,  339,
  340,   -1,  342,  343,   -1,  345,  346,  347,  348,  349,
  350,  351,  352,   -1,  257,  258,  259,  357,  261,   -1,
   -1,   -1,  265,  266,   -1,   -1,   -1,  270,   -1,  272,
  273,  274,  275,  276,  277,  278,   -1,   -1,   -1,   -1,
  283,  284,  285,  286,  287,  288,  289,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  299,   -1,   -1,  302,
  303,  304,  305,  306,  307,  308,  309,  310,  311,   -1,
  313,  314,   -1,  316,  317,   -1,   -1,  126,   -1,   -1,
   -1,   33,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,  336,   -1,   -1,  339,  340,   -1,  342,
  343,   -1,  345,  346,  347,  348,  349,  350,  351,  352,
   -1,  257,  258,  259,  357,  261,   -1,   -1,   -1,  265,
  266,   -1,   -1,   -1,  270,   -1,  272,  273,  274,  275,
  276,  277,  278,   -1,   -1,   -1,   -1,  283,  284,  285,
  286,  287,  288,  289,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,  299,   -1,   -1,  302,  303,  304,  305,
  306,  307,  308,  309,  310,  311,   -1,  313,  314,   -1,
  316,  317,   -1,   -1,  126,   -1,   -1,   -1,   33,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  336,   -1,   -1,  339,  340,   -1,  342,  343,   -1,  345,
  346,  347,  348,  349,  350,  351,  352,   -1,  257,  258,
  259,  357,  261,   -1,   -1,   -1,  265,  266,   -1,   -1,
   -1,  270,   -1,  272,  273,  274,  275,  276,  277,  278,
   -1,   -1,   -1,   -1,  283,  284,  285,  286,  287,  288,
  289,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  299,   -1,   -1,  302,  303,  304,  305,  306,  307,  308,
  309,  310,  311,   -1,  313,  314,   -1,  316,  317,   -1,
   -1,  126,   -1,   -1,   -1,   33,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,  336,   -1,   -1,
  339,  340,   -1,  342,  343,   -1,  345,  346,  347,  348,
  349,  350,  351,  352,   -1,  257,  258,  259,  357,  261,
   -1,   -1,   -1,  265,  266,   -1,   -1,   -1,  270,   -1,
  272,  273,  274,  275,  276,  277,  278,   -1,   -1,   -1,
   -1,  283,  284,  285,  286,  287,  288,  289,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,  299,   -1,   -1,
  302,  303,  304,  305,  306,  307,  308,  309,  310,  311,
   -1,  313,  314,   -1,  316,  317,   -1,   -1,  126,   -1,
   -1,   -1,   33,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  336,   -1,   -1,  339,  340,   -1,
  342,  343,   -1,  345,  346,  347,  348,  349,  350,  351,
  352,   -1,  257,  258,  259,  357,  261,   -1,   -1,   -1,
  265,  266,   -1,   -1,   -1,  270,   -1,  272,  273,  274,
  275,  276,  277,  278,   -1,   -1,   -1,   -1,  283,  284,
  285,  286,  287,  288,  289,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  299,   -1,   -1,  302,  303,  304,
  305,  306,  307,  308,  309,  310,  311,   -1,  313,  314,
   -1,  316,  317,   -1,   -1,  126,   -1,   -1,   -1,   33,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  336,   -1,   -1,  339,  340,   -1,  342,  343,   -1,
  345,  346,  347,  348,  349,  350,  351,  352,   -1,  257,
  258,  259,  357,  261,   -1,   -1,   -1,  265,  266,   -1,
   -1,   -1,  270,   -1,  272,  273,  274,  275,  276,  277,
  278,   -1,   -1,   -1,   -1,  283,  284,  285,  286,  287,
  288,  289,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  299,   -1,   -1,  302,  303,  304,  305,  306,  307,
  308,  309,  310,  311,   -1,  313,  314,   -1,  316,  317,
   -1,   -1,  126,   -1,   -1,   -1,   33,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  336,   -1,
   -1,  339,  340,   -1,  342,  343,   -1,  345,  346,  347,
  348,  349,  350,  351,  352,   -1,  257,  258,  259,  357,
  261,   -1,   -1,   -1,  265,  266,   -1,   -1,   -1,  270,
   -1,  272,  273,  274,  275,  276,  277,  278,   -1,   -1,
   -1,   -1,  283,  284,  285,  286,  287,  288,  289,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  299,   -1,
   -1,  302,  303,  304,  305,  306,  307,  308,  309,  310,
  311,   -1,  313,  314,   -1,  316,  317,   -1,   -1,  126,
   -1,   -1,   -1,   33,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  336,   -1,   -1,  339,  340,
   -1,  342,  343,   -1,  345,  346,  347,  348,  349,  350,
  351,  352,   -1,  257,  258,  259,  357,  261,   -1,   -1,
   -1,  265,  266,   -1,   -1,   -1,  270,   -1,  272,  273,
  274,  275,  276,  277,  278,   -1,   -1,   -1,   -1,  283,
  284,  285,  286,  287,  288,  289,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  299,   -1,   -1,  302,  303,
  304,  305,  306,  307,  308,  309,  310,  311,   -1,  313,
  314,   -1,  316,  317,   -1,   -1,  126,   -1,   -1,   -1,
   33,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  336,   -1,   -1,  339,  340,   -1,  342,  343,
   -1,  345,  346,  347,  348,  349,  350,  351,  352,   -1,
  257,  258,  259,  357,  261,   -1,   -1,   -1,  265,  266,
   -1,   -1,   -1,  270,   -1,  272,  273,  274,  275,  276,
  277,  278,   -1,   -1,   -1,   -1,  283,  284,  285,  286,
  287,  288,  289,   -1,   -1,  292,   -1,   -1,   -1,   -1,
   -1,   -1,  299,   -1,   -1,  302,  303,  304,  305,  306,
  307,  308,  309,  310,  311,   -1,  313,  314,   -1,  316,
  317,   -1,   -1,  126,   -1,   -1,   -1,   33,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  336,
   -1,   -1,  339,  340,   -1,  342,  343,   -1,   -1,   -1,
  347,  348,  349,  350,  351,  352,   -1,  257,  258,  259,
  357,  261,   -1,   -1,   -1,  265,  266,   -1,   -1,   -1,
  270,   -1,  272,  273,  274,  275,  276,  277,  278,   -1,
   -1,   -1,   -1,  283,  284,  285,  286,  287,  288,  289,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  299,
   -1,   -1,  302,  303,  304,  305,  306,  307,  308,  309,
  310,  311,   -1,  313,  314,   -1,  316,  317,   -1,   -1,
  126,   -1,   -1,   -1,   33,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  336,   -1,   -1,  339,
  340,   -1,  342,  343,   -1,  345,   -1,  347,  348,  349,
  350,  351,  352,   -1,  257,  258,  259,  357,  261,   -1,
   -1,   -1,  265,  266,   -1,   -1,   -1,  270,   -1,  272,
  273,  274,  275,  276,  277,  278,   -1,   -1,   -1,   -1,
  283,  284,  285,  286,  287,  288,  289,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  299,   -1,   -1,  302,
  303,  304,  305,  306,  307,  308,  309,  310,  311,   -1,
  313,  314,   -1,  316,  317,   -1,   -1,  126,   -1,   -1,
   -1,   33,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,  336,   -1,   -1,  339,  340,   -1,  342,
  343,   -1,  345,   -1,  347,  348,  349,  350,  351,  352,
   -1,  257,  258,  259,  357,  261,   -1,   -1,   -1,  265,
  266,   -1,   -1,   -1,  270,   -1,  272,  273,  274,  275,
  276,  277,  278,   -1,   -1,   -1,   -1,  283,  284,  285,
  286,  287,  288,  289,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,  299,   -1,   -1,  302,  303,  304,  305,
  306,  307,  308,  309,  310,  311,   -1,  313,  314,   -1,
  316,  317,   -1,   -1,  126,   -1,   -1,   -1,   33,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  336,   -1,   -1,  339,  340,   -1,  342,  343,   -1,  345,
   -1,  347,  348,  349,  350,  351,  352,   -1,  257,  258,
  259,  357,  261,   -1,   -1,   -1,  265,  266,   -1,   -1,
   -1,  270,   -1,  272,  273,  274,  275,  276,  277,  278,
   -1,   -1,   -1,   -1,  283,  284,  285,  286,  287,  288,
  289,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  299,   -1,   -1,  302,  303,  304,  305,  306,  307,  308,
  309,  310,  311,   -1,  313,  314,   -1,  316,  317,   -1,
   -1,  126,   -1,   -1,   -1,   33,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,  336,   -1,   -1,
  339,  340,   -1,  342,  343,   -1,  345,   -1,  347,  348,
  349,  350,  351,  352,   -1,  257,  258,  259,  357,  261,
   -1,   -1,   -1,  265,  266,   -1,   -1,   -1,  270,   -1,
  272,  273,  274,  275,  276,  277,  278,   -1,   -1,   -1,
   -1,  283,  284,  285,  286,  287,  288,  289,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,  299,   -1,   -1,
  302,  303,  304,  305,  306,  307,  308,  309,  310,  311,
   -1,  313,  314,   -1,  316,  317,   -1,   -1,  126,   -1,
   -1,   -1,   33,   -1,   -1,   -1,   -1,   -1,   -1,   40,
   -1,   -1,   -1,   -1,  336,   -1,   -1,  339,  340,   -1,
  342,  343,   -1,  345,   -1,  347,  348,  349,  350,  351,
  352,   -1,  257,  258,  259,  357,  261,   -1,   -1,   -1,
  265,  266,   -1,   -1,   -1,  270,   -1,  272,  273,  274,
  275,  276,  277,  278,   -1,   -1,   -1,   -1,  283,  284,
  285,  286,  287,  288,  289,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  299,   -1,   -1,  302,  303,  304,
  305,  306,  307,  308,  309,  310,  311,   -1,  313,  314,
   -1,  316,  317,   -1,   -1,  126,   -1,   -1,   -1,   33,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  336,   -1,   -1,  339,  340,   -1,  342,  343,   -1,
  345,   -1,  347,  348,  349,  350,  351,  352,   -1,  257,
  258,  259,  357,  261,   -1,   -1,   -1,  265,  266,   -1,
   -1,   -1,  270,   -1,  272,  273,  274,  275,  276,  277,
  278,   -1,   -1,   -1,   -1,  283,  284,  285,  286,  287,
  288,  289,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  299,   -1,   -1,  302,  303,  304,  305,  306,  307,
  308,  309,  310,  311,   -1,  313,  314,   -1,  316,  317,
   -1,   -1,  126,   -1,   -1,   -1,   33,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  336,   -1,
   -1,  339,  340,   -1,  342,  343,   -1,  345,   -1,  347,
  348,  349,  350,  351,  352,   -1,  257,  258,  259,  357,
  261,   -1,   -1,   -1,  265,  266,   -1,   -1,   -1,  270,
   -1,  272,  273,  274,  275,  276,  277,  278,   -1,   -1,
   -1,   -1,  283,  284,  285,  286,  287,  288,  289,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  299,   -1,
   -1,  302,  303,  304,  305,  306,  307,  308,  309,  310,
  311,   -1,  313,  314,   -1,  316,  317,   -1,   -1,  126,
   -1,   -1,   -1,   33,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  336,   -1,   -1,  339,  340,
   -1,  342,  343,   -1,   -1,   -1,  347,  348,  349,  350,
  351,  352,   -1,  257,  258,  259,  357,  261,   -1,   -1,
   -1,  265,  266,   -1,   -1,   -1,  270,   -1,  272,  273,
  274,  275,  276,  277,  278,   -1,   -1,   -1,   -1,  283,
  284,  285,  286,  287,  288,  289,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  299,   -1,   -1,  302,  303,
  304,  305,  306,  307,  308,  309,  310,  311,   -1,  313,
  314,   -1,  316,  317,   -1,   -1,  126,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  336,   -1,   -1,  339,  340,   -1,  342,  343,
   -1,   -1,   -1,  347,  348,  349,  350,  351,  352,   -1,
  257,  258,  259,  357,  261,   -1,   -1,   -1,  265,  266,
   -1,   -1,   -1,  270,   -1,  272,  273,  274,  275,  276,
  277,  278,   -1,   -1,   -1,   -1,  283,  284,  285,  286,
  287,  288,  289,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  299,   -1,   -1,  302,  303,  304,  305,  306,
  307,  308,  309,  310,  311,   -1,  313,  314,   -1,  316,
  317,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  336,
   -1,   -1,  339,  340,   -1,  342,  343,   -1,   -1,   -1,
  347,  348,  349,  350,  351,  352,   -1,  257,  258,  259,
  357,  261,   -1,   -1,   -1,  265,  266,   -1,   -1,   -1,
  270,   -1,  272,  273,  274,  275,  276,  277,  278,   -1,
   -1,   -1,   -1,  283,  284,  285,  286,  287,  288,  289,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  299,
   -1,   -1,  302,  303,  304,  305,  306,  307,  308,  309,
  310,  311,   -1,  313,  314,   -1,  316,  317,   -1,   -1,
   -1,   37,   38,   -1,   -1,   -1,   42,   43,   -1,   45,
   -1,   47,   -1,   -1,   -1,   -1,  336,   -1,   -1,  339,
  340,   -1,  342,  343,   60,   -1,   62,  347,  348,  349,
  350,  351,  352,   -1,   -1,   -1,   -1,  357,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   94,   -1,
   96,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  124,   -1,
  126,   -1,   37,   38,   -1,   -1,   -1,   42,   43,   -1,
   45,   -1,   47,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   60,   -1,   62,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   37,   38,   -1,   -1,   -1,   42,   43,   94,
   45,   96,   47,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   60,   -1,   62,   63,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  124,
   -1,  126,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   94,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  257,  258,  259,  260,  261,  262,  263,  264,  124,
   -1,  267,  268,  269,  270,  271,   -1,   -1,  274,  275,
  276,  277,  278,  279,  280,   -1,   -1,  283,  284,  285,
  286,  287,  288,  289,  290,  291,  292,  293,  294,  295,
  296,  297,  298,  299,  300,  301,  302,  303,  304,  305,
  306,   -1,  308,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  316,  317,  318,  319,  320,  321,   -1,  323,  324,   -1,
   -1,  327,   -1,   -1,   -1,  331,  332,  333,  334,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  345,
   -1,  347,  257,  258,  259,  260,  261,  262,  263,  264,
   -1,   -1,  267,  268,  269,  270,  271,   -1,   -1,  274,
  275,  276,  277,  278,  279,  280,   -1,   -1,  283,  284,
  285,  286,  287,  288,  289,  290,  291,  292,  293,  294,
  295,  296,  297,  298,  299,  300,  301,  302,  303,  304,
  305,   -1,   -1,  308,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  316,  317,  318,  319,  320,  321,   -1,  323,  324,
   -1,   -1,  327,   -1,   -1,   -1,  331,  332,  333,  334,
   -1,   37,   38,   -1,   40,   -1,   42,   43,   -1,   45,
  345,   47,  347,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   60,   -1,   62,   -1,   -1,   -1,
   -1,   -1,   -1,  318,  319,  320,  321,  322,  323,  324,
  325,  326,  327,  328,  329,  330,   -1,   -1,  333,  334,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   94,   -1,
   96,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   37,   38,   -1,   -1,   -1,   42,   43,  124,   45,
  126,   47,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   60,   -1,   62,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   37,   38,   -1,
   -1,   -1,   42,   43,   -1,   45,   -1,   47,   94,   -1,
   96,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   60,   -1,   62,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  124,   -1,
  126,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   94,   -1,   96,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  124,   -1,  126,   -1,   -1,   -1,
   -1,  257,  258,  259,  260,  261,  262,  263,  264,   -1,
   -1,  267,  268,  269,  270,  271,   -1,   -1,  274,  275,
  276,  277,  278,  279,  280,   -1,   -1,  283,  284,  285,
  286,  287,  288,  289,  290,  291,  292,  293,  294,  295,
  296,  297,  298,  299,  300,  301,  302,  303,  304,  305,
  306,  307,  308,  309,   -1,   -1,   -1,   -1,   -1,   -1,
  316,  317,  318,  319,  320,  321,   -1,  323,  324,   -1,
   -1,  327,   -1,   -1,   -1,  331,  332,  333,  334,   -1,
   -1,  257,  258,  259,  260,  261,  262,  263,  264,  345,
   -1,  267,  268,  269,  270,  271,   -1,   -1,  274,  275,
  276,  277,  278,  279,  280,   -1,   -1,  283,  284,  285,
  286,  287,  288,  289,  290,  291,  292,  293,  294,  295,
  296,  297,  298,  299,  300,  301,  302,  303,  304,  305,
  306,  307,  308,  309,   -1,   -1,   -1,   -1,   -1,   -1,
  316,  317,  318,  319,  320,  321,   -1,  323,  324,   -1,
   -1,  327,   -1,   -1,   -1,  331,  332,  333,  334,   -1,
   37,   38,   -1,   -1,   -1,   42,   43,   -1,   45,  345,
   47,   -1,   -1,   -1,  304,  305,   -1,   -1,  308,   -1,
   -1,   -1,   -1,   60,   -1,   62,  316,  317,  318,  319,
  320,  321,   -1,  323,  324,   -1,   -1,  327,   -1,   -1,
   -1,  331,  332,  333,  334,   -1,   37,   38,   -1,   -1,
   -1,   42,   43,   -1,   45,  345,   47,   94,   -1,   96,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   60,
   -1,   62,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   37,   38,  124,   -1,  126,
   42,   43,   -1,   45,   -1,   47,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   94,   -1,   96,   -1,   -1,   60,   -1,
   62,   -1,   -1,   -1,   -1,   -1,   37,   38,   -1,   -1,
   -1,   42,   43,   -1,   45,   -1,   47,   -1,   -1,   -1,
   -1,   -1,   -1,  124,   -1,  126,   -1,   -1,   -1,   60,
   -1,   62,   94,   -1,   96,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  124,   94,  126,   96,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,  124,   -1,  126,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  257,  258,  259,  260,  261,  262,  263,  264,   -1,   -1,
  267,  268,  269,  270,  271,   -1,   -1,  274,  275,  276,
  277,  278,  279,  280,   -1,   -1,  283,  284,  285,  286,
  287,  288,  289,  290,  291,  292,  293,  294,  295,  296,
  297,  298,  299,  300,  301,  302,  303,  304,  305,   -1,
   -1,  308,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  316,
  317,  318,  319,  320,  321,   -1,  323,  324,   -1,   -1,
  327,   -1,   -1,   -1,  331,  332,  333,  334,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  345,   -1,
   -1,   -1,   -1,  304,  305,   -1,   -1,  308,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  316,  317,  318,  319,  320,
  321,   -1,  323,  324,   -1,   -1,  327,   -1,   -1,   -1,
  331,  332,  333,  334,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  304,  305,  345,   -1,  308,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  316,  317,  318,  319,  320,  321,
   -1,  323,  324,   -1,   -1,  327,   -1,   -1,   -1,  331,
  332,  333,  334,  304,  305,   -1,   -1,  308,   -1,   -1,
   -1,   -1,   -1,  345,   -1,  316,  317,  318,  319,  320,
  321,   -1,  323,  324,   -1,   -1,  327,   -1,   -1,   -1,
  331,  332,  333,  334,   -1,   37,   38,   -1,   -1,   -1,
   42,   43,   -1,   45,  345,   47,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   60,   -1,
   62,   -1,   -1,   -1,   -1,   -1,   37,   38,   -1,   -1,
   -1,   42,   43,   -1,   45,   -1,   47,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   60,
   -1,   62,   94,   -1,   96,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   37,   38,   -1,   -1,   -1,   42,
   43,   -1,   45,   -1,   47,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  124,   94,  126,   96,   -1,   60,   -1,   62,
   -1,   -1,   -1,   -1,   -1,   37,   38,   -1,   -1,   -1,
   42,   43,   -1,   45,   -1,   47,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,  124,   -1,  126,   -1,   -1,   60,   -1,
   62,   94,   -1,   96,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   37,   38,   -1,   -1,   -1,   42,   43,
   -1,   45,   -1,   47,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  124,   94,  126,   96,   -1,   60,   -1,   62,   -1,
   -1,   -1,   -1,   -1,   37,   38,   -1,   -1,   -1,   42,
   43,   -1,   45,   -1,   47,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  124,   -1,  126,   -1,   -1,   60,   -1,   62,
   94,   -1,   96,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  124,   94,  126,   96,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  124,   -1,  126,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  304,  305,   -1,   -1,  308,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  316,  317,  318,  319,  320,  321,
   -1,  323,  324,   -1,   -1,  327,   -1,   -1,   -1,  331,
  332,  333,  334,  304,  305,   -1,   -1,  308,   -1,   -1,
   -1,   -1,   -1,  345,   -1,  316,  317,  318,  319,  320,
  321,   -1,  323,  324,   -1,   -1,  327,   -1,   -1,   -1,
  331,  332,  333,  334,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  304,  305,   -1,  345,  308,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,  316,  317,  318,  319,  320,  321,   -1,
  323,  324,   -1,   -1,  327,   -1,   -1,   -1,  331,  332,
  333,  334,  304,  305,   -1,   -1,  308,   -1,   -1,   -1,
   -1,   -1,  345,   -1,  316,  317,  318,  319,  320,  321,
   -1,  323,  324,   -1,   -1,  327,   -1,   -1,   -1,  331,
  332,  333,  334,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  304,  305,   -1,  345,  308,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  316,  317,  318,  319,  320,  321,   -1,  323,
  324,   -1,   -1,  327,   -1,  124,   -1,  331,  332,  333,
  334,  304,  305,   -1,   -1,  308,   -1,   -1,   -1,   -1,
   -1,  345,   -1,  316,  317,  318,  319,  320,  321,   -1,
  323,  324,   -1,   -1,  327,   -1,   -1,   -1,  331,  332,
  333,  334,   -1,   37,   38,   -1,   -1,   -1,   42,   43,
   -1,   45,  345,   47,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   60,   -1,   62,   -1,
   -1,   -1,   -1,   -1,   37,   38,   -1,   -1,   -1,   42,
   43,   -1,   45,   -1,   47,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   60,   -1,   62,
   94,   -1,   96,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  124,   94,  126,   96,   -1,   -1,   -1,   -1,  257,  258,
  259,   -1,  261,   -1,   -1,   -1,  265,  266,   -1,   -1,
   -1,  270,   -1,  272,  273,  274,  275,  276,  277,  278,
   -1,  124,   -1,  126,  283,  284,  285,  286,  287,  288,
  289,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  299,   -1,   -1,  302,  303,  304,  305,  306,  307,  308,
  309,  310,  311,   -1,  313,  314,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,  336,   -1,   -1,
  339,  340,   -1,  342,  343,   -1,  345,   -1,  347,  348,
  349,  350,  351,  352,   -1,   -1,   -1,   -1,  357,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  304,  305,   -1,   -1,  308,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  316,  317,  318,  319,  320,  321,   -1,  323,
  324,   -1,   -1,  327,   -1,   -1,   -1,  331,  332,  333,
  334,  304,  305,   -1,   -1,  308,   -1,   -1,   -1,   -1,
   -1,  345,   -1,  316,  317,  318,  319,  320,  321,   -1,
  323,  324,   -1,   -1,  327,   -1,   -1,   -1,  331,  332,
  333,  334,  257,  258,  259,   -1,  261,   -1,   -1,   -1,
  265,  266,  345,   -1,   -1,  270,   -1,  272,  273,  274,
  275,  276,  277,  278,   -1,   -1,   -1,   -1,  283,  284,
  285,  286,  287,  288,  289,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  299,   -1,   -1,  302,  303,  304,
  305,  306,  307,  308,  309,  310,  311,   -1,  313,  314,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  333,   -1,
   -1,  336,   -1,   -1,  339,  340,   -1,  342,  343,   -1,
   -1,   -1,  347,  348,  349,  350,  351,  352,   -1,  257,
  258,  259,  357,  261,   -1,   -1,   -1,  265,  266,   -1,
   -1,   -1,  270,   -1,  272,  273,  274,  275,  276,  277,
  278,   -1,   -1,   -1,   -1,  283,  284,  285,  286,  287,
  288,  289,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  299,   -1,   -1,  302,  303,  304,  305,  306,  307,
  308,  309,  310,  311,   -1,  313,  314,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  336,   -1,
   -1,  339,  340,   -1,  342,  343,   -1,  345,   -1,  347,
  348,  349,  350,  351,  352,   -1,  257,  258,  259,  357,
  261,   -1,   -1,   -1,  265,  266,   -1,   -1,   -1,  270,
   -1,  272,  273,  274,  275,  276,  277,  278,   -1,   -1,
   -1,   -1,  283,  284,  285,  286,  287,  288,  289,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  299,   -1,
   -1,  302,  303,  304,  305,  306,  307,  308,  309,  310,
  311,   -1,  313,  314,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  336,   -1,   -1,  339,  340,
   -1,  342,  343,   -1,  345,   -1,  347,  348,  349,  350,
  351,  352,   -1,  257,  258,  259,  357,  261,   -1,   -1,
   -1,  265,  266,   -1,   -1,   -1,  270,   -1,  272,  273,
  274,  275,  276,  277,  278,   -1,   -1,   -1,   -1,  283,
  284,  285,  286,  287,  288,  289,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  299,   -1,   -1,  302,  303,
  304,  305,  306,  307,  308,  309,  310,  311,   -1,  313,
  314,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  336,   -1,   -1,  339,  340,   -1,  342,  343,
   -1,   -1,   -1,  347,  348,  349,  350,  351,  352,   -1,
  257,  258,  259,  357,  261,   -1,   -1,   -1,  265,  266,
   -1,   -1,   -1,  270,   -1,  272,  273,  274,  275,  276,
  277,  278,   -1,   -1,   -1,   -1,  283,  284,  285,  286,
  287,  288,  289,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  299,   -1,   -1,  302,  303,  304,  305,  306,
  307,  308,  309,  310,  311,   -1,  313,  314,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  336,
   -1,   -1,  339,  340,   -1,  342,  343,   -1,   -1,   -1,
  347,  348,  349,  350,  351,  352,   -1,  257,  258,  259,
  357,  261,   -1,   -1,   -1,  265,  266,   -1,   -1,   -1,
  270,   -1,  272,  273,  274,  275,  276,  277,  278,   -1,
   -1,   -1,   -1,  283,  284,  285,  286,  287,  288,  289,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  299,
   -1,   -1,  302,  303,  304,  305,  306,  307,  308,  309,
  310,  311,   -1,  313,  314,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  336,   -1,   -1,  339,
  340,   -1,  342,  343,   -1,   -1,   -1,  347,  348,  349,
  350,  351,  352,   -1,   -1,   -1,   -1,  357,
};
#define YYFINAL 1
#ifndef YYDEBUG
#define YYDEBUG 0
#endif
#define YYMAXTOKEN 358
#if YYDEBUG
const char * const yyname[] = {
"end-of-file",0,0,0,0,0,0,0,0,0,"'\\n'",0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,"' '","'!'",0,0,0,"'%'","'&'",0,"'('","')'","'*'","'+'","','","'-'","'.'",
"'/'",0,0,0,0,0,0,0,0,0,0,"':'","';'","'<'","'='","'>'","'?'",0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,"'['",0,"']'","'^'",0,"'`'",0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,"'{'","'|'","'}'","'~'",0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
"kCLASS","kMODULE","kDEF","kUNDEF","kBEGIN","kRESCUE","kENSURE","kEND","kIF",
"kUNLESS","kTHEN","kELSIF","kELSE","kCASE","kWHEN","kWHILE","kUNTIL","kFOR",
"kBREAK","kNEXT","kREDO","kRETRY","kIN","kDO","kDO_COND","kDO_BLOCK","kRETURN",
"kYIELD","kSUPER","kSELF","kNIL","kTRUE","kFALSE","kAND","kOR","kNOT","kIF_MOD",
"kUNLESS_MOD","kWHILE_MOD","kUNTIL_MOD","kRESCUE_MOD","kALIAS","kDEFINED",
"klBEGIN","klEND","k__LINE__","k__FILE__","tIDENTIFIER","tFID","tGVAR","tIVAR",
"tCONSTANT","tCVAR","tINTEGER","tFLOAT","tSTRING_CONTENT","tNTH_REF",
"tBACK_REF","tREGEXP_END","tUPLUS","tUMINUS","tPOW","tCMP","tEQ","tEQQ","tNEQ",
"tGEQ","tLEQ","tANDOP","tOROP","tMATCH","tNMATCH","tDOT2","tDOT3","tAREF",
"tASET","tLSHFT","tRSHFT","tCOLON2","tCOLON3","tOP_ASGN","tASSOC","tLPAREN",
"tLPAREN_ARG","tRPAREN","tLBRACK","tLBRACE","tLBRACE_ARG","tSTAR","tAMPER",
"tSYMBEG","tSTRING_BEG","tXSTRING_BEG","tREGEXP_BEG","tWORDS_BEG","tQWORDS_BEG",
"tSTRING_DBEG","tSTRING_DVAR","tSTRING_END","tLOWEST","tUMINUS_NUM",
"tLAST_TOKEN",
};
const char * const yyrule[] = {
"$accept : program",
"$$1 :",
"program : $$1 compstmt",
"bodystmt : compstmt opt_rescue opt_else opt_ensure",
"compstmt : stmts opt_terms",
"stmts : none",
"stmts : stmt",
"stmts : stmts terms stmt",
"stmts : error stmt",
"$$2 :",
"stmt : kALIAS fitem $$2 fitem",
"stmt : kALIAS tGVAR tGVAR",
"stmt : kALIAS tGVAR tBACK_REF",
"stmt : kALIAS tGVAR tNTH_REF",
"stmt : kUNDEF undef_list",
"stmt : stmt kIF_MOD expr_value",
"stmt : stmt kUNLESS_MOD expr_value",
"stmt : stmt kWHILE_MOD expr_value",
"stmt : stmt kUNTIL_MOD expr_value",
"stmt : stmt kRESCUE_MOD stmt",
"$$3 :",
"stmt : klBEGIN $$3 '{' compstmt '}'",
"stmt : klEND '{' compstmt '}'",
"stmt : lhs '=' command_call",
"stmt : mlhs '=' command_call",
"stmt : var_lhs tOP_ASGN command_call",
"stmt : primary_value '[' aref_args ']' tOP_ASGN command_call",
"stmt : primary_value '.' tIDENTIFIER tOP_ASGN command_call",
"stmt : primary_value '.' tCONSTANT tOP_ASGN command_call",
"stmt : primary_value tCOLON2 tIDENTIFIER tOP_ASGN command_call",
"stmt : backref tOP_ASGN command_call",
"stmt : lhs '=' mrhs",
"stmt : mlhs '=' arg_value",
"stmt : mlhs '=' mrhs",
"stmt : expr",
"expr : command_call",
"expr : expr kAND expr",
"expr : expr kOR expr",
"expr : kNOT expr",
"expr : '!' command_call",
"expr : arg",
"expr_value : expr",
"command_call : command",
"command_call : block_command",
"command_call : kRETURN call_args",
"command_call : kBREAK call_args",
"command_call : kNEXT call_args",
"block_command : block_call",
"block_command : block_call '.' operation2 command_args",
"block_command : block_call tCOLON2 operation2 command_args",
"$$4 :",
"$$5 :",
"cmd_brace_block : tLBRACE_ARG $$4 opt_block_var $$5 compstmt '}'",
"command : operation command_args",
"command : operation command_args cmd_brace_block",
"command : primary_value '.' operation2 command_args",
"command : primary_value '.' operation2 command_args cmd_brace_block",
"command : primary_value tCOLON2 operation2 command_args",
"command : primary_value tCOLON2 operation2 command_args cmd_brace_block",
"command : kSUPER command_args",
"command : kYIELD command_args",
"mlhs : mlhs_basic",
"mlhs : tLPAREN mlhs_entry ')'",
"mlhs_entry : mlhs_basic",
"mlhs_entry : tLPAREN mlhs_entry ')'",
"mlhs_basic : mlhs_head",
"mlhs_basic : mlhs_head mlhs_item",
"mlhs_basic : mlhs_head tSTAR mlhs_node",
"mlhs_basic : mlhs_head tSTAR",
"mlhs_basic : tSTAR mlhs_node",
"mlhs_basic : tSTAR",
"mlhs_item : mlhs_node",
"mlhs_item : tLPAREN mlhs_entry ')'",
"mlhs_head : mlhs_item ','",
"mlhs_head : mlhs_head mlhs_item ','",
"mlhs_node : variable",
"mlhs_node : primary_value '[' aref_args ']'",
"mlhs_node : primary_value '.' tIDENTIFIER",
"mlhs_node : primary_value tCOLON2 tIDENTIFIER",
"mlhs_node : primary_value '.' tCONSTANT",
"mlhs_node : primary_value tCOLON2 tCONSTANT",
"mlhs_node : tCOLON3 tCONSTANT",
"mlhs_node : backref",
"lhs : variable",
"lhs : primary_value '[' aref_args ']'",
"lhs : primary_value '.' tIDENTIFIER",
"lhs : primary_value tCOLON2 tIDENTIFIER",
"lhs : primary_value '.' tCONSTANT",
"lhs : primary_value tCOLON2 tCONSTANT",
"lhs : tCOLON3 tCONSTANT",
"lhs : backref",
"cname : tIDENTIFIER",
"cname : tCONSTANT",
"cpath : tCOLON3 cname",
"cpath : cname",
"cpath : primary_value tCOLON2 cname",
"fname : tIDENTIFIER",
"fname : tCONSTANT",
"fname : tFID",
"fname : op",
"fname : reswords",
"fitem : fname",
"fitem : symbol",
"undef_list : fitem",
"$$6 :",
"undef_list : undef_list ',' $$6 fitem",
"op : '|'",
"op : '^'",
"op : '&'",
"op : tCMP",
"op : tEQ",
"op : tEQQ",
"op : tMATCH",
"op : '>'",
"op : tGEQ",
"op : '<'",
"op : tLEQ",
"op : tLSHFT",
"op : tRSHFT",
"op : '+'",
"op : '-'",
"op : '*'",
"op : tSTAR",
"op : '/'",
"op : '%'",
"op : tPOW",
"op : '~'",
"op : tUPLUS",
"op : tUMINUS",
"op : tAREF",
"op : tASET",
"op : '`'",
"reswords : k__LINE__",
"reswords : k__FILE__",
"reswords : klBEGIN",
"reswords : klEND",
"reswords : kALIAS",
"reswords : kAND",
"reswords : kBEGIN",
"reswords : kBREAK",
"reswords : kCASE",
"reswords : kCLASS",
"reswords : kDEF",
"reswords : kDEFINED",
"reswords : kDO",
"reswords : kELSE",
"reswords : kELSIF",
"reswords : kEND",
"reswords : kENSURE",
"reswords : kFALSE",
"reswords : kFOR",
"reswords : kIN",
"reswords : kMODULE",
"reswords : kNEXT",
"reswords : kNIL",
"reswords : kNOT",
"reswords : kOR",
"reswords : kREDO",
"reswords : kRESCUE",
"reswords : kRETRY",
"reswords : kRETURN",
"reswords : kSELF",
"reswords : kSUPER",
"reswords : kTHEN",
"reswords : kTRUE",
"reswords : kUNDEF",
"reswords : kWHEN",
"reswords : kYIELD",
"reswords : kIF_MOD",
"reswords : kUNLESS_MOD",
"reswords : kWHILE_MOD",
"reswords : kUNTIL_MOD",
"reswords : kRESCUE_MOD",
"arg : lhs '=' arg",
"arg : lhs '=' arg kRESCUE_MOD arg",
"arg : var_lhs tOP_ASGN arg",
"arg : primary_value '[' aref_args ']' tOP_ASGN arg",
"arg : primary_value '.' tIDENTIFIER tOP_ASGN arg",
"arg : primary_value '.' tCONSTANT tOP_ASGN arg",
"arg : primary_value tCOLON2 tIDENTIFIER tOP_ASGN arg",
"arg : primary_value tCOLON2 tCONSTANT tOP_ASGN arg",
"arg : tCOLON3 tCONSTANT tOP_ASGN arg",
"arg : backref tOP_ASGN arg",
"arg : arg tDOT2 arg",
"arg : arg tDOT3 arg",
"arg : arg '+' arg",
"arg : arg '-' arg",
"arg : arg '*' arg",
"arg : arg '/' arg",
"arg : arg '%' arg",
"arg : arg tPOW arg",
"arg : tUMINUS_NUM tINTEGER tPOW arg",
"arg : tUMINUS_NUM tFLOAT tPOW arg",
"arg : tUPLUS arg",
"arg : tUMINUS arg",
"arg : arg '|' arg",
"arg : arg '^' arg",
"arg : arg '&' arg",
"arg : arg tCMP arg",
"arg : arg '>' arg",
"arg : arg tGEQ arg",
"arg : arg '<' arg",
"arg : arg tLEQ arg",
"arg : arg tEQ arg",
"arg : arg tEQQ arg",
"arg : arg tNEQ arg",
"arg : arg tMATCH arg",
"arg : arg tNMATCH arg",
"arg : '!' arg",
"arg : '~' arg",
"arg : arg tLSHFT arg",
"arg : arg tRSHFT arg",
"arg : arg tANDOP arg",
"arg : arg tOROP arg",
"$$7 :",
"arg : kDEFINED opt_nl $$7 arg",
"arg : arg '?' arg ':' arg",
"arg : primary",
"arg_value : arg",
"aref_args : none",
"aref_args : command opt_nl",
"aref_args : args trailer",
"aref_args : args ',' tSTAR arg opt_nl",
"aref_args : assocs trailer",
"aref_args : tSTAR arg opt_nl",
"paren_args : '(' none ')'",
"paren_args : '(' call_args opt_nl ')'",
"paren_args : '(' block_call opt_nl ')'",
"paren_args : '(' args ',' block_call opt_nl ')'",
"opt_paren_args : none",
"opt_paren_args : paren_args",
"call_args : command",
"call_args : args opt_block_arg",
"call_args : args ',' tSTAR arg_value opt_block_arg",
"call_args : assocs opt_block_arg",
"call_args : assocs ',' tSTAR arg_value opt_block_arg",
"call_args : args ',' assocs opt_block_arg",
"call_args : args ',' assocs ',' tSTAR arg opt_block_arg",
"call_args : tSTAR arg_value opt_block_arg",
"call_args : block_arg",
"call_args2 : arg_value ',' args opt_block_arg",
"call_args2 : arg_value ',' block_arg",
"call_args2 : arg_value ',' tSTAR arg_value opt_block_arg",
"call_args2 : arg_value ',' args ',' tSTAR arg_value opt_block_arg",
"call_args2 : assocs opt_block_arg",
"call_args2 : assocs ',' tSTAR arg_value opt_block_arg",
"call_args2 : arg_value ',' assocs opt_block_arg",
"call_args2 : arg_value ',' args ',' assocs opt_block_arg",
"call_args2 : arg_value ',' assocs ',' tSTAR arg_value opt_block_arg",
"call_args2 : arg_value ',' args ',' assocs ',' tSTAR arg_value opt_block_arg",
"call_args2 : tSTAR arg_value opt_block_arg",
"call_args2 : block_arg",
"$$8 :",
"command_args : $$8 open_args",
"open_args : call_args",
"$$9 :",
"open_args : tLPAREN_ARG $$9 ')'",
"$$10 :",
"open_args : tLPAREN_ARG call_args2 $$10 ')'",
"block_arg : tAMPER arg_value",
"opt_block_arg : ',' block_arg",
"opt_block_arg : none",
"args : arg_value",
"args : args ',' arg_value",
"mrhs : args ',' arg_value",
"mrhs : args ',' tSTAR arg_value",
"mrhs : tSTAR arg_value",
"primary : literal",
"primary : strings",
"primary : xstring",
"primary : regexp",
"primary : words",
"primary : qwords",
"primary : var_ref",
"primary : backref",
"primary : tFID",
"$$11 :",
"primary : kBEGIN $$11 bodystmt kEND",
"$$12 :",
"primary : tLPAREN_ARG expr $$12 opt_nl ')'",
"primary : tLPAREN compstmt ')'",
"primary : primary_value tCOLON2 tCONSTANT",
"primary : tCOLON3 tCONSTANT",
"primary : primary_value '[' aref_args ']'",
"primary : tLBRACK aref_args ']'",
"primary : tLBRACE assoc_list '}'",
"primary : kRETURN",
"primary : kYIELD '(' call_args ')'",
"primary : kYIELD '(' ')'",
"primary : kYIELD",
"$$13 :",
"primary : kDEFINED opt_nl '(' $$13 expr ')'",
"primary : operation brace_block",
"primary : method_call",
"primary : method_call brace_block",
"primary : kIF expr_value then compstmt if_tail kEND",
"primary : kUNLESS expr_value then compstmt opt_else kEND",
"$$14 :",
"$$15 :",
"primary : kWHILE $$14 expr_value do $$15 compstmt kEND",
"$$16 :",
"$$17 :",
"primary : kUNTIL $$16 expr_value do $$17 compstmt kEND",
"primary : kCASE expr_value opt_terms case_body kEND",
"primary : kCASE expr_value opt_terms kELSE compstmt kEND",
"primary : kCASE opt_terms case_body kEND",
"primary : kCASE opt_terms kELSE compstmt kEND",
"$$18 :",
"$$19 :",
"primary : kFOR block_var kIN $$18 expr_value do $$19 compstmt kEND",
"$$20 :",
"primary : kCLASS cpath superclass $$20 bodystmt kEND",
"$$21 :",
"$$22 :",
"primary : kCLASS tLSHFT expr $$21 term $$22 bodystmt kEND",
"$$23 :",
"primary : kMODULE cpath $$23 bodystmt kEND",
"$$24 :",
"primary : kDEF fname $$24 f_arglist bodystmt kEND",
"$$25 :",
"$$26 :",
"primary : kDEF singleton dot_or_colon $$25 fname $$26 f_arglist bodystmt kEND",
"primary : kBREAK",
"primary : kNEXT",
"primary : kREDO",
"primary : kRETRY",
"primary_value : primary",
"then : term",
"then : ':'",
"then : kTHEN",
"then : term kTHEN",
"do : term",
"do : ':'",
"do : kDO_COND",
"if_tail : opt_else",
"if_tail : kELSIF expr_value then compstmt if_tail",
"opt_else : none",
"opt_else : kELSE compstmt",
"block_var : lhs",
"block_var : mlhs",
"opt_block_var : none",
"opt_block_var : '|' '|'",
"opt_block_var : tOROP",
"opt_block_var : '|' block_var '|'",
"$$27 :",
"$$28 :",
"do_block : kDO_BLOCK $$27 opt_block_var $$28 compstmt kEND",
"block_call : command do_block",
"block_call : block_call '.' operation2 opt_paren_args",
"block_call : block_call tCOLON2 operation2 opt_paren_args",
"method_call : operation paren_args",
"method_call : primary_value '.' operation2 opt_paren_args",
"method_call : primary_value tCOLON2 operation2 paren_args",
"method_call : primary_value tCOLON2 operation3",
"method_call : kSUPER paren_args",
"method_call : kSUPER",
"$$29 :",
"$$30 :",
"brace_block : '{' $$29 opt_block_var $$30 compstmt '}'",
"$$31 :",
"$$32 :",
"brace_block : kDO $$31 opt_block_var $$32 compstmt kEND",
"case_body : kWHEN when_args then compstmt cases",
"when_args : args",
"when_args : args ',' tSTAR arg_value",
"when_args : tSTAR arg_value",
"cases : opt_else",
"cases : case_body",
"opt_rescue : kRESCUE exc_list exc_var then compstmt opt_rescue",
"opt_rescue : none",
"exc_list : arg_value",
"exc_list : mrhs",
"exc_list : none",
"exc_var : tASSOC lhs",
"exc_var : none",
"opt_ensure : kENSURE compstmt",
"opt_ensure : none",
"literal : numeric",
"literal : symbol",
"literal : dsym",
"strings : string",
"string : string1",
"string : string string1",
"string1 : tSTRING_BEG string_contents tSTRING_END",
"xstring : tXSTRING_BEG xstring_contents tSTRING_END",
"regexp : tREGEXP_BEG xstring_contents tREGEXP_END",
"words : tWORDS_BEG ' ' tSTRING_END",
"words : tWORDS_BEG word_list tSTRING_END",
"word_list :",
"word_list : word_list word ' '",
"word : string_content",
"word : word string_content",
"qwords : tQWORDS_BEG ' ' tSTRING_END",
"qwords : tQWORDS_BEG qword_list tSTRING_END",
"qword_list :",
"qword_list : qword_list tSTRING_CONTENT ' '",
"string_contents :",
"string_contents : string_contents string_content",
"xstring_contents :",
"xstring_contents : xstring_contents string_content",
"string_content : tSTRING_CONTENT",
"$$33 :",
"string_content : tSTRING_DVAR $$33 string_dvar",
"$$34 :",
"string_content : tSTRING_DBEG $$34 compstmt '}'",
"string_dvar : tGVAR",
"string_dvar : tIVAR",
"string_dvar : tCVAR",
"string_dvar : backref",
"symbol : tSYMBEG sym",
"sym : fname",
"sym : tIVAR",
"sym : tGVAR",
"sym : tCVAR",
"dsym : tSYMBEG xstring_contents tSTRING_END",
"numeric : tINTEGER",
"numeric : tFLOAT",
"numeric : tUMINUS_NUM tINTEGER",
"numeric : tUMINUS_NUM tFLOAT",
"variable : tIDENTIFIER",
"variable : tIVAR",
"variable : tGVAR",
"variable : tCONSTANT",
"variable : tCVAR",
"variable : kNIL",
"variable : kSELF",
"variable : kTRUE",
"variable : kFALSE",
"variable : k__FILE__",
"variable : k__LINE__",
"var_ref : variable",
"var_lhs : variable",
"backref : tNTH_REF",
"backref : tBACK_REF",
"superclass : term",
"$$35 :",
"superclass : '<' $$35 expr_value term",
"superclass : error term",
"f_arglist : '(' f_args opt_nl ')'",
"f_arglist : f_args term",
"f_args : f_arg ',' f_optarg ',' f_rest_arg opt_f_block_arg",
"f_args : f_arg ',' f_optarg opt_f_block_arg",
"f_args : f_arg ',' f_rest_arg opt_f_block_arg",
"f_args : f_arg opt_f_block_arg",
"f_args : f_optarg ',' f_rest_arg opt_f_block_arg",
"f_args : f_optarg opt_f_block_arg",
"f_args : f_rest_arg opt_f_block_arg",
"f_args : f_block_arg",
"f_args :",
"f_norm_arg : tCONSTANT",
"f_norm_arg : tIVAR",
"f_norm_arg : tGVAR",
"f_norm_arg : tCVAR",
"f_norm_arg : tIDENTIFIER",
"f_arg : f_norm_arg",
"f_arg : f_arg ',' f_norm_arg",
"f_opt : tIDENTIFIER '=' arg_value",
"f_optarg : f_opt",
"f_optarg : f_optarg ',' f_opt",
"restarg_mark : '*'",
"restarg_mark : tSTAR",
"f_rest_arg : restarg_mark tIDENTIFIER",
"f_rest_arg : restarg_mark",
"blkarg_mark : '&'",
"blkarg_mark : tAMPER",
"f_block_arg : blkarg_mark tIDENTIFIER",
"opt_f_block_arg : ',' f_block_arg",
"opt_f_block_arg : none",
"singleton : var_ref",
"$$36 :",
"singleton : '(' $$36 expr opt_nl ')'",
"assoc_list : none",
"assoc_list : assocs trailer",
"assoc_list : args trailer",
"assocs : assoc",
"assocs : assocs ',' assoc",
"assoc : arg_value tASSOC arg_value",
"operation : tIDENTIFIER",
"operation : tCONSTANT",
"operation : tFID",
"operation2 : tIDENTIFIER",
"operation2 : tCONSTANT",
"operation2 : tFID",
"operation2 : op",
"operation3 : tIDENTIFIER",
"operation3 : tFID",
"operation3 : op",
"dot_or_colon : '.'",
"dot_or_colon : tCOLON2",
"opt_terms :",
"opt_terms : terms",
"opt_nl :",
"opt_nl : '\\n'",
"trailer :",
"trailer : '\\n'",
"trailer : ','",
"term : ';'",
"term : '\\n'",
"terms : term",
"terms : terms ';'",
"none :",
};
#endif
#if YYDEBUG
#include <stdio.h>
#endif
#ifdef YYSTACKSIZE
#undef YYMAXDEPTH
#define YYMAXDEPTH YYSTACKSIZE
#else
#ifdef YYMAXDEPTH
#define YYSTACKSIZE YYMAXDEPTH
#else
#define YYSTACKSIZE 10000
#define YYMAXDEPTH 10000
#endif
#endif
#define YYINITSTACKSIZE 200
int yydebug;
int yynerrs;
int yyerrflag;
int yychar;
short *yyssp;
YYSTYPE *yyvsp;
YYSTYPE yyval;
YYSTYPE yylval;
short *yyss;
short *yysslim;
YYSTYPE *yyvs;
int yystacksize;
#line 2432 "parse.y"
#include "regex.h"
#include "util.h"

/* We remove any previous definition of `SIGN_EXTEND_CHAR',
   since ours (we hope) works properly with all combinations of
   machines, compilers, `char' and `unsigned char' argument types.
   (Per Bothner suggested the basic approach.)  */
#undef SIGN_EXTEND_CHAR
#if __STDC__
# define SIGN_EXTEND_CHAR(c) ((signed char)(c))
#else  /* not __STDC__ */
/* As in Harbison and Steele.  */
# define SIGN_EXTEND_CHAR(c) ((((unsigned char)(c)) ^ 128) - 128)
#endif
#define is_identchar(c) (SIGN_EXTEND_CHAR(c)!=-1&&(ISALNUM(c) || (c) == '_' || ismbchar(c)))

static char *tokenbuf = NULL;
static int   tokidx, toksiz = 0;

#define LEAVE_BS 1

static VALUE (*lex_gets)();	/* gets function */
static VALUE lex_input;		/* non-nil if File */
static VALUE lex_lastline;	/* gc protect */
static char *lex_pbeg;
static char *lex_p;
static char *lex_pend;

static int
yyerror(msg)
    const char *msg;
{
    char *p, *pe, *buf;
    int len, i;

    rb_compile_error("%s", msg);
    p = lex_p;
    while (lex_pbeg <= p) {
	if (*p == '\n') break;
	p--;
    }
    p++;

    pe = lex_p;
    while (pe < lex_pend) {
	if (*pe == '\n') break;
	pe++;
    }

    len = pe - p;
    if (len > 4) {
	buf = ALLOCA_N(char, len+2);
	MEMCPY(buf, p, char, len);
	buf[len] = '\0';
	rb_compile_error_append("%s", buf);

	i = lex_p - p;
	p = buf; pe = p + len;

	while (p < pe) {
	    if (*p != '\t') *p = ' ';
	    p++;
	}
	buf[i] = '^';
	buf[i+1] = '\0';
	rb_compile_error_append("%s", buf);
    }

    return 0;
}

static int heredoc_end;
static int command_start = Qtrue;

int ruby_in_compile = 0;
int ruby__end__seen;

static VALUE ruby_debug_lines;

static NODE*
yycompile(f, line)
    char *f;
    int line;
{
    int n;
    NODE *node = 0;
    struct RVarmap *vp, *vars = ruby_dyna_vars;

    ruby_in_compile = 1;
    if (!compile_for_eval && rb_safe_level() == 0 &&
	rb_const_defined(rb_cObject, rb_intern("SCRIPT_LINES__"))) {
	VALUE hash, fname;

	hash = rb_const_get(rb_cObject, rb_intern("SCRIPT_LINES__"));
	if (TYPE(hash) == T_HASH) {
	    fname = rb_str_new2(f);
	    ruby_debug_lines = rb_hash_aref(hash, fname);
	    if (NIL_P(ruby_debug_lines)) {
		ruby_debug_lines = rb_ary_new();
		rb_hash_aset(hash, fname, ruby_debug_lines);
	    }
	}
	if (line > 1) {
	    VALUE str = rb_str_new(0,0);
	    while (line > 1) {
		rb_ary_push(ruby_debug_lines, str);
		line--;
	    }
	}
    }

    ruby__end__seen = 0;
    ruby_eval_tree = 0;
    heredoc_end = 0;
    lex_strterm = 0;
    ruby_current_node = 0;
    ruby_sourcefile = rb_source_filename(f);
    n = yyparse();
    ruby_debug_lines = 0;
    compile_for_eval = 0;
    ruby_in_compile = 0;
    cond_stack = 0;
    cmdarg_stack = 0;
    command_start = 1;		  
    class_nest = 0;
    in_single = 0;
    in_def = 0;
    cur_mid = 0;

    vp = ruby_dyna_vars;
    ruby_dyna_vars = vars;
    lex_strterm = 0;
    while (vp && vp != vars) {
	struct RVarmap *tmp = vp;
	vp = vp->next;
	rb_gc_force_recycle((VALUE)tmp);
    }
    if (n == 0) node = ruby_eval_tree;
    else ruby_eval_tree_begin = 0;
    return node;
}

static int lex_gets_ptr;

static VALUE
lex_get_str(s)
    VALUE s;
{
    char *beg, *end, *pend;

    beg = RSTRING(s)->ptr;
    if (lex_gets_ptr) {
	if (RSTRING(s)->len == lex_gets_ptr) return Qnil;
	beg += lex_gets_ptr;
    }
    pend = RSTRING(s)->ptr + RSTRING(s)->len;
    end = beg;
    while (end < pend) {
	if (*end++ == '\n') break;
    }
    lex_gets_ptr = end - RSTRING(s)->ptr;
    return rb_str_new(beg, end - beg);
}

static VALUE
lex_getline()
{
    VALUE line = (*lex_gets)(lex_input);
    if (ruby_debug_lines && !NIL_P(line)) {
	rb_ary_push(ruby_debug_lines, line);
    }
    return line;
}

NODE*
rb_compile_string(f, s, line)
    const char *f;
    VALUE s;
    int line;
{
    lex_gets = lex_get_str;
    lex_gets_ptr = 0;
    lex_input = s;
    lex_pbeg = lex_p = lex_pend = 0;
    ruby_sourceline = line - 1;
    compile_for_eval = ruby_in_eval;

    return yycompile(f, line);
}

NODE*
rb_compile_cstr(f, s, len, line)
    const char *f, *s;
    int len, line;
{
    return rb_compile_string(f, rb_str_new(s, len), line);
}

NODE*
rb_compile_file(f, file, start)
    const char *f;
    VALUE file;
    int start;
{
    lex_gets = rb_io_gets;
    lex_input = file;
    lex_pbeg = lex_p = lex_pend = 0;
    ruby_sourceline = start - 1;

    return yycompile(f, start);
}

static inline int
nextc()
{
    int c;

    if (lex_p == lex_pend) {
	if (lex_input) {
	    VALUE v = lex_getline();

	    if (NIL_P(v)) return -1;
	    if (heredoc_end > 0) {
		ruby_sourceline = heredoc_end;
		heredoc_end = 0;
	    }
	    ruby_sourceline++;
	    lex_pbeg = lex_p = RSTRING(v)->ptr;
	    lex_pend = lex_p + RSTRING(v)->len;
	    lex_lastline = v;
	}
	else {
	    lex_lastline = 0;
	    return -1;
	}
    }
    c = (unsigned char)*lex_p++;
    if (c == '\r' && lex_p < lex_pend && *lex_p == '\n') {
	lex_p++;
	c = '\n';
    }

    return c;
}

static void
pushback(c)
    int c;
{
    if (c == -1) return;
    lex_p--;
}

#define was_bol() (lex_p == lex_pbeg + 1)
#define peek(c) (lex_p != lex_pend && (c) == *lex_p)

#define tokfix() (tokenbuf[tokidx]='\0')
#define tok() tokenbuf
#define toklen() tokidx
#define toklast() (tokidx>0?tokenbuf[tokidx-1]:0)

static char*
newtok()
{
    tokidx = 0;
    if (!tokenbuf) {
	toksiz = 60;
	tokenbuf = ALLOC_N(char, 60);
    }
    if (toksiz > 4096) {
	toksiz = 60;
	REALLOC_N(tokenbuf, char, 60);
    }
    return tokenbuf;
}

static void
tokadd(c)
    char c;
{
    tokenbuf[tokidx++] = c;
    if (tokidx >= toksiz) {
	toksiz *= 2;
	REALLOC_N(tokenbuf, char, toksiz);
    }
}

static int
read_escape()
{
    int c;

    switch (c = nextc()) {
      case '\\':	/* Backslash */
	return c;

      case 'n':	/* newline */
	return '\n';

      case 't':	/* horizontal tab */
	return '\t';

      case 'r':	/* carriage-return */
	return '\r';

      case 'f':	/* form-feed */
	return '\f';

      case 'v':	/* vertical tab */
	return '\13';

      case 'a':	/* alarm(bell) */
	return '\007';

      case 'e':	/* escape */
	return 033;

      case '0': case '1': case '2': case '3': /* octal constant */
      case '4': case '5': case '6': case '7':
	{
	    int numlen;

	    pushback(c);
	    c = scan_oct(lex_p, 3, &numlen);
	    lex_p += numlen;
	}
	return c;

      case 'x':	/* hex constant */
	{
	    int numlen;

	    c = scan_hex(lex_p, 2, &numlen);
	    if (numlen == 0) {
		yyerror("Invalid escape character syntax");
		return 0;
	    }
	    lex_p += numlen;
	}
	return c;

      case 'b':	/* backspace */
	return '\010';

      case 's':	/* space */
	return ' ';

      case 'M':
	if ((c = nextc()) != '-') {
	    yyerror("Invalid escape character syntax");
	    pushback(c);
	    return '\0';
	}
	if ((c = nextc()) == '\\') {
	    return read_escape() | 0x80;
	}
	else if (c == -1) goto eof;
	else {
	    return ((c & 0xff) | 0x80);
	}

      case 'C':
	if ((c = nextc()) != '-') {
	    yyerror("Invalid escape character syntax");
	    pushback(c);
	    return '\0';
	}
      case 'c':
	if ((c = nextc())== '\\') {
	    c = read_escape();
	}
	else if (c == '?')
	    return 0177;
	else if (c == -1) goto eof;
	return c & 0x9f;

      eof:
      case -1:
        yyerror("Invalid escape character syntax");
	return '\0';

      default:
	return c;
    }
}

static int
tokadd_escape(term)
    int term;
{
    int c;

    switch (c = nextc()) {
      case '\n':
	return 0;		/* just ignore */

      case '0': case '1': case '2': case '3': /* octal constant */
      case '4': case '5': case '6': case '7':
	{
	    int i;

	    tokadd('\\');
	    tokadd(c);
	    for (i=0; i<2; i++) {
		c = nextc();
		if (c == -1) goto eof;
		if (c < '0' || '7' < c) {
		    pushback(c);
		    break;
		}
		tokadd(c);
	    }
	}
	return 0;

      case 'x':	/* hex constant */
	{
	    int numlen;

	    tokadd('\\');
	    tokadd(c);
	    scan_hex(lex_p, 2, &numlen);
	    if (numlen == 0) {
		yyerror("Invalid escape character syntax");
		return -1;
	    }
	    while (numlen--)
		tokadd(nextc());
	}
	return 0;

      case 'M':
	if ((c = nextc()) != '-') {
	    yyerror("Invalid escape character syntax");
	    pushback(c);
	    return 0;
	}
	tokadd('\\'); tokadd('M'); tokadd('-');
	goto escaped;

      case 'C':
	if ((c = nextc()) != '-') {
	    yyerror("Invalid escape character syntax");
	    pushback(c);
	    return 0;
	}
	tokadd('\\'); tokadd('C'); tokadd('-');
	goto escaped;

      case 'c':
	tokadd('\\'); tokadd('c');
      escaped:
	if ((c = nextc()) == '\\') {
	    return tokadd_escape(term);
	}
	else if (c == -1) goto eof;
	tokadd(c);
	return 0;

      eof:
      case -1:
        yyerror("Invalid escape character syntax");
	return -1;

      default:
	if (c != '\\' || c != term)
	    tokadd('\\');
	tokadd(c);
    }
    return 0;
}

static int
regx_options()
{
    char kcode = 0;
    int options = 0;
    int c;

    newtok();
    while (c = nextc(), ISALPHA(c)) {
	switch (c) {
	  case 'i':
	    options |= RE_OPTION_IGNORECASE;
	    break;
	  case 'x':
	    options |= RE_OPTION_EXTENDED;
	    break;
	  case 'm':
	    options |= RE_OPTION_MULTILINE;
	    break;
	  case 'o':
	    options |= RE_OPTION_ONCE;
	    break;
	  case 'n':
	    kcode = 16;
	    break;
	  case 'e':
	    kcode = 32;
	    break;
	  case 's':
	    kcode = 48;
	    break;
	  case 'u':
	    kcode = 64;
	    break;
	  default:
	    tokadd(c);
	    break;
	}
    }
    pushback(c);
    if (toklen()) {
	tokfix();
	rb_compile_error("unknown regexp option%s - %s",
			 toklen() > 1 ? "s" : "", tok());
    }
    return options | kcode;
}

#define STR_FUNC_ESCAPE 0x01
#define STR_FUNC_EXPAND 0x02
#define STR_FUNC_REGEXP 0x04
#define STR_FUNC_QWORDS 0x08
#define STR_FUNC_SYMBOL 0x10
#define STR_FUNC_INDENT 0x20

enum string_type {
    str_squote = (0),
    str_dquote = (STR_FUNC_EXPAND),
    str_xquote = (STR_FUNC_EXPAND),
    str_regexp = (STR_FUNC_REGEXP|STR_FUNC_ESCAPE|STR_FUNC_EXPAND),
    str_sword  = (STR_FUNC_QWORDS),
    str_dword  = (STR_FUNC_QWORDS|STR_FUNC_EXPAND),
    str_ssym   = (STR_FUNC_SYMBOL),
    str_dsym   = (STR_FUNC_SYMBOL|STR_FUNC_EXPAND),
};

static void
dispose_string(str)
    VALUE str;
{
    free(RSTRING(str)->ptr);
    rb_gc_force_recycle(str);
}

static int
tokadd_string(func, term, paren, nest)
    int func, term, paren, *nest;
{
    int c;

    while ((c = nextc()) != -1) {
	if (paren && c == paren) {
	    ++*nest;
	}
	else if (c == term) {
	    if (!nest || !*nest) {
		pushback(c);
		break;
	    }
	    --*nest;
	}
	else if ((func & STR_FUNC_EXPAND) && c == '#' && lex_p < lex_pend) {
	    int c2 = *lex_p;
	    if (c2 == '$' || c2 == '@' || c2 == '{') {
		pushback(c);
		break;
	    }
	}
	else if (c == '\\') {
	    c = nextc();
	    switch (c) {
	      case '\n':
		if (func & STR_FUNC_QWORDS) break;
		if (func & STR_FUNC_EXPAND) continue;
		tokadd('\\');
		break;

	      case '\\':
		if (func & STR_FUNC_ESCAPE) tokadd(c);
		break;

	      default:
		if (func & STR_FUNC_REGEXP) {
		    pushback(c);
		    if (tokadd_escape(term) < 0)
			return -1;
		    continue;
		}
		else if (func & STR_FUNC_EXPAND) {
		    pushback(c);
		    if (func & STR_FUNC_ESCAPE) tokadd('\\');
		    c = read_escape();
		}
		else if ((func & STR_FUNC_QWORDS) && ISSPACE(c)) {
		    /* ignore backslashed spaces in %w */
		}
		else if (c != term && !(paren && c == paren)) {
		    tokadd('\\');
		}
	    }
	}
	else if (ismbchar(c)) {
	    int i, len = mbclen(c)-1;

	    for (i = 0; i < len; i++) {
		tokadd(c);
		c = nextc();
	    }
	}
	else if ((func & STR_FUNC_QWORDS) && ISSPACE(c)) {
	    pushback(c);
	    break;
	}
	if (!c && (func & STR_FUNC_SYMBOL)) {
	    func &= ~STR_FUNC_SYMBOL;
	    rb_compile_error("symbol cannot contain '\\0'");
	    continue;
	}
	tokadd(c);
    }
    return c;
}

#define NEW_STRTERM(func, term, paren) \
	rb_node_newnode(NODE_STRTERM, (func), (term) | ((paren) << (CHAR_BIT * 2)), 0)

static int
parse_string(quote)
    NODE *quote;
{
    int func = quote->nd_func;
    int term = nd_term(quote);
    int paren = nd_paren(quote);
    int c, space = 0;

    if (func == -1) return tSTRING_END;
    c = nextc();
    if ((func & STR_FUNC_QWORDS) && ISSPACE(c)) {
	do {c = nextc();} while (ISSPACE(c));
	space = 1;
    }
    if (c == term && !quote->nd_nest) {
	if (func & STR_FUNC_QWORDS) {
	    quote->nd_func = -1;
	    return ' ';
	}
	if (!(func & STR_FUNC_REGEXP)) return tSTRING_END;
	yylval.num = regx_options();
	return tREGEXP_END;
    }
    if (space) {
	pushback(c);
	return ' ';
    }
    newtok();
    if ((func & STR_FUNC_EXPAND) && c == '#') {
	switch (c = nextc()) {
	  case '$':
	  case '@':
	    pushback(c);
	    return tSTRING_DVAR;
	  case '{':
	    return tSTRING_DBEG;
	}
	tokadd('#');
    }
    pushback(c);
    if (tokadd_string(func, term, paren, &quote->nd_nest) == -1) {
	ruby_sourceline = nd_line(quote);
	rb_compile_error("unterminated string meets end of file");
	return tSTRING_END;
    }

    tokfix();
    yylval.node = NEW_STR(rb_str_new(tok(), toklen()));
    return tSTRING_CONTENT;
}

static int
heredoc_identifier()
{
    int c = nextc(), term, func = 0, len;

    if (c == '-') {
	c = nextc();
	func = STR_FUNC_INDENT;
    }
    switch (c) {
      case '\'':
	func |= str_squote; goto quoted;
      case '"':
	func |= str_dquote; goto quoted;
      case '`':
	func |= str_xquote;
      quoted:
	newtok();
	tokadd(func);
	term = c;
	while ((c = nextc()) != -1 && c != term) {
	    len = mbclen(c);
	    do {tokadd(c);} while (--len > 0 && (c = nextc()) != -1);
	}
	if (c == -1) {
	    rb_compile_error("unterminated here document identifier");
	    return 0;
	}
	break;

      default:
	if (!is_identchar(c)) {
	    pushback(c);
	    if (func & STR_FUNC_INDENT) {
		pushback('-');
	    }
	    return 0;
	}
	newtok();
	term = '"';
	tokadd(func |= str_dquote);
	do {
	    len = mbclen(c);
	    do {tokadd(c);} while (--len > 0 && (c = nextc()) != -1);
	} while ((c = nextc()) != -1 && is_identchar(c));
	pushback(c);
	break;
    }

    tokfix();
    len = lex_p - lex_pbeg;
    lex_p = lex_pend;
    lex_strterm = rb_node_newnode(NODE_HEREDOC,
				  rb_str_new(tok(), toklen()),	/* nd_lit */
				  len,				/* nd_nth */
				  lex_lastline);		/* nd_orig */
    return term == '`' ? tXSTRING_BEG : tSTRING_BEG;
}

static void
heredoc_restore(here)
    NODE *here;
{
    VALUE line = here->nd_orig;
    lex_lastline = line;
    lex_pbeg = RSTRING(line)->ptr;
    lex_pend = lex_pbeg + RSTRING(line)->len;
    lex_p = lex_pbeg + here->nd_nth;
    heredoc_end = ruby_sourceline;
    ruby_sourceline = nd_line(here);
    dispose_string(here->nd_lit);
    rb_gc_force_recycle((VALUE)here);
}

static int
whole_match_p(eos, len, indent)
    char *eos;
    int len, indent;
{
    char *p = lex_pbeg;
    int n;

    if (indent) {
	while (*p && ISSPACE(*p)) p++;
    }
    n= lex_pend - (p + len);
    if (n < 0 || (n > 0 && p[len] != '\n' && p[len] != '\r')) return Qfalse;
    if (strncmp(eos, p, len) == 0) return Qtrue;
    return Qfalse;
}

static int
here_document(here)
    NODE *here;
{
    int c, func, indent = 0;
    char *eos, *p, *pend;
    long len;
    VALUE str = 0;

    eos = RSTRING(here->nd_lit)->ptr;
    len = RSTRING(here->nd_lit)->len - 1;
    indent = (func = *eos++) & STR_FUNC_INDENT;

    if ((c = nextc()) == -1) {
      error:
	rb_compile_error("can't find string \"%s\" anywhere before EOF", eos);
	heredoc_restore(lex_strterm);
	lex_strterm = 0;
	return 0;
    }
    if (was_bol() && whole_match_p(eos, len, indent)) {
	heredoc_restore(lex_strterm);
	return tSTRING_END;
    }

    if (!(func & STR_FUNC_EXPAND)) {
	do {
	    p = RSTRING(lex_lastline)->ptr;
	    pend = lex_pend;
	    if (pend > p) {
		switch (pend[-1]) {
		  case '\n':
		    if (--pend == p || pend[-1] != '\r') {
			pend++;
			break;
		    }
		  case '\r':
		    --pend;
		}
	    }
	    if (str)
		rb_str_cat(str, p, pend - p);
	    else
		str = rb_str_new(p, pend - p);
	    if (pend < lex_pend) rb_str_cat(str, "\n", 1);
	    lex_p = lex_pend;
	    if (nextc() == -1) {
		if (str) dispose_string(str);
		goto error;
	    }
	} while (!whole_match_p(eos, len, indent));
    }
    else {
	newtok();
	if (c == '#') {
	    switch (c = nextc()) {
	      case '$':
	      case '@':
		pushback(c);
		return tSTRING_DVAR;
	      case '{':
		return tSTRING_DBEG;
	    }
	    tokadd('#');
	}
	do {
	    pushback(c);
	    if ((c = tokadd_string(func, '\n', 0, NULL)) == -1) goto error;
	    if (c != '\n') {
		yylval.node = NEW_STR(rb_str_new(tok(), toklen()));
		return tSTRING_CONTENT;
	    }
	    tokadd(nextc());
	    if ((c = nextc()) == -1) goto error;
	} while (!whole_match_p(eos, len, indent));
	str = rb_str_new(tok(), toklen());
    }
    heredoc_restore(lex_strterm);
    lex_strterm = NEW_STRTERM(-1, 0, 0);
    yylval.node = NEW_STR(str);
    return tSTRING_CONTENT;
}

#include "lex.c"

static void
arg_ambiguous()
{
    rb_warning("ambiguous first argument; put parentheses or even spaces");
}

#define IS_ARG() (lex_state == EXPR_ARG || lex_state == EXPR_CMDARG)

static int
yylex()
{
    register int c;
    int space_seen = 0;
    int cmd_state;

    if (lex_strterm) {
	int token;
	if (nd_type(lex_strterm) == NODE_HEREDOC) {
	    token = here_document(lex_strterm);
	    if (token == tSTRING_END) {
		lex_strterm = 0;
		lex_state = EXPR_END;
	    }
	}
	else {
	    token = parse_string(lex_strterm);
	    if (token == tSTRING_END || token == tREGEXP_END) {
		rb_gc_force_recycle((VALUE)lex_strterm);
		lex_strterm = 0;
		lex_state = EXPR_END;
	    }
	}
	return token;
    }
    cmd_state = command_start;
    command_start = Qfalse;
  retry:
    switch (c = nextc()) {
      case '\0':		/* NUL */
      case '\004':		/* ^D */
      case '\032':		/* ^Z */
      case -1:			/* end of script. */
	return 0;

	/* white spaces */
      case ' ': case '\t': case '\f': case '\r':
      case '\13': /* '\v' */
	space_seen++;
	goto retry;

      case '#':		/* it's a comment */
	while ((c = nextc()) != '\n') {
	    if (c == -1)
		return 0;
	}
	/* fall through */
      case '\n':
	switch (lex_state) {
	  case EXPR_BEG:
	  case EXPR_FNAME:
	  case EXPR_DOT:
	  case EXPR_CLASS:
	    goto retry;
	  default:
	    break;
	}
	command_start = Qtrue;
	lex_state = EXPR_BEG;
	return '\n';

      case '*':
	if ((c = nextc()) == '*') {
	    if ((c = nextc()) == '=') {
		yylval.id = tPOW;
		lex_state = EXPR_BEG;
		return tOP_ASGN;
	    }
	    pushback(c);
	    c = tPOW;
	}
	else {
	    if (c == '=') {
		yylval.id = '*';
		lex_state = EXPR_BEG;
		return tOP_ASGN;
	    }
	    pushback(c);
	    if (IS_ARG() && space_seen && !ISSPACE(c)){
		rb_warning("`*' interpreted as argument prefix");
		c = tSTAR;
	    }
	    else if (lex_state == EXPR_BEG || lex_state == EXPR_MID) {
		c = tSTAR;
	    }
	    else {
		c = '*';
	    }
	}
	switch (lex_state) {
	  case EXPR_FNAME: case EXPR_DOT:
	    lex_state = EXPR_ARG; break;
	  default:
	    lex_state = EXPR_BEG; break;
	}
	return c;

      case '!':
	lex_state = EXPR_BEG;
	if ((c = nextc()) == '=') {
	    return tNEQ;
	}
	if (c == '~') {
	    return tNMATCH;
	}
	pushback(c);
	return '!';

      case '=':
	if (was_bol()) {
	    /* skip embedded rd document */
	    if (strncmp(lex_p, "begin", 5) == 0 && ISSPACE(lex_p[5])) {
		for (;;) {
		    lex_p = lex_pend;
		    c = nextc();
		    if (c == -1) {
			rb_compile_error("embedded document meets end of file");
			return 0;
		    }
		    if (c != '=') continue;
		    if (strncmp(lex_p, "end", 3) == 0 &&
			(lex_p + 3 == lex_pend || ISSPACE(lex_p[3]))) {
			break;
		    }
		}
		lex_p = lex_pend;
		goto retry;
	    }
	}

	switch (lex_state) {
	  case EXPR_FNAME: case EXPR_DOT:
	    lex_state = EXPR_ARG; break;
	  default:
	    lex_state = EXPR_BEG; break;
	}
	if ((c = nextc()) == '=') {
	    if ((c = nextc()) == '=') {
		return tEQQ;
	    }
	    pushback(c);
	    return tEQ;
	}
	if (c == '~') {
	    return tMATCH;
	}
	else if (c == '>') {
	    return tASSOC;
	}
	pushback(c);
	return '=';

      case '<':
	c = nextc();
	if (c == '<' &&
	    lex_state != EXPR_END &&
	    lex_state != EXPR_DOT &&
	    lex_state != EXPR_ENDARG && 
	    lex_state != EXPR_CLASS &&
	    (!IS_ARG() || space_seen)) {
	    int token = heredoc_identifier();
	    if (token) return token;
	}
	switch (lex_state) {
	  case EXPR_FNAME: case EXPR_DOT:
	    lex_state = EXPR_ARG; break;
	  default:
	    lex_state = EXPR_BEG; break;
	}
	if (c == '=') {
	    if ((c = nextc()) == '>') {
		return tCMP;
	    }
	    pushback(c);
	    return tLEQ;
	}
	if (c == '<') {
	    if ((c = nextc()) == '=') {
		yylval.id = tLSHFT;
		lex_state = EXPR_BEG;
		return tOP_ASGN;
	    }
	    pushback(c);
	    return tLSHFT;
	}
	pushback(c);
	return '<';

      case '>':
	switch (lex_state) {
	  case EXPR_FNAME: case EXPR_DOT:
	    lex_state = EXPR_ARG; break;
	  default:
	    lex_state = EXPR_BEG; break;
	}
	if ((c = nextc()) == '=') {
	    return tGEQ;
	}
	if (c == '>') {
	    if ((c = nextc()) == '=') {
		yylval.id = tRSHFT;
		lex_state = EXPR_BEG;
		return tOP_ASGN;
	    }
	    pushback(c);
	    return tRSHFT;
	}
	pushback(c);
	return '>';

      case '"':
	lex_strterm = NEW_STRTERM(str_dquote, '"', 0);
	return tSTRING_BEG;

      case '`':
	if (lex_state == EXPR_FNAME) {
	    lex_state = EXPR_END;
	    return c;
	}
	if (lex_state == EXPR_DOT) {
	    if (cmd_state)
		lex_state = EXPR_CMDARG;
	    else
		lex_state = EXPR_ARG;
	    return c;
	}
	lex_strterm = NEW_STRTERM(str_xquote, '`', 0);
	return tXSTRING_BEG;

      case '\'':
	lex_strterm = NEW_STRTERM(str_squote, '\'', 0);
	return tSTRING_BEG;

      case '?':
	if (lex_state == EXPR_END || lex_state == EXPR_ENDARG) {
	    lex_state = EXPR_BEG;
	    return '?';
	}
	c = nextc();
	if (c == -1) {
	    rb_compile_error("incomplete character syntax");
	    return 0;
	}
	if (ISSPACE(c)){
	    if (!IS_ARG()){
		int c2 = 0;
		switch (c) {
		  case ' ':
		    c2 = 's';
		    break;
		  case '\n':
		    c2 = 'n';
		    break;
		  case '\t':
		    c2 = 't';
		    break;
		  case '\v':
		    c2 = 'v';
		    break;
		  case '\r':
		    c2 = 'r';
		    break;
		  case '\f':
		    c2 = 'f';
		    break;
		}
		if (c2) {
		    rb_warn("invalid character syntax; use ?\\%c", c2);
		}
	    }
	  ternary:
	    pushback(c);
	    lex_state = EXPR_BEG;
	    return '?';
	}
	else if (ismbchar(c)) {
	    rb_warn("multibyte character literal not supported yet; use ?\\%.3o", c);
	    goto ternary;
	}
	else if ((ISALNUM(c) || c == '_') && lex_p < lex_pend && is_identchar(*lex_p)) {
	    goto ternary;
	}
	else if (c == '\\') {
	    c = read_escape();
	}
	c &= 0xff;
	lex_state = EXPR_END;
	yylval.node = NEW_LIT(INT2FIX(c));
	return tINTEGER;

      case '&':
	if ((c = nextc()) == '&') {
	    lex_state = EXPR_BEG;
	    if ((c = nextc()) == '=') {
		yylval.id = tANDOP;
		lex_state = EXPR_BEG;
		return tOP_ASGN;
	    }
	    pushback(c);
	    return tANDOP;
	}
	else if (c == '=') {
	    yylval.id = '&';
	    lex_state = EXPR_BEG;
	    return tOP_ASGN;
	}
	pushback(c);
	if (IS_ARG() && space_seen && !ISSPACE(c)){
	    rb_warning("`&' interpreted as argument prefix");
	    c = tAMPER;
	}
	else if (lex_state == EXPR_BEG || lex_state == EXPR_MID) {
	    c = tAMPER;
	}
	else {
	    c = '&';
	}
	switch (lex_state) {
	  case EXPR_FNAME: case EXPR_DOT:
	    lex_state = EXPR_ARG; break;
	  default:
	    lex_state = EXPR_BEG;
	}
	return c;

      case '|':
	if ((c = nextc()) == '|') {
	    lex_state = EXPR_BEG;
	    if ((c = nextc()) == '=') {
		yylval.id = tOROP;
		lex_state = EXPR_BEG;
		return tOP_ASGN;
	    }
	    pushback(c);
	    return tOROP;
	}
	if (c == '=') {
	    yylval.id = '|';
	    lex_state = EXPR_BEG;
	    return tOP_ASGN;
	}
	if (lex_state == EXPR_FNAME || lex_state == EXPR_DOT) {
	    lex_state = EXPR_ARG;
	}
	else {
	    lex_state = EXPR_BEG;
	}
	pushback(c);
	return '|';

      case '+':
	c = nextc();
	if (lex_state == EXPR_FNAME || lex_state == EXPR_DOT) {
	    lex_state = EXPR_ARG;
	    if (c == '@') {
		return tUPLUS;
	    }
	    pushback(c);
	    return '+';
	}
	if (c == '=') {
	    yylval.id = '+';
	    lex_state = EXPR_BEG;
	    return tOP_ASGN;
	}
	if (lex_state == EXPR_BEG || lex_state == EXPR_MID ||
	    (IS_ARG() && space_seen && !ISSPACE(c))) {
	    if (IS_ARG()) arg_ambiguous();
	    lex_state = EXPR_BEG;
	    pushback(c);
	    if (ISDIGIT(c)) {
		c = '+';
		goto start_num;
	    }
	    return tUPLUS;
	}
	lex_state = EXPR_BEG;
	pushback(c);
	return '+';

      case '-':
	c = nextc();
	if (lex_state == EXPR_FNAME || lex_state == EXPR_DOT) {
	    lex_state = EXPR_ARG;
	    if (c == '@') {
		return tUMINUS;
	    }
	    pushback(c);
	    return '-';
	}
	if (c == '=') {
	    yylval.id = '-';
	    lex_state = EXPR_BEG;
	    return tOP_ASGN;
	}
	if (lex_state == EXPR_BEG || lex_state == EXPR_MID ||
	    (IS_ARG() && space_seen && !ISSPACE(c))) {
	    if (IS_ARG()) arg_ambiguous();
	    lex_state = EXPR_BEG;
	    pushback(c);
	    if (ISDIGIT(c)) {
		return tUMINUS_NUM;
	    }
	    return tUMINUS;
	}
	lex_state = EXPR_BEG;
	pushback(c);
	return '-';

      case '.':
	lex_state = EXPR_BEG;
	if ((c = nextc()) == '.') {
	    if ((c = nextc()) == '.') {
		return tDOT3;
	    }
	    pushback(c);
	    return tDOT2;
	}
	pushback(c);
	if (ISDIGIT(c)) {
	    yyerror("no .<digit> floating literal anymore; put 0 before dot");
	}
	lex_state = EXPR_DOT;
	return '.';

      start_num:
      case '0': case '1': case '2': case '3': case '4':
      case '5': case '6': case '7': case '8': case '9':
	{
	    int is_float, seen_point, seen_e, nondigit;

	    is_float = seen_point = seen_e = nondigit = 0;
	    lex_state = EXPR_END;
	    newtok();
	    if (c == '-' || c == '+') {
		tokadd(c);
		c = nextc();
	    }
	    if (c == '0') {
		int start = toklen();
		c = nextc();
		if (c == 'x' || c == 'X') {
		    /* hexadecimal */
		    c = nextc();
		    if (ISXDIGIT(c)) {
			do {
			    if (c == '_') {
				if (nondigit) break;
				nondigit = c;
				continue;
			    }
			    if (!ISXDIGIT(c)) break;
			    nondigit = 0;
			    tokadd(c);
			} while ((c = nextc()) != -1);
		    }
		    pushback(c);
		    tokfix();
		    if (toklen() == start) {
			yyerror("numeric literal without digits");
		    }
		    else if (nondigit) goto trailing_uc;
		    yylval.node = NEW_LIT(rb_cstr_to_inum(tok(), 16, Qfalse));
		    return tINTEGER;
		}
		if (c == 'b' || c == 'B') {
		    /* binary */
		    c = nextc();
		    if (c == '0' || c == '1') {
			do {
			    if (c == '_') {
				if (nondigit) break;
				nondigit = c;
				continue;
			    }
			    if (c != '0' && c != '1') break;
			    nondigit = 0;
			    tokadd(c);
			} while ((c = nextc()) != -1);
		    }
		    pushback(c);
		    tokfix();
		    if (toklen() == start) {
			yyerror("numeric literal without digits");
		    }
		    else if (nondigit) goto trailing_uc;
		    yylval.node = NEW_LIT(rb_cstr_to_inum(tok(), 2, Qfalse));
		    return tINTEGER;
		}
		if (c == 'd' || c == 'D') {
		    /* decimal */
		    c = nextc();
		    if (ISDIGIT(c)) {
			do {
			    if (c == '_') {
				if (nondigit) break;
				nondigit = c;
				continue;
			    }
			    if (!ISDIGIT(c)) break;
			    nondigit = 0;
			    tokadd(c);
			} while ((c = nextc()) != -1);
		    }
		    pushback(c);
		    tokfix();
		    if (toklen() == start) {
			yyerror("numeric literal without digits");
		    }
		    else if (nondigit) goto trailing_uc;
		    yylval.node = NEW_LIT(rb_cstr_to_inum(tok(), 10, Qfalse));
		    return tINTEGER;
		}
		if (c == '_') {
		    /* 0_0 */
		    goto octal_number;
		}
		if (c == 'o' || c == 'O') {
		    /* prefixed octal */
		    c = nextc();
		    if (c == '_') {
			yyerror("numeric literal without digits");
		    }
		}
		if (c >= '0' && c <= '7') {
		    /* octal */
		  octal_number:
	            do {
			if (c == '_') {
			    if (nondigit) break;
			    nondigit = c;
			    continue;
			}
			if (c < '0' || c > '7') break;
			nondigit = 0;
			tokadd(c);
		    } while ((c = nextc()) != -1);
		    if (toklen() > start) {
			pushback(c);
			tokfix();
			if (nondigit) goto trailing_uc;
			yylval.node = NEW_LIT(rb_cstr_to_inum(tok(), 8, Qfalse));
			return tINTEGER;
		    }
		    if (nondigit) {
			pushback(c);
			goto trailing_uc;
		    }
		}
		if (c > '7' && c <= '9') {
		    yyerror("Illegal octal digit");
		}
		else if (c == '.' || c == 'e' || c == 'E') {
		    tokadd('0');
		}
		else {
		    pushback(c);
		    yylval.node = NEW_LIT(INT2FIX(0));
		    return tINTEGER;
		}
	    }

	    for (;;) {
		switch (c) {
		  case '0': case '1': case '2': case '3': case '4':
		  case '5': case '6': case '7': case '8': case '9':
		    nondigit = 0;
		    tokadd(c);
		    break;

		  case '.':
		    if (nondigit) goto trailing_uc;
		    if (seen_point || seen_e) {
			goto decode_num;
		    }
		    else {
			int c0 = nextc();
			if (!ISDIGIT(c0)) {
			    pushback(c0);
			    goto decode_num;
			}
			c = c0;
		    }
		    tokadd('.');
		    tokadd(c);
		    is_float++;
		    seen_point++;
		    nondigit = 0;
		    break;

		  case 'e':
		  case 'E':
		    if (nondigit) {
			pushback(c);
			c = nondigit;
			goto decode_num;
		    }
		    if (seen_e) {
			goto decode_num;
		    }
		    tokadd(c);
		    seen_e++;
		    is_float++;
		    nondigit = c;
		    c = nextc();
		    if (c != '-' && c != '+') continue;
		    tokadd(c);
		    nondigit = c;
		    break;

		  case '_':	/* `_' in number just ignored */
		    if (nondigit) goto decode_num;
		    nondigit = c;
		    break;

		  default:
		    goto decode_num;
		}
		c = nextc();
	    }

	  decode_num:
	    pushback(c);
	    tokfix();
	    if (nondigit) {
		char tmp[30];
	      trailing_uc:
		sprintf(tmp, "trailing `%c' in number", nondigit);
		yyerror(tmp);
	    }
	    if (is_float) {
		double d = strtod(tok(), 0);
		if (errno == ERANGE) {
		    rb_warn("Float %s out of range", tok());
		    errno = 0;
		}
		yylval.node = NEW_LIT(rb_float_new(d));
		return tFLOAT;
	    }
	    yylval.node = NEW_LIT(rb_cstr_to_inum(tok(), 10, Qfalse));
	    return tINTEGER;
	}

      case ']':
      case '}':
      case ')':
	COND_LEXPOP();
	CMDARG_LEXPOP();
	lex_state = EXPR_END;
	return c;

      case ':':
	c = nextc();
	if (c == ':') {
	    if (lex_state == EXPR_BEG ||  lex_state == EXPR_MID ||
		lex_state == EXPR_CLASS || (IS_ARG() && space_seen)) {
		lex_state = EXPR_BEG;
		return tCOLON3;
	    }
	    lex_state = EXPR_DOT;
	    return tCOLON2;
	}
	if (lex_state == EXPR_END || lex_state == EXPR_ENDARG || ISSPACE(c)) {
	    pushback(c);
	    lex_state = EXPR_BEG;
	    return ':';
	}
	switch (c) {
	  case '\'':
	    lex_strterm = NEW_STRTERM(str_ssym, c, 0);
	    break;
	  case '"':
	    lex_strterm = NEW_STRTERM(str_dsym, c, 0);
	    break;
	  default:
	    pushback(c);
	    break;
	}
	lex_state = EXPR_FNAME;
	return tSYMBEG;

      case '/':
	if (lex_state == EXPR_BEG || lex_state == EXPR_MID) {
	    lex_strterm = NEW_STRTERM(str_regexp, '/', 0);
	    return tREGEXP_BEG;
	}
	if ((c = nextc()) == '=') {
	    yylval.id = '/';
	    lex_state = EXPR_BEG;
	    return tOP_ASGN;
	}
	pushback(c);
	if (IS_ARG() && space_seen) {
	    if (!ISSPACE(c)) {
		arg_ambiguous();
		lex_strterm = NEW_STRTERM(str_regexp, '/', 0);
		return tREGEXP_BEG;
	    }
	}
	switch (lex_state) {
	  case EXPR_FNAME: case EXPR_DOT:
	    lex_state = EXPR_ARG; break;
	  default:
	    lex_state = EXPR_BEG; break;
	}
	return '/';

      case '^':
	if ((c = nextc()) == '=') {
	    yylval.id = '^';
	    lex_state = EXPR_BEG;
	    return tOP_ASGN;
	}
	switch (lex_state) {
	  case EXPR_FNAME: case EXPR_DOT:
	    lex_state = EXPR_ARG; break;
	  default:
	    lex_state = EXPR_BEG; break;
	}
	pushback(c);
	return '^';

      case ';':
	command_start = Qtrue;
      case ',':
	lex_state = EXPR_BEG;
	return c;

      case '~':
	if (lex_state == EXPR_FNAME || lex_state == EXPR_DOT) {
	    if ((c = nextc()) != '@') {
		pushback(c);
	    }
	}
	switch (lex_state) {
	  case EXPR_FNAME: case EXPR_DOT:
	    lex_state = EXPR_ARG; break;
	  default:
	    lex_state = EXPR_BEG; break;
	}
	return '~';

      case '(':
	command_start = Qtrue;
	if (lex_state == EXPR_BEG || lex_state == EXPR_MID) {
	    c = tLPAREN;
	}
	else if (space_seen) {
	    if (lex_state == EXPR_CMDARG) {
		c = tLPAREN_ARG;
	    }
	    else if (lex_state == EXPR_ARG) {
		rb_warn("don't put space before argument parentheses");
		c = '(';
	    }
	}
	COND_PUSH(0);
	CMDARG_PUSH(0);
	lex_state = EXPR_BEG;
	return c;

      case '[':
	if (lex_state == EXPR_FNAME || lex_state == EXPR_DOT) {
	    lex_state = EXPR_ARG;
	    if ((c = nextc()) == ']') {
		if ((c = nextc()) == '=') {
		    return tASET;
		}
		pushback(c);
		return tAREF;
	    }
	    pushback(c);
	    return '[';
	}
	else if (lex_state == EXPR_BEG || lex_state == EXPR_MID) {
	    c = tLBRACK;
	}
	else if (IS_ARG() && space_seen) {
	    c = tLBRACK;
	}
	lex_state = EXPR_BEG;
	COND_PUSH(0);
	CMDARG_PUSH(0);
	return c;

      case '{':
	if (IS_ARG() || lex_state == EXPR_END)
	    c = '{';          /* block (primary) */
	else if (lex_state == EXPR_ENDARG)
	    c = tLBRACE_ARG;  /* block (expr) */
	else
	    c = tLBRACE;      /* hash */
	COND_PUSH(0);
	CMDARG_PUSH(0);
	lex_state = EXPR_BEG;
	return c;

      case '\\':
	c = nextc();
	if (c == '\n') {
	    space_seen = 1;
	    goto retry; /* skip \\n */
	}
	pushback(c);
	return '\\';

      case '%':
	if (lex_state == EXPR_BEG || lex_state == EXPR_MID) {
	    int term;
	    int paren;

	    c = nextc();
	  quotation:
	    if (!ISALNUM(c)) {
		term = c;
		c = 'Q';
	    }
	    else {
		term = nextc();
		if (ISALNUM(term) || ismbchar(term)) {
		    yyerror("unknown type of %string");
		    return 0;
		}
	    }
	    if (c == -1 || term == -1) {
		rb_compile_error("unterminated quoted string meets end of file");
		return 0;
	    }
	    paren = term;
	    if (term == '(') term = ')';
	    else if (term == '[') term = ']';
	    else if (term == '{') term = '}';
	    else if (term == '<') term = '>';
	    else paren = 0;

	    switch (c) {
	      case 'Q':
		lex_strterm = NEW_STRTERM(str_dquote, term, paren);
		return tSTRING_BEG;

	      case 'q':
		lex_strterm = NEW_STRTERM(str_squote, term, paren);
		return tSTRING_BEG;

	      case 'W':
		lex_strterm = NEW_STRTERM(str_dquote | STR_FUNC_QWORDS, term, paren);
		do {c = nextc();} while (ISSPACE(c));
		pushback(c);
		return tWORDS_BEG;

	      case 'w':
		lex_strterm = NEW_STRTERM(str_squote | STR_FUNC_QWORDS, term, paren);
		do {c = nextc();} while (ISSPACE(c));
		pushback(c);
		return tQWORDS_BEG;

	      case 'x':
		lex_strterm = NEW_STRTERM(str_xquote, term, paren);
		return tXSTRING_BEG;

	      case 'r':
		lex_strterm = NEW_STRTERM(str_regexp, term, paren);
		return tREGEXP_BEG;

	      case 's':
		lex_strterm = NEW_STRTERM(str_ssym, term, paren);
		lex_state = EXPR_FNAME;
		return tSYMBEG;

	      default:
		yyerror("unknown type of %string");
		return 0;
	    }
	}
	if ((c = nextc()) == '=') {
	    yylval.id = '%';
	    lex_state = EXPR_BEG;
	    return tOP_ASGN;
	}
	if (IS_ARG() && space_seen && !ISSPACE(c)) {
	    goto quotation;
	}
	switch (lex_state) {
	  case EXPR_FNAME: case EXPR_DOT:
	    lex_state = EXPR_ARG; break;
	  default:
	    lex_state = EXPR_BEG; break;
	}
	pushback(c);
	return '%';

      case '$':
	lex_state = EXPR_END;
	newtok();
	c = nextc();
	switch (c) {
	  case '_':		/* $_: last read line string */
	    c = nextc();
	    if (is_identchar(c)) {
		tokadd('$');
		tokadd('_');
		break;
	    }
	    pushback(c);
	    c = '_';
	    /* fall through */
	  case '~':		/* $~: match-data */
	    local_cnt(c);
	    /* fall through */
	  case '*':		/* $*: argv */
	  case '$':		/* $$: pid */
	  case '?':		/* $?: last status */
	  case '!':		/* $!: error string */
	  case '@':		/* $@: error position */
	  case '/':		/* $/: input record separator */
	  case '\\':		/* $\: output record separator */
	  case ';':		/* $;: field separator */
	  case ',':		/* $,: output field separator */
	  case '.':		/* $.: last read line number */
	  case '=':		/* $=: ignorecase */
	  case ':':		/* $:: load path */
	  case '<':		/* $<: reading filename */
	  case '>':		/* $>: default output handle */
	  case '\"':		/* $": already loaded files */
	    tokadd('$');
	    tokadd(c);
	    tokfix();
	    yylval.id = rb_intern(tok());
	    return tGVAR;

	  case '-':
	    tokadd('$');
	    tokadd(c);
	    c = nextc();
	    tokadd(c);
	    tokfix();
	    yylval.id = rb_intern(tok());
	    /* xxx shouldn't check if valid option variable */
	    return tGVAR;

	  case '&':		/* $&: last match */
	  case '`':		/* $`: string before last match */
	  case '\'':		/* $': string after last match */
	  case '+':		/* $+: string matches last paren. */
	    yylval.node = NEW_BACK_REF(c);
	    return tBACK_REF;

	  case '1': case '2': case '3':
	  case '4': case '5': case '6':
	  case '7': case '8': case '9':
	    tokadd('$');
	    do {
		tokadd(c);
		c = nextc();
	    } while (ISDIGIT(c));
	    pushback(c);
	    tokfix();
	    yylval.node = NEW_NTH_REF(atoi(tok()+1));
	    return tNTH_REF;

	  default:
	    if (!is_identchar(c)) {
		pushback(c);
		return '$';
	    }
	  case '0':
	    tokadd('$');
	}
	break;

      case '@':
	c = nextc();
	newtok();
	tokadd('@');
	if (c == '@') {
	    tokadd('@');
	    c = nextc();
	}
	if (ISDIGIT(c)) {
	    if (tokidx == 1) {
		rb_compile_error("`@%c' is not allowed as an instance variable name", c);
	    }
	    else {
		rb_compile_error("`@@%c' is not allowed as a class variable name", c);
	    }
	}
	if (!is_identchar(c)) {
	    pushback(c);
	    return '@';
	}
	break;

      case '_':
	if (was_bol() && whole_match_p("__END__", 7, 0)) {
	    ruby__end__seen = 1;
	    lex_lastline = 0;
	    return -1;
	}
	newtok();
	break;

      default:
	if (!is_identchar(c)) {
	    rb_compile_error("Invalid char `\\%03o' in expression", c);
	    goto retry;
	}

	newtok();
	break;
    }

    do {
	tokadd(c);
	if (ismbchar(c)) {
	    int i, len = mbclen(c)-1;

	    for (i = 0; i < len; i++) {
		c = nextc();
		tokadd(c);
	    }
	}
	c = nextc();
    } while (is_identchar(c));
    if ((c == '!' || c == '?') && is_identchar(tok()[0]) && !peek('=')) {
	tokadd(c);
    }
    else {
	pushback(c);
    }
    tokfix();

    {
	int result = 0;

	switch (tok()[0]) {
	  case '$':
	    lex_state = EXPR_END;
	    result = tGVAR;
	    break;
	  case '@':
	    lex_state = EXPR_END;
	    if (tok()[1] == '@')
		result = tCVAR;
	    else
		result = tIVAR;
	    break;

	  default:
	    if (toklast() == '!' || toklast() == '?') {
		result = tFID;
	    }
	    else {
		if (lex_state == EXPR_FNAME) {
		    if ((c = nextc()) == '=' && !peek('~') && !peek('>') &&
			(!peek('=') || (lex_p + 1 < lex_pend && lex_p[1] == '>'))) {
			result = tIDENTIFIER;
			tokadd(c);
			tokfix();
		    }
		    else {
			pushback(c);
		    }
		}
		if (result == 0 && ISUPPER(tok()[0])) {
		    result = tCONSTANT;
		}
		else {
		    result = tIDENTIFIER;
		}
	    }

	    if (lex_state != EXPR_DOT) {
		struct kwtable *kw;

		/* See if it is a reserved word.  */
		kw = rb_reserved_word(tok(), toklen());
		if (kw) {
		    enum lex_state state = lex_state;
		    lex_state = kw->state;
		    if (state == EXPR_FNAME) {
			yylval.id = rb_intern(kw->name);
		    }
		    if (kw->id[0] == kDO) {
			if (COND_P()) return kDO_COND;
			if (CMDARG_P() && state != EXPR_CMDARG)
			    return kDO_BLOCK;
			if (state == EXPR_ENDARG)
			    return kDO_BLOCK;
			return kDO;
		    }
		    if (state == EXPR_BEG)
			return kw->id[0];
		    else {
			if (kw->id[0] != kw->id[1])
			    lex_state = EXPR_BEG;
			return kw->id[1];
		    }
		}
	    }

	    if (lex_state == EXPR_BEG ||
		lex_state == EXPR_MID ||
		lex_state == EXPR_DOT ||
		lex_state == EXPR_ARG ||
		lex_state == EXPR_CMDARG) {
		if (cmd_state) {
		    lex_state = EXPR_CMDARG;
		}
		else {
		    lex_state = EXPR_ARG;
		}
	    }
	    else {
		lex_state = EXPR_END;
	    }
	}
	yylval.id = rb_intern(tok());
	if (is_local_id(yylval.id) &&
	    ((dyna_in_block() && rb_dvar_defined(yylval.id)) || local_id(yylval.id))) {
	    lex_state = EXPR_END;
	}
	return result;
    }
}

NODE*
rb_node_newnode(type, a0, a1, a2)
    enum node_type type;
    VALUE a0, a1, a2;
{
    NODE *n = (NODE*)rb_newobj();

    n->flags |= T_NODE;
    nd_set_type(n, type);
    nd_set_line(n, ruby_sourceline);
    n->nd_file = ruby_sourcefile;

    n->u1.value = a0;
    n->u2.value = a1;
    n->u3.value = a2;

    return n;
}

static enum node_type
nodetype(node)			/* for debug */
    NODE *node;
{
    return (enum node_type)nd_type(node);
}

static int
nodeline(node)
    NODE *node;
{
    return nd_line(node);
}

static NODE*
newline_node(node)
    NODE *node;
{
    FL_SET(node, NODE_NEWLINE);
    return node;
}

static void
fixpos(node, orig)
    NODE *node, *orig;
{
    if (!node) return;
    if (!orig) return;
    if (orig == (NODE*)1) return;
    node->nd_file = orig->nd_file;
    nd_set_line(node, nd_line(orig));
}

static void
parser_warning(node, mesg)
    NODE *node;
    const char *mesg;
{
    int line = ruby_sourceline;
    ruby_sourceline = nd_line(node);
    rb_warning(mesg);
    ruby_sourceline = line;
}

static void
parser_warn(node, mesg)
    NODE *node;
    const char *mesg;
{
    int line = ruby_sourceline;
    ruby_sourceline = nd_line(node);
    rb_warn(mesg);
    ruby_sourceline = line;
}

static NODE*
block_append(head, tail)
    NODE *head, *tail;
{
    NODE *end, *h = head;

    if (tail == 0) return head;

  again:
    if (h == 0) return tail;
    switch (nd_type(h)) {
      case NODE_LIT:
      case NODE_STR:
	parser_warning(h, "unused literal ignored");
	return tail;
      default:
	h = end = NEW_BLOCK(head);
	end->nd_end = end;
	fixpos(end, head);
	head = end;
	break;
      case NODE_BLOCK:
	end = h->nd_end;
	break;
    }

    if (RTEST(ruby_verbose)) {
	NODE *nd = end->nd_head;
      newline:
	switch (nd_type(nd)) {
	  case NODE_RETURN:
	  case NODE_BREAK:
	  case NODE_NEXT:
	  case NODE_REDO:
	  case NODE_RETRY:
	    parser_warning(nd, "statement not reached");
	    break;

	  default:
	    break;
	}
    }

    if (nd_type(tail) != NODE_BLOCK) {
	tail = NEW_BLOCK(tail);
	tail->nd_end = tail;
    }
    end->nd_next = tail;
    h->nd_end = tail->nd_end;
    return head;
}

/* append item to the list */
static NODE*
list_append(list, item)
    NODE *list, *item;
{
    NODE *last;

    if (list == 0) return NEW_LIST(item);
    if (list->nd_next) {
	last = list->nd_next->nd_end;
    }
    else {
	last = list;
    }

    list->nd_alen += 1;
    last->nd_next = NEW_LIST(item);
    list->nd_next->nd_end = last->nd_next;
    return list;
}

/* concat two lists */
static NODE*
list_concat(head, tail)
    NODE *head, *tail;
{
    NODE *last;

    if (head->nd_next) {
	last = head->nd_next->nd_end;
    }
    else {
	last = head;
    }

    head->nd_alen += tail->nd_alen;
    last->nd_next = tail;
    if (tail->nd_next) {
	head->nd_next->nd_end = tail->nd_next->nd_end;
    }
    else {
	head->nd_next->nd_end = tail;
    }

    return head;
}

/* concat two string literals */
static NODE *
literal_concat(head, tail)
    NODE *head, *tail;
{
    enum node_type htype;

    if (!head) return tail;
    if (!tail) return head;

    htype = nd_type(head);
    if (htype == NODE_EVSTR) {
	NODE *node = NEW_DSTR(rb_str_new(0, 0));
	head = list_append(node, head);
    }
    switch (nd_type(tail)) {
      case NODE_STR:
	if (htype == NODE_STR) {
	    rb_str_concat(head->nd_lit, tail->nd_lit);
	    rb_gc_force_recycle((VALUE)tail);
	}
	else {
	    list_append(head, tail);
	}
	break;

      case NODE_DSTR:
	if (htype == NODE_STR) {
	    rb_str_concat(head->nd_lit, tail->nd_lit);
	    tail->nd_lit = head->nd_lit;
	    rb_gc_force_recycle((VALUE)head);
	    head = tail;
	}
	else {
	    nd_set_type(tail, NODE_ARRAY);
	    tail->nd_head = NEW_STR(tail->nd_lit);
	    list_concat(head, tail);
	}
	break;

      case NODE_EVSTR:
	if (htype == NODE_STR) {
	    nd_set_type(head, NODE_DSTR);
	}
	list_append(head, tail);
	break;
    }
    return head;
}

static NODE *
evstr2dstr(node)
    NODE *node;
{
    if (nd_type(node) == NODE_EVSTR) {
	node = list_append(NEW_DSTR(rb_str_new(0, 0)), node);
    }
    return node;
}

static NODE *
new_evstr(node)
    NODE *node;
{
    NODE *head = node;

  again:
    if (node) {
	switch (nd_type(node)) {
	  case NODE_STR: case NODE_DSTR: case NODE_EVSTR:
	    return node;
	}
    }
    return NEW_EVSTR(head);
}

static NODE *
call_op(recv, id, narg, arg1)
    NODE *recv;
    ID id;
    int narg;
    NODE *arg1;
{
    value_expr(recv);
    if (narg == 1) {
	value_expr(arg1);
	arg1 = NEW_LIST(arg1);
    }
    else {
	arg1 = 0;
    }
    return NEW_CALL(recv, id, arg1);
}

static NODE*
match_gen(node1, node2)
    NODE *node1;
    NODE *node2;
{
    local_cnt('~');

    value_expr(node1);
    value_expr(node2);
    if (node1) {
	switch (nd_type(node1)) {
	  case NODE_DREGX:
	  case NODE_DREGX_ONCE:
	    return NEW_MATCH2(node1, node2);

	  case NODE_LIT:
	    if (TYPE(node1->nd_lit) == T_REGEXP) {
		return NEW_MATCH2(node1, node2);
	    }
	}
    }

    if (node2) {
	switch (nd_type(node2)) {
	  case NODE_DREGX:
	  case NODE_DREGX_ONCE:
	    return NEW_MATCH3(node2, node1);

	  case NODE_LIT:
	    if (TYPE(node2->nd_lit) == T_REGEXP) {
		return NEW_MATCH3(node2, node1);
	    }
	}
    }

    return NEW_CALL(node1, tMATCH, NEW_LIST(node2));
}

static NODE*
gettable(id)
    ID id;
{
    if (id == kSELF) {
	return NEW_SELF();
    }
    else if (id == kNIL) {
	return NEW_NIL();
    }
    else if (id == kTRUE) {
	return NEW_TRUE();
    }
    else if (id == kFALSE) {
	return NEW_FALSE();
    }
    else if (id == k__FILE__) {
	return NEW_STR(rb_str_new2(ruby_sourcefile));
    }
    else if (id == k__LINE__) {
	return NEW_LIT(INT2FIX(ruby_sourceline));
    }
    else if (is_local_id(id)) {
	if (dyna_in_block() && rb_dvar_defined(id)) return NEW_DVAR(id);
	if (local_id(id)) return NEW_LVAR(id);
	/* method call without arguments */
#if 0
	/* Rite will warn this */
	rb_warn("ambiguous identifier; %s() or self.%s is better for method call",
		rb_id2name(id), rb_id2name(id));
#endif
	return NEW_VCALL(id);
    }
    else if (is_global_id(id)) {
	return NEW_GVAR(id);
    }
    else if (is_instance_id(id)) {
	return NEW_IVAR(id);
    }
    else if (is_const_id(id)) {
	return NEW_CONST(id);
    }
    else if (is_class_id(id)) {
	return NEW_CVAR(id);
    }
    rb_compile_error("identifier %s is not valid", rb_id2name(id));
    return 0;
}

static NODE*
assignable(id, val)
    ID id;
    NODE *val;
{
    value_expr(val);
    if (id == kSELF) {
	yyerror("Can't change the value of self");
    }
    else if (id == kNIL) {
	yyerror("Can't assign to nil");
    }
    else if (id == kTRUE) {
	yyerror("Can't assign to true");
    }
    else if (id == kFALSE) {
	yyerror("Can't assign to false");
    }
    else if (id == k__FILE__) {
	yyerror("Can't assign to __FILE__");
    }
    else if (id == k__LINE__) {
	yyerror("Can't assign to __LINE__");
    }
    else if (is_local_id(id)) {
	if (rb_dvar_curr(id)) {
	    return NEW_DASGN_CURR(id, val);
	}
	else if (rb_dvar_defined(id)) {
	    return NEW_DASGN(id, val);
	}
	else if (local_id(id) || !dyna_in_block()) {
	    return NEW_LASGN(id, val);
	}
	else{
	    rb_dvar_push(id, Qnil);
	    return NEW_DASGN_CURR(id, val);
	}
    }
    else if (is_global_id(id)) {
	return NEW_GASGN(id, val);
    }
    else if (is_instance_id(id)) {
	return NEW_IASGN(id, val);
    }
    else if (is_const_id(id)) {
	if (in_def || in_single)
	    yyerror("dynamic constant assignment");
	return NEW_CDECL(id, val, 0);
    }
    else if (is_class_id(id)) {
	if (in_def || in_single) return NEW_CVASGN(id, val);
	return NEW_CVDECL(id, val);
    }
    else {
	rb_compile_error("identifier %s is not valid", rb_id2name(id));
    }
    return 0;
}

static NODE *
aryset(recv, idx)
    NODE *recv, *idx;
{
    if (recv && nd_type(recv) == NODE_SELF)
	recv = (NODE *)1;
    else
	value_expr(recv);
    return NEW_ATTRASGN(recv, tASET, idx);
}

ID
rb_id_attrset(id)
    ID id;
{
    id &= ~ID_SCOPE_MASK;
    id |= ID_ATTRSET;
    return id;
}

static NODE *
attrset(recv, id)
    NODE *recv;
    ID id;
{
    if (recv && nd_type(recv) == NODE_SELF)
	recv = (NODE *)1;
    else
	value_expr(recv);
    return NEW_ATTRASGN(recv, rb_id_attrset(id), 0);
}

static void
rb_backref_error(node)
    NODE *node;
{
    switch (nd_type(node)) {
      case NODE_NTH_REF:
	rb_compile_error("Can't set variable $%d", node->nd_nth);
	break;
      case NODE_BACK_REF:
	rb_compile_error("Can't set variable $%c", (int)node->nd_nth);
	break;
    }
}

static NODE *
arg_concat(node1, node2)
    NODE *node1;
    NODE *node2;
{
    if (!node2) return node1;
    return NEW_ARGSCAT(node1, node2);
}

static NODE *
arg_add(node1, node2)
    NODE *node1;
    NODE *node2;
{
    if (!node1) return NEW_LIST(node2);
    if (nd_type(node1) == NODE_ARRAY) {
	return list_append(node1, node2);
    }
    else {
	return NEW_ARGSPUSH(node1, node2);
    }
}

static NODE*
node_assign(lhs, rhs)
    NODE *lhs, *rhs;
{
    if (!lhs) return 0;

    value_expr(rhs);
    switch (nd_type(lhs)) {
      case NODE_GASGN:
      case NODE_IASGN:
      case NODE_LASGN:
      case NODE_DASGN:
      case NODE_DASGN_CURR:
      case NODE_MASGN:
      case NODE_CDECL:
      case NODE_CVDECL:
      case NODE_CVASGN:
	lhs->nd_value = rhs;
	break;

      case NODE_ATTRASGN:
      case NODE_CALL:
	lhs->nd_args = arg_add(lhs->nd_args, rhs);
	break;

      default:
	/* should not happen */
	break;
    }

    return lhs;
}

static int
value_expr0(node)
    NODE *node;
{
    int cond = 0;

    while (node) {
	switch (nd_type(node)) {
	  case NODE_DEFN:
	  case NODE_DEFS:
	    parser_warning(node, "void value expression");
	    return Qfalse;

	  case NODE_RETURN:
	  case NODE_BREAK:
	  case NODE_NEXT:
	  case NODE_REDO:
	  case NODE_RETRY:
	    if (!cond) yyerror("void value expression");
	    /* or "control never reach"? */
	    return Qfalse;

	  case NODE_BLOCK:
	    while (node->nd_next) {
		node = node->nd_next;
	    }
	    node = node->nd_head;
	    break;

	  case NODE_BEGIN:
	    node = node->nd_body;
	    break;

	  case NODE_IF:
	    if (!value_expr(node->nd_body)) return Qfalse;
	    node = node->nd_else;
	    break;

	  case NODE_AND:
	  case NODE_OR:
	    cond = 1;
	    node = node->nd_2nd;
	    break;

	  default:
	    return Qtrue;
	}
    }

    return Qtrue;
}

static void
void_expr0(node)
    NODE *node;
{
    char *useless = 0;

    if (!RTEST(ruby_verbose)) return;

  again:
    if (!node) return;
    switch (nd_type(node)) {
      case NODE_CALL:
	switch (node->nd_mid) {
	  case '+':
	  case '-':
	  case '*':
	  case '/':
	  case '%':
	  case tPOW:
	  case tUPLUS:
	  case tUMINUS:
	  case '|':
	  case '^':
	  case '&':
	  case tCMP:
	  case '>':
	  case tGEQ:
	  case '<':
	  case tLEQ:
	  case tEQ:
	  case tNEQ:
	    useless = rb_id2name(node->nd_mid);
	    break;
	}
	break;

      case NODE_LVAR:
      case NODE_DVAR:
      case NODE_GVAR:
      case NODE_IVAR:
      case NODE_CVAR:
      case NODE_NTH_REF:
      case NODE_BACK_REF:
	useless = "a variable";
	break;
      case NODE_CONST:
      case NODE_CREF:
	useless = "a constant";
	break;
      case NODE_LIT:
      case NODE_STR:
      case NODE_DSTR:
      case NODE_DREGX:
      case NODE_DREGX_ONCE:
	useless = "a literal";
	break;
      case NODE_COLON2:
      case NODE_COLON3:
	useless = "::";
	break;
      case NODE_DOT2:
	useless = "..";
	break;
      case NODE_DOT3:
	useless = "...";
	break;
      case NODE_SELF:
	useless = "self";
	break;
      case NODE_NIL:
	useless = "nil";
	break;
      case NODE_TRUE:
	useless = "true";
	break;
      case NODE_FALSE:
	useless = "false";
	break;
      case NODE_DEFINED:
	useless = "defined?";
	break;
    }

    if (useless) {
	int line = ruby_sourceline;

	ruby_sourceline = nd_line(node);
	rb_warn("useless use of %s in void context", useless);
	ruby_sourceline = line;
    }
}

static void
void_stmts(node)
    NODE *node;
{
    if (!RTEST(ruby_verbose)) return;
    if (!node) return;
    if (nd_type(node) != NODE_BLOCK) return;

    for (;;) {
	if (!node->nd_next) return;
	void_expr(node->nd_head);
	node = node->nd_next;
    }
}

static NODE *
remove_begin(node)
    NODE *node;
{
    NODE **n = &node;
    while (*n) {
	if (nd_type(*n) != NODE_BEGIN) {
	    return node;
	}
	*n = (*n)->nd_body;
    }
    return node;
}

static int
assign_in_cond(node)
    NODE *node;
{
    switch (nd_type(node)) {
      case NODE_MASGN:
	yyerror("multiple assignment in conditional");
	return 1;

      case NODE_LASGN:
      case NODE_DASGN:
      case NODE_GASGN:
      case NODE_IASGN:
	break;

      default:
	return 0;
    }

    switch (nd_type(node->nd_value)) {
      case NODE_LIT:
      case NODE_STR:
      case NODE_NIL:
      case NODE_TRUE:
      case NODE_FALSE:
	/* reports always */
	parser_warn(node->nd_value, "found = in conditional, should be ==");
	return 1;

      case NODE_DSTR:
      case NODE_XSTR:
      case NODE_DXSTR:
      case NODE_EVSTR:
      case NODE_DREGX:
      default:
	break;
    }
#if 0
    if (assign_in_cond(node->nd_value) == 0) {
	parser_warning(node->nd_value, "assignment in condition");
    }
#endif
    return 1;
}

static int
e_option_supplied()
{
    if (strcmp(ruby_sourcefile, "-e") == 0)
	return Qtrue;
    return Qfalse;
}

static void
warn_unless_e_option(node, str)
    NODE *node;
    const char *str;
{
    if (!e_option_supplied()) parser_warn(node, str);
}

static void
warning_unless_e_option(node, str)
    NODE *node;
    const char *str;
{
    if (!e_option_supplied()) parser_warning(node, str);
}

static NODE *cond0();

static NODE*
range_op(node)
    NODE *node;
{
    enum node_type type;

    if (!e_option_supplied()) return node;
    if (node == 0) return 0;

    value_expr(node);
    node = cond0(node);
    type = nd_type(node);
    if (type == NODE_LIT && FIXNUM_P(node->nd_lit)) {
	warn_unless_e_option(node, "integer literal in conditional range");
	return call_op(node,tEQ,1,NEW_GVAR(rb_intern("$.")));
    }
    return node;
}

static int
literal_node(node)
    NODE *node;
{
    if (!node) return 1;	/* same as NODE_NIL */
    switch (nd_type(node)) {
      case NODE_LIT:
      case NODE_STR:
      case NODE_DSTR:
      case NODE_EVSTR:
      case NODE_DREGX:
      case NODE_DREGX_ONCE:
      case NODE_DSYM:
	return 2;
      case NODE_TRUE:
      case NODE_FALSE:
      case NODE_NIL:
	return 1;
    }
    return 0;
}

static NODE*
cond0(node)
    NODE *node;
{
    enum node_type type = nd_type(node);

    assign_in_cond(node);

    switch (type) {
      case NODE_DSTR:
      case NODE_EVSTR:
      case NODE_STR:
	rb_warn("string literal in condition");
	break;

      case NODE_DREGX:
      case NODE_DREGX_ONCE:
	warning_unless_e_option(node, "regex literal in condition");
	local_cnt('_');
	local_cnt('~');
	return NEW_MATCH2(node, NEW_GVAR(rb_intern("$_")));

      case NODE_AND:
      case NODE_OR:
	node->nd_1st = cond0(node->nd_1st);
	node->nd_2nd = cond0(node->nd_2nd);
	break;

      case NODE_DOT2:
      case NODE_DOT3:
	node->nd_beg = range_op(node->nd_beg);
	node->nd_end = range_op(node->nd_end);
	if (type == NODE_DOT2) nd_set_type(node,NODE_FLIP2);
	else if (type == NODE_DOT3) nd_set_type(node, NODE_FLIP3);
	node->nd_cnt = local_append(internal_id());
	if (!e_option_supplied()) {
	    int b = literal_node(node->nd_beg);
	    int e = literal_node(node->nd_end);
	    if ((b == 1 && e == 1) || (b + e >= 2 && RTEST(ruby_verbose))) {
		parser_warn(node, "range literal in condition");
	    }
	}
	break;

      case NODE_DSYM:
	parser_warning(node, "literal in condition");
	break;

      case NODE_LIT:
	if (TYPE(node->nd_lit) == T_REGEXP) {
	    warn_unless_e_option(node, "regex literal in condition");
	    nd_set_type(node, NODE_MATCH);
	    local_cnt('_');
	    local_cnt('~');
	}
	else {
	    parser_warning(node, "literal in condition");
	}
      default:
	break;
    }
    return node;
}

static NODE*
cond(node)
    NODE *node;
{
    if (node == 0) return 0;
    value_expr(node);
    return cond0(node);
}

static NODE*
logop(type, left, right)
    enum node_type type;
    NODE *left, *right;
{
    value_expr(left);
    if (left && nd_type(left) == type) {
	NODE *node = left, *second;
	while ((second = node->nd_2nd) != 0 && nd_type(second) == type) {
	    node = second;
	}
	node->nd_2nd = NEW_NODE(type, second, right, 0);
	return left;
    }
    return NEW_NODE(type, left, right, 0);
}

static int
cond_negative(nodep)
    NODE **nodep;
{
    NODE *c = *nodep;

    if (!c) return 0;
    switch (nd_type(c)) {
      case NODE_NOT:
	*nodep = c->nd_body;
	return 1;
    }
    return 0;
}

static void
no_blockarg(node)
    NODE *node;
{
    if (node && nd_type(node) == NODE_BLOCK_PASS) {
	rb_compile_error("block argument should not be given");
    }
}

static NODE *
ret_args(node)
    NODE *node;
{
    if (node) {
	no_blockarg(node);
	if (nd_type(node) == NODE_ARRAY && node->nd_next == 0) {
	    node = node->nd_head;
	}
	else if (node && nd_type(node) == NODE_SPLAT) {
	    node = NEW_SVALUE(node);
	}
    }
    return node;
}

static NODE *
new_yield(node)
    NODE *node;
{
    long state = Qtrue;

    if (node) {
        no_blockarg(node);
        if (nd_type(node) == NODE_ARRAY && node->nd_next == 0) {
            node = node->nd_head;
            state = Qfalse;
        }
        else if (node && nd_type(node) == NODE_SPLAT) {
            state = Qtrue;
        }
    }
    else {
        state = Qfalse;
    }
    return NEW_YIELD(node, state);
}

static NODE*
negate_lit(node)
    NODE *node;
{
    switch (TYPE(node->nd_lit)) {
      case T_FIXNUM:
	node->nd_lit = LONG2FIX(-FIX2LONG(node->nd_lit));
	break;
      case T_BIGNUM:
	node->nd_lit = rb_funcall(node->nd_lit,tUMINUS,0,0);
	break;
      case T_FLOAT:
	RFLOAT(node->nd_lit)->value = -RFLOAT(node->nd_lit)->value;
	break;
      default:
	break;
    }
    return node;
}

static NODE *
arg_blk_pass(node1, node2)
    NODE *node1;
    NODE *node2;
{
    if (node2) {
	node2->nd_head = node1;
	return node2;
    }
    return node1;
}

static NODE*
arg_prepend(node1, node2)
    NODE *node1, *node2;
{
    switch (nd_type(node2)) {
      case NODE_ARRAY:
	return list_concat(NEW_LIST(node1), node2);

      case NODE_SPLAT:
	return arg_concat(node1, node2->nd_head);

      case NODE_BLOCK_PASS:
	node2->nd_body = arg_prepend(node1, node2->nd_body);
	return node2;

      default:
	rb_bug("unknown nodetype(%d) for arg_prepend", nd_type(node2));
    }
    return 0;			/* not reached */
}

static NODE*
new_call(r,m,a)
    NODE *r;
    ID m;
    NODE *a;
{
    if (a && nd_type(a) == NODE_BLOCK_PASS) {
	a->nd_iter = NEW_CALL(r,m,a->nd_head);
	return a;
    }
    return NEW_CALL(r,m,a);
}

static NODE*
new_fcall(m,a)
    ID m;
    NODE *a;
{
    if (a && nd_type(a) == NODE_BLOCK_PASS) {
	a->nd_iter = NEW_FCALL(m,a->nd_head);
	return a;
    }
    return NEW_FCALL(m,a);
}

static NODE*
new_super(a)
    NODE *a;
{
    if (a && nd_type(a) == NODE_BLOCK_PASS) {
	a->nd_iter = NEW_SUPER(a->nd_head);
	return a;
    }
    return NEW_SUPER(a);
}

static struct local_vars {
    ID *tbl;
    int nofree;
    int cnt;
    int dlev;
    struct RVarmap* dyna_vars;
    struct local_vars *prev;
} *lvtbl;

static void
local_push(top)
    int top;
{
    struct local_vars *local;

    local = ALLOC(struct local_vars);
    local->prev = lvtbl;
    local->nofree = 0;
    local->cnt = 0;
    local->tbl = 0;
    local->dlev = 0;
    local->dyna_vars = ruby_dyna_vars;
    lvtbl = local;
    if (!top) {
	/* preserve reference for GC, but link should be cut. */
	rb_dvar_push(0, (VALUE)ruby_dyna_vars);
	ruby_dyna_vars->next = 0;
    }
}

static void
local_pop()
{
    struct local_vars *local = lvtbl->prev;

    if (lvtbl->tbl) {
	if (!lvtbl->nofree) free(lvtbl->tbl);
	else lvtbl->tbl[0] = lvtbl->cnt;
    }
    ruby_dyna_vars = lvtbl->dyna_vars;
    free(lvtbl);
    lvtbl = local;
}

static ID*
local_tbl()
{
    lvtbl->nofree = 1;
    return lvtbl->tbl;
}

static int
local_append(id)
    ID id;
{
    if (lvtbl->tbl == 0) {
	lvtbl->tbl = ALLOC_N(ID, 4);
	lvtbl->tbl[0] = 0;
	lvtbl->tbl[1] = '_';
	lvtbl->tbl[2] = '~';
	lvtbl->cnt = 2;
	if (id == '_') return 0;
	if (id == '~') return 1;
    }
    else {
	REALLOC_N(lvtbl->tbl, ID, lvtbl->cnt+2);
    }

    lvtbl->tbl[lvtbl->cnt+1] = id;
    return lvtbl->cnt++;
}

static int
local_cnt(id)
    ID id;
{
    int cnt, max;

    if (id == 0) return lvtbl->cnt;

    for (cnt=1, max=lvtbl->cnt+1; cnt<max;cnt++) {
	if (lvtbl->tbl[cnt] == id) return cnt-1;
    }
    return local_append(id);
}

static int
local_id(id)
    ID id;
{
    int i, max;

    if (lvtbl == 0) return Qfalse;
    for (i=3, max=lvtbl->cnt+1; i<max; i++) {
	if (lvtbl->tbl[i] == id) return Qtrue;
    }
    return Qfalse;
}

static void
top_local_init()
{
    local_push(1);
    lvtbl->cnt = ruby_scope->local_tbl?ruby_scope->local_tbl[0]:0;
    if (lvtbl->cnt > 0) {
	lvtbl->tbl = ALLOC_N(ID, lvtbl->cnt+3);
	MEMCPY(lvtbl->tbl, ruby_scope->local_tbl, ID, lvtbl->cnt+1);
    }
    else {
	lvtbl->tbl = 0;
    }
    if (ruby_dyna_vars)
	lvtbl->dlev = 1;
    else
	lvtbl->dlev = 0;
}

static void
top_local_setup()
{
    int len = lvtbl->cnt;
    int i;

    if (len > 0) {
	i = ruby_scope->local_tbl?ruby_scope->local_tbl[0]:0;

	if (i < len) {
	    if (i == 0 || (ruby_scope->flags & SCOPE_MALLOC) == 0) {
		VALUE *vars = ALLOC_N(VALUE, len+1);
		if (ruby_scope->local_vars) {
		    *vars++ = ruby_scope->local_vars[-1];
		    MEMCPY(vars, ruby_scope->local_vars, VALUE, i);
		    rb_mem_clear(vars+i, len-i);
		}
		else {
		    *vars++ = 0;
		    rb_mem_clear(vars, len);
		}
		ruby_scope->local_vars = vars;
		ruby_scope->flags |= SCOPE_MALLOC;
	    }
	    else {
		VALUE *vars = ruby_scope->local_vars-1;
		REALLOC_N(vars, VALUE, len+1);
		ruby_scope->local_vars = vars+1;
		rb_mem_clear(ruby_scope->local_vars+i, len-i);
	    }
	    if (ruby_scope->local_tbl && ruby_scope->local_vars[-1] == 0) {
		free(ruby_scope->local_tbl);
	    }
	    ruby_scope->local_vars[-1] = 0;
	    ruby_scope->local_tbl = local_tbl();
	}
    }
    local_pop();
}

static struct RVarmap*
dyna_push()
{
    struct RVarmap* vars = ruby_dyna_vars;

    rb_dvar_push(0, 0);
    lvtbl->dlev++;
    return vars;
}

static void
dyna_pop(vars)
    struct RVarmap* vars;
{
    lvtbl->dlev--;
    ruby_dyna_vars = vars;
}

static int
dyna_in_block()
{
    return (lvtbl->dlev > 0);
}

static NODE *
dyna_init(node, pre)
    NODE *node;
    struct RVarmap *pre;
{
    struct RVarmap *post = ruby_dyna_vars;
    NODE *var;

    if (!node || !post || pre == post) return node;
    for (var = 0; post != pre && post->id; post = post->next) {
	var = NEW_DASGN_CURR(post->id, var);
    }
    return block_append(var, node);
}

int
ruby_parser_stack_on_heap()
{
#if defined(YYBISON) && !defined(C_ALLOCA)
    return Qfalse;
#else
    return Qtrue;
#endif
}

void
rb_gc_mark_parser()
{
    if (!ruby_in_compile) return;

    rb_gc_mark_maybe((VALUE)yylval.node);
    rb_gc_mark(ruby_debug_lines);
    rb_gc_mark(lex_lastline);
    rb_gc_mark(lex_input);
    rb_gc_mark((VALUE)lex_strterm);
}

void
rb_parser_append_print()
{
    ruby_eval_tree =
	block_append(ruby_eval_tree,
		     NEW_FCALL(rb_intern("print"),
			       NEW_ARRAY(NEW_GVAR(rb_intern("$_")))));
}

void
rb_parser_while_loop(chop, split)
    int chop, split;
{
    if (split) {
	ruby_eval_tree =
	    block_append(NEW_GASGN(rb_intern("$F"),
				   NEW_CALL(NEW_GVAR(rb_intern("$_")),
					    rb_intern("split"), 0)),
				   ruby_eval_tree);
    }
    if (chop) {
	ruby_eval_tree =
	    block_append(NEW_CALL(NEW_GVAR(rb_intern("$_")),
				  rb_intern("chop!"), 0), ruby_eval_tree);
    }
    ruby_eval_tree = NEW_OPT_N(ruby_eval_tree);
}

static struct {
    ID token;
    char *name;
} op_tbl[] = {
    {tDOT2,	".."},
    {tDOT3,	"..."},
    {'+',	"+"},
    {'-',	"-"},
    {'+',	"+(binary)"},
    {'-',	"-(binary)"},
    {'*',	"*"},
    {'/',	"/"},
    {'%',	"%"},
    {tPOW,	"**"},
    {tUPLUS,	"+@"},
    {tUMINUS,	"-@"},
    {tUPLUS,	"+(unary)"},
    {tUMINUS,	"-(unary)"},
    {'|',	"|"},
    {'^',	"^"},
    {'&',	"&"},
    {tCMP,	"<=>"},
    {'>',	">"},
    {tGEQ,	">="},
    {'<',	"<"},
    {tLEQ,	"<="},
    {tEQ,	"=="},
    {tEQQ,	"==="},
    {tNEQ,	"!="},
    {tMATCH,	"=~"},
    {tNMATCH,	"!~"},
    {'!',	"!"},
    {'~',	"~"},
    {'!',	"!(unary)"},
    {'~',	"~(unary)"},
    {'!',	"!@"},
    {'~',	"~@"},
    {tAREF,	"[]"},
    {tASET,	"[]="},
    {tLSHFT,	"<<"},
    {tRSHFT,	">>"},
    {tCOLON2,	"::"},
    {'`',	"`"},
    {0,	0}
};

static st_table *sym_tbl;
static st_table *sym_rev_tbl;

void
Init_sym()
{
    sym_tbl = st_init_strtable_with_size(200);
    sym_rev_tbl = st_init_numtable_with_size(200);
}

static ID last_id = tLAST_TOKEN;

static ID
internal_id()
{
    return ID_INTERNAL | (++last_id << ID_SCOPE_SHIFT);
}

ID
rb_intern(name)
    const char *name;
{
    const char *m = name;
    ID id;
    int last;

    if (st_lookup(sym_tbl, (st_data_t)name, (st_data_t *)&id))
	return id;

    last = strlen(name)-1;
    id = 0;
    switch (*name) {
      case '$':
	id |= ID_GLOBAL;
	m++;
	if (!is_identchar(*m)) m++;
	break;
      case '@':
	if (name[1] == '@') {
	    m++;
	    id |= ID_CLASS;
	}
	else {
	    id |= ID_INSTANCE;
	}
	m++;
	break;
      default:
	if (name[0] != '_' && !ISALPHA(name[0]) && !ismbchar(name[0])) {
	    /* operators */
	    int i;

	    for (i=0; op_tbl[i].token; i++) {
		if (*op_tbl[i].name == *name &&
		    strcmp(op_tbl[i].name, name) == 0) {
		    id = op_tbl[i].token;
		    goto id_regist;
		}
	    }
	}

	if (name[last] == '=') {
	    /* attribute assignment */
	    char *buf = ALLOCA_N(char,last+1);

	    strncpy(buf, name, last);
	    buf[last] = '\0';
	    id = rb_intern(buf);
	    if (id > tLAST_TOKEN && !is_attrset_id(id)) {
		id = rb_id_attrset(id);
		goto id_regist;
	    }
	    id = ID_ATTRSET;
	}
	else if (ISUPPER(name[0])) {
	    id = ID_CONST;
        }
	else {
	    id = ID_LOCAL;
	}
	break;
    }
    while (m <= name + last && is_identchar(*m)) {
	m += mbclen(*m);
    }
    if (*m) id = ID_JUNK;
    id |= ++last_id << ID_SCOPE_SHIFT;
  id_regist:
    name = strdup(name);
    st_add_direct(sym_tbl, (st_data_t)name, id);
    st_add_direct(sym_rev_tbl, id, (st_data_t)name);
    return id;
}

char *
rb_id2name(id)
    ID id;
{
    char *name;

    if (id < tLAST_TOKEN) {
	int i = 0;

	for (i=0; op_tbl[i].token; i++) {
	    if (op_tbl[i].token == id)
		return op_tbl[i].name;
	}
    }

    if (st_lookup(sym_rev_tbl, id, (st_data_t *)&name))
	return name;

    if (is_attrset_id(id)) {
	ID id2 = (id & ~ID_SCOPE_MASK) | ID_LOCAL;

      again:
	name = rb_id2name(id2);
	if (name) {
	    char *buf = ALLOCA_N(char, strlen(name)+2);

	    strcpy(buf, name);
	    strcat(buf, "=");
	    rb_intern(buf);
	    return rb_id2name(id);
	}
	if (is_local_id(id2)) {
	    id2 = (id & ~ID_SCOPE_MASK) | ID_CONST;
	    goto again;
	}
    }
    return 0;
}

static int
symbols_i(key, value, ary)
    char *key;
    ID value;
    VALUE ary;
{
    rb_ary_push(ary, ID2SYM(value));
    return ST_CONTINUE;
}

/*
 *  call-seq:
 *     Symbol.all_symbols    => array
 *  
 *  Returns an array of all the symbols currently in Ruby's symbol
 *  table.
 *     
 *     Symbol.all_symbols.size    #=> 903
 *     Symbol.all_symbols[1,20]   #=> [:floor, :ARGV, :Binding, :symlink,
 *                                     :chown, :EOFError, :$;, :String, 
 *                                     :LOCK_SH, :"setuid?", :$<, 
 *                                     :default_proc, :compact, :extend, 
 *                                     :Tms, :getwd, :$=, :ThreadGroup,
 *                                     :wait2, :$>]
 */

VALUE
rb_sym_all_symbols()
{
    VALUE ary = rb_ary_new2(sym_tbl->num_entries);

    st_foreach(sym_tbl, symbols_i, ary);
    return ary;
}

int
rb_is_const_id(id)
    ID id;
{
    if (is_const_id(id)) return Qtrue;
    return Qfalse;
}

int
rb_is_class_id(id)
    ID id;
{
    if (is_class_id(id)) return Qtrue;
    return Qfalse;
}

int
rb_is_instance_id(id)
    ID id;
{
    if (is_instance_id(id)) return Qtrue;
    return Qfalse;
}

int
rb_is_local_id(id)
    ID id;
{
    if (is_local_id(id)) return Qtrue;
    return Qfalse;
}

int
rb_is_junk_id(id)
    ID id;
{
    if (is_junk_id(id)) return Qtrue;
    return Qfalse;
}

static void
special_local_set(c, val)
    char c;
    VALUE val;
{
    int cnt;

    top_local_init();
    cnt = local_cnt(c);
    top_local_setup();
    ruby_scope->local_vars[cnt] = val;
}

VALUE
rb_backref_get()
{
    VALUE *var = rb_svar(1);
    if (var) {
	return *var;
    }
    return Qnil;
}

void
rb_backref_set(val)
    VALUE val;
{
    VALUE *var = rb_svar(1);
    if (var) {
	*var = val;
    }
    else {
	special_local_set('~', val);
    }
}

VALUE
rb_lastline_get()
{
    VALUE *var = rb_svar(0);
    if (var) {
	return *var;
    }
    return Qnil;
}

void
rb_lastline_set(val)
    VALUE val;
{
    VALUE *var = rb_svar(0);
    if (var) {
	*var = val;
    }
    else {
	special_local_set('_', val);
    }
}
#line 8791 "y.tab.c"
/* allocate initial stack or double stack size, up to YYMAXDEPTH */
static int yygrowstack()
{
    int newsize, i;
    short *newss;
    YYSTYPE *newvs;

    if ((newsize = yystacksize) == 0)
        newsize = YYINITSTACKSIZE;
    else if (newsize >= YYMAXDEPTH)
        return -1;
    else if ((newsize *= 2) > YYMAXDEPTH)
        newsize = YYMAXDEPTH;
    i = yyssp - yyss;
    newss = yyss ? (short *)realloc(yyss, newsize * sizeof *newss) :
      (short *)malloc(newsize * sizeof *newss);
    if (newss == NULL)
        return -1;
    yyss = newss;
    yyssp = newss + i;
    newvs = yyvs ? (YYSTYPE *)realloc(yyvs, newsize * sizeof *newvs) :
      (YYSTYPE *)malloc(newsize * sizeof *newvs);
    if (newvs == NULL)
        return -1;
    yyvs = newvs;
    yyvsp = newvs + i;
    yystacksize = newsize;
    yysslim = yyss + newsize - 1;
    return 0;
}

#define YYABORT goto yyabort
#define YYREJECT goto yyabort
#define YYACCEPT goto yyaccept
#define YYERROR goto yyerrlab

#ifndef YYPARSE_PARAM
#if defined(__cplusplus) || __STDC__
#define YYPARSE_PARAM_ARG void
#define YYPARSE_PARAM_DECL
#else	/* ! ANSI-C/C++ */
#define YYPARSE_PARAM_ARG
#define YYPARSE_PARAM_DECL
#endif	/* ANSI-C/C++ */
#else	/* YYPARSE_PARAM */
#ifndef YYPARSE_PARAM_TYPE
#define YYPARSE_PARAM_TYPE void *
#endif
#if defined(__cplusplus) || __STDC__
#define YYPARSE_PARAM_ARG YYPARSE_PARAM_TYPE YYPARSE_PARAM
#define YYPARSE_PARAM_DECL
#else	/* ! ANSI-C/C++ */
#define YYPARSE_PARAM_ARG YYPARSE_PARAM
#define YYPARSE_PARAM_DECL YYPARSE_PARAM_TYPE YYPARSE_PARAM;
#endif	/* ANSI-C/C++ */
#endif	/* ! YYPARSE_PARAM */

int
yyparse (YYPARSE_PARAM_ARG)
    YYPARSE_PARAM_DECL
{
    register int yym, yyn, yystate;
#if YYDEBUG
    register const char *yys;

    if ((yys = getenv("YYDEBUG")))
    {
        yyn = *yys;
        if (yyn >= '0' && yyn <= '9')
            yydebug = yyn - '0';
    }
#endif

    yynerrs = 0;
    yyerrflag = 0;
    yychar = (-1);

    if (yyss == NULL && yygrowstack()) goto yyoverflow;
    yyssp = yyss;
    yyvsp = yyvs;
    *yyssp = yystate = 0;

yyloop:
    if ((yyn = yydefred[yystate])) goto yyreduce;
    if (yychar < 0)
    {
        if ((yychar = yylex()) < 0) yychar = 0;
#if YYDEBUG
        if (yydebug)
        {
            yys = 0;
            if (yychar <= YYMAXTOKEN) yys = yyname[yychar];
            if (!yys) yys = "illegal-symbol";
            printf("%sdebug: state %d, reading %d (%s)\n",
                    YYPREFIX, yystate, yychar, yys);
        }
#endif
    }
    if ((yyn = yysindex[yystate]) && (yyn += yychar) >= 0 &&
            yyn <= YYTABLESIZE && yycheck[yyn] == yychar)
    {
#if YYDEBUG
        if (yydebug)
            printf("%sdebug: state %d, shifting to state %d\n",
                    YYPREFIX, yystate, yytable[yyn]);
#endif
        if (yyssp >= yysslim && yygrowstack())
        {
            goto yyoverflow;
        }
        *++yyssp = yystate = yytable[yyn];
        *++yyvsp = yylval;
        yychar = (-1);
        if (yyerrflag > 0)  --yyerrflag;
        goto yyloop;
    }
    if ((yyn = yyrindex[yystate]) && (yyn += yychar) >= 0 &&
            yyn <= YYTABLESIZE && yycheck[yyn] == yychar)
    {
        yyn = yytable[yyn];
        goto yyreduce;
    }
    if (yyerrflag) goto yyinrecovery;
#if defined(lint) || defined(__GNUC__)
    goto yynewerror;
#endif
yynewerror:
    yyerror("syntax error");
#if defined(lint) || defined(__GNUC__)
    goto yyerrlab;
#endif
yyerrlab:
    ++yynerrs;
yyinrecovery:
    if (yyerrflag < 3)
    {
        yyerrflag = 3;
        for (;;)
        {
            if ((yyn = yysindex[*yyssp]) && (yyn += YYERRCODE) >= 0 &&
                    yyn <= YYTABLESIZE && yycheck[yyn] == YYERRCODE)
            {
#if YYDEBUG
                if (yydebug)
                    printf("%sdebug: state %d, error recovery shifting\
 to state %d\n", YYPREFIX, *yyssp, yytable[yyn]);
#endif
                if (yyssp >= yysslim && yygrowstack())
                {
                    goto yyoverflow;
                }
                *++yyssp = yystate = yytable[yyn];
                *++yyvsp = yylval;
                goto yyloop;
            }
            else
            {
#if YYDEBUG
                if (yydebug)
                    printf("%sdebug: error recovery discarding state %d\n",
                            YYPREFIX, *yyssp);
#endif
                if (yyssp <= yyss) goto yyabort;
                --yyssp;
                --yyvsp;
            }
        }
    }
    else
    {
        if (yychar == 0) goto yyabort;
#if YYDEBUG
        if (yydebug)
        {
            yys = 0;
            if (yychar <= YYMAXTOKEN) yys = yyname[yychar];
            if (!yys) yys = "illegal-symbol";
            printf("%sdebug: state %d, error recovery discards token %d (%s)\n",
                    YYPREFIX, yystate, yychar, yys);
        }
#endif
        yychar = (-1);
        goto yyloop;
    }
yyreduce:
#if YYDEBUG
    if (yydebug)
        printf("%sdebug: state %d, reducing by rule %d (%s)\n",
                YYPREFIX, yystate, yyn, yyrule[yyn]);
#endif
    yym = yylen[yyn];
    yyval = yyvsp[1-yym];
    switch (yyn)
    {
case 1:
#line 328 "parse.y"
{
			lex_state = EXPR_BEG;
                        top_local_init();
			if (ruby_class == rb_cObject) class_nest = 0;
			else class_nest = 1;
		    }
break;
case 2:
#line 335 "parse.y"
{
			if (yyvsp[0].node && !compile_for_eval) {
                            /* last expression should not be void */
			    if (nd_type(yyvsp[0].node) != NODE_BLOCK) void_expr(yyvsp[0].node);
			    else {
				NODE *node = yyvsp[0].node;
				while (node->nd_next) {
				    node = node->nd_next;
				}
				void_expr(node->nd_head);
			    }
			}
			ruby_eval_tree = block_append(ruby_eval_tree, yyvsp[0].node);
                        top_local_setup();
			class_nest = 0;
		    }
break;
case 3:
#line 357 "parse.y"
{
		        yyval.node = yyvsp[-3].node;
			if (yyvsp[-2].node) {
			    yyval.node = NEW_RESCUE(yyvsp[-3].node, yyvsp[-2].node, yyvsp[-1].node);
			}
			else if (yyvsp[-1].node) {
			    rb_warn("else without rescue is useless");
			    yyval.node = block_append(yyval.node, yyvsp[-1].node);
			}
			if (yyvsp[0].node) {
			    yyval.node = NEW_ENSURE(yyval.node, yyvsp[0].node);
			}
			fixpos(yyval.node, yyvsp[-3].node);
		    }
break;
case 4:
#line 374 "parse.y"
{
			void_stmts(yyvsp[-1].node);
		        yyval.node = yyvsp[-1].node;
		    }
break;
case 6:
#line 382 "parse.y"
{
			yyval.node = newline_node(yyvsp[0].node);
		    }
break;
case 7:
#line 386 "parse.y"
{
			yyval.node = block_append(yyvsp[-2].node, newline_node(yyvsp[0].node));
		    }
break;
case 8:
#line 390 "parse.y"
{
			yyval.node = yyvsp[0].node;
		    }
break;
case 9:
#line 395 "parse.y"
{lex_state = EXPR_FNAME;}
break;
case 10:
#line 396 "parse.y"
{
		        yyval.node = NEW_ALIAS(yyvsp[-2].id, yyvsp[0].id);
		    }
break;
case 11:
#line 400 "parse.y"
{
		        yyval.node = NEW_VALIAS(yyvsp[-1].id, yyvsp[0].id);
		    }
break;
case 12:
#line 404 "parse.y"
{
			char buf[3];

			sprintf(buf, "$%c", (char)yyvsp[0].node->nd_nth);
		        yyval.node = NEW_VALIAS(yyvsp[-1].id, rb_intern(buf));
		    }
break;
case 13:
#line 411 "parse.y"
{
		        yyerror("can't make alias for the number variables");
		        yyval.node = 0;
		    }
break;
case 14:
#line 416 "parse.y"
{
			yyval.node = yyvsp[0].node;
		    }
break;
case 15:
#line 420 "parse.y"
{
			yyval.node = NEW_IF(cond(yyvsp[0].node), yyvsp[-2].node, 0);
		        fixpos(yyval.node, yyvsp[0].node);
			if (cond_negative(&yyval.node->nd_cond)) {
		            yyval.node->nd_else = yyval.node->nd_body;
		            yyval.node->nd_body = 0;
			}
		    }
break;
case 16:
#line 429 "parse.y"
{
			yyval.node = NEW_UNLESS(cond(yyvsp[0].node), yyvsp[-2].node, 0);
		        fixpos(yyval.node, yyvsp[0].node);
			if (cond_negative(&yyval.node->nd_cond)) {
		            yyval.node->nd_body = yyval.node->nd_else;
		            yyval.node->nd_else = 0;
			}
		    }
break;
case 17:
#line 438 "parse.y"
{
			if (yyvsp[-2].node && nd_type(yyvsp[-2].node) == NODE_BEGIN) {
			    yyval.node = NEW_WHILE(cond(yyvsp[0].node), yyvsp[-2].node->nd_body, 0);
			}
			else {
			    yyval.node = NEW_WHILE(cond(yyvsp[0].node), yyvsp[-2].node, 1);
			}
			if (cond_negative(&yyval.node->nd_cond)) {
			    nd_set_type(yyval.node, NODE_UNTIL);
			}
		    }
break;
case 18:
#line 450 "parse.y"
{
			if (yyvsp[-2].node && nd_type(yyvsp[-2].node) == NODE_BEGIN) {
			    yyval.node = NEW_UNTIL(cond(yyvsp[0].node), yyvsp[-2].node->nd_body, 0);
			}
			else {
			    yyval.node = NEW_UNTIL(cond(yyvsp[0].node), yyvsp[-2].node, 1);
			}
			if (cond_negative(&yyval.node->nd_cond)) {
			    nd_set_type(yyval.node, NODE_WHILE);
			}
		    }
break;
case 19:
#line 462 "parse.y"
{
			yyval.node = NEW_RESCUE(yyvsp[-2].node, NEW_RESBODY(0,yyvsp[0].node,0), 0);
		    }
break;
case 20:
#line 466 "parse.y"
{
			if (in_def || in_single) {
			    yyerror("BEGIN in method");
			}
			local_push(0);
		    }
break;
case 21:
#line 473 "parse.y"
{
			ruby_eval_tree_begin = block_append(ruby_eval_tree_begin,
						            NEW_PREEXE(yyvsp[-1].node));
		        local_pop();
		        yyval.node = 0;
		    }
break;
case 22:
#line 480 "parse.y"
{
			if (in_def || in_single) {
			    rb_warn("END in method; use at_exit");
			}

			yyval.node = NEW_ITER(0, NEW_POSTEXE(), yyvsp[-1].node);
		    }
break;
case 23:
#line 488 "parse.y"
{
			value_expr(yyvsp[0].node);
			yyval.node = node_assign(yyvsp[-2].node, yyvsp[0].node);
		    }
break;
case 24:
#line 493 "parse.y"
{
			value_expr(yyvsp[0].node);
			yyvsp[-2].node->nd_value = (yyvsp[-2].node->nd_head) ? NEW_TO_ARY(yyvsp[0].node) : NEW_ARRAY(yyvsp[0].node);
			yyval.node = yyvsp[-2].node;
		    }
break;
case 25:
#line 499 "parse.y"
{
			value_expr(yyvsp[0].node);
			if (yyvsp[-2].node) {
			    ID vid = yyvsp[-2].node->nd_vid;
			    if (yyvsp[-1].id == tOROP) {
				yyvsp[-2].node->nd_value = yyvsp[0].node;
				yyval.node = NEW_OP_ASGN_OR(gettable(vid), yyvsp[-2].node);
				if (is_asgn_or_id(vid)) {
				    yyval.node->nd_aid = vid;
				}
			    }
			    else if (yyvsp[-1].id == tANDOP) {
				yyvsp[-2].node->nd_value = yyvsp[0].node;
				yyval.node = NEW_OP_ASGN_AND(gettable(vid), yyvsp[-2].node);
			    }
			    else {
				yyval.node = yyvsp[-2].node;
				yyval.node->nd_value = call_op(gettable(vid),yyvsp[-1].id,1,yyvsp[0].node);
			    }
			}
			else {
			    yyval.node = 0;
			}
		    }
break;
case 26:
#line 524 "parse.y"
{
                        NODE *args;

			value_expr(yyvsp[0].node);
		        args = NEW_LIST(yyvsp[0].node);
			yyvsp[-3].node = list_append(yyvsp[-3].node, NEW_NIL());
			list_concat(args, yyvsp[-3].node);
			if (yyvsp[-1].id == tOROP) {
			    yyvsp[-1].id = 0;
			}
			else if (yyvsp[-1].id == tANDOP) {
			    yyvsp[-1].id = 1;
			}
			yyval.node = NEW_OP_ASGN1(yyvsp[-5].node, yyvsp[-1].id, args);
		        fixpos(yyval.node, yyvsp[-5].node);
		    }
break;
case 27:
#line 541 "parse.y"
{
			value_expr(yyvsp[0].node);
			if (yyvsp[-1].id == tOROP) {
			    yyvsp[-1].id = 0;
			}
			else if (yyvsp[-1].id == tANDOP) {
			    yyvsp[-1].id = 1;
			}
			yyval.node = NEW_OP_ASGN2(yyvsp[-4].node, yyvsp[-2].id, yyvsp[-1].id, yyvsp[0].node);
		        fixpos(yyval.node, yyvsp[-4].node);
		    }
break;
case 28:
#line 553 "parse.y"
{
			value_expr(yyvsp[0].node);
			if (yyvsp[-1].id == tOROP) {
			    yyvsp[-1].id = 0;
			}
			else if (yyvsp[-1].id == tANDOP) {
			    yyvsp[-1].id = 1;
			}
			yyval.node = NEW_OP_ASGN2(yyvsp[-4].node, yyvsp[-2].id, yyvsp[-1].id, yyvsp[0].node);
		        fixpos(yyval.node, yyvsp[-4].node);
		    }
break;
case 29:
#line 565 "parse.y"
{
			value_expr(yyvsp[0].node);
			if (yyvsp[-1].id == tOROP) {
			    yyvsp[-1].id = 0;
			}
			else if (yyvsp[-1].id == tANDOP) {
			    yyvsp[-1].id = 1;
			}
			yyval.node = NEW_OP_ASGN2(yyvsp[-4].node, yyvsp[-2].id, yyvsp[-1].id, yyvsp[0].node);
		        fixpos(yyval.node, yyvsp[-4].node);
		    }
break;
case 30:
#line 577 "parse.y"
{
		        rb_backref_error(yyvsp[-2].node);
			yyval.node = 0;
		    }
break;
case 31:
#line 582 "parse.y"
{
			yyval.node = node_assign(yyvsp[-2].node, NEW_SVALUE(yyvsp[0].node));
		    }
break;
case 32:
#line 586 "parse.y"
{
			yyvsp[-2].node->nd_value = (yyvsp[-2].node->nd_head) ? NEW_TO_ARY(yyvsp[0].node) : NEW_ARRAY(yyvsp[0].node);
			yyval.node = yyvsp[-2].node;
		    }
break;
case 33:
#line 591 "parse.y"
{
			yyvsp[-2].node->nd_value = yyvsp[0].node;
			yyval.node = yyvsp[-2].node;
		    }
break;
case 36:
#line 600 "parse.y"
{
			yyval.node = logop(NODE_AND, yyvsp[-2].node, yyvsp[0].node);
		    }
break;
case 37:
#line 604 "parse.y"
{
			yyval.node = logop(NODE_OR, yyvsp[-2].node, yyvsp[0].node);
		    }
break;
case 38:
#line 608 "parse.y"
{
			yyval.node = NEW_NOT(cond(yyvsp[0].node));
		    }
break;
case 39:
#line 612 "parse.y"
{
			yyval.node = NEW_NOT(cond(yyvsp[0].node));
		    }
break;
case 41:
#line 619 "parse.y"
{
			value_expr(yyval.node);
			yyval.node = yyvsp[0].node;
		    }
break;
case 44:
#line 628 "parse.y"
{
			yyval.node = NEW_RETURN(ret_args(yyvsp[0].node));
		    }
break;
case 45:
#line 632 "parse.y"
{
			yyval.node = NEW_BREAK(ret_args(yyvsp[0].node));
		    }
break;
case 46:
#line 636 "parse.y"
{
			yyval.node = NEW_NEXT(ret_args(yyvsp[0].node));
		    }
break;
case 48:
#line 643 "parse.y"
{
			yyval.node = new_call(yyvsp[-3].node, yyvsp[-1].id, yyvsp[0].node);
		    }
break;
case 49:
#line 647 "parse.y"
{
			yyval.node = new_call(yyvsp[-3].node, yyvsp[-1].id, yyvsp[0].node);
		    }
break;
case 50:
#line 653 "parse.y"
{
			yyval.vars = dyna_push();
			yyvsp[0].num = ruby_sourceline;
		    }
break;
case 51:
#line 657 "parse.y"
{yyval.vars = ruby_dyna_vars;}
break;
case 52:
#line 660 "parse.y"
{
			yyval.node = NEW_ITER(yyvsp[-3].node, 0, dyna_init(yyvsp[-1].node, yyvsp[-2].vars));
			nd_set_line(yyval.node, yyvsp[-5].num);
			dyna_pop(yyvsp[-4].vars);
		    }
break;
case 53:
#line 668 "parse.y"
{
			yyval.node = new_fcall(yyvsp[-1].id, yyvsp[0].node);
		        fixpos(yyval.node, yyvsp[0].node);
		   }
break;
case 54:
#line 673 "parse.y"
{
			yyval.node = new_fcall(yyvsp[-2].id, yyvsp[-1].node);
			if (yyvsp[0].node) {
			    if (nd_type(yyval.node) == NODE_BLOCK_PASS) {
				rb_compile_error("both block arg and actual block given");
			    }
			    yyvsp[0].node->nd_iter = yyval.node;
			    yyval.node = yyvsp[0].node;
			}
		        fixpos(yyval.node, yyvsp[-1].node);
		   }
break;
case 55:
#line 685 "parse.y"
{
			yyval.node = new_call(yyvsp[-3].node, yyvsp[-1].id, yyvsp[0].node);
		        fixpos(yyval.node, yyvsp[-3].node);
		    }
break;
case 56:
#line 690 "parse.y"
{
			yyval.node = new_call(yyvsp[-4].node, yyvsp[-2].id, yyvsp[-1].node);
			if (yyvsp[0].node) {
			    if (nd_type(yyval.node) == NODE_BLOCK_PASS) {
				rb_compile_error("both block arg and actual block given");
			    }
			    yyvsp[0].node->nd_iter = yyval.node;
			    yyval.node = yyvsp[0].node;
			}
		        fixpos(yyval.node, yyvsp[-4].node);
		   }
break;
case 57:
#line 702 "parse.y"
{
			yyval.node = new_call(yyvsp[-3].node, yyvsp[-1].id, yyvsp[0].node);
		        fixpos(yyval.node, yyvsp[-3].node);
		    }
break;
case 58:
#line 707 "parse.y"
{
			yyval.node = new_call(yyvsp[-4].node, yyvsp[-2].id, yyvsp[-1].node);
			if (yyvsp[0].node) {
			    if (nd_type(yyval.node) == NODE_BLOCK_PASS) {
				rb_compile_error("both block arg and actual block given");
			    }
			    yyvsp[0].node->nd_iter = yyval.node;
			    yyval.node = yyvsp[0].node;
			}
		        fixpos(yyval.node, yyvsp[-4].node);
		   }
break;
case 59:
#line 719 "parse.y"
{
			yyval.node = new_super(yyvsp[0].node);
		        fixpos(yyval.node, yyvsp[0].node);
		    }
break;
case 60:
#line 724 "parse.y"
{
			yyval.node = new_yield(yyvsp[0].node);
		        fixpos(yyval.node, yyvsp[0].node);
		    }
break;
case 62:
#line 732 "parse.y"
{
			yyval.node = yyvsp[-1].node;
		    }
break;
case 64:
#line 739 "parse.y"
{
			yyval.node = NEW_MASGN(NEW_LIST(yyvsp[-1].node), 0);
		    }
break;
case 65:
#line 745 "parse.y"
{
			yyval.node = NEW_MASGN(yyvsp[0].node, 0);
		    }
break;
case 66:
#line 749 "parse.y"
{
			yyval.node = NEW_MASGN(list_append(yyvsp[-1].node,yyvsp[0].node), 0);
		    }
break;
case 67:
#line 753 "parse.y"
{
			yyval.node = NEW_MASGN(yyvsp[-2].node, yyvsp[0].node);
		    }
break;
case 68:
#line 757 "parse.y"
{
			yyval.node = NEW_MASGN(yyvsp[-1].node, -1);
		    }
break;
case 69:
#line 761 "parse.y"
{
			yyval.node = NEW_MASGN(0, yyvsp[0].node);
		    }
break;
case 70:
#line 765 "parse.y"
{
			yyval.node = NEW_MASGN(0, -1);
		    }
break;
case 72:
#line 772 "parse.y"
{
			yyval.node = yyvsp[-1].node;
		    }
break;
case 73:
#line 778 "parse.y"
{
			yyval.node = NEW_LIST(yyvsp[-1].node);
		    }
break;
case 74:
#line 782 "parse.y"
{
			yyval.node = list_append(yyvsp[-2].node, yyvsp[-1].node);
		    }
break;
case 75:
#line 788 "parse.y"
{
			yyval.node = assignable(yyvsp[0].id, 0);
		    }
break;
case 76:
#line 792 "parse.y"
{
			yyval.node = aryset(yyvsp[-3].node, yyvsp[-1].node);
		    }
break;
case 77:
#line 796 "parse.y"
{
			yyval.node = attrset(yyvsp[-2].node, yyvsp[0].id);
		    }
break;
case 78:
#line 800 "parse.y"
{
			yyval.node = attrset(yyvsp[-2].node, yyvsp[0].id);
		    }
break;
case 79:
#line 804 "parse.y"
{
			yyval.node = attrset(yyvsp[-2].node, yyvsp[0].id);
		    }
break;
case 80:
#line 808 "parse.y"
{
			if (in_def || in_single)
			    yyerror("dynamic constant assignment");
			yyval.node = NEW_CDECL(0, 0, NEW_COLON2(yyvsp[-2].node, yyvsp[0].id));
		    }
break;
case 81:
#line 814 "parse.y"
{
			if (in_def || in_single)
			    yyerror("dynamic constant assignment");
			yyval.node = NEW_CDECL(0, 0, NEW_COLON3(yyvsp[0].id));
		    }
break;
case 82:
#line 820 "parse.y"
{
		        rb_backref_error(yyvsp[0].node);
			yyval.node = 0;
		    }
break;
case 83:
#line 827 "parse.y"
{
			yyval.node = assignable(yyvsp[0].id, 0);
		    }
break;
case 84:
#line 831 "parse.y"
{
			yyval.node = aryset(yyvsp[-3].node, yyvsp[-1].node);
		    }
break;
case 85:
#line 835 "parse.y"
{
			yyval.node = attrset(yyvsp[-2].node, yyvsp[0].id);
		    }
break;
case 86:
#line 839 "parse.y"
{
			yyval.node = attrset(yyvsp[-2].node, yyvsp[0].id);
		    }
break;
case 87:
#line 843 "parse.y"
{
			yyval.node = attrset(yyvsp[-2].node, yyvsp[0].id);
		    }
break;
case 88:
#line 847 "parse.y"
{
			if (in_def || in_single)
			    yyerror("dynamic constant assignment");
			yyval.node = NEW_CDECL(0, 0, NEW_COLON2(yyvsp[-2].node, yyvsp[0].id));
		    }
break;
case 89:
#line 853 "parse.y"
{
			if (in_def || in_single)
			    yyerror("dynamic constant assignment");
			yyval.node = NEW_CDECL(0, 0, NEW_COLON3(yyvsp[0].id));
		    }
break;
case 90:
#line 859 "parse.y"
{
		        rb_backref_error(yyvsp[0].node);
			yyval.node = 0;
		    }
break;
case 91:
#line 866 "parse.y"
{
			yyerror("class/module name must be CONSTANT");
		    }
break;
case 93:
#line 873 "parse.y"
{
			yyval.node = NEW_COLON3(yyvsp[0].id);
		    }
break;
case 94:
#line 877 "parse.y"
{
			yyval.node = NEW_COLON2(0, yyval.node);
		    }
break;
case 95:
#line 881 "parse.y"
{
			yyval.node = NEW_COLON2(yyvsp[-2].node, yyvsp[0].id);
		    }
break;
case 99:
#line 890 "parse.y"
{
			lex_state = EXPR_END;
			yyval.id = yyvsp[0].id;
		    }
break;
case 100:
#line 895 "parse.y"
{
			lex_state = EXPR_END;
			yyval.id = yyvsp[0].id;
		    }
break;
case 103:
#line 906 "parse.y"
{
			yyval.node = NEW_UNDEF(yyvsp[0].id);
		    }
break;
case 104:
#line 909 "parse.y"
{lex_state = EXPR_FNAME;}
break;
case 105:
#line 910 "parse.y"
{
			yyval.node = block_append(yyvsp[-3].node, NEW_UNDEF(yyvsp[0].id));
		    }
break;
case 106:
#line 915 "parse.y"
{ yyval.id = '|'; }
break;
case 107:
#line 916 "parse.y"
{ yyval.id = '^'; }
break;
case 108:
#line 917 "parse.y"
{ yyval.id = '&'; }
break;
case 109:
#line 918 "parse.y"
{ yyval.id = tCMP; }
break;
case 110:
#line 919 "parse.y"
{ yyval.id = tEQ; }
break;
case 111:
#line 920 "parse.y"
{ yyval.id = tEQQ; }
break;
case 112:
#line 921 "parse.y"
{ yyval.id = tMATCH; }
break;
case 113:
#line 922 "parse.y"
{ yyval.id = '>'; }
break;
case 114:
#line 923 "parse.y"
{ yyval.id = tGEQ; }
break;
case 115:
#line 924 "parse.y"
{ yyval.id = '<'; }
break;
case 116:
#line 925 "parse.y"
{ yyval.id = tLEQ; }
break;
case 117:
#line 926 "parse.y"
{ yyval.id = tLSHFT; }
break;
case 118:
#line 927 "parse.y"
{ yyval.id = tRSHFT; }
break;
case 119:
#line 928 "parse.y"
{ yyval.id = '+'; }
break;
case 120:
#line 929 "parse.y"
{ yyval.id = '-'; }
break;
case 121:
#line 930 "parse.y"
{ yyval.id = '*'; }
break;
case 122:
#line 931 "parse.y"
{ yyval.id = '*'; }
break;
case 123:
#line 932 "parse.y"
{ yyval.id = '/'; }
break;
case 124:
#line 933 "parse.y"
{ yyval.id = '%'; }
break;
case 125:
#line 934 "parse.y"
{ yyval.id = tPOW; }
break;
case 126:
#line 935 "parse.y"
{ yyval.id = '~'; }
break;
case 127:
#line 936 "parse.y"
{ yyval.id = tUPLUS; }
break;
case 128:
#line 937 "parse.y"
{ yyval.id = tUMINUS; }
break;
case 129:
#line 938 "parse.y"
{ yyval.id = tAREF; }
break;
case 130:
#line 939 "parse.y"
{ yyval.id = tASET; }
break;
case 131:
#line 940 "parse.y"
{ yyval.id = '`'; }
break;
case 173:
#line 953 "parse.y"
{
			yyval.node = node_assign(yyvsp[-2].node, yyvsp[0].node);
		    }
break;
case 174:
#line 957 "parse.y"
{
			yyval.node = node_assign(yyvsp[-4].node, NEW_RESCUE(yyvsp[-2].node, NEW_RESBODY(0,yyvsp[0].node,0), 0));
		    }
break;
case 175:
#line 961 "parse.y"
{
			value_expr(yyvsp[0].node);
			if (yyvsp[-2].node) {
			    ID vid = yyvsp[-2].node->nd_vid;
			    if (yyvsp[-1].id == tOROP) {
				yyvsp[-2].node->nd_value = yyvsp[0].node;
				yyval.node = NEW_OP_ASGN_OR(gettable(vid), yyvsp[-2].node);
				if (is_asgn_or_id(vid)) {
				    yyval.node->nd_aid = vid;
				}
			    }
			    else if (yyvsp[-1].id == tANDOP) {
				yyvsp[-2].node->nd_value = yyvsp[0].node;
				yyval.node = NEW_OP_ASGN_AND(gettable(vid), yyvsp[-2].node);
			    }
			    else {
				yyval.node = yyvsp[-2].node;
				yyval.node->nd_value = call_op(gettable(vid),yyvsp[-1].id,1,yyvsp[0].node);
			    }
			}
			else {
			    yyval.node = 0;
			}
		    }
break;
case 176:
#line 986 "parse.y"
{
                        NODE *args;

			value_expr(yyvsp[0].node);
			args = NEW_LIST(yyvsp[0].node);
			yyvsp[-3].node = list_append(yyvsp[-3].node, NEW_NIL());
			list_concat(args, yyvsp[-3].node);
			if (yyvsp[-1].id == tOROP) {
			    yyvsp[-1].id = 0;
			}
			else if (yyvsp[-1].id == tANDOP) {
			    yyvsp[-1].id = 1;
			}
			yyval.node = NEW_OP_ASGN1(yyvsp[-5].node, yyvsp[-1].id, args);
		        fixpos(yyval.node, yyvsp[-5].node);
		    }
break;
case 177:
#line 1003 "parse.y"
{
			value_expr(yyvsp[0].node);
			if (yyvsp[-1].id == tOROP) {
			    yyvsp[-1].id = 0;
			}
			else if (yyvsp[-1].id == tANDOP) {
			    yyvsp[-1].id = 1;
			}
			yyval.node = NEW_OP_ASGN2(yyvsp[-4].node, yyvsp[-2].id, yyvsp[-1].id, yyvsp[0].node);
		        fixpos(yyval.node, yyvsp[-4].node);
		    }
break;
case 178:
#line 1015 "parse.y"
{
			value_expr(yyvsp[0].node);
			if (yyvsp[-1].id == tOROP) {
			    yyvsp[-1].id = 0;
			}
			else if (yyvsp[-1].id == tANDOP) {
			    yyvsp[-1].id = 1;
			}
			yyval.node = NEW_OP_ASGN2(yyvsp[-4].node, yyvsp[-2].id, yyvsp[-1].id, yyvsp[0].node);
		        fixpos(yyval.node, yyvsp[-4].node);
		    }
break;
case 179:
#line 1027 "parse.y"
{
			value_expr(yyvsp[0].node);
			if (yyvsp[-1].id == tOROP) {
			    yyvsp[-1].id = 0;
			}
			else if (yyvsp[-1].id == tANDOP) {
			    yyvsp[-1].id = 1;
			}
			yyval.node = NEW_OP_ASGN2(yyvsp[-4].node, yyvsp[-2].id, yyvsp[-1].id, yyvsp[0].node);
		        fixpos(yyval.node, yyvsp[-4].node);
		    }
break;
case 180:
#line 1039 "parse.y"
{
			yyerror("constant re-assignment");
			yyval.node = 0;
		    }
break;
case 181:
#line 1044 "parse.y"
{
			yyerror("constant re-assignment");
			yyval.node = 0;
		    }
break;
case 182:
#line 1049 "parse.y"
{
		        rb_backref_error(yyvsp[-2].node);
			yyval.node = 0;
		    }
break;
case 183:
#line 1054 "parse.y"
{
			value_expr(yyvsp[-2].node);
			value_expr(yyvsp[0].node);
			yyval.node = NEW_DOT2(yyvsp[-2].node, yyvsp[0].node);
		    }
break;
case 184:
#line 1060 "parse.y"
{
			value_expr(yyvsp[-2].node);
			value_expr(yyvsp[0].node);
			yyval.node = NEW_DOT3(yyvsp[-2].node, yyvsp[0].node);
		    }
break;
case 185:
#line 1066 "parse.y"
{
			yyval.node = call_op(yyvsp[-2].node, '+', 1, yyvsp[0].node);
		    }
break;
case 186:
#line 1070 "parse.y"
{
		        yyval.node = call_op(yyvsp[-2].node, '-', 1, yyvsp[0].node);
		    }
break;
case 187:
#line 1074 "parse.y"
{
		        yyval.node = call_op(yyvsp[-2].node, '*', 1, yyvsp[0].node);
		    }
break;
case 188:
#line 1078 "parse.y"
{
			yyval.node = call_op(yyvsp[-2].node, '/', 1, yyvsp[0].node);
		    }
break;
case 189:
#line 1082 "parse.y"
{
			yyval.node = call_op(yyvsp[-2].node, '%', 1, yyvsp[0].node);
		    }
break;
case 190:
#line 1086 "parse.y"
{
			yyval.node = call_op(yyvsp[-2].node, tPOW, 1, yyvsp[0].node);
		    }
break;
case 191:
#line 1090 "parse.y"
{
			yyval.node = call_op(call_op(yyvsp[-2].node, tPOW, 1, yyvsp[0].node), tUMINUS, 0, 0);
		    }
break;
case 192:
#line 1094 "parse.y"
{
			yyval.node = call_op(call_op(yyvsp[-2].node, tPOW, 1, yyvsp[0].node), tUMINUS, 0, 0);
		    }
break;
case 193:
#line 1098 "parse.y"
{
			if (yyvsp[0].node && nd_type(yyvsp[0].node) == NODE_LIT) {
			    yyval.node = yyvsp[0].node;
			}
			else {
			    yyval.node = call_op(yyvsp[0].node, tUPLUS, 0, 0);
			}
		    }
break;
case 194:
#line 1107 "parse.y"
{
			yyval.node = call_op(yyvsp[0].node, tUMINUS, 0, 0);
		    }
break;
case 195:
#line 1111 "parse.y"
{
		        yyval.node = call_op(yyvsp[-2].node, '|', 1, yyvsp[0].node);
		    }
break;
case 196:
#line 1115 "parse.y"
{
			yyval.node = call_op(yyvsp[-2].node, '^', 1, yyvsp[0].node);
		    }
break;
case 197:
#line 1119 "parse.y"
{
			yyval.node = call_op(yyvsp[-2].node, '&', 1, yyvsp[0].node);
		    }
break;
case 198:
#line 1123 "parse.y"
{
			yyval.node = call_op(yyvsp[-2].node, tCMP, 1, yyvsp[0].node);
		    }
break;
case 199:
#line 1127 "parse.y"
{
			yyval.node = call_op(yyvsp[-2].node, '>', 1, yyvsp[0].node);
		    }
break;
case 200:
#line 1131 "parse.y"
{
			yyval.node = call_op(yyvsp[-2].node, tGEQ, 1, yyvsp[0].node);
		    }
break;
case 201:
#line 1135 "parse.y"
{
			yyval.node = call_op(yyvsp[-2].node, '<', 1, yyvsp[0].node);
		    }
break;
case 202:
#line 1139 "parse.y"
{
			yyval.node = call_op(yyvsp[-2].node, tLEQ, 1, yyvsp[0].node);
		    }
break;
case 203:
#line 1143 "parse.y"
{
			yyval.node = call_op(yyvsp[-2].node, tEQ, 1, yyvsp[0].node);
		    }
break;
case 204:
#line 1147 "parse.y"
{
			yyval.node = call_op(yyvsp[-2].node, tEQQ, 1, yyvsp[0].node);
		    }
break;
case 205:
#line 1151 "parse.y"
{
			yyval.node = NEW_NOT(call_op(yyvsp[-2].node, tEQ, 1, yyvsp[0].node));
		    }
break;
case 206:
#line 1155 "parse.y"
{
			yyval.node = match_gen(yyvsp[-2].node, yyvsp[0].node);
		    }
break;
case 207:
#line 1159 "parse.y"
{
			yyval.node = NEW_NOT(match_gen(yyvsp[-2].node, yyvsp[0].node));
		    }
break;
case 208:
#line 1163 "parse.y"
{
			yyval.node = NEW_NOT(cond(yyvsp[0].node));
		    }
break;
case 209:
#line 1167 "parse.y"
{
			yyval.node = call_op(yyvsp[0].node, '~', 0, 0);
		    }
break;
case 210:
#line 1171 "parse.y"
{
			yyval.node = call_op(yyvsp[-2].node, tLSHFT, 1, yyvsp[0].node);
		    }
break;
case 211:
#line 1175 "parse.y"
{
			yyval.node = call_op(yyvsp[-2].node, tRSHFT, 1, yyvsp[0].node);
		    }
break;
case 212:
#line 1179 "parse.y"
{
			yyval.node = logop(NODE_AND, yyvsp[-2].node, yyvsp[0].node);
		    }
break;
case 213:
#line 1183 "parse.y"
{
			yyval.node = logop(NODE_OR, yyvsp[-2].node, yyvsp[0].node);
		    }
break;
case 214:
#line 1186 "parse.y"
{in_defined = 1;}
break;
case 215:
#line 1187 "parse.y"
{
		        in_defined = 0;
			yyval.node = NEW_DEFINED(yyvsp[0].node);
		    }
break;
case 216:
#line 1192 "parse.y"
{
			yyval.node = NEW_IF(cond(yyvsp[-4].node), yyvsp[-2].node, yyvsp[0].node);
		        fixpos(yyval.node, yyvsp[-4].node);
		    }
break;
case 217:
#line 1197 "parse.y"
{
			yyval.node = yyvsp[0].node;
		    }
break;
case 218:
#line 1203 "parse.y"
{
			value_expr(yyvsp[0].node);
			yyval.node = yyvsp[0].node;
		    }
break;
case 220:
#line 1211 "parse.y"
{
		        rb_warn("parenthesize argument(s) for future version");
			yyval.node = NEW_LIST(yyvsp[-1].node);
		    }
break;
case 221:
#line 1216 "parse.y"
{
			yyval.node = yyvsp[-1].node;
		    }
break;
case 222:
#line 1220 "parse.y"
{
			value_expr(yyvsp[-1].node);
			yyval.node = arg_concat(yyvsp[-4].node, yyvsp[-1].node);
		    }
break;
case 223:
#line 1225 "parse.y"
{
			yyval.node = NEW_LIST(NEW_HASH(yyvsp[-1].node));
		    }
break;
case 224:
#line 1229 "parse.y"
{
			value_expr(yyvsp[-1].node);
			yyval.node = newline_node(NEW_SPLAT(yyvsp[-1].node));
		    }
break;
case 225:
#line 1236 "parse.y"
{
			yyval.node = yyvsp[-1].node;
		    }
break;
case 226:
#line 1240 "parse.y"
{
			yyval.node = yyvsp[-2].node;
		    }
break;
case 227:
#line 1244 "parse.y"
{
		        rb_warn("parenthesize argument for future version");
			yyval.node = NEW_LIST(yyvsp[-2].node);
		    }
break;
case 228:
#line 1249 "parse.y"
{
		        rb_warn("parenthesize argument for future version");
			yyval.node = list_append(yyvsp[-4].node, yyvsp[-2].node);
		    }
break;
case 231:
#line 1260 "parse.y"
{
		        rb_warn("parenthesize argument(s) for future version");
			yyval.node = NEW_LIST(yyvsp[0].node);
		    }
break;
case 232:
#line 1265 "parse.y"
{
			yyval.node = arg_blk_pass(yyvsp[-1].node, yyvsp[0].node);
		    }
break;
case 233:
#line 1269 "parse.y"
{
			yyval.node = arg_concat(yyvsp[-4].node, yyvsp[-1].node);
			yyval.node = arg_blk_pass(yyval.node, yyvsp[0].node);
		    }
break;
case 234:
#line 1274 "parse.y"
{
			yyval.node = NEW_LIST(NEW_HASH(yyvsp[-1].node));
			yyval.node = arg_blk_pass(yyval.node, yyvsp[0].node);
		    }
break;
case 235:
#line 1279 "parse.y"
{
			yyval.node = arg_concat(NEW_LIST(NEW_HASH(yyvsp[-4].node)), yyvsp[-1].node);
			yyval.node = arg_blk_pass(yyval.node, yyvsp[0].node);
		    }
break;
case 236:
#line 1284 "parse.y"
{
			yyval.node = list_append(yyvsp[-3].node, NEW_HASH(yyvsp[-1].node));
			yyval.node = arg_blk_pass(yyval.node, yyvsp[0].node);
		    }
break;
case 237:
#line 1289 "parse.y"
{
			value_expr(yyvsp[-1].node);
			yyval.node = arg_concat(list_append(yyvsp[-6].node, NEW_HASH(yyvsp[-4].node)), yyvsp[-1].node);
			yyval.node = arg_blk_pass(yyval.node, yyvsp[0].node);
		    }
break;
case 238:
#line 1295 "parse.y"
{
			yyval.node = arg_blk_pass(NEW_SPLAT(yyvsp[-1].node), yyvsp[0].node);
		    }
break;
case 240:
#line 1302 "parse.y"
{
			yyval.node = arg_blk_pass(list_concat(NEW_LIST(yyvsp[-3].node),yyvsp[-1].node), yyvsp[0].node);
		    }
break;
case 241:
#line 1306 "parse.y"
{
                        yyval.node = arg_blk_pass(yyvsp[-2].node, yyvsp[0].node);
                    }
break;
case 242:
#line 1310 "parse.y"
{
			yyval.node = arg_concat(NEW_LIST(yyvsp[-4].node), yyvsp[-1].node);
			yyval.node = arg_blk_pass(yyval.node, yyvsp[0].node);
		    }
break;
case 243:
#line 1315 "parse.y"
{
                       yyval.node = arg_concat(list_concat(NEW_LIST(yyvsp[-6].node),yyvsp[-4].node), yyvsp[-1].node);
			yyval.node = arg_blk_pass(yyval.node, yyvsp[0].node);
		    }
break;
case 244:
#line 1320 "parse.y"
{
			yyval.node = NEW_LIST(NEW_HASH(yyvsp[-1].node));
			yyval.node = arg_blk_pass(yyval.node, yyvsp[0].node);
		    }
break;
case 245:
#line 1325 "parse.y"
{
			yyval.node = arg_concat(NEW_LIST(NEW_HASH(yyvsp[-4].node)), yyvsp[-1].node);
			yyval.node = arg_blk_pass(yyval.node, yyvsp[0].node);
		    }
break;
case 246:
#line 1330 "parse.y"
{
			yyval.node = list_append(NEW_LIST(yyvsp[-3].node), NEW_HASH(yyvsp[-1].node));
			yyval.node = arg_blk_pass(yyval.node, yyvsp[0].node);
		    }
break;
case 247:
#line 1335 "parse.y"
{
			yyval.node = list_append(list_concat(NEW_LIST(yyvsp[-5].node),yyvsp[-3].node), NEW_HASH(yyvsp[-1].node));
			yyval.node = arg_blk_pass(yyval.node, yyvsp[0].node);
		    }
break;
case 248:
#line 1340 "parse.y"
{
			yyval.node = arg_concat(list_append(NEW_LIST(yyvsp[-6].node), NEW_HASH(yyvsp[-4].node)), yyvsp[-1].node);
			yyval.node = arg_blk_pass(yyval.node, yyvsp[0].node);
		    }
break;
case 249:
#line 1345 "parse.y"
{
			yyval.node = arg_concat(list_append(list_concat(NEW_LIST(yyvsp[-8].node), yyvsp[-6].node), NEW_HASH(yyvsp[-4].node)), yyvsp[-1].node);
			yyval.node = arg_blk_pass(yyval.node, yyvsp[0].node);
		    }
break;
case 250:
#line 1350 "parse.y"
{
			yyval.node = arg_blk_pass(NEW_SPLAT(yyvsp[-1].node), yyvsp[0].node);
		    }
break;
case 252:
#line 1356 "parse.y"
{
			yyval.num = cmdarg_stack;
			CMDARG_PUSH(1);
		    }
break;
case 253:
#line 1361 "parse.y"
{
			/* CMDARG_POP() */
		        cmdarg_stack = yyvsp[-1].num;
			yyval.node = yyvsp[0].node;
		    }
break;
case 255:
#line 1369 "parse.y"
{lex_state = EXPR_ENDARG;}
break;
case 256:
#line 1370 "parse.y"
{
		        rb_warn("don't put space before argument parentheses");
			yyval.node = 0;
		    }
break;
case 257:
#line 1374 "parse.y"
{lex_state = EXPR_ENDARG;}
break;
case 258:
#line 1375 "parse.y"
{
		        rb_warn("don't put space before argument parentheses");
			yyval.node = yyvsp[-2].node;
		    }
break;
case 259:
#line 1382 "parse.y"
{
			yyval.node = NEW_BLOCK_PASS(yyvsp[0].node);
		    }
break;
case 260:
#line 1388 "parse.y"
{
			yyval.node = yyvsp[0].node;
		    }
break;
case 262:
#line 1395 "parse.y"
{
			yyval.node = NEW_LIST(yyvsp[0].node);
		    }
break;
case 263:
#line 1399 "parse.y"
{
			yyval.node = list_append(yyvsp[-2].node, yyvsp[0].node);
		    }
break;
case 264:
#line 1405 "parse.y"
{
			yyval.node = list_append(yyvsp[-2].node, yyvsp[0].node);
		    }
break;
case 265:
#line 1409 "parse.y"
{
			yyval.node = arg_concat(yyvsp[-3].node, yyvsp[0].node);
		    }
break;
case 266:
#line 1413 "parse.y"
{
			yyval.node = NEW_SPLAT(yyvsp[0].node);
		    }
break;
case 275:
#line 1427 "parse.y"
{
			yyval.node = NEW_FCALL(yyvsp[0].id, 0);
		    }
break;
case 276:
#line 1431 "parse.y"
{
			yyvsp[0].num = ruby_sourceline;
		    }
break;
case 277:
#line 1436 "parse.y"
{
			yyval.node = NEW_BEGIN(yyvsp[-1].node);
			nd_set_line(yyval.node, yyvsp[-3].num);
		    }
break;
case 278:
#line 1440 "parse.y"
{lex_state = EXPR_ENDARG;}
break;
case 279:
#line 1441 "parse.y"
{
		        rb_warning("(...) interpreted as grouped expression");
			yyval.node = yyvsp[-3].node;
		    }
break;
case 280:
#line 1446 "parse.y"
{
			yyval.node = yyvsp[-1].node;
		    }
break;
case 281:
#line 1450 "parse.y"
{
			yyval.node = NEW_COLON2(yyvsp[-2].node, yyvsp[0].id);
		    }
break;
case 282:
#line 1454 "parse.y"
{
			yyval.node = NEW_COLON3(yyvsp[0].id);
		    }
break;
case 283:
#line 1458 "parse.y"
{
			if (yyvsp[-3].node && nd_type(yyvsp[-3].node) == NODE_SELF)
			    yyval.node = NEW_FCALL(tAREF, yyvsp[-1].node);
			else
			    yyval.node = NEW_CALL(yyvsp[-3].node, tAREF, yyvsp[-1].node);
			fixpos(yyval.node, yyvsp[-3].node);
		    }
break;
case 284:
#line 1466 "parse.y"
{
		        if (yyvsp[-1].node == 0) {
			    yyval.node = NEW_ZARRAY(); /* zero length array*/
			}
			else {
			    yyval.node = yyvsp[-1].node;
			}
		    }
break;
case 285:
#line 1475 "parse.y"
{
			yyval.node = NEW_HASH(yyvsp[-1].node);
		    }
break;
case 286:
#line 1479 "parse.y"
{
			yyval.node = NEW_RETURN(0);
		    }
break;
case 287:
#line 1483 "parse.y"
{
			yyval.node = new_yield(yyvsp[-1].node);
		    }
break;
case 288:
#line 1487 "parse.y"
{
			yyval.node = NEW_YIELD(0, Qfalse);
		    }
break;
case 289:
#line 1491 "parse.y"
{
			yyval.node = NEW_YIELD(0, Qfalse);
		    }
break;
case 290:
#line 1494 "parse.y"
{in_defined = 1;}
break;
case 291:
#line 1495 "parse.y"
{
		        in_defined = 0;
			yyval.node = NEW_DEFINED(yyvsp[-1].node);
		    }
break;
case 292:
#line 1500 "parse.y"
{
			yyvsp[0].node->nd_iter = NEW_FCALL(yyvsp[-1].id, 0);
			yyval.node = yyvsp[0].node;
			fixpos(yyvsp[0].node->nd_iter, yyvsp[0].node);
		    }
break;
case 294:
#line 1507 "parse.y"
{
			if (yyvsp[-1].node && nd_type(yyvsp[-1].node) == NODE_BLOCK_PASS) {
			    rb_compile_error("both block arg and actual block given");
			}
			yyvsp[0].node->nd_iter = yyvsp[-1].node;
			yyval.node = yyvsp[0].node;
		        fixpos(yyval.node, yyvsp[-1].node);
		    }
break;
case 295:
#line 1519 "parse.y"
{
			yyval.node = NEW_IF(cond(yyvsp[-4].node), yyvsp[-2].node, yyvsp[-1].node);
		        fixpos(yyval.node, yyvsp[-4].node);
			if (cond_negative(&yyval.node->nd_cond)) {
		            NODE *tmp = yyval.node->nd_body;
		            yyval.node->nd_body = yyval.node->nd_else;
		            yyval.node->nd_else = tmp;
			}
		    }
break;
case 296:
#line 1532 "parse.y"
{
			yyval.node = NEW_UNLESS(cond(yyvsp[-4].node), yyvsp[-2].node, yyvsp[-1].node);
		        fixpos(yyval.node, yyvsp[-4].node);
			if (cond_negative(&yyval.node->nd_cond)) {
		            NODE *tmp = yyval.node->nd_body;
		            yyval.node->nd_body = yyval.node->nd_else;
		            yyval.node->nd_else = tmp;
			}
		    }
break;
case 297:
#line 1541 "parse.y"
{COND_PUSH(1);}
break;
case 298:
#line 1541 "parse.y"
{COND_POP();}
break;
case 299:
#line 1544 "parse.y"
{
			yyval.node = NEW_WHILE(cond(yyvsp[-4].node), yyvsp[-1].node, 1);
		        fixpos(yyval.node, yyvsp[-4].node);
			if (cond_negative(&yyval.node->nd_cond)) {
			    nd_set_type(yyval.node, NODE_UNTIL);
			}
		    }
break;
case 300:
#line 1551 "parse.y"
{COND_PUSH(1);}
break;
case 301:
#line 1551 "parse.y"
{COND_POP();}
break;
case 302:
#line 1554 "parse.y"
{
			yyval.node = NEW_UNTIL(cond(yyvsp[-4].node), yyvsp[-1].node, 1);
		        fixpos(yyval.node, yyvsp[-4].node);
			if (cond_negative(&yyval.node->nd_cond)) {
			    nd_set_type(yyval.node, NODE_WHILE);
			}
		    }
break;
case 303:
#line 1564 "parse.y"
{
			yyval.node = NEW_CASE(yyvsp[-3].node, yyvsp[-1].node);
		        fixpos(yyval.node, yyvsp[-3].node);
		    }
break;
case 304:
#line 1569 "parse.y"
{
			yyval.node = block_append(yyvsp[-4].node, yyvsp[-1].node);
		    }
break;
case 305:
#line 1573 "parse.y"
{
			yyval.node = yyvsp[-1].node;
		    }
break;
case 306:
#line 1577 "parse.y"
{
			yyval.node = yyvsp[-1].node;
		    }
break;
case 307:
#line 1580 "parse.y"
{COND_PUSH(1);}
break;
case 308:
#line 1580 "parse.y"
{COND_POP();}
break;
case 309:
#line 1583 "parse.y"
{
			yyval.node = NEW_FOR(yyvsp[-7].node, yyvsp[-4].node, yyvsp[-1].node);
		        fixpos(yyval.node, yyvsp[-7].node);
		    }
break;
case 310:
#line 1588 "parse.y"
{
			if (in_def || in_single)
			    yyerror("class definition in method body");
			class_nest++;
			local_push(0);
		        yyval.num = ruby_sourceline;
		    }
break;
case 311:
#line 1597 "parse.y"
{
		        yyval.node = NEW_CLASS(yyvsp[-4].node, yyvsp[-1].node, yyvsp[-3].node);
		        nd_set_line(yyval.node, yyvsp[-2].num);
		        local_pop();
			class_nest--;
		    }
break;
case 312:
#line 1604 "parse.y"
{
			yyval.num = in_def;
		        in_def = 0;
		    }
break;
case 313:
#line 1609 "parse.y"
{
		        yyval.num = in_single;
		        in_single = 0;
			class_nest++;
			local_push(0);
		    }
break;
case 314:
#line 1617 "parse.y"
{
		        yyval.node = NEW_SCLASS(yyvsp[-5].node, yyvsp[-1].node);
		        fixpos(yyval.node, yyvsp[-5].node);
		        local_pop();
			class_nest--;
		        in_def = yyvsp[-4].num;
		        in_single = yyvsp[-2].num;
		    }
break;
case 315:
#line 1626 "parse.y"
{
			if (in_def || in_single)
			    yyerror("module definition in method body");
			class_nest++;
			local_push(0);
		        yyval.num = ruby_sourceline;
		    }
break;
case 316:
#line 1635 "parse.y"
{
		        yyval.node = NEW_MODULE(yyvsp[-3].node, yyvsp[-1].node);
		        nd_set_line(yyval.node, yyvsp[-2].num);
		        local_pop();
			class_nest--;
		    }
break;
case 317:
#line 1642 "parse.y"
{
			yyval.id = cur_mid;
			cur_mid = yyvsp[0].id;
			in_def++;
			local_push(0);
		    }
break;
case 318:
#line 1651 "parse.y"
{
			yyval.node = NEW_DEFN(yyvsp[-4].id, yyvsp[-2].node, yyvsp[-1].node, NOEX_PRIVATE);
		        fixpos(yyval.node, yyvsp[-2].node);
		        local_pop();
			in_def--;
			cur_mid = yyvsp[-3].id;
		    }
break;
case 319:
#line 1658 "parse.y"
{lex_state = EXPR_FNAME;}
break;
case 320:
#line 1659 "parse.y"
{
			in_single++;
			local_push(0);
		        lex_state = EXPR_END; /* force for args */
		    }
break;
case 321:
#line 1667 "parse.y"
{
			yyval.node = NEW_DEFS(yyvsp[-7].node, yyvsp[-4].id, yyvsp[-2].node, yyvsp[-1].node);
		        fixpos(yyval.node, yyvsp[-7].node);
		        local_pop();
			in_single--;
		    }
break;
case 322:
#line 1674 "parse.y"
{
			yyval.node = NEW_BREAK(0);
		    }
break;
case 323:
#line 1678 "parse.y"
{
			yyval.node = NEW_NEXT(0);
		    }
break;
case 324:
#line 1682 "parse.y"
{
			yyval.node = NEW_REDO();
		    }
break;
case 325:
#line 1686 "parse.y"
{
			yyval.node = NEW_RETRY();
		    }
break;
case 326:
#line 1692 "parse.y"
{
			value_expr(yyvsp[0].node);
			yyval.node = yyvsp[0].node;
		    }
break;
case 335:
#line 1713 "parse.y"
{
			yyval.node = NEW_IF(cond(yyvsp[-3].node), yyvsp[-1].node, yyvsp[0].node);
		        fixpos(yyval.node, yyvsp[-3].node);
		    }
break;
case 337:
#line 1721 "parse.y"
{
			yyval.node = yyvsp[0].node;
		    }
break;
case 341:
#line 1732 "parse.y"
{
			yyval.node = (NODE*)1;
		    }
break;
case 342:
#line 1736 "parse.y"
{
			yyval.node = (NODE*)1;
		    }
break;
case 343:
#line 1740 "parse.y"
{
			yyval.node = yyvsp[-1].node;
		    }
break;
case 344:
#line 1746 "parse.y"
{
		        yyval.vars = dyna_push();
			yyvsp[0].num = ruby_sourceline;
		    }
break;
case 345:
#line 1750 "parse.y"
{yyval.vars = ruby_dyna_vars;}
break;
case 346:
#line 1753 "parse.y"
{
			yyval.node = NEW_ITER(yyvsp[-3].node, 0, dyna_init(yyvsp[-1].node, yyvsp[-2].vars));
			nd_set_line(yyval.node, yyvsp[-5].num);
			dyna_pop(yyvsp[-4].vars);
		    }
break;
case 347:
#line 1761 "parse.y"
{
			if (yyvsp[-1].node && nd_type(yyvsp[-1].node) == NODE_BLOCK_PASS) {
			    rb_compile_error("both block arg and actual block given");
			}
			yyvsp[0].node->nd_iter = yyvsp[-1].node;
			yyval.node = yyvsp[0].node;
		        fixpos(yyval.node, yyvsp[-1].node);
		    }
break;
case 348:
#line 1770 "parse.y"
{
			yyval.node = new_call(yyvsp[-3].node, yyvsp[-1].id, yyvsp[0].node);
		    }
break;
case 349:
#line 1774 "parse.y"
{
			yyval.node = new_call(yyvsp[-3].node, yyvsp[-1].id, yyvsp[0].node);
		    }
break;
case 350:
#line 1780 "parse.y"
{
			yyval.node = new_fcall(yyvsp[-1].id, yyvsp[0].node);
		        fixpos(yyval.node, yyvsp[0].node);
		    }
break;
case 351:
#line 1785 "parse.y"
{
			yyval.node = new_call(yyvsp[-3].node, yyvsp[-1].id, yyvsp[0].node);
		        fixpos(yyval.node, yyvsp[-3].node);
		    }
break;
case 352:
#line 1790 "parse.y"
{
			yyval.node = new_call(yyvsp[-3].node, yyvsp[-1].id, yyvsp[0].node);
		        fixpos(yyval.node, yyvsp[-3].node);
		    }
break;
case 353:
#line 1795 "parse.y"
{
			yyval.node = new_call(yyvsp[-2].node, yyvsp[0].id, 0);
		    }
break;
case 354:
#line 1799 "parse.y"
{
			yyval.node = new_super(yyvsp[0].node);
		    }
break;
case 355:
#line 1803 "parse.y"
{
			yyval.node = NEW_ZSUPER();
		    }
break;
case 356:
#line 1809 "parse.y"
{
		        yyval.vars = dyna_push();
			yyvsp[0].num = ruby_sourceline;
		    }
break;
case 357:
#line 1813 "parse.y"
{yyval.vars = ruby_dyna_vars;}
break;
case 358:
#line 1815 "parse.y"
{
			yyval.node = NEW_ITER(yyvsp[-3].node, 0, dyna_init(yyvsp[-1].node, yyvsp[-2].vars));
			nd_set_line(yyval.node, yyvsp[-5].num);
			dyna_pop(yyvsp[-4].vars);
		    }
break;
case 359:
#line 1821 "parse.y"
{
		        yyval.vars = dyna_push();
			yyvsp[0].num = ruby_sourceline;
		    }
break;
case 360:
#line 1825 "parse.y"
{yyval.vars = ruby_dyna_vars;}
break;
case 361:
#line 1827 "parse.y"
{
			yyval.node = NEW_ITER(yyvsp[-3].node, 0, dyna_init(yyvsp[-1].node, yyvsp[-2].vars));
			nd_set_line(yyval.node, yyvsp[-5].num);
			dyna_pop(yyvsp[-4].vars);
		    }
break;
case 362:
#line 1837 "parse.y"
{
			yyval.node = NEW_WHEN(yyvsp[-3].node, yyvsp[-1].node, yyvsp[0].node);
		    }
break;
case 364:
#line 1843 "parse.y"
{
			yyval.node = list_append(yyvsp[-3].node, NEW_WHEN(yyvsp[0].node, 0, 0));
		    }
break;
case 365:
#line 1847 "parse.y"
{
			yyval.node = NEW_LIST(NEW_WHEN(yyvsp[0].node, 0, 0));
		    }
break;
case 368:
#line 1859 "parse.y"
{
		        if (yyvsp[-3].node) {
		            yyvsp[-3].node = node_assign(yyvsp[-3].node, NEW_ERRINFO());
			    yyvsp[-1].node = block_append(yyvsp[-3].node, yyvsp[-1].node);
			}
			yyval.node = NEW_RESBODY(yyvsp[-4].node, yyvsp[-1].node, yyvsp[0].node);
		        fixpos(yyval.node, yyvsp[-4].node?yyvsp[-4].node:yyvsp[-1].node);
		    }
break;
case 370:
#line 1871 "parse.y"
{
			yyval.node = NEW_LIST(yyvsp[0].node);
		    }
break;
case 373:
#line 1879 "parse.y"
{
			yyval.node = yyvsp[0].node;
		    }
break;
case 375:
#line 1886 "parse.y"
{
			if (yyvsp[0].node)
			    yyval.node = yyvsp[0].node;
			else
			    /* place holder */
			    yyval.node = NEW_NIL();
		    }
break;
case 378:
#line 1898 "parse.y"
{
			yyval.node = NEW_LIT(ID2SYM(yyvsp[0].id));
		    }
break;
case 380:
#line 1905 "parse.y"
{
			NODE *node = yyvsp[0].node;
			if (!node) {
			    node = NEW_STR(rb_str_new(0, 0));
			}
			else {
			    node = evstr2dstr(node);
			}
			yyval.node = node;
		    }
break;
case 382:
#line 1919 "parse.y"
{
			yyval.node = literal_concat(yyvsp[-1].node, yyvsp[0].node);
		    }
break;
case 383:
#line 1925 "parse.y"
{
			yyval.node = yyvsp[-1].node;
		    }
break;
case 384:
#line 1931 "parse.y"
{
			NODE *node = yyvsp[-1].node;
			if (!node) {
			    node = NEW_XSTR(rb_str_new(0, 0));
			}
			else {
			    switch (nd_type(node)) {
			      case NODE_STR:
				nd_set_type(node, NODE_XSTR);
				break;
			      case NODE_DSTR:
				nd_set_type(node, NODE_DXSTR);
				break;
			      default:
				node = NEW_NODE(NODE_DXSTR, rb_str_new(0, 0), 1, NEW_LIST(node));
				break;
			    }
			}
			yyval.node = node;
		    }
break;
case 385:
#line 1954 "parse.y"
{
			int options = yyvsp[0].num;
			NODE *node = yyvsp[-1].node;
			if (!node) {
			    node = NEW_LIT(rb_reg_new("", 0, options & ~RE_OPTION_ONCE));
			}
			else switch (nd_type(node)) {
			  case NODE_STR:
			    {
				VALUE src = node->nd_lit;
				nd_set_type(node, NODE_LIT);
				node->nd_lit = rb_reg_new(RSTRING(src)->ptr,
							  RSTRING(src)->len,
							  options & ~RE_OPTION_ONCE);
			    }
			    break;
			  default:
			    node = NEW_NODE(NODE_DSTR, rb_str_new(0, 0), 1, NEW_LIST(node));
			  case NODE_DSTR:
			    if (options & RE_OPTION_ONCE) {
				nd_set_type(node, NODE_DREGX_ONCE);
			    }
			    else {
				nd_set_type(node, NODE_DREGX);
			    }
			    node->nd_cflag = options & ~RE_OPTION_ONCE;
			    break;
			}
			yyval.node = node;
		    }
break;
case 386:
#line 1987 "parse.y"
{
			yyval.node = NEW_ZARRAY();
		    }
break;
case 387:
#line 1991 "parse.y"
{
			yyval.node = yyvsp[-1].node;
		    }
break;
case 388:
#line 1997 "parse.y"
{
			yyval.node = 0;
		    }
break;
case 389:
#line 2001 "parse.y"
{
			yyval.node = list_append(yyvsp[-2].node, evstr2dstr(yyvsp[-1].node));
		    }
break;
case 391:
#line 2008 "parse.y"
{
			yyval.node = literal_concat(yyvsp[-1].node, yyvsp[0].node);
		    }
break;
case 392:
#line 2014 "parse.y"
{
			yyval.node = NEW_ZARRAY();
		    }
break;
case 393:
#line 2018 "parse.y"
{
			yyval.node = yyvsp[-1].node;
		    }
break;
case 394:
#line 2024 "parse.y"
{
			yyval.node = 0;
		    }
break;
case 395:
#line 2028 "parse.y"
{
			yyval.node = list_append(yyvsp[-2].node, yyvsp[-1].node);
		    }
break;
case 396:
#line 2034 "parse.y"
{
			yyval.node = 0;
		    }
break;
case 397:
#line 2038 "parse.y"
{
			yyval.node = literal_concat(yyvsp[-1].node, yyvsp[0].node);
		    }
break;
case 398:
#line 2044 "parse.y"
{
			yyval.node = 0;
		    }
break;
case 399:
#line 2048 "parse.y"
{
			yyval.node = literal_concat(yyvsp[-1].node, yyvsp[0].node);
		    }
break;
case 401:
#line 2055 "parse.y"
{
			yyval.node = lex_strterm;
			lex_strterm = 0;
			lex_state = EXPR_BEG;
		    }
break;
case 402:
#line 2061 "parse.y"
{
			lex_strterm = yyvsp[-1].node;
		        yyval.node = NEW_EVSTR(yyvsp[0].node);
		    }
break;
case 403:
#line 2066 "parse.y"
{
			yyval.node = lex_strterm;
			lex_strterm = 0;
			lex_state = EXPR_BEG;
			COND_PUSH(0);
			CMDARG_PUSH(0);
		    }
break;
case 404:
#line 2074 "parse.y"
{
			lex_strterm = yyvsp[-2].node;
			COND_LEXPOP();
			CMDARG_LEXPOP();
			yyval.node = new_evstr(yyvsp[-1].node);
		    }
break;
case 405:
#line 2082 "parse.y"
{yyval.node = NEW_GVAR(yyvsp[0].id);}
break;
case 406:
#line 2083 "parse.y"
{yyval.node = NEW_IVAR(yyvsp[0].id);}
break;
case 407:
#line 2084 "parse.y"
{yyval.node = NEW_CVAR(yyvsp[0].id);}
break;
case 409:
#line 2089 "parse.y"
{
		        lex_state = EXPR_END;
			yyval.id = yyvsp[0].id;
		    }
break;
case 414:
#line 2102 "parse.y"
{
		        lex_state = EXPR_END;
			if (!(yyval.node = yyvsp[-1].node)) {
			    yyerror("empty symbol literal");
			}
			else {
			    switch (nd_type(yyval.node)) {
			      case NODE_DSTR:
				nd_set_type(yyval.node, NODE_DSYM);
				break;
			      case NODE_STR:
				if (strlen(RSTRING(yyval.node->nd_lit)->ptr) == RSTRING(yyval.node->nd_lit)->len) {
				    yyval.node->nd_lit = ID2SYM(rb_intern(RSTRING(yyval.node->nd_lit)->ptr));
				    nd_set_type(yyval.node, NODE_LIT);
				    break;
				}
				/* fall through */
			      default:
				yyval.node = NEW_NODE(NODE_DSYM, rb_str_new(0, 0), 1, NEW_LIST(yyval.node));
				break;
			    }
			}
		    }
break;
case 417:
#line 2130 "parse.y"
{
			yyval.node = negate_lit(yyvsp[0].node);
		    }
break;
case 418:
#line 2134 "parse.y"
{
			yyval.node = negate_lit(yyvsp[0].node);
		    }
break;
case 424:
#line 2144 "parse.y"
{yyval.id = kNIL;}
break;
case 425:
#line 2145 "parse.y"
{yyval.id = kSELF;}
break;
case 426:
#line 2146 "parse.y"
{yyval.id = kTRUE;}
break;
case 427:
#line 2147 "parse.y"
{yyval.id = kFALSE;}
break;
case 428:
#line 2148 "parse.y"
{yyval.id = k__FILE__;}
break;
case 429:
#line 2149 "parse.y"
{yyval.id = k__LINE__;}
break;
case 430:
#line 2153 "parse.y"
{
			yyval.node = gettable(yyvsp[0].id);
		    }
break;
case 431:
#line 2159 "parse.y"
{
			yyval.node = assignable(yyvsp[0].id, 0);
		    }
break;
case 434:
#line 2169 "parse.y"
{
			yyval.node = 0;
		    }
break;
case 435:
#line 2173 "parse.y"
{
			lex_state = EXPR_BEG;
		    }
break;
case 436:
#line 2177 "parse.y"
{
			yyval.node = yyvsp[-1].node;
		    }
break;
case 437:
#line 2180 "parse.y"
{yyerrok; yyval.node = 0;}
break;
case 438:
#line 2184 "parse.y"
{
			yyval.node = yyvsp[-2].node;
			lex_state = EXPR_BEG;
		    }
break;
case 439:
#line 2189 "parse.y"
{
			yyval.node = yyvsp[-1].node;
		    }
break;
case 440:
#line 2195 "parse.y"
{
			yyval.node = block_append(NEW_ARGS(yyvsp[-5].num, yyvsp[-3].node, yyvsp[-1].id), yyvsp[0].node);
		    }
break;
case 441:
#line 2199 "parse.y"
{
			yyval.node = block_append(NEW_ARGS(yyvsp[-3].num, yyvsp[-1].node, -1), yyvsp[0].node);
		    }
break;
case 442:
#line 2203 "parse.y"
{
			yyval.node = block_append(NEW_ARGS(yyvsp[-3].num, 0, yyvsp[-1].id), yyvsp[0].node);
		    }
break;
case 443:
#line 2207 "parse.y"
{
			yyval.node = block_append(NEW_ARGS(yyvsp[-1].num, 0, -1), yyvsp[0].node);
		    }
break;
case 444:
#line 2211 "parse.y"
{
			yyval.node = block_append(NEW_ARGS(0, yyvsp[-3].node, yyvsp[-1].id), yyvsp[0].node);
		    }
break;
case 445:
#line 2215 "parse.y"
{
			yyval.node = block_append(NEW_ARGS(0, yyvsp[-1].node, -1), yyvsp[0].node);
		    }
break;
case 446:
#line 2219 "parse.y"
{
			yyval.node = block_append(NEW_ARGS(0, 0, yyvsp[-1].id), yyvsp[0].node);
		    }
break;
case 447:
#line 2223 "parse.y"
{
			yyval.node = block_append(NEW_ARGS(0, 0, -1), yyvsp[0].node);
		    }
break;
case 448:
#line 2227 "parse.y"
{
			yyval.node = NEW_ARGS(0, 0, -1);
		    }
break;
case 449:
#line 2233 "parse.y"
{
			yyerror("formal argument cannot be a constant");
		    }
break;
case 450:
#line 2237 "parse.y"
{
                        yyerror("formal argument cannot be an instance variable");
		    }
break;
case 451:
#line 2241 "parse.y"
{
                        yyerror("formal argument cannot be a global variable");
		    }
break;
case 452:
#line 2245 "parse.y"
{
                        yyerror("formal argument cannot be a class variable");
		    }
break;
case 453:
#line 2249 "parse.y"
{
			if (!is_local_id(yyvsp[0].id))
			    yyerror("formal argument must be local variable");
			else if (local_id(yyvsp[0].id))
			    yyerror("duplicate argument name");
			local_cnt(yyvsp[0].id);
			yyval.num = 1;
		    }
break;
case 455:
#line 2261 "parse.y"
{
			yyval.num += 1;
		    }
break;
case 456:
#line 2267 "parse.y"
{
			if (!is_local_id(yyvsp[-2].id))
			    yyerror("formal argument must be local variable");
			else if (local_id(yyvsp[-2].id))
			    yyerror("duplicate optional argument name");
			yyval.node = assignable(yyvsp[-2].id, yyvsp[0].node);
		    }
break;
case 457:
#line 2277 "parse.y"
{
			yyval.node = NEW_BLOCK(yyvsp[0].node);
			yyval.node->nd_end = yyval.node;
		    }
break;
case 458:
#line 2282 "parse.y"
{
			yyval.node = block_append(yyvsp[-2].node, yyvsp[0].node);
		    }
break;
case 461:
#line 2292 "parse.y"
{
			if (!is_local_id(yyvsp[0].id))
			    yyerror("rest argument must be local variable");
			else if (local_id(yyvsp[0].id))
			    yyerror("duplicate rest argument name");
			yyval.id = local_cnt(yyvsp[0].id);
		    }
break;
case 462:
#line 2300 "parse.y"
{
			yyval.id = -2;
		    }
break;
case 465:
#line 2310 "parse.y"
{
			if (!is_local_id(yyvsp[0].id))
			    yyerror("block argument must be local variable");
			else if (local_id(yyvsp[0].id))
			    yyerror("duplicate block argument name");
			yyval.node = NEW_BLOCK_ARG(yyvsp[0].id);
		    }
break;
case 466:
#line 2320 "parse.y"
{
			yyval.node = yyvsp[0].node;
		    }
break;
case 468:
#line 2327 "parse.y"
{
			if (nd_type(yyvsp[0].node) == NODE_SELF) {
			    yyval.node = NEW_SELF();
			}
			else {
			    yyval.node = yyvsp[0].node;
		            value_expr(yyval.node);
			}
		    }
break;
case 469:
#line 2336 "parse.y"
{lex_state = EXPR_BEG;}
break;
case 470:
#line 2337 "parse.y"
{
			if (yyvsp[-2].node == 0) {
			    yyerror("can't define singleton method for ().");
			}
			else {
			    switch (nd_type(yyvsp[-2].node)) {
			      case NODE_STR:
			      case NODE_DSTR:
			      case NODE_XSTR:
			      case NODE_DXSTR:
			      case NODE_DREGX:
			      case NODE_LIT:
			      case NODE_ARRAY:
			      case NODE_ZARRAY:
				yyerror("can't define singleton method for literals");
			      default:
				value_expr(yyvsp[-2].node);
				break;
			    }
			}
			yyval.node = yyvsp[-2].node;
		    }
break;
case 472:
#line 2363 "parse.y"
{
			yyval.node = yyvsp[-1].node;
		    }
break;
case 473:
#line 2367 "parse.y"
{
			if (yyvsp[-1].node->nd_alen%2 != 0) {
			    yyerror("odd number list for Hash");
			}
			yyval.node = yyvsp[-1].node;
		    }
break;
case 475:
#line 2377 "parse.y"
{
			yyval.node = list_concat(yyvsp[-2].node, yyvsp[0].node);
		    }
break;
case 476:
#line 2383 "parse.y"
{
			yyval.node = list_append(NEW_LIST(yyvsp[-2].node), yyvsp[0].node);
		    }
break;
case 496:
#line 2421 "parse.y"
{yyerrok;}
break;
case 499:
#line 2426 "parse.y"
{yyerrok;}
break;
case 500:
#line 2429 "parse.y"
{yyval.node = 0;}
break;
#line 11532 "y.tab.c"
    }
    yyssp -= yym;
    yystate = *yyssp;
    yyvsp -= yym;
    yym = yylhs[yyn];
    if (yystate == 0 && yym == 0)
    {
#if YYDEBUG
        if (yydebug)
            printf("%sdebug: after reduction, shifting from state 0 to\
 state %d\n", YYPREFIX, YYFINAL);
#endif
        yystate = YYFINAL;
        *++yyssp = YYFINAL;
        *++yyvsp = yyval;
        if (yychar < 0)
        {
            if ((yychar = yylex()) < 0) yychar = 0;
#if YYDEBUG
            if (yydebug)
            {
                yys = 0;
                if (yychar <= YYMAXTOKEN) yys = yyname[yychar];
                if (!yys) yys = "illegal-symbol";
                printf("%sdebug: state %d, reading %d (%s)\n",
                        YYPREFIX, YYFINAL, yychar, yys);
            }
#endif
        }
        if (yychar == 0) goto yyaccept;
        goto yyloop;
    }
    if ((yyn = yygindex[yym]) && (yyn += yystate) >= 0 &&
            yyn <= YYTABLESIZE && yycheck[yyn] == yystate)
        yystate = yytable[yyn];
    else
        yystate = yydgoto[yym];
#if YYDEBUG
    if (yydebug)
        printf("%sdebug: after reduction, shifting from state %d \
to state %d\n", YYPREFIX, *yyssp, yystate);
#endif
    if (yyssp >= yysslim && yygrowstack())
    {
        goto yyoverflow;
    }
    *++yyssp = yystate;
    *++yyvsp = yyval;
    goto yyloop;
yyoverflow:
    yyerror("yacc stack overflow");
yyabort:
    return (1);
yyaccept:
    return (0);
}
